      subroutine cfftb(n,c,wsave)
      implicit real*8(a-h,o-z)
      parameter(precis=2.)
      parameter(zero=0.d0, one=1.d0, oneneg=-1.d0, two=2.d0, three=3.d0,
     1          four=4.d0, five=5.d0, six=6.d0, seven=7.d0, eight=8.d0,
     2          nine=9.d0, ten=10.d0,
     3          half=0.5d0, quarter=.25d0, eighth=.125d0, tenth=.1d0,
     4          threq=.75d0,
     5          hundr=100.0d0, hundinv=.01d0)
c***begin prologue  cfftb
c***date written   790601   (yymmdd)
c***revision date  830401   (yymmdd)
c***category no.  j1a2
c***keywords  fourier transform
c***author  swarztrauber, p. n., (ncar)
c***purpose  unnormalized inverse of cfftf.
c***description
c
c  subroutine cfftb computes the backward complex discrete fourier
c  transform (the fourier synthesis).  equivalently, cfftb computes
c  a complex periodic sequence from its fourier coefficients.
c  the transform is defined below at output parameter c.
c
c  a call of cfftf followed by a call of cfftb will multiply the
c  sequence by n.
c
c  the array wsave which is used by subroutine cfftb must be
c  initialized by calling subroutine cffti(n,wsave).
c
c  input parameters
c
c
c  n      the length of the complex sequence c.  the method is
c         more efficient when n is the product of small primes.
c
c  c      a complex array of length n which contains the sequence
c
c  wsave   a real work array which must be dimensioned at least 4*n+15
c          in the program that calls cfftb.  the wsave array must be
c          initialized by calling subroutine cffti(n,wsave), and a
c          different wsave array must be used for each different
c          value of n.  this initialization does not have to be
c          repeated so long as n remains unchanged.  thus subsequent
c          transforms can be obtained faster than the first.
c          the same wsave array can be used by cfftf and cfftb.
c
c  output parameters
c
c  c      for j=1,...,n
c
c             c(j)=the sum from k=1,...,n of
c
c                   c(k)*dexp(i*j*k*2*pi/n)
c
c                         where i=dsqrt(-1)
c
c  wsave   contains initialization calculations which must not be
c          destroyed between calls of subroutine cfftf or cfftb
c***references  (none)
c***routines called  cfftb1
c***end prologue  cfftb
      dimension       c(*)       ,wsave(*)
c***first executable statement  cfftb
      if (n .eq. 1) return
      iw1 = n+n+1
      iw2 = iw1+n+n
      call cfftb1 (n,c,wsave,wsave(iw1),wsave(iw2))
      return
      end
