      subroutine cosqb(n,x,wsave)
      implicit real*8(a-h,o-z)
      parameter(precis=2.)
      parameter(zero=0.d0, one=1.d0, oneneg=-1.d0, two=2.d0, three=3.d0,     
     1          four=4.d0, five=5.d0, six=6.d0, seven=7.d0, eight=8.d0,
     2          nine=9.d0, ten=10.d0,
     3          half=0.5d0, quarter=.25d0, eighth=.125d0, tenth=.1d0,     
     4          threq=.75d0,     
     5          hundr=100.0d0, hundinv=.01d0)
c***begin prologue  cosqb
c***date written   790601   (yymmdd)
c***revision date  830401   (yymmdd)
c***category no.  j1a3
c***keywords  fourier transform
c***author  swarztrauber, p. n., (ncar)
c***purpose  unnormalized inverse of cosqf.
c***description
c
c  subroutine cosqb computes the fast fourier transform of quarter
c  wave data. that is, cosqb computes a sequence from its
c  representation in terms of a cosine series with odd wave numbers.
c  the transform is defined below at output parameter x.
c
c  cosqb is the unnormalized inverse of cosqf since a call of cosqb
c  followed by a call of cosqf will multiply the input sequence x
c  by 4*n.
c
c  the array wsave which is used by subroutine cosqb must be
c  initialized by calling subroutine cosqi(n,wsave).
c
c
c  input parameters
c
c  n       the length of the array x to be transformed.  the method
c          is most efficient when n is a product of small primes.
c
c  x       an array which contains the sequence to be transformed
c
c  wsave   a work array that must be dimensioned at least 3*n+15
c          in the program that calls cosqb.  the wsave array must be
c          initialized by calling subroutine cosqi(n,wsave), and a
c          different wsave array must be used for each different
c          value of n.  this initialization does not have to be
c          repeated so long as n remains unchanged.  thus subsequent
c          transforms can be obtained faster than the first.
c
c  output parameters
c
c  x       for i=1,...,n
c
c               x(i)= the sum from k=1 to k=n of
c
c                 4*x(k)*dcos((2*k-1)*(i-1)*pi/(2*n))
c
c               a call of cosqb followed by a call of
c               cosqf will multiply the sequence x by 4*n.
c               therefore cosqf is the unnormalized inverse
c               of cosqb.
c
c  wsave   contains initialization calculations which must not
c          be destroyed between calls of cosqb or cosqf.
c***references  (none)
c***routines called  cosqb1
c***end prologue  cosqb
      dimension       x(*)       ,wsave(*)
      data tsqrt2 /2.82842712474619/
c***first executable statement  cosqb
      if (n-2) 101,102,103
  101 x(1) = 4.*x(1)
      return
  102 x1 = 4.*(x(1)+x(2))
      x(2) = tsqrt2*(x(1)-x(2))
      x(1) = x1
      return
  103 call cosqb1 (n,x,wsave,wsave(n+1))
      return
      end
