      subroutine cosqf(n,x,wsave)
      implicit real*8(a-h,o-z)
      parameter(precis=2.)
      parameter(zero=0.d0, one=1.d0, oneneg=-1.d0, two=2.d0, three=3.d0,     
     1          four=4.d0, five=5.d0, six=6.d0, seven=7.d0, eight=8.d0,     
     2          nine=9.d0, ten=10.d0,     
     3          half=0.5d0, quarter=.25d0, eighth=.125d0, tenth=.1d0,     
     4          threq=.75d0,     
     5          hundr=100.0d0, hundinv=.01d0)
c***begin prologue  cosqf
c***date written   790601   (yymmdd)
c***revision date  830401   (yymmdd)
c***category no.  j1a3
c***keywords  fourier transform
c***author  swarztrauber, p. n., (ncar)
c***purpose  forward cosine transform with odd wave numbers.
c***description
c
c  subroutine cosqf computes the fast fourier transform of quarter
c  wave data. that is, cosqf computes the coefficients in a cosine
c  series representation with only odd wave numbers.  the transform
c  is defined below at output parameter x
c
c  cosqf is the unnormalized inverse of cosqb since a call of cosqf
c  followed by a call of cosqb will multiply the input sequence x
c  by 4*n.
c
c  the array wsave which is used by subroutine cosqf must be
c  initialized by calling subroutine cosqi(n,wsave).
c
c
c  input parameters
c
c  n       the length of the array x to be transformed.  the method
c          is most efficient when n is a product of small primes.
c
c  x       an array which contains the sequence to be transformed
c
c  wsave   a work array which must be dimensioned at least 3*n+15
c          in the program that calls cosqf.  the wsave array must be
c          initialized by calling subroutine cosqi(n,wsave), and a
c          different wsave array must be used for each different
c          value of n.  this initialization does not have to be
c          repeated so long as n remains unchanged.  thus subsequent
c          transforms can be obtained faster than the first.
c
c  output parameters
c
c  x       for i=1,...,n
c
c               x(i) = x(1) plus the sum from k=2 to k=n of
c
c                  2*x(k)*dcos((2*i-1)*(k-1)*pi/(2*n))
c
c               a call of cosqf followed by a call of
c               cosqb will multiply the sequence x by 4*n.
c               therefore cosqb is the unnormalized inverse
c               of cosqf.
c
c  wsave   contains initialization calculations which must not
c          be destroyed between calls of cosqf or cosqb.
c***references  (none)
c***routines called  cosqf1
c***end prologue  cosqf
      dimension       x(*)       ,wsave(*)
      data sqrt2 /1.4142135623731/
c***first executable statement  cosqf
      if (n-2) 102,101,103
  101 tsqx = sqrt2*x(2)
      x(2) = x(1)-tsqx
      x(1) = x(1)+tsqx
  102 return
  103 call cosqf1 (n,x,wsave,wsave(n+1))
      return
      end
