      subroutine costi(n,wsave)
      implicit real*8(a-h,o-z)
      parameter(precis=2.)
      parameter(zero=0.d0, one=1.d0, oneneg=-1.d0, two=2.d0, three=3.d0,
     1          four=4.d0, five=5.d0, six=6.d0, seven=7.d0, eight=8.d0,
     2          nine=9.d0, ten=10.d0,
     3          half=0.5d0, quarter=.25d0, eighth=.125d0, tenth=.1d0,
     4          threq=.75d0,
     5          hundr=100.0d0, hundinv=.01d0)
c***begin prologue  costi
c***date written   790601   (yymmdd)
c***revision date  830401   (yymmdd)
c***category no.  j1a3
c***keywords  fourier transform
c***author  swarztrauber, p. n., (ncar)
c***purpose  initialize for cost.
c***description
c
c  subroutine costi initializes the array wsave which is used in
c  subroutine cost.  the prime factorization of n together with
c  a tabulation of the trigonometric functions are computed and
c  stored in wsave.
c
c  input parameter
c
c  n       the length of the sequence to be transformed.  the method
c          is most efficient when n-1 is a product of small primes.
c
c  output parameter
c
c  wsave   a work array which must be dimensioned at least 3*n+15.
c          different wsave arrays are required for different values
c          of n.  the contents of wsave must not be changed between
c          calls of cost.
c***references  (none)
c***routines called  rffti
c***end prologue  costi
      dimension       wsave(*)
      data pi /3.14159265358979/
c***first executable statement  costi
      if (n .le. 3) return
      nm1 = n-1
      np1 = n+1
      ns2 = n/2
      dt = pi/dble(nm1)
      fk = zero
      do 101 k=2,ns2
         kc = np1-k
         fk = fk+one
         wsave(k) = two*dsin(fk*dt)
         wsave(kc) = two*dcos(fk*dt)
  101 continue
      call rffti (nm1,wsave(n+1))
      return
      end
