      subroutine rfftb(n,r,wsave)
      implicit real*8(a-h,o-z)
      parameter(precis=2.)
      parameter(zero=0.d0, one=1.d0, oneneg=-1.d0, two=2.d0, three=3.d0,
     1          four=4.d0, five=5.d0, six=6.d0, seven=7.d0, eight=8.d0,
     2          nine=9.d0, ten=10.d0,
     3          half=0.5d0, quarter=.25d0, eighth=.125d0, tenth=.1d0,
     4          threq=.75d0,
     5          hundr=100.0d0, hundinv=.01d0)
c***begin prologue  rfftb
c***date written   790601   (yymmdd)
c***revision date  830401   (yymmdd)
c***category no.  j1a1
c***keywords  fourier transform
c***author  swarztrauber, p. n., (ncar)
c***purpose  backward transform of a real coefficient array.
c***description
c
c  subroutine rfftb computes the real perodic sequence from its
c  fourier coefficients (fourier synthesis).  the transform is defined
c  below at output parameter r.
c
c  input parameters
c
c  n       the length of the array r to be transformed.  the method
c          is most efficient when n is a product of small primes.
c          n may change so long as different work arrays are provided.
c
c  r       a real array of length n which contains the sequence
c          to be transformed
c
c  wsave   a work array which must be dimensioned at least 2*n+15
c          in the program that calls rfftb.  the wsave array must be
c          initialized by calling subroutine rffti(n,wsave), and a
c          different wsave array must be used for each different
c          value of n.  this initialization does not have to be
c          repeated so long as n remains unchanged.  thus subsequent
c          transforms can be obtained faster than the first.
c          the same wsave array can be used by rfftf and rfftb.
c
c
c  output parameters
c
c  r       for n even and for i = 1,...,n
c
c               r(i) = r(1)+(-1)**(i-1)*r(n)
c
c                    plus the sum from k=2 to k=n/2 of
c
c                     2.*r(2*k-2)*dcos((k-1)*(i-1)*2*pi/n)
c
c                    -2.*r(2*k-1)*dsin((k-1)*(i-1)*2*pi/n)
c
c          for n odd and for i = 1,...,n
c
c               r(i) = r(1) plus the sum from k=2 to k=(n+1)/2 of
c
c                    2.*r(2*k-2)*dcos((k-1)*(i-1)*2*pi/n)
c
c                   -2.*r(2*k-1)*dsin((k-1)*(i-1)*2*pi/n)
c
c   *****  note:
c               this transform is unnormalized since a call of rfftf
c               followed by a call of rfftb will multiply the input
c               sequence by n.
c
c  wsave   contains results which must not be destroyed between
c          calls of rfftb or rfftf.
c***references  (none)
c***routines called  rfftb1
c***end prologue  rfftb
      dimension       r(*)       ,wsave(*)
c***first executable statement  rfftb
      if (n .eq. 1) return
      call rfftb1 (n,r,wsave,wsave(n+1),wsave(2*n+1))
      return
      end
