      subroutine rfftf(n,r,wsave)
      implicit real*8(a-h,o-z)
      parameter(precis=2.)
      parameter(zero=0.d0, one=1.d0, oneneg=-1.d0, two=2.d0, three=3.d0,
     1          four=4.d0, five=5.d0, six=6.d0, seven=7.d0, eight=8.d0,
     2          nine=9.d0, ten=10.d0,
     3          half=0.5d0, quarter=.25d0, eighth=.125d0, tenth=.1d0,
     4          threq=.75d0,
     5          hundr=100.0d0, hundinv=.01d0)
c***begin prologue  rfftf
c***date written   790601   (yymmdd)
c***revision date  830401   (yymmdd)
c***category no.  j1a1
c***keywords  fourier transform
c***author  swarztrauber, p. n., (ncar)
c***purpose  forward transform of a real, periodic sequence.
c***description
c
c  subroutine rfftf computes the fourier coefficients of a real
c  perodic sequence (fourier analysis).  the transform is defined
c  below at output parameter r.
c
c  input parameters
c
c  n       the length of the array r to be transformed.  the method
c          is most efficient when n is a product of small primes.
c          n may change so long as different work arrays are provided
c
c  r       a real array of length n which contains the sequence
c          to be transformed
c
c  wsave   a work array which must be dimensioned at least 2*n+15
c          in the program that calls rfftf.  the wsave array must be
c          initialized by calling subroutine rffti(n,wsave), and a
c          different wsave array must be used for each different
c          value of n.  this initialization does not have to be
c          repeated so long as n remains unchanged.  thus subsequent
c          transforms can be obtained faster than the first.
c          the same wsave array can be used by rfftf and rfftb.
c
c
c  output parameters
c
c  r       r(1) = the sum from i=1 to i=n of r(i)
c
c          if n is even set l = n/2; if n is odd set l = (n+1)/2
c
c            then for k = 2,...,l
c
c               r(2*k-2) = the sum from i = 1 to i = n of
c
c                    r(i)*dcos((k-1)*(i-1)*2*pi/n)
c
c               r(2*k-1) = the sum from i = 1 to i = n of
c
c                   -r(i)*dsin((k-1)*(i-1)*2*pi/n)
c
c          if n is even
c
c               r(n) = the sum from i = 1 to i = n of
c
c                    (-1)**(i-1)*r(i)
c
c   *****  note:
c               this transform is unnormalized since a call of rfftf
c               followed by a call of rfftb will multiply the input
c               sequence by n.
c
c  wsave   contains results which must not be destroyed between
c          calls of rfftf or rfftb.
c***references  (none)
c***routines called  rfftf1
c***end prologue  rfftf
      dimension       r(*)       ,wsave(*)
c***first executable statement  rfftf
      if (n .eq. 1) return
      call rfftf1 (n,r,wsave,wsave(n+1),wsave(2*n+1))
      return
      end
