      subroutine sinqf(n,x,wsave)
      implicit real*8(a-h,o-z)
      parameter(precis=2.)
      parameter(zero=0.d0, one=1.d0, oneneg=-1.d0, two=2.d0, three=3.d0,
     1         four=4.d0, five=5.d0, six=6.d0, seven=7.d0, eight=8.d0,
     2         nine=9.d0, ten=10.d0,
     3         half=0.5d0, quarter=0.25d0, eighth=.125d0, tenth=.1d0,
     4         threq=.75d0,
     5         hundr=100.0d0, hundinv=.01d0)
c***begin prologue  sinqf
c***date written   790601   (yymmdd)
c***revision date  830401   (yymmdd)
c***category no.  j1a3
c***keywords  fourier transform
c***author  swarztrauber, p. n., (ncar)
c***purpose  forward sine transform with odd wave numbers.
c***description
c
c  subroutine sinqf computes the fast fourier transform of quarter
c  wave data.  that is, sinqf computes the coefficients in a sine
c  series representation with only odd wave numbers.  the transform
c  is defined below at output parameter x.
c
c  sinqb is the unnormalized inverse of sinqf since a call of sinqf
c  followed by a call of sinqb will multiply the input sequence x
c  by 4*n.
c
c  the array wsave which is used by subroutine sinqf must be
c  initialized by calling subroutine sinqi(n,wsave).
c
c
c  input parameters
c
c  n       the length of the array x to be transformed.  the method
c          is most efficient when n is a product of small primes.
c
c  x       an array which contains the sequence to be transformed
c
c  wsave   a work array which must be dimensioned at least 3*n+15
c          in the program that calls sinqf.  the wsave array must be
c          initialized by calling subroutine sinqi(n,wsave), and a
c          different wsave array must be used for each different
c          value of n.  this initialization does not have to be
c          repeated so long as n remains unchanged.  thus subsequent
c          transforms can be obtained faster than the first.
c
c  output parameters
c
c  x       for i=1,...,n
c
c               x(i) = (-1)**(i-1)*x(n)
c
c                  + the sum from k=1 to k=n-1 of
c
c                  2*x(k)*dsin((2*i-1)*k*pi/(2*n))
c
c               a call of sinqf followed by a call of
c               sinqb will multiply the sequence x by 4*n.
c               therefore sinqb is the unnormalized inverse
c               of sinqf.
c
c  wsave   contains initialization calculations which must not
c          be destroyed between calls of sinqf or sinqb.
c***references  (none)
c***routines called  cosqf
c***end prologue  sinqf
      dimension       x(*)       ,wsave(*)
c***first executable statement  sinqf
      if (n .eq. 1) return
      ns2 = n/2
      do 101 k=1,ns2
         kc = n-k
         xhold = x(k)
         x(k) = x(kc+1)
         x(kc+1) = xhold
  101 continue
      call cosqf (n,x,wsave)
      do 102 k=2,n,2
         x(k) = -x(k)
  102 continue
      return
      end
