      subroutine sint(n,x,wsave)
      implicit real*8(a-h,o-z)
      parameter(precis=2.)
      parameter(zero=0.d0, one=1.d0, oneneg=-1.d0, two=2.d0, three=3.d0,
     1          four=4.d0, five=5.d0, six=6.d0, seven=7.d0, eight=8.d0,
     2          nine=9.d0, ten=10.d0,
     3          half=0.5d0, quarter=.25d0, eighth=.125d0, tenth=.1d0,
     4          threq=.75d0,
     5          hundr=100.0d0, hundinv=.01d0)
c***begin prologue  sint
c***date written   790601   (yymmdd)
c***revision date  830401   (yymmdd)
c***category no.  j1a3
c***keywords  fourier transform
c***author  swarztrauber, p. n., (ncar)
c***purpose  sine transform of a real, odd sequence.
c***description
c
c  subroutine sint computes the discrete fourier sine transform
c  of an odd sequence x(i).  the transform is defined below at
c  output parameter x.
c
c  sint is the unnormalized inverse of itself since a call of sint
c  followed by another call of sint will multiply the input sequence
c  x by 2*(n+1).
c
c  the array wsave which is used by subroutine sint must be
c  initialized by calling subroutine sinti(n,wsave).
c
c  input parameters
c
c  n       the length of the sequence to be transformed.  the method
c          is most efficient when n+1 is the product of small primes.
c
c  x       an array which contains the sequence to be transformed
c
c
c  wsave   a work array with dimension at least idint(3.5*n+16)
c          in the program that calls sint.  the wsave array must be
c          initialized by calling subroutine sinti(n,wsave), and a
c          different wsave array must be used for each different
c          value of n.  this initialization does not have to be
c          repeated so long as n remains unchanged.  thus subsequent
c          transforms can be obtained faster than the first.
c
c  output parameters
c
c  x       for i=1,...,n
c
c               x(i)= the sum from k=1 to k=n
c
c                    2*x(k)*dsin(k*i*pi/(n+1))
c
c               a call of sint followed by another call of
c               sint will multiply the sequence x by 2*(n+1).
c               hence sint is the unnormalized inverse
c               of itself.
c
c  wsave   contains initialization calculations which must not be
c          destroyed between calls of sint.
c***references  (none)
c***routines called  rfftf
c***end prologue  sint
      dimension       x(*)       ,wsave(*)
      data sqrt3 /1.73205080756888/
c***first executable statement  sint
      if (n-2) 101,102,103
  101 x(1) = x(1)+x(1)
      return
  102 xh = sqrt3*(x(1)+x(2))
      x(2) = sqrt3*(x(1)-x(2))
      x(1) = xh
      return
  103 np1 = n+1
      ns2 = n/2
      wsave(1) = zero
      kw = np1
      do 104 k=1,ns2
1        kw = kw+1
         kc = np1-k
         t1 = x(k)-x(kc)
         t2 = wsave(kw)*(x(k)+x(kc))
         wsave(k+1) = t1+t2
         wsave(kc+1) = t2-t1
  104 continue
      modn = mod(n,2)
      if (modn .ne. 0) wsave(ns2+2) = 4.*x(ns2+1)
      nf = np1+ns2+1
      call rfftf (np1,wsave,wsave(nf))
      x(1) = .5*wsave(1)
      do 105 i=3,n,2
         x(i-1) = -wsave(i)
         x(i) = x(i-2)+wsave(i-1)
  105 continue
      if (modn .ne. 0) return
      x(n) = -wsave(n+1)
      return
      end
