      subroutine dgefs(a,lda,n,v,itask,ind,work,iwork)                  dgefs  3
c***begin prologue  dgefs                                               dgefs  4
c***date written   800326   (yymmdd)                                    dgefs  5
c***revision date  861211   (yymmdd)                                    bb812406
c***category no.  d2a1                                                  bb812407
c***keywords  library=slatec,                                           bb812408
c             type=double precision(sgefs-s dgefs-d cgefs-c),           bb812409
c             complex linear equations,general matrix,                  bb812410
c             general system of linear equations,linear equations       dgefs  9
c***author  voorhees, e., (lanl)                                        dgefs 10
c***purpose  dgefs solves a general double precision                    dgefs 11
c            nxn system of linear equations.                            dgefs 12
c***description                                                         dgefs 13
c                                                                       dgefs 14
c    subroutine dgefs solves a general nxn system of double             dgefs 15
c    precision linear equations using linpack subroutines dgeco         dgefs 16
c    and dgesl.  that is, if a is an nxn double precision matrix        dgefs 17
c    and if x and b are double precision n-vectors, then dgefs          dgefs 18
c    solves the equation                                                dgefs 19
c                                                                       dgefs 20
c                          a*x=b.                                       dgefs 21
c                                                                       dgefs 22
c    the matrix a is first factored into upper and lower tri-           dgefs 23
c    angular matrices u and l using partial pivoting.  these            dgefs 24
c    factors and the pivoting information are used to find the          dgefs 25
c    solution vector x.  an approximate condition number is             dgefs 26
c    calculated to provide a rough estimate of the number of            dgefs 27
c    digits of accuracy in the computed solution.                       dgefs 28
c                                                                       dgefs 29
c    if the equation a*x=b is to be solved for more than one vector     dgefs 30
c    b, the factoring of a does not need to be performed again and      dgefs 31
c    the option to only solve (itask.gt.1) will be faster for           dgefs 32
c    the succeeding solutions.  in this case, the contents of a,        dgefs 33
c    lda, n and iwork must not have been altered by the user follow-    dgefs 34
c    ing factorization (itask=1).  ind will not be changed by dgefs     dgefs 35
c    in this case.                                                      dgefs 36
c                                                                       dgefs 37
c  argument description ***                                             dgefs 38
c                                                                       dgefs 39
c    a      double precision(lda,n)                                     dgefs 40
c             on entry, the doubly subscripted array with dimension     dgefs 41
c               (lda,n) which contains the coefficient matrix.          dgefs 42
c             on return, an upper triangular matrix u and the           dgefs 43
c               multipliers necessary to construct a matrix l           dgefs 44
c               so that a=l*u.                                          dgefs 45
c    lda    integer                                                     dgefs 46
c             the leading dimension of the array a.  lda must be great- dgefs 47
c             er than or equal to n.  (terminal error message ind=-1)   dgefs 48
c    n      integer                                                     dgefs 49
c             the order of the matrix a.  the first n elements of       dgefs 50
c             the array a are the elements of the first column of       dgefs 51
c             the matrix a.  n must be greater than or equal to 1.      dgefs 52
c             (terminal error message ind=-2)                           dgefs 53
c    v      double precision(n)                                         dgefs 54
c             on entry, the singly subscripted array(vector) of di-     dgefs 55
c               mension n which contains the right hand side b of a     dgefs 56
c               system of simultaneous linear equations a*x=b.          dgefs 57
c             on return, v contains the solution vector, x .            dgefs 58
c    itask  integer                                                     dgefs 59
c             if itask=1, the matrix a is factored and then the         dgefs 60
c               linear equation is solved.                              dgefs 61
c             if itask .gt. 1, the equation is solved using the existingdgefs 62
c               factored matrix a and iwork.                            dgefs 63
c             if itask .lt. 1, then terminal error message ind=-3 is    dgefs 64
c               printed.                                                dgefs 65
c    ind    integer                                                     dgefs 66
c             gt. 0  ind is a rough estimate of the number of digits    dgefs 67
c                     of accuracy in the solution, x.                   dgefs 68
c             lt. 0  see error message corresponding to ind below.      dgefs 69
c    work   double precision(n)                                         dgefs 70
c             a singly subscripted array of dimension at least n.       dgefs 71
c    iwork  integer(n)                                                  dgefs 72
c             a singly subscripted array of dimension at least n.       dgefs 73
c                                                                       dgefs 74
c  error messages printed ***                                           dgefs 75
c                                                                       dgefs 76
c    ind=-1  terminal   n is greater than lda.                          dgefs 77
c    ind=-2  terminal   n is less than 1.                               dgefs 78
c    ind=-3  terminal   itask is less than 1.                           dgefs 79
c    ind=-4  terminal   the matrix a is computationally singular.       dgefs 80
c                         a solution has not been computed.             dgefs 81
c    ind=-10 warning    the solution has no apparent significance.      dgefs 82
c                         the solution may be inaccurate or the matrix  dgefs 83
c                         a may be poorly scaled.                       dgefs 84
c                                                                       dgefs 85
c               note-  the above terminal(*fatal*) error messages are   dgefs 86
c                      designed to be handled by xerrwv in which        dgefs 87
c                      level=1 (recoverable) and iflag=2 .  level=0     dgefs 88
c                      for warning error messages from xerror.  unless  dgefs 89
c                      the user provides otherwise, an error message    dgefs 90
c                      will be printed followed by an abort.            dgefs 91
c***references  subroutine dgefs was developed by group c-3, los alamos dgefs 92
c                 scientific laboratory, los alamos, nm 87545.          dgefs 93
c                 the linpack subroutines used by dgefs are described indgefs 94
c                 detail in the *linpack users guide* published by      dgefs 95
c                 the society for industrial and applied mathematics    dgefs 96
c                 (siam) dated 1979.                                    dgefs 97
c***routines called  d1mach,dgeco,dgesl,xerror,xerrwv                   dgefs 98
c***end prologue  dgefs                                                 dgefs 99
c                                                                       dgefs100
      integer lda,n,itask,ind,iwork(n)                                  dgefs101
      double precision a(lda,n),v(n),work(n),d1mach                     dgefs102
      double precision rcond                                            dgefs103
c***first executable statement  dgefs                                   dgefs104
      if (lda.lt.n)  go to 101                                          dgefs105
      if (n.le.0)  go to 102                                            dgefs106
      if (itask.lt.1) go to 103                                         dgefs107
      if (itask.gt.1) go to 20                                          dgefs108
c                                                                       dgefs109
c     factor matrix a into lu                                           dgefs110
      call dgeco(a,lda,n,iwork,rcond,work)                              dgefs111
c                                                                       dgefs112
c     check for computationally singular matrix                         dgefs113
      if (rcond.eq.0.0d0)  go to 104                                    dgefs114
c                                                                       dgefs115
c     compute ind (estimate of no. of significant digits)               dgefs116
      ind=-idint(dlog10(d1mach(4)/rcond))                               dgefs117
c                                                                       dgefs118
c     check for ind greater than zero                                   dgefs119
      if (ind.gt.0)  go to 20                                           dgefs120
      ind=-10                                                           dgefs121
      call xerror( 'dgefs error (ind=-10) -- solution may have no signifdgefs122
     1icance',58,-10,0)                                                 dgefs123
c                                                                       dgefs124
c     solve after factoring                                             dgefs125
   20 call dgesl(a,lda,n,iwork,v,0)                                     dgefs126
      return                                                            dgefs127
c                                                                       dgefs128
c     if lda.lt.n, ind=-1, terminal xerrwv message                      dgefs129
  101 ind=-1                                                            dgefs130
      call xerrwv( 'dgefs error (ind=-1) -- lda=i1 is less than n=i2',  dgefs131
     148,-1,1,2,lda,n,0,0,0)                                            dgefs132
      return                                                            dgefs133
c                                                                       dgefs134
c     if n.lt.1, ind=-2, terminal xerrwv message                        dgefs135
  102 ind=-2                                                            dgefs136
      call xerrwv( 'dgefs error (ind=-2) -- n=i1 is less than 1',       dgefs137
     143,-2,1,1,n,0,0,0,0)                                              dgefs138
      return                                                            dgefs139
c                                                                       dgefs140
c     if itask.lt.1, ind=-3, terminal xerrwv message                    dgefs141
  103 ind=-3                                                            dgefs142
      call xerrwv( 'dgefs error (ind=-3) -- itask=i1 is less than 1',   dgefs143
     147,-3,1,1,itask,0,0,0,0)                                          dgefs144
      return                                                            dgefs145
c                                                                       dgefs146
c     if singular matrix, ind=-4, terminal xerrwv message               dgefs147
  104 ind=-4                                                            dgefs148
      call xerrwv( 'dgefs error (ind=-4) -- singular matrix a - no solutdgefs149
     1ion',55,-4,1,0,0,0,0,0,0)                                         dgefs150
      return                                                            dgefs151
c                                                                       dgefs152
      end                                                               dgefs153
