      subroutine dnbco(abe,lda,n,ml,mu,ipvt,rcond,z)                    dnbco  3
c***begin prologue  dnbco                                               dnbco  4
c***date written   800728   (yymmdd)                                    dnbco  5
c***revision date  861211   (yymmdd)                                    bb812506
c***category no.  d2a2                                                  bb812507
c***keywords  library=slatec,                                           bb812508
c             type=double precision(snbco-s dnbco-d cnbco-c),banded,    bb812509
c             linear equations,nonsymmetric                             bb812510
c***author  voorhees, e., (lanl)                                        dnbco  9
c***purpose  factors a double precision band matrix by gaussian         dnbco 10
c            elimination and estimates the condition of the matrix.     dnbco 11
c***description                                                         dnbco 12
c                                                                       dnbco 13
c     dnbco factors a double precision band matrix by gaussian          dnbco 14
c     elimination and estimates the condition of the matrix.            dnbco 15
c                                                                       dnbco 16
c     if rcond is not needed, dnbfa is slightly faster.                 dnbco 17
c     to solve  a*x = b , follow dnbco by dnbsl.                        dnbco 18
c     to compute  inverse(a)*c , follow dnbco by dnbsl.                 dnbco 19
c     to compute  determinant(a) , follow dnbco by dnbdi.               dnbco 20
c                                                                       dnbco 21
c     on entry                                                          dnbco 22
c                                                                       dnbco 23
c        abe     double precision(lda, nc)                              dnbco 24
c                contains the matrix in band storage.  the rows         dnbco 25
c                of the original matrix are stored in the rows          dnbco 26
c                of abe and the diagonals of the original matrix        dnbco 27
c                are stored in columns 1 through ml+mu+1 of abe.        dnbco 28
c                nc must be .ge. 2*ml+mu+1 .                            dnbco 29
c                see the comments below for details.                    dnbco 30
c                                                                       dnbco 31
c        lda     integer                                                dnbco 32
c                the leading dimension of the array abe.                dnbco 33
c                lda must be .ge.  n .                                  dnbco 34
c                                                                       dnbco 35
c        n       integer                                                dnbco 36
c                the order of the original matrix.                      dnbco 37
c                                                                       dnbco 38
c        ml      integer                                                dnbco 39
c                number of diagonals below the main diagonal.           dnbco 40
c                0 .le. ml .lt.  n .                                    dnbco 41
c                                                                       dnbco 42
c        mu      integer                                                dnbco 43
c                number of diagonals above the main diagonal.           dnbco 44
c                0 .le. mu .lt.  n .                                    dnbco 45
c                more efficient if ml .le. mu .                         dnbco 46
c                                                                       dnbco 47
c     on return                                                         dnbco 48
c                                                                       dnbco 49
c        abe     an upper triangular matrix in band storage             dnbco 50
c                and the multipliers which were used to obtain it.      dnbco 51
c                the factorization can be written  a = l*u  where       dnbco 52
c                l is a product of permutation and unit lower           dnbco 53
c                triangular matrices and  u  is upper triangular.       dnbco 54
c                                                                       dnbco 55
c        ipvt    integer(n)                                             dnbco 56
c                an integer vector of pivot indices.                    dnbco 57
c                                                                       dnbco 58
c        rcond   double precision                                       dnbco 59
c                an estimate of the reciprocal condition of  a .        dnbco 60
c                for the system  a*x = b , relative perturbations       dnbco 61
c                in  a  and  b  of size  epsilon  may cause             dnbco 62
c                relative perturbations in  x  of size  epsilon/rcond . dnbco 63
c                if  rcond  is so small that the logical expression     dnbco 64
c                         1.0 + rcond .eq. 1.0                          dnbco 65
c                is true, then  a  may be singular to working           dnbco 66
c                precision.  in particular,  rcond  is zero  if         dnbco 67
c                exact singularity is detected or the estimate          dnbco 68
c                underflows.                                            dnbco 69
c                                                                       dnbco 70
c        z       double precision(n)                                    dnbco 71
c                a work vector whose contents are usually unimportant.  dnbco 72
c                if  a  is close to a singular matrix, then  z  is      dnbco 73
c                an approximate null vector in the sense that           dnbco 74
c                norm(a*z) = rcond*norm(a)*norm(z) .                    dnbco 75
c                                                                       dnbco 76
c     band storage                                                      dnbco 77
c                                                                       dnbco 78
c           if  a  is a band matrix, the following program segment      dnbco 79
c           will set up the input.                                      dnbco 80
c                                                                       dnbco 81
c                   ml = (band width below the diagonal)                dnbco 82
c                   mu = (band width above the diagonal)                dnbco 83
c                   do 20 i = 1, n                                      dnbco 84
c                      j1 = max0(1, i-ml)                               dnbco 85
c                      j2 = min0(n, i+mu)                               dnbco 86
c                      do 10 j = j1, j2                                 dnbco 87
c                         k = j - i + ml + 1                            dnbco 88
c                         abe(i,k) = a(i,j)                             dnbco 89
c                10    continue                                         dnbco 90
c                20 continue                                            dnbco 91
c                                                                       dnbco 92
c           this uses columns  1  through  ml+mu+1  of abe .            dnbco 93
c           furthermore,  ml  additional columns are needed in          dnbco 94
c           abe  starting with column  ml+mu+2  for elements            dnbco 95
c           generated during the triangularization.  the total          dnbco 96
c           number of columns needed in  abe  is  2*ml+mu+1 .           dnbco 97
c                                                                       dnbco 98
c     example:  if the original matrix is                               dnbco 99
c                                                                       dnbco100
c           11 12 13  0  0  0                                           dnbco101
c           21 22 23 24  0  0                                           dnbco102
c            0 32 33 34 35  0                                           dnbco103
c            0  0 43 44 45 46                                           dnbco104
c            0  0  0 54 55 56                                           dnbco105
c            0  0  0  0 65 66                                           dnbco106
c                                                                       dnbco107
c      then  n = 6, ml = 1, mu = 2, lda .ge. 5  and abe should contain  dnbco108
c                                                                       dnbco109
c            * 11 12 13  +     , * = not used                           dnbco110
c           21 22 23 24  +     , + = used for pivoting                  dnbco111
c           32 33 34 35  +                                              dnbco112
c           43 44 45 46  +                                              dnbco113
c           54 55 56  *  +                                              dnbco114
c           65 66  *  *  +                                              dnbco115
c                                                                       dnbco116
c     slatec.  this version dated 07/28/80 .                            dnbco117
c     e. a. voorhees, los alamos scientific laboratory                  dnbco118
c                                                                       dnbco119
c     subroutines and functions                                         dnbco120
c                                                                       dnbco121
c     slatec dnbfa                                                      dnbco122
c      ddot,daxpy,dscal,dasum                                           dnbco123
c     fortran  dabs,dmax1,max0,min0,dsign                               dnbco124
c***references  subroutine dnbco was developed by group c-3, los alamos dnbco125
c                 scientific laboratory, los alamos, nm 87545.          dnbco126
c***routines called  dasum,daxpy,ddot,dnbfa,dscal                       dnbco127
c***end prologue  dnbco                                                 dnbco128
      integer lda,n,ml,mu,ipvt(1)                                       dnbco129
      double precision abe(lda,1),z(1)                                  dnbco130
      double precision rcond                                            dnbco131
c                                                                       dnbco132
      double precision ddot,ek,t,wk,wkm                                 dnbco133
      double precision anorm,s,dasum,sm,ynorm                           dnbco134
      integer i,info,j,ju,k,kb,kp1,l,ldb,lm,lz,m,ml1,mm,nl,nu           dnbco135
c***first executable statement  dnbco                                   dnbco136
      ml1=ml+1                                                          dnbco137
      ldb = lda - 1                                                     dnbco138
      anorm = 0.0d0                                                     dnbco139
      do 10 j = 1, n                                                    dnbco140
        nu = min0(mu,j-1)                                               dnbco141
        nl = min0(ml,n-j)                                               dnbco142
        l = 1 + nu + nl                                                 dnbco143
        anorm = dmax1(anorm,dasum(l,abe(j+nl,ml1-nl),ldb))              dnbco144
   10 continue                                                          dnbco145
c                                                                       dnbco146
c     factor                                                            dnbco147
c                                                                       dnbco148
      call dnbfa(abe,lda,n,ml,mu,ipvt,info)                             dnbco149
c                                                                       dnbco150
c     rcond = 1/(norm(a)*(estimate of norm(inverse(a)))) .              dnbco151
c     estimate = norm(z)/norm(y) where  a*z = y  and trans(a)*y = e .   dnbco152
c     trans(a)  is the transpose of a .  the components of  e  are      dnbco153
c     chosen to cause maximum local growth in the elements of  w where  dnbco154
c     trans(u)*w = e .  the vectors are frequently rescaled to avoid    dnbco155
c     overflow.                                                         dnbco156
c                                                                       dnbco157
c     solve trans(u)*w = e                                              dnbco158
c                                                                       dnbco159
      ek = 1.0d0                                                        dnbco160
      do 20 j = 1, n                                                    dnbco161
        z(j) = 0.0d0                                                    dnbco162
   20 continue                                                          dnbco163
      m = ml + mu + 1                                                   dnbco164
      ju = 0                                                            dnbco165
      do 100 k = 1, n                                                   dnbco166
        if (z(k) .ne. 0.0d0) ek = dsign(ek,-z(k))                       dnbco167
        if (dabs(ek-z(k)) .le. dabs(abe(k,ml1))) go to 30               dnbco168
          s = dabs(abe(k,ml1))/dabs(ek-z(k))                            dnbco169
          call dscal(n,s,z,1)                                           dnbco170
          ek = s*ek                                                     dnbco171
   30   continue                                                        dnbco172
        wk = ek - z(k)                                                  dnbco173
        wkm = -ek - z(k)                                                dnbco174
        s = dabs(wk)                                                    dnbco175
        sm = dabs(wkm)                                                  dnbco176
        if (abe(k,ml1) .eq. 0.0d0) go to 40                             dnbco177
          wk = wk/abe(k,ml1)                                            dnbco178
          wkm = wkm/abe(k,ml1)                                          dnbco179
        go to 50                                                        dnbco180
   40   continue                                                        dnbco181
          wk = 1.0d0                                                    dnbco182
          wkm = 1.0d0                                                   dnbco183
   50   continue                                                        dnbco184
        kp1 = k + 1                                                     dnbco185
        ju = min0(max0(ju,mu+ipvt(k)),n)                                dnbco186
        mm = ml1                                                        dnbco187
        if (kp1 .gt. ju) go to 90                                       dnbco188
          do 60 i = kp1, ju                                             dnbco189
            mm = mm + 1                                                 dnbco190
            sm = sm + dabs(z(i)+wkm*abe(k,mm))                          dnbco191
            z(i) = z(i) + wk*abe(k,mm)                                  dnbco192
            s = s + dabs(z(i))                                          dnbco193
   60     continue                                                      dnbco194
          if (s .ge. sm) go to 80                                       dnbco195
            t = wkm -wk                                                 dnbco196
            wk = wkm                                                    dnbco197
            mm = ml1                                                    dnbco198
            do 70 i = kp1, ju                                           dnbco199
              mm = mm + 1                                               dnbco200
              z(i) = z(i) + t*abe(k,mm)                                 dnbco201
   70       continue                                                    dnbco202
   80     continue                                                      dnbco203
   90   continue                                                        dnbco204
      z(k) = wk                                                         dnbco205
  100 continue                                                          dnbco206
      s = 1.0d0/dasum(n,z,1)                                            dnbco207
      call dscal(n,s,z,1)                                               dnbco208
c                                                                       dnbco209
c     solve trans(l)*y = w                                              dnbco210
c                                                                       dnbco211
      do 120 kb = 1, n                                                  dnbco212
        k = n + 1 - kb                                                  dnbco213
        nl = min0(ml,n-k)                                               dnbco214
        if (k .lt. n) z(k) = z(k) + ddot(nl,abe(k+nl,ml1-nl),-ldb,z(k+1)dnbco215
     1  ,1)                                                             dnbco216
        if (dabs(z(k)) .le. 1.0d0) go to 110                            dnbco217
          s = 1.0d0/dabs(z(k))                                          dnbco218
          call dscal(n,s,z,1)                                           dnbco219
  110   continue                                                        dnbco220
        l = ipvt(k)                                                     dnbco221
        t = z(l)                                                        dnbco222
        z(l) = z(k)                                                     dnbco223
        z(k) = t                                                        dnbco224
  120 continue                                                          dnbco225
      s = 1.0d0/dasum(n,z,1)                                            dnbco226
      call dscal(n,s,z,1)                                               dnbco227
c                                                                       dnbco228
      ynorm = 1.0d0                                                     dnbco229
c                                                                       dnbco230
c     solve l*v = y                                                     dnbco231
c                                                                       dnbco232
      do 140 k = 1, n                                                   dnbco233
        l = ipvt(k)                                                     dnbco234
        t = z(l)                                                        dnbco235
        z(l) = z(k)                                                     dnbco236
        z(k) = t                                                        dnbco237
        nl = min0(ml,n-k)                                               dnbco238
        if (k .lt. n) call daxpy(nl,t,abe(k+nl,ml1-nl),-ldb,z(k+1),1)   dnbco239
        if (dabs(z(k)) .le. 1.0d0) go to 130                            dnbco240
          s = 1.0d0/dabs(z(k))                                          dnbco241
          call dscal(n,s,z,1)                                           dnbco242
          ynorm = s*ynorm                                               dnbco243
  130   continue                                                        dnbco244
  140 continue                                                          dnbco245
      s = 1.0d0/dasum(n,z,1)                                            dnbco246
      call dscal(n,s,z,1)                                               dnbco247
      ynorm = s*ynorm                                                   dnbco248
c                                                                       dnbco249
c     solve  u*z = v                                                    dnbco250
c                                                                       dnbco251
      do 160 kb = 1, n                                                  dnbco252
        k = n + 1 - kb                                                  dnbco253
        if (dabs(z(k)) .le. dabs(abe(k,ml1))) go to 150                 dnbco254
          s = dabs(abe(k,ml1))/dabs(z(k))                               dnbco255
          call dscal(n,s,z,1)                                           dnbco256
          ynorm = s*ynorm                                               dnbco257
  150   continue                                                        dnbco258
        if (abe(k,ml1) .ne. 0.0d0) z(k) = z(k)/abe(k,ml1)               dnbco259
        if (abe(k,ml1) .eq. 0.0d0) z(k) = 1.0d0                         dnbco260
        lm = min0(k,m) - 1                                              dnbco261
        lz = k - lm                                                     dnbco262
        t = -z(k)                                                       dnbco263
        call daxpy(lm,t,abe(k-1,ml+2),-ldb,z(lz),1)                     dnbco264
  160 continue                                                          dnbco265
c     make znorm = 1.0d0                                                dnbco266
      s = 1.0d0/dasum(n,z,1)                                            dnbco267
      call dscal(n,s,z,1)                                               dnbco268
      ynorm = s*ynorm                                                   dnbco269
c                                                                       dnbco270
      if (anorm .ne. 0.0d0) rcond = ynorm/anorm                         dnbco271
      if (anorm .eq. 0.0d0) rcond = 0.0d0                               dnbco272
      return                                                            dnbco273
      end                                                               dnbco274
