      subroutine dnbfa(abe,lda,n,ml,mu,ipvt,info)                       dnbfa  3
c***begin prologue  dnbfa                                               dnbfa  4
c***date written   800728   (yymmdd)                                    dnbfa  5
c***revision date  861211   (yymmdd)                                    bb812516
c***category no.  d2a2                                                  bb812517
c***keywords  library=slatec,                                           bb812518
c             type=double precision(snbfa-s dnbfa-d cnbfa-c),banded,    bb812519
c             linear equations,nonsymmetric                             bb812520
c***author  voorhees, e., (lanl)                                        dnbfa  9
c***purpose  factors a double precision band matrix by elimination.     dnbfa 10
c***description                                                         dnbfa 11
c                                                                       dnbfa 12
c     dnbfa factors a double precision band matrix by elimination.      dnbfa 13
c                                                                       dnbfa 14
c     dnbfa is usually called by dnbco, but it can be called            dnbfa 15
c     directly with a saving in time if rcond is not needed.            dnbfa 16
c                                                                       dnbfa 17
c     on entry                                                          dnbfa 18
c                                                                       dnbfa 19
c        abe     double precision(lda, nc)                              dnbfa 20
c                contains the matrix in band storage.  the rows         dnbfa 21
c                of the original matrix are stored in the rows          dnbfa 22
c                of abe and the diagonals of the original matrix        dnbfa 23
c                are stored in columns 1 through ml+mu+1 of abe.        dnbfa 24
c                nc must be .ge. 2*ml+mu+1 .                            dnbfa 25
c                see the comments below for details.                    dnbfa 26
c                                                                       dnbfa 27
c        lda     integer                                                dnbfa 28
c                the leading dimension of the array abe.                dnbfa 29
c                lda must be .ge.  n .                                  dnbfa 30
c                                                                       dnbfa 31
c        n       integer                                                dnbfa 32
c                the order of the original matrix.                      dnbfa 33
c                                                                       dnbfa 34
c        ml      integer                                                dnbfa 35
c                number of diagonals below the main diagonal.           dnbfa 36
c                0 .le. ml .lt.  n .                                    dnbfa 37
c                                                                       dnbfa 38
c        mu      integer                                                dnbfa 39
c                number of diagonals above the main diagonal.           dnbfa 40
c                0 .le. mu .lt.  n .                                    dnbfa 41
c                more efficient if ml .le. mu .                         dnbfa 42
c                                                                       dnbfa 43
c     on return                                                         dnbfa 44
c                                                                       dnbfa 45
c        abe     an upper triangular matrix in band storage             dnbfa 46
c                and the multipliers which were used to obtain it.      dnbfa 47
c                the factorization can be written  a = l*u  where       dnbfa 48
c                l is a product of permutation and unit lower           dnbfa 49
c                triangular matrices and  u  is upper triangular.       dnbfa 50
c                                                                       dnbfa 51
c        ipvt    integer(n)                                             dnbfa 52
c                an integer vector of pivot indices.                    dnbfa 53
c                                                                       dnbfa 54
c        info    integer                                                dnbfa 55
c                =0  normal value                                       dnbfa 56
c                =k  if  u(k,k) .eq. 0.0 .  this is not an error        dnbfa 57
c                condition for this subroutine, but it does             dnbfa 58
c                indicate that dnbsl will divide by zero if             dnbfa 59
c                called.  use rcond in dnbco for a reliable             dnbfa 60
c                indication of singularity.                             dnbfa 61
c                                                                       dnbfa 62
c     band storage                                                      dnbfa 63
c                                                                       dnbfa 64
c           if  a  is a band matrix, the following program segment      dnbfa 65
c           will set up the input.                                      dnbfa 66
c                                                                       dnbfa 67
c                   ml = (band width below the diagonal)                dnbfa 68
c                   mu = (band width above the diagonal)                dnbfa 69
c                   do 20 i = 1, n                                      dnbfa 70
c                      j1 = max0(1, i-ml)                               dnbfa 71
c                      j2 = min0(n, i+mu)                               dnbfa 72
c                      do 10 j = j1, j2                                 dnbfa 73
c                         k = j - i + ml + 1                            dnbfa 74
c                         abe(i,k) = a(i,j)                             dnbfa 75
c                10    continue                                         dnbfa 76
c                20 continue                                            dnbfa 77
c                                                                       dnbfa 78
c           this uses columns  1  through  ml+mu+1  of abe .            dnbfa 79
c           furthermore,  ml  additional columns are needed in          dnbfa 80
c           abe  starting with column  ml+mu+2  for elements            dnbfa 81
c           generated during the triangularization.  the total          dnbfa 82
c           number of columns needed in  abe  is  2*ml+mu+1 .           dnbfa 83
c                                                                       dnbfa 84
c     example:  if the original matrix is                               dnbfa 85
c                                                                       dnbfa 86
c           11 12 13  0  0  0                                           dnbfa 87
c           21 22 23 24  0  0                                           dnbfa 88
c            0 32 33 34 35  0                                           dnbfa 89
c            0  0 43 44 45 46                                           dnbfa 90
c            0  0  0 54 55 56                                           dnbfa 91
c            0  0  0  0 65 66                                           dnbfa 92
c                                                                       dnbfa 93
c      then  n = 6, ml = 1, mu = 2, lda .ge. 5  and abe should contain  dnbfa 94
c                                                                       dnbfa 95
c            * 11 12 13  +     , * = not used                           dnbfa 96
c           21 22 23 24  +     , + = used for pivoting                  dnbfa 97
c           32 33 34 35  +                                              dnbfa 98
c           43 44 45 46  +                                              dnbfa 99
c           54 55 56  *  +                                              dnbfa100
c           65 66  *  *  +                                              dnbfa101
c                                                                       dnbfa102
c     slatec.  this version dated 07/28/80 .                            dnbfa103
c     e. a. voorhees, los alamos scientific laboratory                  dnbfa104
c                                                                       dnbfa105
c     subroutines and functions                                         dnbfa106
c                                                                       dnbfa107
c      idamax,daxpy,dscal,dswap                                         dnbfa108
c     fortran  min0                                                     dnbfa109
c***references  subroutine dnbfa was developed by group c-3, los alamos dnbfa110
c                 scientific laboratory, los alamos, nm 87545.          dnbfa111
c***routines called  daxpy,dscal,dswap,idamax                           dnbfa112
c***end prologue  dnbfa                                                 dnbfa113
      integer lda,n,ml,mu,ipvt(1),info                                  dnbfa114
      double precision abe(lda,1)                                       dnbfa115
c                                                                       dnbfa116
      integer ml1,mb,m,n1,ldb,i,j,k,l,lm,lm1,lm2,mp,idamax              dnbfa117
      double precision t                                                dnbfa118
c***first executable statement  dnbfa                                   dnbfa119
      ml1=ml+1                                                          dnbfa120
      mb=ml+mu                                                          dnbfa121
      m=ml+mu+1                                                         dnbfa122
      n1=n-1                                                            dnbfa123
      ldb=lda-1                                                         dnbfa124
      info=0                                                            dnbfa125
c                                                                       dnbfa126
c     set fill-in columns to zero                                       dnbfa127
c                                                                       dnbfa128
      if(n.le.1)go to 50                                                dnbfa129
      if(ml.le.0)go to 7                                                dnbfa130
      do 6 j=1,ml                                                       dnbfa131
        do 5 i=1,n                                                      dnbfa132
          abe(i,m+j)=0.0d0                                              dnbfa133
    5   continue                                                        dnbfa134
    6 continue                                                          dnbfa135
    7 continue                                                          dnbfa136
c                                                                       dnbfa137
c     gaussian elimination with partial elimination                     dnbfa138
c                                                                       dnbfa139
      do 40 k=1,n1                                                      dnbfa140
        lm=min0(n-k,ml)                                                 dnbfa141
        lm1=lm+1                                                        dnbfa142
        lm2=ml1-lm                                                      dnbfa143
c                                                                       dnbfa144
c     search for pivot index                                            dnbfa145
c                                                                       dnbfa146
        l=-idamax(lm1,abe(lm+k,lm2),ldb)+lm1+k                          dnbfa147
        ipvt(k)=l                                                       dnbfa148
        mp=min0(mb,n-k)                                                 dnbfa149
c                                                                       dnbfa150
c     swap rows if necessary                                            dnbfa151
c                                                                       dnbfa152
        if(l.ne.k)call dswap(mp+1,abe(k,ml1),lda,abe(l,ml1+k-l),lda)    dnbfa153
c                                                                       dnbfa154
c     skip column reduction if pivot is zero                            dnbfa155
c                                                                       dnbfa156
        if(abe(k,ml1).eq.0.0d0) go to 20                                dnbfa157
c                                                                       dnbfa158
c     compute multipliers                                               dnbfa159
c                                                                       dnbfa160
        t=-1.0/abe(k,ml1)                                               dnbfa161
        call dscal(lm,t,abe(lm+k,lm2),ldb)                              dnbfa162
c                                                                       dnbfa163
c     row elimination with column indexing                              dnbfa164
c                                                                       dnbfa165
        do 10 j=1,mp                                                    dnbfa166
          call daxpy(lm,abe(k,ml1+j),abe(lm+k,lm2),ldb,abe(lm+k,lm2+j),ldnbfa167
     1db)                                                               dnbfa168
   10   continue                                                        dnbfa169
        go to 30                                                        dnbfa170
   20   continue                                                        dnbfa171
        info=k                                                          dnbfa172
   30   continue                                                        dnbfa173
   40 continue                                                          dnbfa174
   50 continue                                                          dnbfa175
      ipvt(n)=n                                                         dnbfa176
      if(abe(n,ml1).eq.0.0d0) info=n                                    dnbfa177
      return                                                            dnbfa178
      end                                                               dnbfa179
