      subroutine dnbfs(abe,lda,n,ml,mu,v,itask,ind,work,iwork)          dnbfs  3
c***begin prologue  dnbfs                                               dnbfs  4
c***date written   800812   (yymmdd)                                    dnbfs  5
c***revision date  861211   (yymmdd)                                    bb812521
c***category no.  d2a2                                                  bb812522
c***keywords  library=slatec,                                           bb812523
c             type=double precision(snbfs-s dnbfs-d cnbfs-c),banded,    bb812524
c             linear equations,nonsymmetric                             bb812525
c***author  voorhees, e., (lanl)                                        dnbfs  9
c***purpose  dnbfs solves a general nonsymmetric banded                 dnbfs 10
c            double precision nxn system of linear equations            dnbfs 11
c***description                                                         dnbfs 12
c                                                                       dnbfs 13
c    subroutine dnbfs solves a general nonsymmetric banded nxn          dnbfs 14
c    system of double precision real linear equations using             dnbfs 15
c    slatec subroutines dnbco and dnbsl.  these are adaptations         dnbfs 16
c    of the linpack subroutines dgbco and dgbsl which require           dnbfs 17
c    a different format for storing the matrix elements.  if            dnbfs 18
c    a  is an nxn double precision matrix and if  x  and  b  are        dnbfs 19
c    double precision n-vectors, then dnbfs solves the equation         dnbfs 20
c                                                                       dnbfs 21
c                          a*x=b.                                       dnbfs 22
c                                                                       dnbfs 23
c    a band matrix is a matrix whose nonzero elements are all           dnbfs 24
c    fairly near the main diagonal, specifically  a(i,j) = 0            dnbfs 25
c    if  i-j is greater than  ml  or  j-i  is greater than              dnbfs 26
c    mu .  the integers ml and mu are called the lower and upper        dnbfs 27
c    band widths and  m = ml+mu+1  is the total band width.             dnbfs 28
c    dnbfs uses less time and storage than the corresponding            dnbfs 29
c    program for general matrices (dgefs) if 2*ml+mu .lt.  n .          dnbfs 30
c                                                                       dnbfs 31
c    the matrix a is first factored into upper and lower tri-           dnbfs 32
c    angular matrices u and l using partial pivoting.  these            dnbfs 33
c    factors and the pivoting information are used to find the          dnbfs 34
c    solution vector x.  an approximate condition number is             dnbfs 35
c    calculated to provide a rough estimate of the number of            dnbfs 36
c    digits of accuracy in the computed solution.                       dnbfs 37
c                                                                       dnbfs 38
c    if the equation a*x=b is to be solved for more than one vector     dnbfs 39
c    b, the factoring of a does not need to be performed again and      dnbfs 40
c    the option to only solve (itask .gt. 1) will be faster for         dnbfs 41
c    the succeeding solutions.  in this case, the contents of a,        dnbfs 42
c    lda, n and iwork must not have been altered by the user follow-    dnbfs 43
c    ing factorization (itask=1).  ind will not be changed by dnbfs     dnbfs 44
c    in this case.                                                      dnbfs 45
c                                                                       dnbfs 46
c                                                                       dnbfs 47
c    band storage                                                       dnbfs 48
c                                                                       dnbfs 49
c          if  a  is a band matrix, the following program segment       dnbfs 50
c          will set up the input.                                       dnbfs 51
c                                                                       dnbfs 52
c                  ml = (band width below the diagonal)                 dnbfs 53
c                  mu = (band width above the diagonal)                 dnbfs 54
c                  do 20 i = 1, n                                       dnbfs 55
c                     j1 = max0(1, i-ml)                                dnbfs 56
c                     j2 = min0(n, i+mu)                                dnbfs 57
c                     do 10 j = j1, j2                                  dnbfs 58
c                        k = j - i + ml + 1                             dnbfs 59
c                        abe(i,k) = a(i,j)                              dnbfs 60
c               10    continue                                          dnbfs 61
c               20 continue                                             dnbfs 62
c                                                                       dnbfs 63
c          this uses columns  1  through  ml+mu+1  of abe .             dnbfs 64
c          furthermore,  ml  additional columns are needed in           dnbfs 65
c          abe  starting with column  ml+mu+2  for elements             dnbfs 66
c          generated during the triangularization.  the total           dnbfs 67
c          number of columns needed in  abe  is  2*ml+mu+1 .            dnbfs 68
c                                                                       dnbfs 69
c    example:  if the original matrix is                                dnbfs 70
c                                                                       dnbfs 71
c          11 12 13  0  0  0                                            dnbfs 72
c          21 22 23 24  0  0                                            dnbfs 73
c           0 32 33 34 35  0                                            dnbfs 74
c           0  0 43 44 45 46                                            dnbfs 75
c           0  0  0 54 55 56                                            dnbfs 76
c           0  0  0  0 65 66                                            dnbfs 77
c                                                                       dnbfs 78
c     then  n = 6, ml = 1, mu = 2, lda .ge. 5  and abe should contain   dnbfs 79
c                                                                       dnbfs 80
c           * 11 12 13  +     , * = not used                            dnbfs 81
c          21 22 23 24  +     , + = used for pivoting                   dnbfs 82
c          32 33 34 35  +                                               dnbfs 83
c          43 44 45 46  +                                               dnbfs 84
c          54 55 56  *  +                                               dnbfs 85
c          65 66  *  *  +                                               dnbfs 86
c                                                                       dnbfs 87
c                                                                       dnbfs 88
c  argument description ***                                             dnbfs 89
c                                                                       dnbfs 90
c    abe    double precision(lda,nc)                                    dnbfs 91
c             on entry, contains the matrix in band storage as          dnbfs 92
c               described above.  nc  must not be less than             dnbfs 93
c               2*ml+mu+1 .  the user is cautioned to specify  nc       dnbfs 94
c               with care since it is not an argument and cannot        dnbfs 95
c               be checked by dnbfs.  the rows of the original          dnbfs 96
c               matrix are stored in the rows of  abe  and the          dnbfs 97
c               diagonals of the original matrix are stored in          dnbfs 98
c               columns  1  through  ml+mu+1  of  abe .                 dnbfs 99
c             on return, contains an upper triangular matrix u and      dnbfs100
c               the multipliers necessary to construct a matrix l       dnbfs101
c               so that a=l*u.                                          dnbfs102
c    lda    integer                                                     dnbfs103
c             the leading dimension of array abe.  lda must be great-   dnbfs104
c             er than or equal to n.  (terminal error message ind=-1)   dnbfs105
c    n      integer                                                     dnbfs106
c             the order of the matrix a.  n must be greater             dnbfs107
c             than or equal to 1 .  (terminal error message ind=-2)     dnbfs108
c    ml     integer                                                     dnbfs109
c             the number of diagonals below the main diagonal.          dnbfs110
c             ml  must not be less than zero nor greater than or        dnbfs111
c             equal to  n .  (terminal error message ind=-5)            dnbfs112
c    mu     integer                                                     dnbfs113
c             the number of diagonals above the main diagonal.          dnbfs114
c             mu  must not be less than zero nor greater than or        dnbfs115
c             equal to  n .  (terminal error message ind=-6)            dnbfs116
c    v      double precision(n)                                         dnbfs117
c             on entry, the singly subscripted array(vector) of di-     dnbfs118
c               mension n which contains the right hand side b of a     dnbfs119
c               system of simultaneous linear equations a*x=b.          dnbfs120
c             on return, v contains the solution vector, x .            dnbfs121
c    itask  integer                                                     dnbfs122
c             if itask=1, the matrix a is factored and then the         dnbfs123
c               linear equation is solved.                              dnbfs124
c             if itask .gt. 1, the equation is solved using the existingdnbfs125
c               factored matrix a and iwork.                            dnbfs126
c             if itask .lt. 1, then terminal error message ind=-3 is    dnbfs127
c               printed.                                                dnbfs128
c    ind    integer                                                     dnbfs129
c             gt. 0  ind is a rough estimate of the number of digits    dnbfs130
c                     of accuracy in the solution, x.                   dnbfs131
c             lt. 0  see error message corresponding to ind below.      dnbfs132
c    work   double precision(n)                                         dnbfs133
c             a singly subscripted array of dimension at least n.       dnbfs134
c    iwork  integer(n)                                                  dnbfs135
c             a singly subscripted array of dimension at least n.       dnbfs136
c                                                                       dnbfs137
c  error messages printed ***                                           dnbfs138
c                                                                       dnbfs139
c    ind=-1  terminal   n is greater than lda.                          dnbfs140
c    ind=-2  terminal   n is less than 1.                               dnbfs141
c    ind=-3  terminal   itask is less than 1.                           dnbfs142
c    ind=-4  terminal   the matrix a is computationally singular.       dnbfs143
c                         a solution has not been computed.             dnbfs144
c    ind=-5  terminal   ml is less than zero or is greater than         dnbfs145
c                         or equal to n .                               dnbfs146
c    ind=-6  terminal   mu is less than zero or is greater than         dnbfs147
c                         or equal to n .                               dnbfs148
c    ind=-10 warning    the solution has no apparent significance.      dnbfs149
c                         the solution may be inaccurate or the matrix  dnbfs150
c                         a may be poorly scaled.                       dnbfs151
c                                                                       dnbfs152
c               note-  the above terminal(*fatal*) error messages are   dnbfs153
c                      designed to be handled by xerrwv in which        dnbfs154
c                      level=1 (recoverable) and iflag=2 .  level=0     dnbfs155
c                      for warning error messages from xerror.  unless  dnbfs156
c                      the user provides otherwise, an error message    dnbfs157
c                      will be printed followed by an abort.            dnbfs158
c***references  subroutine dnbfs was developed by group c-3, los alamos dnbfs159
c                 scientific laboratory, los alamos, nm 87545.          dnbfs160
c                 the slatec subroutines used by dnbfs are similar      dnbfs161
c                 (except for matrix storage format) to the correspon-  dnbfs162
c                 ding linpack subroutines (dgbco and dgbsl) described  dnbfs163
c                 in detail in the *linpack users guide* published by   dnbfs164
c                 the society for industrial and applied mathematics    dnbfs165
c                 (siam) dated 1979.                                    dnbfs166
c***routines called  d1mach,dnbco,dnbsl,xerror,xerrwv                   dnbfs167
c***end prologue  dnbfs                                                 dnbfs168
c                                                                       dnbfs169
      integer lda,n,itask,ind,iwork(n),ml,mu                            dnbfs170
      double precision abe(lda,1),v(n),work(n),d1mach                   dnbfs171
      double precision rcond                                            dnbfs172
c***first executable statement  dnbfs                                   dnbfs173
      if (lda.lt.n)  go to 101                                          dnbfs174
      if (n.le.0)  go to 102                                            dnbfs175
      if (itask.lt.1) go to 103                                         dnbfs176
      if((ml.lt.0).or.(ml.ge.n))go to 105                               dnbfs177
      if((mu.lt.0).or.(mu.ge.n))go to 106                               dnbfs178
      if (itask.gt.1) go to 20                                          dnbfs179
c                                                                       dnbfs180
c     factor matrix a into lu                                           dnbfs181
      call dnbco(abe,lda,n,ml,mu,iwork,rcond,work)                      dnbfs182
c                                                                       dnbfs183
c     check for computationally singular matrix                         dnbfs184
      if (rcond.eq.0.0d0)  go to 104                                    dnbfs185
c                                                                       dnbfs186
c     compute ind (estimate of no. of significant digits)               dnbfs187
      ind=-idint(dlog10(d1mach(4)/rcond))                               dnbfs188
c                                                                       dnbfs189
c     check for ind greater than zero                                   dnbfs190
      if (ind.gt.0)  go to 20                                           dnbfs191
      ind=-10                                                           dnbfs192
      call xerror( 'dnbfs error (ind=-10) -- solution may have no signifdnbfs193
     1icance',58,-10,0)                                                 dnbfs194
c                                                                       dnbfs195
c     solve after factoring                                             dnbfs196
   20 call dnbsl(abe,lda,n,ml,mu,iwork,v,0)                             dnbfs197
      return                                                            dnbfs198
c                                                                       dnbfs199
c     if lda.lt.n, ind=-1, terminal xerrwv message                      dnbfs200
  101 ind=-1                                                            dnbfs201
      call xerrwv( 'dnbfs error (ind=-1) -- lda=i1 is less than n=i2',  dnbfs202
     148,-1,1,2,lda,n,0,0,0)                                            dnbfs203
      return                                                            dnbfs204
c                                                                       dnbfs205
c     if n.lt.1, ind=-2, terminal xerrwv message                        dnbfs206
  102 ind=-2                                                            dnbfs207
      call xerrwv( 'dnbfs error (ind=-2) -- n=i1 is less than 1',       dnbfs208
     143,-2,1,1,n,0,0,0,0)                                              dnbfs209
      return                                                            dnbfs210
c                                                                       dnbfs211
c     if itask.lt.1, ind=-3, terminal xerrwv message                    dnbfs212
  103 ind=-3                                                            dnbfs213
      call xerrwv( 'dnbfs error (ind=-3) -- itask=i1 is less than 1',   dnbfs214
     147,-3,1,1,itask,0,0,0,0)                                          dnbfs215
      return                                                            dnbfs216
c                                                                       dnbfs217
c     if singular matrix, ind=-4, terminal xerrwv message               dnbfs218
  104 ind=-4                                                            dnbfs219
      call xerrwv( 'dnbfs error (ind=-4) -- singular matrix a - no solutdnbfs220
     1ion',55,-4,1,0,0,0,0,0,0)                                         dnbfs221
      return                                                            dnbfs222
c                                                                       dnbfs223
c     if improper ml value, ind=-5, terminal xerrwv message             dnbfs224
  105 ind=-5                                                            dnbfs225
      call xerrwv( 'dnbfs error (ind=-5) -- ml=i1 is out of range',45,-5dnbfs226
     1,1,1,ml,0,0,0,0)                                                  dnbfs227
      return                                                            dnbfs228
c                                                                       dnbfs229
c     if improper mu value, ind=-6, terminal xerrwv message             dnbfs230
  106 ind=-6                                                            dnbfs231
      call xerrwv( 'dnbfs error (ind=-6) -- mu=i1 is out of range',45,-6dnbfs232
     1,1,1,mu,0,0,0,0)                                                  dnbfs233
      return                                                            dnbfs234
c                                                                       dnbfs235
      end                                                               dnbfs236
