      subroutine dnbsl(abe,lda,n,ml,mu,ipvt,b,job)                      dnbsl  3
c***begin prologue  dnbsl                                               dnbsl  4
c***date written   800728   (yymmdd)                                    dnbsl  5
c***revision date  861211   (yymmdd)                                    bb812526
c***category no.  d2a2                                                  bb812527
c***keywords  library=slatec,                                           bb812528
c             type=double precision(snbsl-s dnbsl-d cnbsl-c),banded,    bb812529
c             linear equations,nonsymmetric                             bb812530
c***author  voorhees, e., (lanl)                                        dnbsl  9
c***purpose  solves a double precision band system using                dnbsl 10
c            the factors computed by dnbco or dnbfa.                    dnbsl 11
c***description                                                         dnbsl 12
c                                                                       dnbsl 13
c     dnbsl solves the double precision band system                     dnbsl 14
c     a * x = b  or  trans(a) * x = b                                   dnbsl 15
c     using the factors computed by dnbco or dnbfa.                     dnbsl 16
c                                                                       dnbsl 17
c     on entry                                                          dnbsl 18
c                                                                       dnbsl 19
c        abe     double precision(lda, nc)                              dnbsl 20
c                the output from dnbco or dnbfa.                        dnbsl 21
c                nc must be .ge. 2*ml+mu+1 .                            dnbsl 22
c                                                                       dnbsl 23
c        lda     integer                                                dnbsl 24
c                the leading dimension of the array  abe .              dnbsl 25
c                                                                       dnbsl 26
c        n       integer                                                dnbsl 27
c                the order of the original matrix.                      dnbsl 28
c                                                                       dnbsl 29
c        ml      integer                                                dnbsl 30
c                number of diagonals below the main diagonal.           dnbsl 31
c                                                                       dnbsl 32
c        mu      integer                                                dnbsl 33
c                number of diagonals above the main diagonal.           dnbsl 34
c                                                                       dnbsl 35
c        ipvt    integer(n)                                             dnbsl 36
c                the pivot vector from dnbco or dnbfa.                  dnbsl 37
c                                                                       dnbsl 38
c        b       double precision(n)                                    dnbsl 39
c                the right hand side vector.                            dnbsl 40
c                                                                       dnbsl 41
c        job     integer                                                dnbsl 42
c                = 0         to solve  a*x = b .                        dnbsl 43
c                = nonzero   to solve  trans(a)*x = b , where           dnbsl 44
c                            trans(a)  is the transpose.                dnbsl 45
c                                                                       dnbsl 46
c     on return                                                         dnbsl 47
c                                                                       dnbsl 48
c        b       the solution vector  x .                               dnbsl 49
c                                                                       dnbsl 50
c     error condition                                                   dnbsl 51
c                                                                       dnbsl 52
c        a division by zero will occur if the input factor contains a   dnbsl 53
c        zero on the diagonal.  technically this indicates singularity  dnbsl 54
c        but it is often caused by improper arguments or improper       dnbsl 55
c        setting of lda.  it will not occur if the subroutines are      dnbsl 56
c        called correctly and if dnbco has set rcond .gt. 0.0           dnbsl 57
c        or dnbfa has set info .eq. 0 .                                 dnbsl 58
c                                                                       dnbsl 59
c     to compute  inverse(a) * c  where  c  is a matrix                 dnbsl 60
c     with  p  columns                                                  dnbsl 61
c           call dnbco(abe,lda,n,ml,mu,ipvt,rcond,z)                    dnbsl 62
c           if (rcond is too small) go to ...                           dnbsl 63
c           do 10 j = 1, p                                              dnbsl 64
c             call dnbsl(abe,lda,n,ml,mu,ipvt,c(1,j),0)                 dnbsl 65
c        10 continue                                                    dnbsl 66
c                                                                       dnbsl 67
c     slatec.  this version dated 07/28/80 .                            dnbsl 68
c     e. a. voorhees, los alamos scientific laboratory                  dnbsl 69
c                                                                       dnbsl 70
c     subroutines and functions                                         dnbsl 71
c                                                                       dnbsl 72
c      daxpy,ddot                                                       dnbsl 73
c     fortran  min0                                                     dnbsl 74
c***references  subroutine dnbsl was developed by group c-3, los alamos dnbsl 75
c                 scientific laboratory, los alamos, nm 87545.          dnbsl 76
c***routines called  daxpy,ddot                                         dnbsl 77
c***end prologue  dnbsl                                                 dnbsl 78
      integer lda,n,ml,mu,ipvt(1),job                                   dnbsl 79
      double precision abe(lda,1),b(1)                                  dnbsl 80
c                                                                       dnbsl 81
      double precision ddot,t                                           dnbsl 82
      integer k,kb,l,lb,ldb,lm,m,mlm,nm1                                dnbsl 83
c***first executable statement  dnbsl                                   dnbsl 84
      m=mu+ml+1                                                         dnbsl 85
      nm1=n-1                                                           dnbsl 86
      ldb=1-lda                                                         dnbsl 87
      if(job.ne.0)go to 50                                              dnbsl 88
c                                                                       dnbsl 89
c       job = 0 , solve  a * x = b                                      dnbsl 90
c       first solve l*y = b                                             dnbsl 91
c                                                                       dnbsl 92
        if(ml.eq.0)go to 30                                             dnbsl 93
        if(nm1.lt.1)go to 30                                            dnbsl 94
          do 20 k=1,nm1                                                 dnbsl 95
            lm=min0(ml,n-k)                                             dnbsl 96
            l=ipvt(k)                                                   dnbsl 97
            t=b(l)                                                      dnbsl 98
            if(l.eq.k)go to 10                                          dnbsl 99
              b(l)=b(k)                                                 dnbsl100
              b(k)=t                                                    dnbsl101
   10       continue                                                    dnbsl102
            mlm=ml-(lm-1)                                               dnbsl103
            call daxpy(lm,t,abe(k+lm,mlm),ldb,b(k+1),1)                 dnbsl104
   20     continue                                                      dnbsl105
   30   continue                                                        dnbsl106
c                                                                       dnbsl107
c       now solve  u*x = y                                              dnbsl108
c                                                                       dnbsl109
        do 40 kb=1,n                                                    dnbsl110
          k=n+1-kb                                                      dnbsl111
          b(k)=b(k)/abe(k,ml+1)                                         dnbsl112
          lm=min0(k,m)-1                                                dnbsl113
          lb=k-lm                                                       dnbsl114
          t=-b(k)                                                       dnbsl115
          call daxpy(lm,t,abe(k-1,ml+2),ldb,b(lb),1)                    dnbsl116
   40   continue                                                        dnbsl117
      go to 100                                                         dnbsl118
   50 continue                                                          dnbsl119
c                                                                       dnbsl120
c       job = nonzero, solve trans(a) * x = b                           dnbsl121
c       first solve  trans(u)*y = b                                     dnbsl122
c                                                                       dnbsl123
        do 60 k = 1, n                                                  dnbsl124
          lm = min0(k,m) - 1                                            dnbsl125
          lb = k - lm                                                   dnbsl126
          t = ddot(lm,abe(k-1,ml+2),ldb,b(lb),1)                        dnbsl127
          b(k) = (b(k) - t)/abe(k,ml+1)                                 dnbsl128
   60   continue                                                        dnbsl129
c                                                                       dnbsl130
c       now solve trans(l)*x = y                                        dnbsl131
c                                                                       dnbsl132
        if (ml .eq. 0) go to 90                                         dnbsl133
        if (nm1 .lt. 1) go to 90                                        dnbsl134
          do 80 kb = 1, nm1                                             dnbsl135
            k = n - kb                                                  dnbsl136
            lm = min0(ml,n-k)                                           dnbsl137
            mlm = ml - (lm - 1)                                         dnbsl138
            b(k) = b(k) + ddot(lm,abe(k+lm,mlm),ldb,b(k+1),1)           dnbsl139
            l = ipvt(k)                                                 dnbsl140
            if (l .eq. k) go to 70                                      dnbsl141
              t = b(l)                                                  dnbsl142
              b(l) = b(k)                                               dnbsl143
              b(k) = t                                                  dnbsl144
   70       continue                                                    dnbsl145
   80     continue                                                      dnbsl146
   90   continue                                                        dnbsl147
  100 continue                                                          dnbsl148
      return                                                            dnbsl149
      end                                                               dnbsl150
