      integer function i1mach(i)                                        i1mach 4
c***begin prologue  i1mach                                              i1mach 5
c***date written   750101   (yymmdd)                                    i1mach 6
c***revision date  880315   (yymmdd)                                    bb831524
c***category no.  r1                                                    bb815155
c***keywords  library=slatec,type=integer(i1mach-i),machine constants   bb815156
c***author  fox, p. a., (bell labs)                                     i1mach10
c           hall, a. d., (bell labs)                                    i1mach11
c           schryer, n. l., (bell labs)                                 i1mach12
c***purpose  returns integer machine dependent constants                i1mach13
c***description                                                         i1mach14
c                                                                       i1mach15
c     i1mach can be used to obtain machine-dependent parameters         i1mach21
c     for the local machine environment.  it is a function              i1mach22
c     subroutine with one (input) argument, and can be called           i1mach23
c     as follows, for example                                           i1mach24
c                                                                       i1mach25
c          k = i1mach(i)                                                i1mach26
c                                                                       i1mach27
c     where i=1,...,16.  the (output) value of k above is               i1mach28
c     determined by the (input) value of i.  the results for            i1mach29
c     various values of i are discussed below.                          i1mach30
c                                                                       i1mach31
c  i/o unit numbers.                                                    i1mach32
c    i1mach( 1) = the standard input unit.                              i1mach33
c    i1mach( 2) = the standard output unit.                             i1mach34
c    i1mach( 3) = the standard punch unit.                              i1mach35
c    i1mach( 4) = the standard error message unit.                      i1mach36
c                                                                       i1mach37
c  words.                                                               i1mach38
c    i1mach( 5) = the number of bits per integer storage unit.          i1mach39
c    i1mach( 6) = the number of characters per integer storage unit.    i1mach40
c                                                                       i1mach41
c  integers.                                                            i1mach42
c    assume integers are represented in the s-digit, base-a form        i1mach43
c                                                                       i1mach44
c               sign ( x(s-1)*a**(s-1) + ... + x(1)*a + x(0) )          i1mach45
c                                                                       i1mach46
c               where 0 .le. x(i) .lt. a for i=0,...,s-1.               i1mach47
c    i1mach( 7) = a, the base.                                          i1mach48
c    i1mach( 8) = s, the number of base-a digits.                       i1mach49
c    i1mach( 9) = a**s - 1, the largest magnitude.                      i1mach50
c                                                                       i1mach51
c  floating-point numbers.                                              i1mach52
c    assume floating-point numbers are represented in the t-digit,      i1mach53
c    base-b form                                                        i1mach54
c               sign (b**e)*( (x(1)/b) + ... + (x(t)/b**t) )            i1mach55
c                                                                       i1mach56
c               where 0 .le. x(i) .lt. b for i=1,...,t,                 i1mach57
c               0 .lt. x(1), and emin .le. e .le. emax.                 i1mach58
c    i1mach(10) = b, the base.                                          i1mach59
c                                                                       i1mach60
c  single-precision                                                     i1mach61
c    i1mach(11) = t, the number of base-b digits.                       i1mach62
c    i1mach(12) = emin, the smallest exponent e.                        i1mach63
c    i1mach(13) = emax, the largest exponent e.                         i1mach64
c                                                                       i1mach65
c  double-precision                                                     i1mach66
c    i1mach(14) = t, the number of base-b digits.                       i1mach67
c    i1mach(15) = emin, the smallest exponent e.                        i1mach68
c    i1mach(16) = emax, the largest exponent e.                         i1mach69
c                                                                       i1mach70
c  to alter this function for a particular environment,                 i1mach71
c  the desired set of data statements should be activated by            i1mach72
c  removing the c from column 1.  also, the values of                   i1mach73
c  i1mach(1) - i1mach(4) should be checked for consistency              i1mach74
c  with the local operating system.                                     i1mach75
c                                                                       bb815157
c***references  fox p.a., hall a.d., schryer n.l.,*framework for a      i1mach76
c                 portable library*, acm transactions on mathematical   i1mach77
c                 software, vol. 4, no. 2, june 1978, pp. 177-188.      i1mach78
c***routines called  (none)                                             i1mach79
c***end prologue  i1mach                                                i1mach80
c                                                                       i1mach81
      integer imach(16),output                                          i1mach82
      equivalence (imach(4),output)                                     i1mach83
c                                                                       i1mach84
c     machine constants for the apollo                                  bb815158
c                                                                       bb815159
c     data imach(1) /    5 /                                            bb815160
c     data imach(2) /    6 /                                            bb815161
c     data imach(3) /    6 /                                            bb815162
c     data imach(4) /    6 /                                            bb815163
c     data imach(5) /   32 /                                            bb815164
c     data imach(6) /    4 /                                            bb815165
c     data imach(7) /    2 /                                            bb815166
c     data imach(8) /   31 /                                            bb815167
c     data imach(9) /2147483647 /                                       bb815168
c     data imach(10)/    2 /                                            bb815169
c     data imach(11)/   24 /                                            bb815170
c     data imach(12)/ -125 /                                            bb815171
c     data imach(13)/  129 /                                            bb815172
c     data imach(14)/   53 /                                            bb815173
c     data imach(15)/ -1021 /                                           bb815174
c     data imach(16)/  1025 /                                           bb815175
c                                                                       bb815176
c     machine constants for the burroughs 1700 system                   bb815177
c                                                                       i1mach86
c     data imach( 1) /    7 /                                           i1mach87
c     data imach( 2) /    2 /                                           i1mach88
c     data imach( 3) /    2 /                                           i1mach89
c     data imach( 4) /    2 /                                           i1mach90
c     data imach( 5) /   36 /                                           i1mach91
c     data imach( 6) /    4 /                                           i1mach92
c     data imach( 7) /    2 /                                           i1mach93
c     data imach( 8) /   33 /                                           i1mach94
c     data imach( 9) / z1ffffffff /                                     i1mach95
c     data imach(10) /    2 /                                           i1mach96
c     data imach(11) /   24 /                                           i1mach97
c     data imach(12) / -256 /                                           i1mach98
c     data imach(13) /  255 /                                           i1mach99
c     data imach(14) /   60 /                                           i1mac100
c     data imach(15) / -256 /                                           i1mac101
c     data imach(16) /  255 /                                           i1mac102
c                                                                       i1mac103
c     machine constants for the burroughs 5700 system                   bb815178
c                                                                       i1mac105
c     data imach( 1) /   5 /                                            i1mac106
c     data imach( 2) /   6 /                                            i1mac107
c     data imach( 3) /   7 /                                            i1mac108
c     data imach( 4) /   6 /                                            i1mac109
c     data imach( 5) /  48 /                                            i1mac110
c     data imach( 6) /   6 /                                            i1mac111
c     data imach( 7) /   2 /                                            i1mac112
c     data imach( 8) /  39 /                                            i1mac113
c     data imach( 9) / o0007777777777777 /                              i1mac114
c     data imach(10) /   8 /                                            i1mac115
c     data imach(11) /  13 /                                            i1mac116
c     data imach(12) / -50 /                                            i1mac117
c     data imach(13) /  76 /                                            i1mac118
c     data imach(14) /  26 /                                            i1mac119
c     data imach(15) / -50 /                                            i1mac120
c     data imach(16) /  76 /                                            i1mac121
c                                                                       i1mac122
c     machine constants for the burroughs 6700/7700 systems             bb815179
c                                                                       i1mac124
c     data imach( 1) /   5 /                                            i1mac125
c     data imach( 2) /   6 /                                            i1mac126
c     data imach( 3) /   7 /                                            i1mac127
c     data imach( 4) /   6 /                                            i1mac128
c     data imach( 5) /  48 /                                            i1mac129
c     data imach( 6) /   6 /                                            i1mac130
c     data imach( 7) /   2 /                                            i1mac131
c     data imach( 8) /  39 /                                            i1mac132
c     data imach( 9) / o0007777777777777 /                              i1mac133
c     data imach(10) /   8 /                                            i1mac134
c     data imach(11) /  13 /                                            i1mac135
c     data imach(12) / -50 /                                            i1mac136
c     data imach(13) /  76 /                                            i1mac137
c     data imach(14) /  26 /                                            i1mac138
c     data imach(15) / -32754 /                                         i1mac139
c     data imach(16) /  32780 /                                         i1mac140
c                                                                       i1mac141
c     machine constants for the cdc 170/180 series using nos/ve         bb815180
c                                                                       bb815181
c     data imach( 1) /     5 /                                          bb815182
c     data imach( 2) /     6 /                                          bb815183
c     data imach( 3) /     7 /                                          bb815184
c     data imach( 4) /     6 /                                          bb815185
c     data imach( 5) /    64 /                                          bb815186
c     data imach( 6) /     8 /                                          bb815187
c     data imach( 7) /     2 /                                          bb815188
c     data imach( 8) /    63 /                                          bb815189
c     data imach( 9) / 9223372036854775807 /                            bb815190
c     data imach(10) /     2 /                                          bb815191
c     data imach(11) /    47 /                                          bb815192
c     data imach(12) / -4095 /                                          bb815193
c     data imach(13) /  4094 /                                          bb815194
c     data imach(14) /    94 /                                          bb815195
c     data imach(15) / -4095 /                                          bb815196
c     data imach(16) /  4094 /                                          bb815197
c                                                                       bb815198
c     machine constants for the cdc 6000/7000 series                    bb815199
c                                                                       bb815200
c     data imach( 1) /    5 /                                           bb815201
c     data imach( 2) /    6 /                                           bb815202
c     data imach( 3) /    7 /                                           bb815203
c     data imach( 4) / l"output" /                                      bb831525
c     data imach( 5) /   60 /                                           bb815205
c     data imach( 6) /   10 /                                           bb815206
c     data imach( 7) /    2 /                                           bb815207
c     data imach( 8) /   48 /                                           bb815208
c     data imach( 9) / o"00007777777777777777" /                        bb831526
c     data imach(10) /    2 /                                           bb815210
c     data imach(11) /   47 /                                           bb815211
c     data imach(12) / -929 /                                           bb815212
c     data imach(13) / 1070 /                                           bb815213
c     data imach(14) /   94 /                                           bb815214
c     data imach(15) / -929 /                                           bb815215
c     data imach(16) / 1069 /                                           bb815216
c                                                                       bb811859
c     machine constants for the cray-1                                  bb815218
c                                                                       bb815219
c     data imach( 1) /   100 /                                          i1mac163
c     data imach( 2) /   101 /                                          i1mac164
c     data imach( 3) /   102 /                                          i1mac165
c     data imach( 4) /   101 /                                          i1mac166
c     data imach( 5) /    64 /                                          i1mac167
c     data imach( 6) /     8 /                                          i1mac168
c     data imach( 7) /     2 /                                          i1mac169
c     data imach( 8) /    63 /                                          i1mac170
c     data imach( 9) /  777777777777777777777b /                        i1mac171
c     data imach(10) /     2 /                                          i1mac172
c     data imach(11) /    47 /                                          i1mac173
c     data imach(12) / -8189 /                                          i1mac174
c     data imach(13) /  8190 /                                          i1mac175
c     data imach(14) /    94 /                                          i1mac176
c     data imach(15) / -8099 /                                          i1mac177
c     data imach(16) /  8190 /                                          i1mac178
c                                                                       i1mac179
c     machine constants for the data general eclipse s/200              i1mac180
c                                                                       i1mac181
c     data imach( 1) /   11 /                                           i1mac182
c     data imach( 2) /   12 /                                           i1mac183
c     data imach( 3) /    8 /                                           i1mac184
c     data imach( 4) /   10 /                                           i1mac185
c     data imach( 5) /   16 /                                           i1mac186
c     data imach( 6) /    2 /                                           i1mac187
c     data imach( 7) /    2 /                                           i1mac188
c     data imach( 8) /   15 /                                           i1mac189
c     data imach( 9) /32767 /                                           i1mac190
c     data imach(10) /   16 /                                           i1mac191
c     data imach(11) /    6 /                                           i1mac192
c     data imach(12) /  -64 /                                           i1mac193
c     data imach(13) /   63 /                                           i1mac194
c     data imach(14) /   14 /                                           i1mac195
c     data imach(15) /  -64 /                                           i1mac196
c     data imach(16) /   63 /                                           i1mac197
c                                                                       i1mac198
c     machine constants for the elxsi 6400                              bb815220
c                                                                       bb815221
c     data imach( 1) /     5/                                           bb815222
c     data imach( 2) /     6/                                           bb815223
c     data imach( 3) /     6/                                           bb815224
c     data imach( 4) /     6/                                           bb815225
c     data imach( 5) /    32/                                           bb815226
c     data imach( 6) /     4/                                           bb815227
c     data imach( 7) /     2/                                           bb815228
c     data imach( 8) /    32/                                           bb815229
c     data imach( 9) /2147483647/                                       bb815230
c     data imach(10) /     2/                                           bb815231
c     data imach(11) /    24/                                           bb815232
c     data imach(12) /  -126/                                           bb815233
c     data imach(13) /   127/                                           bb815234
c     data imach(14) /    53/                                           bb815235
c     data imach(15) / -1022/                                           bb815236
c     data imach(16) /  1023/                                           bb815237
c                                                                       bb815238
c     machine constants for the harris 220                              i1mac199
c                                                                       i1mac200
c     data imach( 1) /       5 /                                        i1mac201
c     data imach( 2) /       6 /                                        i1mac202
c     data imach( 3) /       0 /                                        i1mac203
c     data imach( 4) /       6 /                                        i1mac204
c     data imach( 5) /      24 /                                        i1mac205
c     data imach( 6) /       3 /                                        i1mac206
c     data imach( 7) /       2 /                                        i1mac207
c     data imach( 8) /      23 /                                        i1mac208
c     data imach( 9) / 8388607 /                                        i1mac209
c     data imach(10) /       2 /                                        i1mac210
c     data imach(11) /      23 /                                        i1mac211
c     data imach(12) /    -127 /                                        i1mac212
c     data imach(13) /     127 /                                        i1mac213
c     data imach(14) /      38 /                                        i1mac214
c     data imach(15) /    -127 /                                        i1mac215
c     data imach(16) /     127 /                                        i1mac216
c                                                                       i1mac217
c     machine constants for the honeywell 600/6000 series               bb815239
c                                                                       i1mac219
c     data imach( 1) /    5 /                                           i1mac220
c     data imach( 2) /    6 /                                           i1mac221
c     data imach( 3) /   43 /                                           i1mac222
c     data imach( 4) /    6 /                                           i1mac223
c     data imach( 5) /   36 /                                           i1mac224
c     data imach( 6) /    6 /                                           i1mac225
c     data imach( 7) /    2 /                                           i1mac226
c     data imach( 8) /   35 /                                           i1mac227
c     data imach( 9) / o377777777777 /                                  i1mac228
c     data imach(10) /    2 /                                           i1mac229
c     data imach(11) /   27 /                                           i1mac230
c     data imach(12) / -127 /                                           i1mac231
c     data imach(13) /  127 /                                           i1mac232
c     data imach(14) /   63 /                                           i1mac233
c     data imach(15) / -127 /                                           i1mac234
c     data imach(16) /  127 /                                           i1mac235
c                                                                       i1mac236
c     machine constants for the hp 2100                                 i1mac237
c     3 word double precision option with ftn4                          i1mac238
c                                                                       i1mac239
c     data imach(1) /      5/                                           i1mac240
c     data imach(2) /      6 /                                          i1mac241
c     data imach(3) /      4 /                                          i1mac242
c     data imach(4) /      1 /                                          i1mac243
c     data imach(5) /     16 /                                          i1mac244
c     data imach(6) /      2 /                                          i1mac245
c     data imach(7) /      2 /                                          i1mac246
c     data imach(8) /     15 /                                          i1mac247
c     data imach(9) /  32767 /                                          i1mac248
c     data imach(10)/      2 /                                          i1mac249
c     data imach(11)/     23 /                                          i1mac250
c     data imach(12)/   -128 /                                          i1mac251
c     data imach(13)/    127 /                                          i1mac252
c     data imach(14)/     39 /                                          i1mac253
c     data imach(15)/   -128 /                                          i1mac254
c     data imach(16)/    127 /                                          i1mac255
c                                                                       i1mac256
c     machine constants for the hp 2100                                 i1mac257
c     4 word double precision option with ftn4                          i1mac258
c                                                                       i1mac259
c     data imach(1) /      5 /                                          i1mac260
c     data imach(2) /      6 /                                          i1mac261
c     data imach(3) /      4 /                                          i1mac262
c     data imach(4) /      1 /                                          i1mac263
c     data imach(5) /     16 /                                          i1mac264
c     data imach(6) /      2 /                                          i1mac265
c     data imach(7) /      2 /                                          i1mac266
c     data imach(8) /     15 /                                          i1mac267
c     data imach(9) /  32767 /                                          i1mac268
c     data imach(10)/      2 /                                          i1mac269
c     data imach(11)/     23 /                                          i1mac270
c     data imach(12)/   -128 /                                          i1mac271
c     data imach(13)/    127 /                                          i1mac272
c     data imach(14)/     55 /                                          i1mac273
c     data imach(15)/   -128 /                                          i1mac274
c     data imach(16)/    127 /                                          i1mac275
c                                                                       i1mac276
c     machine constants for the hp 9000                                 bb815240
c                                                                       bb815241
c     data imach(1)  /    5 /                                           bb815242
c     data imach(2)  /    6 /                                           bb815243
c     data imach(3)  /    6 /                                           bb815244
c     data imach(3)  /    7 /                                           bb815245
c     data imach(5)  /   32 /                                           bb815246
c     data imach(6)  /    4 /                                           bb815247
c     data imach(7)  /    2 /                                           bb815248
c     data imach(8)  /   32 /                                           bb815249
c     data imach(9)  /2147483647 /                                      bb815250
c     data imach(10) /    2 /                                           bb815251
c     data imach(11) /   24 /                                           bb815252
c     data imach(12) / -126 /                                           bb815253
c     data imach(13) /  127 /                                           bb815254
c     data imach(14) /   53 /                                           bb815255
c     data imach(15) /-1015 /                                           bb815256
c     data imach(16) / 1017 /                                           bb815257
c                                                                       bb815258
c     machine constants for the ibm 360/370 series,                     i1mac277
c     the xerox sigma 5/7/9, the sel systems 85/86, and                 i1mac278
c     the perkin elmer (interdata) 7/32.                                i1mac279
c                                                                       i1mac280
c     data imach( 1) /   5 /                                            i1mac281
c     data imach( 2) /   6 /                                            i1mac282
c     data imach( 3) /   7 /                                            i1mac283
c     data imach( 4) /   6 /                                            i1mac284
c     data imach( 5) /  32 /                                            i1mac285
c     data imach( 6) /   4 /                                            i1mac286
c     data imach( 7) /  16 /                                            i1mac287
c     data imach( 8) /  31 /                                            i1mac288
c     data imach( 9) / z7fffffff /                                      i1mac289
c     data imach(10) /  16 /                                            i1mac290
c     data imach(11) /   6 /                                            i1mac291
c     data imach(12) / -64 /                                            i1mac292
c     data imach(13) /  63 /                                            i1mac293
c     data imach(14) /  14 /                                            i1mac294
c     data imach(15) / -64 /                                            i1mac295
c     data imach(16) /  63 /                                            i1mac296
c                                                                       i1mac297
c     machine constants for the ibm pc                                  bb815259
c                                                                       bb815260
c     data imach( 1) /     5 /                                          bb815261
c     data imach( 2) /     6 /                                          bb815262
c     data imach( 3) /     0 /                                          bb815263
c     data imach( 4) /     0 /                                          bb815264
c     data imach( 5) /    32 /                                          bb815265
c     data imach( 6) /     4 /                                          bb815266
c     data imach( 7) /     2 /                                          bb815267
c     data imach( 8) /    31 /                                          bb815268
c     data imach( 9) / 2147483647 /                                     bb815269
c     data imach(10) /     2 /                                          bb815270
c     data imach(11) /    24 /                                          bb815271
c     data imach(12) /  -125 /                                          bb815272
c     data imach(13) /   127 /                                          bb815273
c     data imach(14) /    53 /                                          bb815274
c     data imach(15) / -1021 /                                          bb815275
c     data imach(16) /  1023 /                                          bb815276
c                                                                       bb815277
c     machine constants for the pdp-10 (ka processor)                   bb815278
c                                                                       i1mac299
c     data imach( 1) /    5 /                                           i1mac300
c     data imach( 2) /    6 /                                           i1mac301
c     data imach( 3) /    5 /                                           i1mac302
c     data imach( 4) /    6 /                                           i1mac303
c     data imach( 5) /   36 /                                           i1mac304
c     data imach( 6) /    5 /                                           i1mac305
c     data imach( 7) /    2 /                                           i1mac306
c     data imach( 8) /   35 /                                           i1mac307
c     data imach( 9) / "377777777777 /                                  i1mac308
c     data imach(10) /    2 /                                           i1mac309
c     data imach(11) /   27 /                                           i1mac310
c     data imach(12) / -128 /                                           i1mac311
c     data imach(13) /  127 /                                           i1mac312
c     data imach(14) /   54 /                                           i1mac313
c     data imach(15) / -101 /                                           i1mac314
c     data imach(16) /  127 /                                           i1mac315
c                                                                       i1mac316
c     machine constants for the pdp-10 (ki processor)                   bb815279
c                                                                       i1mac318
c     data imach( 1) /    5 /                                           i1mac319
c     data imach( 2) /    6 /                                           i1mac320
c     data imach( 3) /    5 /                                           i1mac321
c     data imach( 4) /    6 /                                           i1mac322
c     data imach( 5) /   36 /                                           i1mac323
c     data imach( 6) /    5 /                                           i1mac324
c     data imach( 7) /    2 /                                           i1mac325
c     data imach( 8) /   35 /                                           i1mac326
c     data imach( 9) / "377777777777 /                                  i1mac327
c     data imach(10) /    2 /                                           i1mac328
c     data imach(11) /   27 /                                           i1mac329
c     data imach(12) / -128 /                                           i1mac330
c     data imach(13) /  127 /                                           i1mac331
c     data imach(14) /   62 /                                           i1mac332
c     data imach(15) / -128 /                                           i1mac333
c     data imach(16) /  127 /                                           i1mac334
c                                                                       i1mac335
c     machine constants for pdp-11 fortran supporting                   i1mac336
c     32-bit integer arithmetic.                                        i1mac337
c                                                                       i1mac338
c     data imach( 1) /    5 /                                           i1mac339
c     data imach( 2) /    6 /                                           i1mac340
c     data imach( 3) /    5 /                                           i1mac341
c     data imach( 4) /    6 /                                           i1mac342
c     data imach( 5) /   32 /                                           i1mac343
c     data imach( 6) /    4 /                                           i1mac344
c     data imach( 7) /    2 /                                           i1mac345
c     data imach( 8) /   31 /                                           i1mac346
c     data imach( 9) / 2147483647 /                                     i1mac347
c     data imach(10) /    2 /                                           i1mac348
c     data imach(11) /   24 /                                           i1mac349
c     data imach(12) / -127 /                                           i1mac350
c     data imach(13) /  127 /                                           i1mac351
c     data imach(14) /   56 /                                           i1mac352
c     data imach(15) / -127 /                                           i1mac353
c     data imach(16) /  127 /                                           i1mac354
c                                                                       i1mac355
c     machine constants for pdp-11 fortran supporting                   i1mac356
c     16-bit integer arithmetic.                                        i1mac357
c                                                                       i1mac358
c     data imach( 1) /    5 /                                           i1mac359
c     data imach( 2) /    6 /                                           i1mac360
c     data imach( 3) /    5 /                                           i1mac361
c     data imach( 4) /    6 /                                           i1mac362
c     data imach( 5) /   16 /                                           i1mac363
c     data imach( 6) /    2 /                                           i1mac364
c     data imach( 7) /    2 /                                           i1mac365
c     data imach( 8) /   15 /                                           i1mac366
c     data imach( 9) / 32767 /                                          i1mac367
c     data imach(10) /    2 /                                           i1mac368
c     data imach(11) /   24 /                                           i1mac369
c     data imach(12) / -127 /                                           i1mac370
c     data imach(13) /  127 /                                           i1mac371
c     data imach(14) /   56 /                                           i1mac372
c     data imach(15) / -127 /                                           i1mac373
c     data imach(16) /  127 /                                           i1mac374
c                                                                       i1mac375
c     machine constants for the sun                                     bb815280
c                                                                       bb815281
      data imach(1) /    5 /                                            bb815282
      data imach(2) /    6 /                                            bb815283
      data imach(3) /    6 /                                            bb815284
      data imach(4) /    6 /                                            bb815285
      data imach(5) /   32 /                                            bb815286
      data imach(6) /    4 /                                            bb815287
      data imach(7) /    2 /                                            bb815288
      data imach(8) /   31 /                                            bb815289
      data imach(9) /2147483647 /                                       bb815290
      data imach(10)/    2 /                                            bb815291
      data imach(11)/   24 /                                            bb815292
      data imach(12)/ -125 /                                            bb815293
      data imach(13)/  128 /                                            bb815294
      data imach(14)/   53 /                                            bb815295
      data imach(15)/ -1021 /                                           bb815296
      data imach(16)/  1024 /                                           bb815297
c                                                                       bb815298
c     machine constants for the univac 1100 series ftn compiler         bb815299
c                                                                       i1mac377
c                                                                       i1mac378
c     data imach( 1) /    5 /                                           i1mac379
c     data imach( 2) /    6 /                                           i1mac380
c     data imach( 3) /    1 /                                           i1mac381
c     data imach( 4) /    6 /                                           i1mac382
c     data imach( 5) /   36 /                                           i1mac383
c     data imach( 6) /    4 /                                           i1mac384
c     data imach( 7) /    2 /                                           i1mac385
c     data imach( 8) /   35 /                                           i1mac386
c     data imach( 9) / o377777777777 /                                  i1mac387
c     data imach(10) /    2 /                                           i1mac388
c     data imach(11) /   27 /                                           i1mac389
c     data imach(12) / -128 /                                           i1mac390
c     data imach(13) /  127 /                                           i1mac391
c     data imach(14) /   60 /                                           i1mac392
c     data imach(15) /-1024 /                                           i1mac393
c     data imach(16) / 1023 /                                           i1mac394
c                                                                       i1mac395
c                                                                       i1mac396
c     machine constants for the vax 11/780                              i1mac397
c                                                                       i1mac398
c     data imach(1) /    5 /                                            i1mac399
c     data imach(2) /    6 /                                            i1mac400
c     data imach(3) /    5 /                                            i1mac401
c     data imach(4) /    6 /                                            i1mac402
c     data imach(5) /   32 /                                            i1mac403
c     data imach(6) /    4 /                                            i1mac404
c     data imach(7) /    2 /                                            i1mac405
c     data imach(8) /   31 /                                            i1mac406
c     data imach(9) /2147483647 /                                       i1mac407
c     data imach(10)/    2 /                                            i1mac408
c     data imach(11)/   24 /                                            i1mac409
c     data imach(12)/ -127 /                                            i1mac410
c     data imach(13)/  127 /                                            i1mac411
c     data imach(14)/   56 /                                            i1mac412
c     data imach(15)/ -127 /                                            i1mac413
c     data imach(16)/  127 /                                            i1mac414
c                                                                       i1mac415
c     machine constants for the z80 microprocessor                      i1mac416
c                                                                       i1mac417
c     data imach( 1) /     1/                                           i1mac418
c     data imach( 2) /     1/                                           i1mac419
c     data imach( 3) /     0/                                           i1mac420
c     data imach( 4) /     1/                                           i1mac421
c     data imach( 5) /    16/                                           i1mac422
c     data imach( 6) /     2/                                           i1mac423
c     data imach( 7) /     2/                                           i1mac424
c     data imach( 8) /    15/                                           i1mac425
c     data imach( 9) / 32767/                                           i1mac426
c     data imach(10) /     2/                                           i1mac427
c     data imach(11) /    24/                                           i1mac428
c     data imach(12) /  -127/                                           i1mac429
c     data imach(13) /   127/                                           i1mac430
c     data imach(14) /    56/                                           i1mac431
c     data imach(15) /  -127/                                           i1mac432
c     data imach(16) /   127/                                           i1mac433
c                                                                       i1mac434
c                                                                       i1mac435
c***first executable statement  i1mach                                  i1mac436
      if (i .lt. 1  .or.  i .gt. 16) go to 10                           i1mac437
c                                                                       i1mac438
      i1mach = imach(i)                                                 bb815300
      return                                                            i1mac440
c                                                                       i1mac441
   10 continue                                                          bb815301
      write (unit = output, fmt = 9000)                                 bb815302
 9000 format ('1error    1 in i1mach - i out of bounds')                bb815303
c                                                                       bb811102
c     call fdump                                                        bb815305
c                                                                       bb815306
      stop                                                              bb815308
      end                                                               bb815309
