c-$Log:   /pvcs.config/mc3dvv/src/allocmem.F_a  $
c
c   Rev 1.4   Fri Jul 30 15:36:22 1999   lowman
c  Added Nu, slvelu, tstats and afftw includes and added
c  memory allocatation statements for all arrays that
c  appear in these includes blocks.
c
c   Rev 1.3   Tue Mar 23 14:09:36 1999   lowman
cAdded option of allocating memory for abe matrix for
cdifferent job cases. For example, the size of the abe
cmatrix is dependent on whether isx and jsx are even or
codd, whether lbd=mbd=0 and whether, xl=yl and nx=ny.
c
c   Rev 1.2   Tue Feb 09 14:55:16 1999   lowman
cIntroduced memory reduction with abe=-1 option and
cchanged arguments for floating-point matrix memory 
callocation by mmgetblk
c
c   Rev 1.1   Tue Oct 06 16:03:20 1998   lowman
cAdded second single precision work array spwk2
cAdded memory to hold tauz, tau13 and tau23
c
c   Rev 1.0   Mon Jul 14 14:20:48 1997   pvcs
c 
c *************************************************************
c *    ALLOCMEM
c *************************************************************
c *
c***begin prologue  allocmem
c***date written   930101   (yymmdd)
c***revision date  930803   (yymmdd)
c***category no.  
c***keywords
c***author   Lynn Trease   (Los Alamos National Laboratory)
c***purpose
c***   Allocate memory for arrays. 
c***description
c***references (none)
c***routines called  mallocf
c***modifications (yymmdd) modification
c***modifications (930726):[PvK] Use C-overlay of malloc to make this routine 
c***                             general for several platforms 
c***modifications (930803) [cwg] Add variable names to memory count output.
c***modifications (930809) [PvK] Changed memory declaration for 3-D arrays
c***                             in ctensr and a77
c***modifications (930823) [cwg] Use mmgetblk instead of malloc for memory
c***                             allocation. This is better for debugging.
c***modifications (930824) [cwg] Need to include a3.h to test for problem size.
c***modifications (930824) [cwg] Eliminate t43d as work array, use spwk instead.
c***modifications (950729) [PvK] Allocate space for DASPK arrays if necessary
c***modifications (950807) [GI]  Allocate space for variable viscosity arrays
c***modifications (9601xx) [PvK] Error correction: patch.h
c***end prologue  allocmem
c *
c *    SDK 6/99  Inclusion of MPI version. 
c *    PvK 7/99  Partial completion of memory reduction for MPI
c *              So far, only help arrays in flux/ctensr/heatflo have
c *              been reduced to comprise only the local parts.
c *              Savings in 2D are huge, reasonable in 3D.
c *
c *************************************************************
      subroutine allocmem
      use module_precision
      use module_param
      use module_giparam
      use module_a1
      use module_gia1
      use module_a2
      use module_gia3
      use module_a3
      use module_a22
      use module_ac1
      use module_a77
      use module_ftcoef
      use module_intfac
      use module_ctensr
      use module_ctoft
      use module_viscom
      use module_a9
      use module_gia9
      use module_spwork
      use module_a6
      use module_a7
      use module_a11
      use module_a13
      use module_a14
      use module_a20
      use module_a26
      use module_a30
      use module_flow
      use module_cgeoid
      use module_inttauc
      use module_a23
      use module_work1d
      use module_temp4
      use module_output
      use module_patch
      use module_prtcl
      use module_a24
      use module_a25
      use module_tauout
      use module_pttouv
      use module_timeint
      use module_alam
      use module_cvrms
      use module_petracer
      use module_nu
      use module_slvelu
      use module_tstats
      use module_afftw
      use module_vstats

#ifdef MPI 
      use module_mpibuf
      include 'mpif.h'
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )
      call MPI_COMM_SIZE( MPI_COMM_WORLD, numprocs, ierr)
      ntdeclared=0
      kdeclared=0
      do itest=0,numprocs-1
        call mpilayer(nz+1,numprocs,itest,
     &               izstart,izstop)
        izstart = max(izstart - 3,1)
        izstop = min(izstop + 2,nz+2)
        if (ny.le.1) then
c         *** 2D
          isize = (nx+2)*(izstop-izstart+1)
        else
c         *** 3D
          isize = (nx+2)*(ny+2)*(izstop-izstart+1)
        endif
        kdeclared =  max(kdeclared,izstop-izstart+1)
        ntdeclared = max(ntdeclared,isize)
      enddo
#else
      myid = 0
      ntdeclared=ijknx
      kdeclared =nz+1
#endif
       isaving = 0
c a1
       if (itimechoice.eq.0) then
         if(.not. allocated (t)) allocate(t(inx,jnx,knx))
         if(.not. allocated (dtensr)) allocate(dtensr(inx,jnx,knx))
         na1 = 2*ijknx
       else
c        *** DASPK
         if(.not. allocated (t)) allocate(t(itx,jtx,ktx))
         if(.not. allocated (tn)) allocate(tn(itx,jtx,ktx))
         na1 = 2*ijktx
       endif
c      >>>> GITO
       if(.not. allocated (visc)) allocate(visc(isl,jsl,ksl))
       if(.not. allocated (vnuc)) allocate(vnuc(isl,jsl,ksl))
       na1 = na1 + 2*ijksl

c a2

       if(.not. allocated (u)) allocate(u(inx,jnx,knx))
       if(.not. allocated (v)) allocate(v(inx,jnx,knx))
       if(.not. allocated (w)) allocate(w(inx,jnx,knx))
       na2 = 3*ijknx

c a3
       if(.not. allocated (avx)) allocate(avx(inx))
       if(.not. allocated (avy)) allocate(avy(jnx))
       if(.not. allocated (avz)) allocate(avz(knx))
       if(.not. allocated (z)) allocate(z(knx))
c a22

#ifdef MPI
       if(.not. allocated (vp)) allocate(vp(knxx_max,knyy_max,ksx))
       if(.not. allocated (v3)) allocate(v3(knxx_max,knyy_max,ksx))
       if(.not. allocated (vtf)) allocate(vtf(knxx_max,knyy_max,ksx))
       if(.not. allocated (taut)) allocate(taut(knxx_max,knyy_max,ksx))
       if(.not. allocated (taup)) allocate(taup(knxx_max,knyy_max,ksx))
       if(.not. allocated (tau3)) allocate(tau3(knxx_max,knyy_max,ksx))
#else
       if(.not. allocated (vp)) allocate(vp(isx,jsx,ksx))
       if(.not. allocated (v3)) allocate(v3(isx,jsx,ksx))
       if(.not. allocated (vtf)) allocate(vtf(isx,jsx,ksx))
       if(.not. allocated (taut)) allocate(taut(isx,jsx,ksx))
       if(.not. allocated (taup)) allocate(taup(isx,jsx,ksx))
       if(.not. allocated (tau3)) allocate(tau3(isx,jsx,ksx))
#endif
       na22 = 6*ijkred
       isaving = isaving + 6*(ijksx - ijkred)
c ac1

       if(.not. allocated (c)) 
     & allocate(c(icx,jcx,kcx,nspecies))
       if(.not. allocated (cn)) 
     & allocate(cn(icx,jcx,kcx,nspecies))
       if(.not. allocated (cmax)) allocate(cmax(nspecies))

       nac1 = 2*nspecies*ijkcx + nspecies

c a77

#ifdef MPI
       if(.not. allocated (advctd)) 
     & allocate(advctd(nx+2,ny+2,izstop-izstart+1))
#else
       if(.not. allocated (advctd)) allocate(advctd(inx,jnx,knx))
#endif
       na77 = ntdeclared
       isaving = isaving + (ijknx-ntdeclared)
       if (ny.le.2) then
c       *** 2-D situation

       if(.not. allocated (fluxv)) allocate(fluxv(inx,jnx,3,knx))
       if(.not. allocated (difshn)) allocate(difshn(inx,jnx,3,knx))
       na77 = 7*ijknx
       endif

c ftcoef

      if(.not. allocated (ft)) allocate(ft(isx,jsx,ksx))
      nftcoef = ijksx

c intfac

      if(.not. allocated (ytb)) allocate(ytb(nxyz,4,3))
      if(.not. allocated (ybb)) allocate(ybb(nxyz,4,3))
      if(.not. allocated (xlb)) allocate(xlb(nxyz,4,3))
      if(.not. allocated (xrb)) allocate(xrb(nxyz,4,3))
      if(.not. allocated (d3)) allocate(d3(nxyz,3))

      nintfac = 19*nxyz

c ctensr

#ifdef MPI
      if(.not. allocated (qshad)) 
     & allocate(qshad(nx+2,ny+2,izstop-izstart+1))
#else
      if(.not. allocated (qshad)) allocate(qshad(inx,jnx,knx))
#endif
      nctensr = ntdeclared
      isaving = isaving + ijknx - ntdeclared


       if (ny.le.2) then
c         *** 2D problem: 4D arrays (3,inx,iny,inz) are still used
         if(.not. allocated (fluxd)) allocate(fluxd(inx,jnx,3,knx))
         nctensr = ijknx * 3
       endif

c ctoft

       if(.not. allocated (cc)) allocate(cc(icx,jcx,kcx))
       nctoft = ijkcx

c viscom

       if(.not. allocated (vis)) allocate(vis(ksx))
       if(.not. allocated (vnu)) allocate(vnu(ksx))
       nviscom = 2*ksx

c a9

       if(.not. allocated (rr)) allocate(rr(kmx))
       na9 = kmx

c GIa9
       if(.not. allocated (r1)) allocate(r1(ksl))
       if(.not. allocated (r2)) allocate(r2(ksl))
       if(.not. allocated (r3)) allocate(r3(ksl))
       if(.not. allocated (r4)) allocate(r4(ksl))
       nGIa9 = 4*ksl

c spwork
c currently a double because used as a temp array.
c
       if(.not. allocated (spwk)) allocate(spwk(inx,jnx,knx))
       nspwork = ijknx

c a6

       if(.not. allocated (template)) allocate(template(iqx,jqx,npla))
       if(.not. allocated (template2)) allocate(template2(iqx,jqx,npla))
       if(.not. allocated (iplgeo)) allocate(iplgeo(iqx,jqx))

       na6 = (npla+1)*iqx*jqx
c a7

       if(.not. allocated (vsurf)) allocate(vsurf(iqx,jqx,2))

       na7 = 2*iqx*jqx

c a11

       if(.not. allocated (fli)) allocate(fli(inx,jnx))
       if(.not. allocated (flj)) allocate(flj(inx,jnx))
       if(.not. allocated (fl)) allocate(fl(inx,jnx))
       if(.not. allocated (wni)) allocate(wni(inx))
       if(.not. allocated (wnj)) allocate(wnj(jnx))

       na11 = 3*inx*jnx + inx+jnx

c a13

       if(.not. allocated (fftvx)) allocate(fftvx(iqx,jqx))
       if(.not. allocated (fftvy)) allocate(fftvy(iqx,jqx))

       na13 = 2*iqx*jqx

c a14

       if(.not. allocated (vpb)) allocate(vpb(iqx,jqx))
       if(.not. allocated (vtb)) allocate(vtb(iqx,jqx))

       na14 = 2*iqx*jqx

c a20

c     >>> GITO 10/9/94 (Changed ijksx -> ijknx, PvK 8/21/95)
      if ((iftau.eq.1).or.(ivistype.lt.0))then
         if(.not. allocated (tauz)) allocate(tauz(iqx,jqx,kqx))
         if(.not. allocated (tau13)) allocate(tau13(iqx,jqx,kqx))
         if(.not. allocated (tau23)) allocate(tau23(iqx,jqx,kqx))
     
         na20 = 3*ijknx
      else
         if(.not. allocated (tauz)) allocate(tauz(iqx,jqx,1))
         if(.not. allocated (tau13)) allocate(tau13(iqx,jqx,1))
         if(.not. allocated (tau23)) allocate(tau23(iqx,jqx,1))

         na20 = 3*inx*jnx
      endif

c a26

       if(.not. allocated (upl1)) allocate(upl1(kmx,npssd))
       if(.not. allocated (vpl1)) allocate(vpl1(kmx,npssd))
       if(.not. allocated (tau13pl1)) allocate(tau13pl1(kmx,npssd))
       if(.not. allocated (tau23pl1)) allocate(tau23pl1(kmx,npssd))

       na26 = 4*kmx*npssd
    
c a30

       if(.not. allocated (pc)) allocate(pc(npla,npla))
       if(.not. allocated (tc)) allocate(tc(npla))
       if(.not. allocated (vpl)) allocate(vpl(2,npla))

       na30 = 3*npla + npla*npla

c timeint
       ntimeint = 0
       if (itimechoice.eq.1) then
c         *** Allocate memory for DASPK arrays
          if(.not. allocated (yprime)) allocate(yprime(itx,jtx,ktx))
          if(.not. allocated (iiwork)) allocate(iiwork(niwork))
          if(.not. allocated (rwork)) allocate(rwork(nrwork))

          ntimeint = ijktx + niwork + nrwork
       else if (itimechoice.eq.-1) then

          if(.not. allocated (yprime)) allocate(yprime(itx,jtx,ktx))
          ntimeint = ijktx 
       endif

c flow

       if(.not. allocated (iabeaddr)) allocate(iabeaddr(inx,jnx))
       if(.not. allocated (yy)) allocate(yy(2,kmx))
       if(.not. allocated (y)) allocate(y(4,kmx))
       if(.not. allocated (b)) allocate(b(4*kmx))
       if(.not. allocated (yy1)) allocate(yy1(2,kmx+1))
       if(.not. allocated (y1)) allocate(y1(4,kmx+1))
       if(.not. allocated (ccc1)) allocate(ccc1(4,3,knx+1))
       if(.not. allocated (cc1)) allocate(cc1(4,3,knx+1))

       nflow = 10*kmx+6*(kmx+1) + 24*(knx+1) + inx*jnx

c cgeoid

       if(.not. allocated (gd)) allocate(gd(isx,jsx))
       ncgeoid = isx*jsx

c inttauc

       if(.not. allocated (a)) allocate(a(npla,npla))
       if(.not. allocated (x)) allocate(x(npla))
       if(.not. allocated (work)) allocate(work(npla))
       if(.not. allocated (iwork)) allocate(iwork(npla))

       ninttauc = 3*npla+npla*npla

c a23

       if(.not. allocated (tau13pl)) allocate(tau13pl(iqx,jqx,npla))
       if(.not. allocated (tau23pl)) allocate(tau23pl(iqx,jqx,npla))

       na23 = 2*npla*iqx*jqx

c work1d

       if(.not. allocated (wk1d)) allocate(wk1d(nxyz))

       nwork1d = nxyz

c temp4

       if(.not. allocated (ttemp)) allocate(ttemp(isx,ksx))

       ntemp4 = isx*ksx/2

c output

       if(.not. allocated (tavg)) allocate(tavg(isx,jsx,2))
       if(.not. allocated (dvpl)) allocate(dvpl(2,npla))
       if(.not. allocated (vplo)) allocate(vplo(2,npla))

       noutput = 2*isx*jsx + 4*npla

c patch

       if(.not. allocated (tave)) allocate(tave(inx,maxout))
       if(.not. allocated (wave)) allocate(wave(inx,maxout))
       if(.not. allocated (told)) allocate(told(inx,maxout))
       if(.not. allocated (kave)) allocate(kave(maxout))

       npatch = 3*inx*maxout + maxout

c a24

       if(ifsavepl.eq.1)then

       if(.not. allocated (vppl)) allocate(vppl(ixpl,jypl,kzpl,npssd))
       if(.not. allocated (v3pl)) allocate(v3pl(ixpl,jypl,kzpl,npssd))
       if(.not. allocated (vtfpl)) allocate(vtfpl(ixpl,jypl,kzpl,npssd))

       na24 = 3*npssd*ijkqx
       else
       if(.not. allocated (vppl)) allocate(vppl(1,1,1,npssd))
       if(.not. allocated (v3pl)) allocate(v3pl(1,1,1,npssd))
       if(.not. allocated (vtfpl)) allocate(vtfpl(1,1,1,npssd))

       na24 = 3*npssd
       endif

c prtcl

       if(.not. allocated (xpi)) allocate(xpi(nparticle))
       if(.not. allocated (ypi)) allocate(ypi(nparticle))
       if(.not. allocated (zpi)) allocate(zpi(nparticle))
       if(.not. allocated (locp)) allocate(locp(nparticle))
       if(.not. allocated (ipf)) allocate(ipf(nparticle))
       if(.not. allocated (jpf)) allocate(jpf(nparticle))
       if(.not. allocated (kpf)) allocate(kpf(nparticle))

       nprtcl = 4*nparticle + 3*nparticle*integ*1d0/ireal

c pttouv

       if(.not. allocated (akep)) allocate(akep(kmx))
       if(.not. allocated (aket)) allocate(aket(kmx))
       if(.not. allocated (ake)) allocate(ake(kmx))
       if(.not. allocated (akephv)) allocate(akephv(kmx))
 
      npttouv = 4*kmx

c tauout

       if(.not. allocated (ttmax)) allocate(ttmax(npla))
       if(.not. allocated (tmin)) allocate(tmin(npla))
       if(.not. allocated (tmean)) allocate(tmean(npla))
       if(.not. allocated (xmax)) allocate(xmax(npla))
       if(.not. allocated (xmin)) allocate(xmin(npla))

       ntauout = 5*npla

c alam
       ijsl=isl*jsl
       if(.not. allocated (xlam1)) allocate(xlam1(isl,jsl,ksl))
       if(.not. allocated (xlam2)) allocate(xlam2(isl,jsl,ksl))
       if(.not. allocated (xlam3)) allocate(xlam3(isl,jsl,ksl))
       if(.not. allocated (xlam4)) allocate(xlam4(isl,jsl,ksl))
       if(.not. allocated (work1)) allocate(work1(isl,jsl))
       if(.not. allocated (work2)) allocate(work2(isl,jsl))
       if(.not. allocated (work3)) allocate(work3(isl,jsl))
       if(.not. allocated (work4)) allocate(work4(isl,jsl))
       if(.not. allocated (workvisx)) allocate(workvisx(isl,jsl))
       if(.not. allocated (workvisy)) allocate(workvisy(isl,jsl))
       if(.not. allocated (workt)) allocate(workt(isl,jsl))

       nalam = (4*ijksl)+(7*ijsl)

       ncvrms = jmx*kmx + kmx

c petracer
       if (itracoption.eq.1) then

        npetracer = nmarkdim
       else
        npetracer = 0
       endif

c Nu

       if(.not. allocated (tavrg)) allocate(tavrg(knx))
       if(.not. allocated (znu)) allocate(znu(knx))

       nnu=2*knx


c solvlu

       if(.not. allocated (zwork)) allocate(zwork(knx))
       nsolvlu=knx

c tstats

       if(.not. allocated (havg_oft)) allocate(havg_oft(inx,10))
       if(.not. allocated (yavg_oft)) allocate(yavg_oft(inx,10,3))

       ntstats=40*inx

c vstats

       if(.not. allocated (vstats)) allocate(vstats(8))
c fft1d
      numfftmax=max(inx,jnx)
      num2fftmax = 3.5*numfftmax

       if(.not. allocated (fft1d)) allocate(fft1d(numfftmax))
       if(.not. allocated (wxs)) allocate(wxs(num2fftmax+16))
       if(.not. allocated (wx)) allocate(wx(2*numfftmax+15))
       if(.not. allocated (wxc)) allocate(wxc(3*numfftmax+15))
       if(.not. allocated (wys)) allocate(wys(num2fftmax+16))
       if(.not. allocated (wy)) allocate(wy(2*numfftmax+15))
       if(.not. allocated (wyc)) allocate(wyc(3*numfftmax+15))
 
      nfft1d=numfftmax+2*((num2fftmax+16)+(2*numfftmax+15)+
     & (3*numfftmax+15))

c JPL/CWG change, Dec. 98
c The following changes were introduced to implement significant memory
c requirement reductions. The abe matrix can be reduced by nearly a factor
c of 8 if lbd=mbd=0 and isx=jsx and xl=yl.
c If tau fields are not required, the abe field can tyipically
c share the same space as the taupl fields when there are 3 or more plates.
c taupl s are now allocated as a single block with pointers to make sure that
c upon deallocation of this memory a continuous large block is deallocated
c rather than 3 small unconnected blocks.

       if(ifsavepl.eq.1)then
       if(.not. allocated (tautpl)) 
     & allocate(tautpl(ixpl,jypl,kzpl,npssd))
       if(.not. allocated (tauppl))
     & allocate(tauppl(ixpl,jypl,kzpl,npssd))
       if(.not. allocated (tau3pl))
     & allocate(tau3pl(ixpl,jypl,kzpl,npssd))

       na25 = 3*npssd*ijkqx
       else
       if(.not. allocated (tautpl))
     & allocate(tautpl(1,1,1,npssd))
       if(.not. allocated (tauppl))
     & allocate(tauppl(1,1,1,npssd))
       if(.not. allocated (tau3pl))
     & allocate(tau3pl(1,1,1,npssd))

       na25 = 3*npssd
       endif

       if(.not. allocated (ipvt)) allocate(ipvt(4*kmx,nabe))

c     The following considers the amount of memory required to store all
c     of the abe matrices. The number of abe matrices required by a problem
c     is dependent on whether isx and jsx are even or odd and whether isx=jsx
c     and xl=yl. 

       if(((lbd.eq.0).and.(mbd.eq.0)).and.(jnx.ne.1))then
     
         if((mod(isx,2).eq.1).and.(mod(jsx,2).eq.1))
     & mabe=(isx-1)*(jsx-1)/4+(isx-1)/2+(jsx-1)/2+1+1

         if((mod(isx,2).eq.0).and.(mod(jsx,2).eq.0))
     & mabe=(isx-2)*(jsx-2)/4+isx+jsx+2+1

         if((mod(isx,2).eq.1).and.(mod(jsx,2).eq.0))
     & mabe=(jsx-2)*(isx-1)/4+jsx/2+1+(isx-1)/2+1

         if((mod(isx,2).eq.0).and.(mod(jsx,2).eq.1))
     & mabe=(isx-2)*(jsx-1)/4+isx/2+1+(jsx-1)/2+1

         if((isx.eq.jsx).and.(xl.eq.yl))then
  
           if((mod(isx,2).eq.1).and.(mod(jsx,2).eq.1))
     & mabe=1+(isx-1)/2+(isx-1)/2+((isx-1)*(jsx-3))/8+1

           if((mod(isx,2).eq.0).and.(mod(jsx,2).eq.0))
     & mabe=3+(isx-2)/2+(isx-2)+((isx-2)*(jsx-4))/8+1

         endif

       if(abs(ifabe).ne.1)then

       if(.not. allocated (abe)) allocate(abe(4*kmx,ncabe,mabe))
       nflow = 10*kmx+4*kmx*nabe+4*kmx*ncabe*mabe+
     v                6*(kmx+1) + 24*(knx+1)

       elseif(ifabe.eq.-1)then
         if(.not. allocated (abe)) allocate(abe(4*kmx,ncabe,1))

         if(mabe.lt.na25)then
          nflow=nflow+4*kmx*nabe+4*kmx*ncabe*mabe 
         else
          stop 'ERROR:allocmem, abe=-1 wont work'
         endif

c    Enters this else when b.c's are periodic and ifabe=1

       else
         if(.not. allocated (abe)) allocate(abe(4*kmx,ncabe,1))
         nflow  = 10*kmx + 4*kmx*nabe + 4*kmx*ncabe*1 +
     v                6*(kmx+1) + 24*(knx+1)      
       endif       

c    Enters the following elseif when b.c.'s are not periodic or job is 2D

      elseif(ifabe.eq.0)then
      if(.not. allocated (abe)) allocate(abe(4*kmx,ncabe,nabe))
       nflow  = 10*kmx + 4*kmx*nabe + 4*kmx*ncabe*nabe +
     v                6*(kmx+1) + 24*(knx+1)
      else
       if(ifabe.eq.-1)stop 'ifabe=-1 only works with periodic b.c.'        

       if(.not. allocated (abe)) allocate(abe(4*kmx,ncabe,1))
       nflow  = 10*kmx + 4*kmx*1 + 4*kmx*1*ncabe +
     v                6*(kmx+1) + 24*(knx+1)
      endif


c    calculate apx size of arrays in common blocks
      
      if (myid.eq.0) then
         write(6,239)
      endif
  239 format(' Number of words allocated in allocmem: ')

      ntotal =          na1 + na2 + na22 + nac1 + na77 + nftcoef
      if (myid.eq.0) then
         write(6,240) na1,na2,na22,nac1,na77,nftcoef
      endif
  240 format('na1      = ',i12,' t, dtensr ',/,
     v       'na2      = ',i12,' u, v, w ',/,
     v       'na22     = ',i12,' vp, v3, vtf, taut, taup, tau3 ',/,
     v       'nac1     = ',i12,' c, cn',/,
     v       'na77     = ',i12,' fluxv, difshn, advctd ',/,
     v       'nftcoef  = ',i12,' ft ')

      ntotal = ntotal + nintfac + nctensr + nctoft + nviscom
      if (myid.eq.0) then
        write(6,241) nintfac,nctensr,nctoft,nviscom
      endif
  241 format('nintfac  = ',i12,' ytb, ybb, xlb, xrb, d3 ',/,
     v       'nctensr  = ',i12,' fluxd, qshad',/,
     v       'nctoft   = ',i12,' cc ',/,
     v       'nviscom  = ',i12,' vis, vnu ')

      ntotal = ntotal + na9 +nGIa9 + nspwork + na6 + na7 + na11 + na13 
      if (myid.eq.0) then
         write(6,242) na9,nGIa9,nspwork,na6,na7,na11,na13
      endif
  242 format('na9      = ',i12,' rr ',/,
     v       'nGIa9    = ',i12,' r1 r2 r3 r4',/,
     v       'naspwork = ',i12,' spwork ',/,
     v       'na6      = ',i12,' qtemplate, iplgeo ',/,
     v       'na7      = ',i12,' vsurf ',/,
     v       'na11     = ',i12,' fli, flj, fl, wni, wnj ',/,
     v       'na13     = ',i12,' fftvx, fftvy ')

      ntotal = ntotal + na14 + na20 + na26 + na30 + ntimeint + nflow 
     v + ncgeoid 
      if (myid.eq.0) then
         write(6,244) na14,na20,na26,na30,ntimeint,nflow,ncgeoid
      endif
  244 format('na14     = ',i12,' vpb, vtb ',/,
     v       'na20     = ',i12,' tauz, tau13, tau23 ',/,
     v       'na26     = ',i12,' upl1, vpl1, tau13pl1, tau23pl1 ',/,
     v       'na30     = ',i12,' pc, tc, vpl ',/,
     v       'timeint  = ',i12,' yprime, iwork, rwork ',/,
     v       'nflow    = ',i12,' yy,y,abe,b,ipvt,yy1,y1,ccc1,cc1',/,
     v       'ncgeoid  = ',i12,' gd ')

      ntotal = ntotal + ninttauc + na23 + nwork1d + ntemp4 + noutput
      if (myid.eq.0) then
         write(6,246) ninttauc,na23,nwork1d,ntemp4,noutput
      endif
  246 format('ninttauc = ',i12,' a, x, work, iwork ',/,
     v       'na23     = ',i12,' tau13pl, tau23pl ',/,
     v       'nwork1d  = ',i12,' wk1d ',/,
     v       'ntemp4   = ',i12,' ttemp ',/,
     v       'noutput  = ',i12,' tavg, dvpl, vplo ')

      ntotal = ntotal + npatch + na24 + na25 + nprtcl + npttouv 
      if (myid.eq.0) then
         write(6,248) npatch,na24,na25,nprtcl,npttouv
      endif
  248 format('npatch   = ',i12,' tave, wave, told, kave ',/,
     v       'na24     = ',i12,' vppl, v3pl, vtfpl ',/,
     v       'na25     = ',i12,' tautpl, tauppl, tau3pl ',/,
     v       'nprtcl   = ',i12,' xpi, ypi, zpi, locp, ipf, jpf, kpf ',/,
     v       'npttouv  = ',i12,' akep, aket, ake, akephv ')

      ntotal = ntotal + ntauout
      if (myid.eq.0) then
         write(6,250) ntauout
      endif
  250 format('ntauout  = ',i12,' ttmax, tmin, tmean, xmax, xmin ')

      ntotal = ntotal + nalam + ncvrms + npetracer

      ntotal = ntotal + nnu + nsolvlu + ntstats + nfft1d

      if (myid.eq.0) then
         write(6,249) nnu, nsolvlu, ntstats, nfft1d
      endif
  249 format('nnu      = ',i12,'tavrg, znu',/,
     v       'nsolvlu  = ',i12,'zwork',/,
     v       'ntstats  = ',i12,'havg_oft, yavg_oft ',/,
     v       'nfft1d   = ',i12,'fft1d ')

      if (myid.eq.0) then
         write(6,252) nalam
         write(6,253) ncvrms
         write(6,254) npetracer
      endif
  252 format("nalam    = ",i12,' xlam1-xlam4, work1-work11')
  253 format("cvrms    = ",i12,' h2vrms, hvrms')
  254 format("tracer   = ",i12,' coormark ')

#ifdef MPI
        isize = 0
        knxx_max = 0
        knyy_max = 0
        knz_max = 0
        do ichunk = 0, numprocs-1
          if (ny .eq. 1) then
            ntmp = 1
          else
            ntmp = ny+1
          endif
          call mpiblock(nx+1,ntmp,numprocs,ichunk,
     &         ixstart,ixstop,iystart,iystop)
          ixstop = ixstop + 1
          call mpilayer(nz+1,numprocs,ichunk,izstart,izstop)
          izstart = izstart - 2
          if (izstart .lt. 1) izstart = 1
          izstop = izstop + 2
          if (izstop .gt. nz+1) izstop = nz+1
          knxx = ixstop - ixstart + 1
          knyy = iystop - iystart + 1
          knz = izstop - izstart + 1
          knxx_max = max(knxx_max,knxx)
          knyy_max = max(knyy_max,knyy)
          knz_max = max(knz_max,knz)
        enddo
        if(knyy_max .eq. 1) knyy_max=2
        ibuff = max(ibuff,knxx_max*knyy_max*knz_max)
        ibuff = ibuff*numprocs
        if (numprocs .eq. 1) ibuff = 1
c
c loop over all the chunks in the 'z' discritization
c
      if(.not. allocated (sbuff)) allocate(sbuff(ibuff))
      if(.not. allocated (rbuff)) allocate(rbuff(ibuff))
      ntotal = ntotal + 2*ibuff
      if (myid.eq.0) then
        write(6,255) 2*ibuff
      endif
      isaving = isaving - 2*ibuff
#endif

      if (myid.eq.0) then
        write(6,287)
        write(6,288) ntotal
c       rnspbyte=ntotal*8.e-6
c       write(6,290) rnspbyte
        rndpbyte=ntotal*8.e-6
        write(6,292) rndpbyte
      end if
      
  255 format("mpibuff  = ",i12,' sbuff,rbuff ')
  287 format('---------------------------------')
  288 format('ntotal   = ',i12  ,'   Total Words of Memory Program ')
  290 format('nspbyte  = ',f14.2,' Mb if single precision ')
  292 format('ndpbyte  = ',f14.2,' Mb if double precision ')

#ifdef MPI
      if (myid.eq.0) then
        write(6,*)
        write(6,297) isaving
        write(6,298) isaving*8.0e-6
  297 format('Memory saved in MPI routines: ', i12 , ' words')
  298 format('                          or: ', f12.2 , ' Mbytes')
      endif
#endif

#ifdef MEMORY_CHECK
c       call mmverify()
#endif
       return 
       end
