c $Log:   /pvcs.config/mc3dvv/src/calnus.F_a  $
c
c   Rev 1.1   Fri Jul 30 15:24:56 1999   lowman
c  Increased NMAXNUS to 2050
c
c   Rev 1.0   Mon Jul 14 14:20:52 1997   pvcs
c 
c *************************************************************
c *   CALNUS
c *
c *   Calculate normalized surface heat flow from temperature
c *   field y, defined on a NORDERth finite difference grid.
c *   y      - temperature field y(nx,ny)
c *   itx,jtx,ktx  - dimensions of grid (including virtual points)
c *   norder - order of FD approximation (2/4/6..)
c *   dx,dy  - grid spacing
c *   ichois - Choice parameter 1 = symmetric FD (order NORDER)
c *                             2 = one sided FD ( ,,     ,,  )
c *
c *************************************************************
      real*8 function calnus(norder,ichois)
      use module_precision
      use module_a1
      use module_gia1
      use module_gia3
      use module_a3
      use module_param
      use module_giparam
      parameter(NMAXNUS=4098)
      dimension dtdz(NMAXNUS),tava(NMAXNUS)
      dx = xl/(nx-1)
      dy = yl/(ny-1)
      dz = 1d0/(nz-1)
      oneoverh = 1d0/dz

      if (ichois.ne.1) then
          write(6,*) 'PERROR(calnus): use only ichois=1'
          stop 
      endif
      if (itx-2.gt.NMAXNUS) then
         write(6,*) 'PERROR(calnus): itx-2 > NMAX' 
         write(6,*) '    NMAX      : ',NMAX
         write(6,*) '    itx - 2   : ',itx-2
         calnus = 0
         return
      endif

      if (ny.eq.2) then
c        **** 2-D
         tav = 0
         calnus = 0
         if (norder.eq.2) then
c           *** symmetric, 2nd order (use anti-symmetry)
            do i=2,itx-1
               dtdz(i) = -oneoverh*t(i,1,ktx-1)
               tava(i) = t(i,1,1)
c              write(6,'(i5,2f8.3)') i,dtdz(i),tava(i)
            enddo
         else if (norder.eq.4) then
c           *** symmetric, 4th order (use anti-symmetry)
            do i=3,itx-2
               dtdz(i) = oneoverh*(
     v            -4d0/3*t(i,1,ktx-2) + 1d0/6*t(i,1,ktx-3))
               tava(i) = t(i,1,2)
            enddo
         else if (norder.eq.6) then
c           *** symmetric, 6th order (use anti-symmetry)
            do i=4,itx-3
               dtdz(i) =  oneoverh*(
     v             -1.5d0*t(i,1,ktx-3) + 0.3d0*t(i,1,ktx-4) 
     v             -  1d0/30*t(i,1,ktx-5))
               tava(i) = t(i,1,3)
            enddo
         else if (norder.eq.8) then
c            *** symmetric, 8th order (use anti-symmetry)
             do i=5,itx-4
                dtdz(i) = oneoverh*( -1.6d0*t(i,1,ktx-4) 
     v            + 0.4d0*t(i,1,ktx-5)-8d0/105*t(i,1,ktx-6)
     v            +1d0/140*t(i,1,ktx-7)) 
                tava(i) = t(i,1,4)
             enddo
         endif
         ipnus = norder/2 + 1
         npnus = itx - norder 
         tav = caln01(tava(ipnus),npnus,1)
         calnus = -caln01(dtdz(ipnus),npnus,1)/tav
c        write(6,*) 'tav: ',tav,caln01(dtdz(ipnus),npnus,1),npnus

      else

c *************************************************************
c        *** 3-D ***
c *************************************************************
         calnus = 0
         tav = 0
         if (norder.eq.2) then
            do j=2,jtx-1
                do i=2,itx-1
                   calnus = calnus - oneoverh*t(i,j,ktx-1)
                   tav = tav + t(i,j,1)
                enddo
            enddo
         else if (norder.eq.4) then
            do j=3,jtx-2
               do i=3,itx-2
                 calnus = calnus + oneoverh*(
     v              -4d0/3*t(i,j,ktx-2) + 1d0/6*t(i,j,ktx-3))
                 tav = tav + t(i,j,2)
               enddo
            enddo
         else if (norder.eq.6) then
            do j=4,jtx-3
               do i=4,itx-3
                 calnus = calnus + oneoverh*(
     v               -1.5d0*t(i,j,ktx-3) + 0.3d0*t(i,j,ktx-4)
     v               -  1d0/30*t(i,j,ktx-5))
                 tav = tav + t(i,j,3)
               enddo
            enddo
         else if (norder.eq.8) then
            do j=5,jtx-4
               do i=5,itx-4
                 calnus = calnus + oneoverh*( -1.6d0*t(i,j,ktx-4)
     v            + 0.4d0*t(i,j,ktx-5)-8d0/105*t(i,j,ktx-6)
     v            +1d0/140*t(i,j,ktx-7))
                 tav = tav + t(i,j,4)
               enddo
            enddo
         endif

         calnus = -calnus/tav

      endif


      return
      end

      real*8 function caln01(y,n,ichois)
      integer n
      real*8 y(n)

      caln01 = 0d0
      if (ichois.eq.1) then
         caln01 = 0.5*y(1) + 0.5*y(N)
         do i=2,n-1
            caln01 = caln01 + y(i)
         enddo
      else
         do i=1,n-2,2
            caln01 = caln01 + 1d0/3d0*(y(i) + 4*y(i+1) + y(i+2)) 
         enddo
         if ((n/2)*2.eq.n) then
c           *** Number of points is even, integrate over last interval
            caln01 = caln01 + 0.5*y(n-1)+0.5*y(n)
         endif
      endif
     
      return
      end

