c $Log:   /pvcs.config/mc3dvv/src/chem2ft.F_a  $
c
c   Rev 1.0   Mon Jul 14 14:20:56 1997   pvcs
c 
c *************************************************************
c *   CHEM2FT
c *************************************************************
      subroutine chem2ft
c >> date written:    950304   (yymmdd)
c >> author:   G. Ito
c >> description:
c       Calculates and adds body forces due to depletion
c       density reduction (c(i,j,k,1) and melt retention, f0
c       converts depletion at cell centers to cell corners
c
c       tm=temperature of the mantle at cell corners
c       tf=temperature required to melt by f0    
c >> Input:
c >> references (none)
c
c
c===================================================

      use module_precision
      use module_param
      use module_a1
      use module_gia1
      use module_ac1
      use module_gia3
      use module_a3
      use module_giparam
      use module_a12
c     common /a12/ lpu, lpv, lpw, lpt13, lpt23, lpt33, lpft, nxu, nxv,
c    1 nxw, nxt13, nxt23, nxt33, nxft, nxr, mpu, mpv, mpw, mpt13, mpt23,
c    2 mpt33, mpft, nyu, nyv, nyw, nyt13, nyt23, nyt33, nyft, nyr
      use module_ftcoef
      
c      do 10 k=1,knx		!debugging
c      do 10 j=1,jnx
c      do 10 i=1,inx
c10      c(i,j,k,1)=t(i,j,k)

c------------------------------------------------------
c     check to see if melt retention is added
c------------------------------------------------------
      iram=0
      if (ram.gt.0.and.imelt.eq.1.and.f0.gt.0.) iram=1

      if ((lbd.eq.0).and.(mbd.eq.0)) then
c------------------------------------------------------
c        2D Periodic
c------------------------------------------------------
        if (ny.eq.2) then
          do 30 k=2,nz-1
            if (iram.eq.1) then
              dep=bigd*(1.-(dble(k)-.5)/dble(nz))
              call tstl(dep,tl,ts)
            endif
          do 20 i=1,nxr+1
            fd=0.25*(c(i,1,k,1)+c(i+1,1,k,1)+
     &              c(i,1,k+1,1)+c(i+1,1,k+1,1))
            if (iram.eq.1.and.i.lt.nx/2) then
	      fd0=min1(fd,f0)
              tm=ft(i,1,k)*delt/rat+(f*600.)
              df=depl(tm,ts,tl,dep,zero,t0,cc,xlatent)
	      f1=min1(df,f0)
              ft(i,1,k)=ram*f1 + ft(i,1,k) 
            endif
            ft(i,1,k)=rad*fd+ft(i,1,k)  
     
   20     continue
   30     continue
          do 40 i=1,nxr+1
            ft(i,1,1)=rad*0.5*(c(i,1,1,1)+
     &               c(i+1,1,1,1))+ft(i,1,1)
            ft(i,1,nz)=rad*0.5*(c(i,1,nz1,1)+
     &               c(i+1,1,nz1,1))+ft(i,1,nz)
   40     continue
c
        else			
c------------------------------------------------------
c        3D Periodic
c------------------------------------------------------
          do 60 k=2,nz-1
            if (iram.eq.1) then
              dep=bigd*(1.-(dble(k)-.5)/dble(nz))
	      call tstl(dep,tl,ts)
            endif
          do 50 j=1,nyr+1
          do 50 i=1,nxr+1
	    fd=0.125*(c(i,j,k,1)+c(i+1,j,k,1)+
     &      c(i,j+1,k,1)+c(i+1,j+1,k,1)+c(i,j,k+1,1)+
     &      c(i+1,j,k+1,1)+c(i,j+1,k+1,1)+c(i+1,j+1,k+1,1))
            if (iram.eq.1.and.i.lt.nx/2)then
	      fd0=min1(fd,fd0)
              tm=ft(i,j,k)*delt/rat+(fd0*600.d0)
              df=depl(tm,ts,tl,dep,zero,t0,cc,xlatent)
	      f1=min1(df,f0)
              ft(i,j,k)= ram*f1 + ft(i,j,k)
            endif
            ft(i,j,k)=rad*fd+ft(i,j,k)

   50     continue
   60     continue
          do 70 j=1,nyr+1
          do 70 i=1,nxr+1
            ft(i,j,1)=rad*0.25*(c(i,j,1,1)+c(i+1,j,1,1)+
     &        c(i+1,j+1,1,1))+ft(i,j,1)
   70       ft(i,j,nz)=rad*0.25*(c(i,j,nz1,1)+c(i+1,j,nz1,1)+
     &        c(i,j+1,nz1,1)+c(i+1,j+1,nz1,1))+ft(i,j,nz)
        endif

      elseif ((lbd.eq.1).and.(mbd.eq.1)) then
c------------------------------------------------------
c        2D Reflecting
c------------------------------------------------------
        if (ny.eq.2) then		
          do 80 i=2,nx-1
            ft(i,1,1)=rad*0.5*(c(i,1,1,1)+c(i+1,1,1,1))+ft(i,1,1)
            ft(i,1,nz)=rad*0.5*(c(i,1,nz1,1)+c(i+1,1,nz1,1))
     &      +ft(i,1,nz)
   80     continue

          do 90 k=2,nz-1
            if (iram.eq.1) then
              dep=bigd*(1.-(dble(k)-.5)/dble(nz))
	      call tstl(dep,tl,ts)
	    endif
          do 90 i=2,nx-1
            fd=0.25*(c(i,1,k,1)+c(i+1,1,k,1)+
     &          c(i,1,k+1,1)+c(i+1,1,k+1,1))
            if (iram.eq.1.and.i.lt.nx/2)then
              fd0=min1(fd,f0)
              tm=ft(i,1,k)*delt/rat+(fd0*600.d0)	!add up to f0 latent heat 
              df=depl(tm,ts,tl,dep,zero,t0,cc,xlatent)	!how much melt retained
	      f1=min1(df,f0)
              ft(i,1,k)=ram*f1+ft(i,1,k)
	      if (df.gt.zero) then
                print *, '1)>>i,k,t,df=',i,k,tm,ts,df
                print *, ft(i,1,k)
	      endif
            endif

            ft(i,1,k)=rad*fd+ft(i,1,k)
   90     continue

          do 110 k=2,nz-1
            if (iram.eq.1) then
              dep=bigd*(1.-(dble(k)-.5)/dble(nz))
	      call tstl(dep,tl,ts)

              fd=0.5*(c(1,1,k,1)+c(1,1,k+1,1))
              fd0=min1(fd,f0)
              tm=ft(1,1,k)*delt/rat+(fd0*600.d0)
              df=depl(tm,ts,tl,dep,zero,t0,cc,xlatent)
	      f1=min1(df,f0)	    
              ft(1,1,k)=ram*f1+ft(1,1,k)
            endif

           ft(1,1,k)=rad*0.5*(c(1,1,k,1)+c(1,1,k+1,1))+ft(1,1,k)
           ft(nx,1,k)=rad*0.5*(c(nx1,1,k,1)+c(nx1,1,k+1,1))+
     &      ft(nx,1,k)
  110     continue

          ft(1,1,1)=rad*c(1,1,1,1)+ft(1,1,1)
          ft(1,1,nz)=rad*c(1,1,nz1,1)+ft(1,1,nz)
          ft(nx,1,1)=rad*c(nx1,1,1,1)+ft(nx,1,1)
          ft(nx,1,nz)=rad*c(nx1,1,nz1,1)+ft(nx,1,nz)

        else			
c------------------------------------------------------
c        3D Reflecting
c------------------------------------------------------

          do 140 k=2,nz-1
            if (iram.eq.1) then
              dep=bigd*(1.-(dble(k)-.5)/dble(nz))
	      call tstl(dep,tl,ts)
	    endif
          do 130 j=2,ny-1
          do 130 i=2,nx-1
	    fd=0.125*(c(i,j,k,1)+c(i+1,j,k,1)+
     &      c(i,j+1,k,1)+c(i+1,j+1,k,1)+
     &      c(i,j,k+1,1)+c(i+1,j,k+1,1)+c(i,j+1,k+1,1)+
     &      c(i+1,j+1,k+1,1))
          if (iram.eq.1.and.i.lt.nx/2) then
	    fd0=min1(fd,f0)
            tm=ft(i,j,k)*delt/rat+(fd0*600.d0)
            df=depl(tm,ts,tl,dep,zero,t0,cc,xlatent)	
	    f1=min1(df,f0)    
            ft(i,j,k)=ram*f1+ft(i,j,k)
          endif

          ft(i,j,k)=rad*fd+ft(i,j,k)      
  130     continue
  140     continue

          do 150 j=2,ny-1
          do 150 i=2,nx-1
          ft(i,j,1)=rad*0.25*(c(i,j,1,1)+c(i+1,j,1,1)+
     &      c(i,j+1,1,1)+c(i+1,j+1,1,1))+ft(i,j,1)
  150     ft(i,j,nz)=rad*0.25*(c(i,j,nz1,1)+c(i+1,j,nz1,1)+
     &       c(i,j+1,nz1,1)+c(i+1,j+1,nz1,1))+ft(i,j,nz)

          do 160 k=2,nz-1
            if (iram.eq.1) then
              dep=bigd*(1.-(dble(k)-.5)/dble(nz))
	      call tstl(dep,tl,ts)
	    endif
          do 160 i=2,nx-1
          if (iram.eq.1.and.i.lt.nx/2) then
	    fd=0.25*(c(i,1,k,1)+c(i+1,1,k,1)+
     &      c(i,1,k+1,1)+c(i+1,1,k+1,1))
	    fd0=min1(fd,f0)
            tm=ft(i,1,k)*delt/rat+(fd0*600.d0)
            df=depl(tm,ts,tl,dep,zero,t0,cc,xlatent)
	    f1=min1(df,f0)	    
            ft(i,1,k)=ram*f1+ft(i,1,k)

	    fd=0.25*(c(i,ny1,k,1)+c(i+1,ny1,k,1)+
     &      c(i,ny1,k+1,1)+c(i+1,ny1,k+1,1))
	    fd0=min1(fd,f0)
            tm=ft(i,ny,k)*delt/rat+(fd0*600.d0)
            df=depl(tm,ts,tl,dep,zero,t0,cc,xlatent)
	    f1=min1(df,f0)	    
            ft(i,ny,k)=ram*f1+ft(i,ny,k)
          endif

          ft(i,1,k)=rad*0.25*(c(i,1,k,1)+c(i+1,1,k,1)+
     &      c(i,1,k+1,1)+c(i+1,1,k+1,1))+ft(i,1,k)
          ft(i,ny,k)=rad*0.25*(c(i,ny1,k,1)+c(i+1,ny1,k,1)+
     &      c(i,ny1,k+1,1)+c(i+1,ny1,k+1,1))+ft(i,ny,k)
 160      continue

          do 170 i=2,nx-1
          ft(i,1,1)=rad*0.5*(c(i,1,1,1)+c(i+1,1,1,1))+ft(i,1,1)
          ft(i,1,nz)=rad*0.5*(c(i,1,nz1,1)+c(i+1,1,nz1,1))+
     &        ft(i,1,nz)
          ft(i,ny,1)=rad*0.5*(c(i,ny1,1,1)+c(i+1,ny1,1,1))+
     &        ft(i,ny,1)
  170     ft(i,ny,nz)=rad*.5*(c(i,ny1,nz1,1)+c(i+1,ny1,nz1,1))+
     &        ft(i,ny,nz)

          do 180 k=2,nz-1
            if (iram.eq.1) then
              dep=bigd*(1.-(dble(k)-.5)/dble(nz))
	      call tstl(dep,tl,ts)
            endif
          do 180 j=2,ny-1

          if (iram.eq.1) then
	    fd=0.25*(c(1,j,k,1)+c(1,j+1,k,1)+
     &       c(1,j,k+1,1)+c(1,j+1,k+1,1))
	    fd0=min1(fd,f0)
            tm=ft(1,j,k)*delt/rat+(fd0*600.d0)
            df=depl(tm,ts,tl,dep,zero,t0,cc,xlatent)	
	    f1=min1(df,f0)    
            ft(1,j,k)=ram*f1+ft(1,j,k)
	  endif

          ft(1,j,k)=rad*0.25*(c(1,j,k,1)+c(1,j+1,k,1)+
     &       c(1,j,k+1,1)+c(1,j+1,k+1,1))+ft(1,j,k)
          ft(nx,j,k)=rad*0.25*(c(nx1,j,k,1)+c(nx1,j+1,k,1)+
     &        c(nx1,j,k+1,1)+c(nx1,j+1,k+1,1))+ft(nx,j,k)
  180     continue
          do 190 j=2,ny-1
          ft(1,j,1)=rad*0.5*(c(1,j,1,1)+c(1,j+1,1,1))+ft(1,j,1)
          ft(1,j,nz)=rad*0.5*(c(1,j,nz1,1)+c(1,j+1,nz1,1))+
     &       ft(1,j,nz)
          ft(nx,j,1)=rad*0.5*(c(nx1,j,1,1)+c(nx1,j+1,1,1))+
     &       ft(nx,j,1)
  190     ft(nx,j,nz)=rad*0.5*(c(nx1,j,nz1,1)+c(nx1,j+1,nz1,1))+
     &        ft(nx,j,nz)

          do 200 k=2,nz-1
          if (iram.eq.1) then
            dep=bigd*(1.-(dble(k)-.5)/dble(nz))
	    call tstl(dep,tl,ts)

	    fd=0.5*(c(1,1,k,1)+c(1,1,k+1,1))
	    fd0=min1(fd,f0)
            tm=ft(1,1,k)*delt/rat+(fd0*600.d0)
            df=depl(tm,ts,tl,dep,zero,t0,cc,xlatent)	    
            ft(1,1,k)=ram*f1+ft(1,1,k)


	    fd=0.5*(c(1,ny1,k,1)+c(1,ny1,k+1,1))
	    fd0=min1(fd,f0)
            tm=ft(1,ny,k)*delt/rat+(fd0*600.d0)
            df=depl(tm,ts,tl,dep,zero,t0,cc,xlatent)	
	    ft(1,ny,k)=ram*f1+ft(1,ny,k)
	  endif

          ft(1,1,k)=rad*0.5*(c(1,1,k,1)+c(1,1,k+1,1))+ft(1,1,k)  
          ft(1,ny,k)=rad*0.5*(c(1,ny1,k,1)+c(1,ny1,k+1,1))+
     &      ft(1,ny,k)
          ft(nx,1,k)=rad*0.5*(c(nx1,1,k,1)+c(nx1,1,k+1,1))+
     &        ft(nx,1,k)
          ft(nx,ny,k)=rad*0.5*(c(nx1,ny1,k,1)+c(nx1,ny1,k+1,1))+
     &        ft(nx,ny,k)
200       continue

          ft(1,1,1)=rad*c(1,1,1,1)+ft(1,1,1)
          ft(1,1,nz)=rad*c(1,1,nz1,1)+ft(1,1,nz)
          ft(1,ny,1)=rad*c(1,ny1,1,1)+ft(1,ny,1)
          ft(1,ny,nz)=rad*c(1,ny1,nz1,1)+ft(1,ny,nz)
          ft(nx,1,1)=rad*c(nx1,1,1,1)+ft(nx,1,1)
          ft(nx,1,nz)=rad*c(nx1,1,nz1,1)+ft(nx,1,nz)
          ft(nx,ny,1)=rad*c(nx1,ny1,1,1)+ft(nx,ny,1)
          ft(nx,ny,nz)=rad*c(nx1,ny1,nz1,1)+ft(nx,ny,nz)
        endif

      else

c        not set up for mixed b.c.

      endif
       
c 
c      do 10 k=1,nz1		!debugging
c      do 10 j=1,jnx
c      do 10 i=1,inx
c        c(i,j,k,2)=ft(i,j,k)
c 10   continue

      return
      end


