c $Log:   /pvcs.config/mc3dvv/src/detyprime.F_a  $
c
c   Rev 1.0   Mon Jul 14 14:21:08 1997   pvcs
c 
c *************************************************************
c *   DETYPRIME
c *
c *   Calculate YPRIME that is consistent with F(Y',Y,T) = 0
c *
c *   Use NORDERth symmetric finite difference stencil for
c *    2      2
c *   d T    d T     dT      dT
c *   ---  + --- - u*--  - w*--
c *     2      2     
c *   dx     dz      dx      dz
c *************************************************************
      subroutine detyprime
      use module_precision
      use module_param
      use module_giparam
      use module_a1
      use module_gia1
      use module_a2
      use module_gia3
      use module_a3
      use module_timeint

      if (ny.ne.2) then
         write(6,*) 'detyprime: inx: ',inx,jnx,knx
         write(6,*) 'detyprime: itx: ',itx,jtx,ktx
         call detyp3d(t,yprime,itx,jtx,ktx,u,v,w,inx,jnx,knx,norder)
      else
         write(6,*) 'detyprime: inx: ',inx,jnx,knx
         write(6,*) 'detyprime: itx: ',itx,jtx,ktx
         call detyp2d(t,yprime,itx,jtx,ktx,u,v,w,inx,jnx,knx,norder)
      endif
      return
      end

c *************************************************************
c *   DETYP2D
c *************************************************************
      subroutine detyp2d(y,yprime,itx,jtx,ktx,u,v,w,inx,jnx,knx,norder) 
      use module_precision
      use module_a3
      use module_gia3
      dimension y(itx,jtx,ktx),yprime(itx,jtx,ktx)
      dimension u(inx,jnx,knx),v(inx,jnx,knx),w(inx,jnx,knx)
c     *** First order derivatives (w1..)
      parameter(w121=0.5)
      parameter(w141=2d0/3d0,w142=-1d0/12d0)
      parameter(w161=3d0/4d0,w162=-3d0/20d0,w163=1d0/60d0)
      parameter(w181=4d0/5d0,w182=-1d0/5d0 ,w183=4d0/105d0,
     v                                      w184=-1d0/280d0)
c     *** Second order derivatives (w2..)
      parameter(w220=-2d0,       w221=1.0)
      parameter(w240=-5d0/2d0,   w241=4d0/3d0,w242=-1d0/12d0)
      parameter(w260=-49d0/18d0, w261=3d0/2d0,w262=-3d0/20d0
     v                                       ,w263=1d0/90d0)
      parameter(w280=-205d0/72d0,w281=8d0/5d0,w282=-1d0/5d0 ,
     v                                        w283=8d0/315d0,
     v                                        w284=-1d0/560d0)

      dx = xl/(nx-1)
      dz = 1d0/(nz-1)
      oneh2x = 1d0/(dx*dx)
      oneh2z = 1d0/(dz*dz)
      halfhx = 1d0/(2*dx)
      halfhz = 1d0/(2*dz)
      twelvehx = 1d0/(12d0*dx)
      twelvehz = 1d0/(12d0*dz)
      twelveh2x = 1d0/(12d0*dx*dx)
      twelveh2z = 1d0/(12d0*dz*dz)
      sixnhx    = 1d0/(60d0*dx)
      sixnhz    = 1d0/(60d0*dz)
      sixnh2x   = 1d0/(180d0*dx*dx)
      sixnh2z   = 1d0/(180d0*dz*dz)

c *************************************************************
c *   BOUNDARY CONDITIONS
c *************************************************************
      if (norder.eq.2) then
         do k=1,ktx
c           *** Neumann
            y(1,1,k) = y(3,1,k)
            y(itx,1,k) = y(itx-2,1,k)
         enddo
      else if (norder.eq.4) then
         do k=1,ktx
c           *** Neumann
            y(1,1,k) = y(5,1,k)
            y(2,1,k) = y(4,1,k)
            y(itx,1,k) = y(itx-4,1,k)
            y(itx-1,1,k) = y(itx-3,1,k)
         enddo
         do i=1,itx
c           *** Dirichlet
            y(i,1,1) = 2d0-y(i,1,3)
            y(i,1,ktx) = 0d0-y(i,1,ktx-2)
         enddo
      else if (norder.eq.6) then
         do k=1,ktx
c           *** Neumann
            y(1,1,k) = y(7,1,k)
            y(2,1,k) = y(6,1,k)
            y(3,1,k) = y(5,1,k)
            y(itx,1,k) = y(itx-6,1,k)
            y(itx-1,1,k) = y(itx-5,1,k)
            y(itx-2,1,k) = y(itx-4,1,k)
         enddo
         do i=1,itx
c           *** Dirichlet
            y(i,1,1) = 2d0-y(i,1,5)
            y(i,1,2) = 2d0-y(i,1,4)
            y(i,1,ktx) = 0d0-y(i,1,ktx-4)
            y(i,1,ktx-1) = 0d0-y(i,1,ktx-3)
         enddo
      else if (norder.eq.8) then
         do k=1,ktx
c           *** Neumann
            y(1,1,k) = y(9,1,k)
            y(2,1,k) = y(8,1,k)
            y(3,1,k) = y(7,1,k)
            y(4,1,k) = y(6,1,k)
            y(itx,1,k) = y(itx-8,1,k)
            y(itx-1,1,k) = y(itx-7,1,k)
            y(itx-2,1,k) = y(itx-6,1,k)
            y(itx-3,1,k) = y(itx-5,1,k)
         enddo
         do i=1,itx
c           *** Dirichlet
            y(i,1,1) = 2d0-y(i,1,7)
            y(i,1,2) = 2d0-y(i,1,6)
            y(i,1,3) = 2d0-y(i,1,5)
            y(i,1,ktx) = 0d0-y(i,1,ktx-6)
            y(i,1,ktx-1) = 0d0-y(i,1,ktx-5)
            y(i,1,ktx-2) = 0d0-y(i,1,ktx-4)
         enddo
      else 
         write(6,*) 'PERROR(detyprime): norder is not 2,4,6,8' 
         write(6,*) '                   norder = ',norder
         stop
      endif 

c *************************************************************
c *   Calculate Y'
c *************************************************************
      jp = 1
      do k=1,ktx
         kp = k-norder/2+1
         do i=1,itx
            ip = i-norder/2
c           *** ORDER 2
            if (norder.eq.2) then
               if (k.eq.1.or.k.eq.ktx) then
                  yprime(i,1,k) = 0
               else if (i.eq.1.or.i.eq.itx) then
                  yprime(i,1,k) = 0
               else
                  d2tdx = y(i-1,1,k) - 2*y(i,1,k) + y(i+1,1,k)
                  d2tdx = d2tdx * oneh2x
                  d2tdz = y(i,1,k-1) - 2*y(i,1,k) + y(i,1,k+1)
                  d2tdz = d2tdz * oneh2z
                  dtdx  = y(i+1,1,k)-y(i-1,1,k)
                  dtdx  = halfhx*dtdx
                  dtdz  = y(i,1,k+1)-y(i,1,k-1)
                  dtdz  = halfhz*dtdz

                  tprime = d2tdx+d2tdz-u(ip,jp,kp)*dtdx-w(ip,jp,kp)*dtdz
                 yprime(i,1,k) = tprime
               endif

c           *** ORDER 4
            else if (norder.eq.4) then
               if (k.le.2.or.k.ge.ktx-1) then
                  yprime(i,1,k) = 0
               else if (i.le.2.or.i.ge.itx-1) then
                  yprime(i,1,k) = 0
               else
                 d2tdx = w242*y(i+2,1,k) + w241*y(i+1,1,k) +
     v                   w240*y(i,1,k) +
     v                   w241*y(i-1,1,k) + w242*y(i-2,1,k)
                 d2tdx = d2tdx * oneh2x
                 d2tdz = w242*y(i,1,k+2) + w241*y(i,1,k+1) +
     v                   w240*y(i,1,k) +
     v                   w241*y(i,1,k-1) + w242*y(i,1,k-2)
                 d2tdz = d2tdz * oneh2z

                 dtdx = w142*y(i+2,1,k)+w141*y(i+1,1,k)-w141*y(i-1,1,k)
     v                  -w142*y(i-2,1,k)
                 dtdx = dtdx * onehx
                 dtdz = w142*y(i,1,k+2)+w141*y(i,1,k+1)-w141*y(i,1,k-1)
     v                  -w142*y(i,1,k-2)
                 dtdz = dtdz * onehz

                 tprime = d2tdx+d2tdz-u(ip,jp,kp)*dtdx-w(ip,jp,kp)*dtdz
                 yprime(i,1,k) = tprime
               endif
c           *** ORDER 6
            else if (norder.eq.6) then
               if (k.le.3.or.k.ge.ktx-2) then
                  yprime(i,1,k) = 0
               else if (i.le.3.or.i.ge.itx-2) then
                  yprime(i,1,k) = 0
               else
                 d2tdx = 2*y(i+3,1,k) -27*y(i+2,1,k) + 270*y(i+1,1,k) -
     v            490*y(i,1,k)+270*y(i-1,1,k)-27*y(i-2,1,k)+2*y(i-3,1,k)
                 d2tdx = d2tdx * sixnh2x
                 d2tdz = 2*y(i,1,k+3) -27*y(i,1,k+2) + 270*y(i,1,k+1) -
     v            490*y(i,1,k)+270*y(i,1,k-1)-27*y(i,1,k-2)+2*y(i,1,k-3)
                 d2tdz = d2tdz * sixnh2z
                 dtdx=y(i+3,1,k)-9*y(i+2,1,k)+45*y(i+1,1,k)-
     v             45*y(i-1,1,k)+ 9*y(i-2,1,k) - y(i-3,1,k)
                 dtdz = y(i,1,k+3) -9*y(i,1,k+2) + 45*y(i,1,k+1) -
     v             45*y(i,1,k-1)+ 9*y(i,1,k-2) - y(i,1,k-3)
c                write(6,'(4f12.7)') dtdx,y(i+1,1,k)-y(i-1,1,k),dtdz,
c    v                    y(i,1,k+1)-y(i,1,k-1)

                 dtdx = dtdx * sixnhx
                 dtdz = dtdz * sixnhz

c                dtdxl = halfhx*(y(i+1,1,k)-y(i-1,1,k))
c                dtdzl = halfhz*(y(i,1,k+1)-y(i,1,k-1))
c                write(6,'(4f12.7)') dtdx,dtdxl,dtdz,dtdzl
c                d2tdxl = oneh2x*(y(i+1,1,k)-2d0*y(i,1,k)+y(i-1,1,k))
c                d2tdzl = oneh2z*(y(i,1,k+1)-2d0*y(i,1,k)+y(i,1,k-1))

c                write(6,100) i,k,dtdx*u(ip,jp,kp),dtdxl*u(ip,jp,kp),
c    v              dtdz*w(ip,jp,kp),dtdzl*w(ip,jp,kp),d2tdx,d2tdxl,
c    v              d2tdz,d2tdzl,u(ip,jp,kp),w(ip,jp,kp)

                 tprime = d2tdx+d2tdz-u(ip,jp,kp)*dtdx-w(ip,jp,kp)*dtdz
                 yprime(i,1,k) = tprime

               endif

c          *** ORDER 8
           else if (norder.eq.8) then
               if (k.le.4.or.k.ge.ktx-3) then
                  yprime(i,1,k) = 0
               else if (i.le.4.or.i.ge.itx-3) then
                  yprime(i,1,k) = 0
               else
                 d2tdx = w284*y(i+4,1,k) + w283*y(i+3,1,k)
     v             + w282*y(i+2,1,k) + w281*y(i+1,1,k) + w280*y(i,1,k)
     v             + w281*y(i-1,1,k) + w282*y(i-2,1,k) + w283*y(i-3,1,k)
     v             + w284*y(i-4,1,k)
                 d2tdx = d2tdx * oneh2x
                 d2tdz = w284*y(i,1,k+4) + w283*y(i,1,k+3)
     v            + w282*y(i,1,k+2)
     v            + w281*y(i,1,k+1) + w280*y(i,1,k)   + w281*y(i,1,k-1)
     v            + w282*y(i,1,k-2) + w283*y(i,1,k-3) + w284*y(i,1,k-4)
                 d2tdz = d2tdz * oneh2x

                 dtdx = w184*y(i+4,1,k) + w183*y(i+3,1,k)
     v             + w182*y(i+2,1,k)
     v             + w181*y(i+1,1,k) - w181*y(i-1,1,k) - w182*y(i-2,1,k)
     v             - w183*y(i-3,1,k) - w184*y(i-4,1,k)
                 dtdx = dtdx * onehx
                 dtdz = w184*y(i,1,k+4) + w183*y(i,1,k+3)
     v             + w182*y(i,1,k+2)
     v             + w181*y(i,1,k+1) - w181*y(i,1,k-1) - w182*y(i,1,k-2)
     v             - w183*y(i,1,k-3) - w184*y(i,1,k-4)
                 dtdz = dtdz * onehz

                 tprime = d2tdx+d2tdz-u(ip,jp,kp)*dtdx-w(ip,jp,kp)*dtdz
                 yprime(i,1,k) = tprime

               endif
           endif
         enddo
      enddo
100   format(2i5,5(2f12.7,2x))

c     call printy(y,itx,jtx,ktx)
c     call printy(yprime,itx,jtx,ktx)

      return
      end

c *************************************************************
c *   DETYP3D
c *************************************************************
      subroutine detyp3d(y,yprime,itx,jtx,ktx,u,v,w,inx,jnx,knx,norder) 
      use module_precision
      use module_a3
      use module_gia3
      dimension y(itx,jtx,ktx),yprime(itx,jtx,ktx)
      dimension u(inx,jnx,knx),v(inx,jnx,knx),w(inx,jnx,knx)
c     *** First order derivatives (w1..)
      parameter(w121=0.5)
      parameter(w141=2d0/3d0,w142=-1d0/12d0)
      parameter(w161=3d0/4d0,w162=-3d0/20d0,w163=1d0/60d0)
      parameter(w181=4d0/5d0,w182=-1d0/5d0 ,w183=4d0/105d0,
     v                                      w184=-1d0/280d0)
c     *** Second order derivatives (w2..)
      parameter(w220=-2d0,       w221=1.0)
      parameter(w240=-5d0/2d0,   w241=4d0/3d0,w242=-1d0/12d0)
      parameter(w260=-49d0/18d0, w261=3d0/2d0,w262=-3d0/20d0
     v                                       ,w263=1d0/90d0)
      parameter(w280=-205d0/72d0,w281=8d0/5d0,w282=-1d0/5d0 ,
     v                                        w283=8d0/315d0,
     v                                        w284=-1d0/560d0)

      dx = xl/(nx-1)
      dy = yl/(ny-1)
      dz = 1d0/(nz-1)
      oneh2x = 1d0/(dx*dx)
      oneh2y = 1d0/(dy*dy)
      oneh2z = 1d0/(dz*dz)
      onehx = 1d0/dx
      onehy = 1d0/dy
      onehz = 1d0/dz
      halfhx = 1d0/(2*dx)
      halfhy = 1d0/(2*dy)
      halfhz = 1d0/(2*dz)
      twelvehx = 1d0/(12d0*dx)
      twelvehy = 1d0/(12d0*dy)
      twelvehz = 1d0/(12d0*dz)
      twelveh2x = 1d0/(12d0*dx*dx)
      twelveh2y = 1d0/(12d0*dy*dy)
      twelveh2z = 1d0/(12d0*dz*dz)
      sixnhx    = 1d0/(60d0*dx)
      siynhy    = 1d0/(60d0*dy)
      sixnhz    = 1d0/(60d0*dz)
      sixnh2x   = 1d0/(180d0*dx*dx)
      siynh2y   = 1d0/(180d0*dy*dy)
      sixnh2z   = 1d0/(180d0*dz*dz)

c *************************************************************
c *   BOUNDARY CONDITIONS
c *************************************************************
      if (norder.eq.2) then
         do k=1,ktx
c           *** Neumann
            do j=1,jtx
               y(1,j,k) = y(3,j,k)
               y(itx,j,k) = y(itx-2,j,k)
            enddo
            do i=1,itx
               y(i,1,k) = y(i,3,k)
               y(i,jtx,k) = y(i,jtx-2,k)
            enddo
         enddo
      else if (norder.eq.4) then
         do k=1,ktx
c           *** Neumann
            do j=1,jtx
              y(1,j,k) = y(5,j,k)
              y(2,j,k) = y(4,j,k)
              y(itx,j,k) = y(itx-4,j,k)
              y(itx-1,j,k) = y(itx-3,j,k)
            enddo
            do i=1,itx
              y(i,1,k) = y(i,5,k)
              y(i,2,k) = y(i,4,k)
              y(i,jtx,k) = y(i,jtx-4,k)
              y(i,jtx-1,k) = y(i,jtx-3,k)
            enddo
         enddo
         do i=1,itx
c           *** Dirichlet
            do j=1,jtx
              y(i,j,1) = 2d0-y(i,j,3)
              y(i,j,ktx) = 0d0-y(i,j,ktx-2)
            enddo
         enddo
      else if (norder.eq.6) then
         do k=1,ktx
c           *** Neumann
            do j=1,jtx
              y(1,j,k) = y(7,j,k)
              y(2,j,k) = y(6,j,k)
              y(3,j,k) = y(5,j,k)
              y(itx,j,k) = y(itx-6,j,k)
              y(itx-1,j,k) = y(itx-5,j,k)
              y(itx-2,j,k) = y(itx-4,j,k)
            enddo
            do i=1,itx
              y(i,1,k) = y(i,7,k)
              y(i,2,k) = y(i,6,k)
              y(i,3,k) = y(i,5,k)
              y(i,jtx,k) = y(i,jtx-6,k)
              y(i,jtx-1,k) = y(i,jtx-5,k)
              y(i,jtx-2,k) = y(i,jtx-4,k)
            enddo
         enddo
         do j=1,jtx
           do i=1,itx
c            *** Dirichlet
             y(i,j,1) = 2d0-y(i,j,5)
             y(i,j,2) = 2d0-y(i,j,4)
             y(i,j,ktx) = 0d0-y(i,j,ktx-4)
             y(i,j,ktx-1) = 0d0-y(i,j,ktx-3)
           enddo
         enddo
      else if (norder.eq.8) then
         do k=1,ktx
c          *** Neumann
           do j=1,jtx
             y(1,j,k) = y(9,j,k)
             y(2,j,k) = y(8,j,k)
             y(3,j,k) = y(7,j,k)
             y(4,j,k) = y(6,j,k)
             y(itx,j,k) = y(itx-8,j,k)
             y(itx-1,j,k) = y(itx-7,j,k)
             y(itx-2,j,k) = y(itx-6,j,k)
             y(itx-3,j,k) = y(itx-5,j,k)
           enddo
           do i=1,itx
             y(i,1,k) = y(i,9,k)
             y(i,2,k) = y(i,8,k)
             y(i,3,k) = y(i,7,k)
             y(i,4,k) = y(i,6,k)
             y(i,jtx,k) = y(i,jtx-8,k)
             y(i,jtx-1,k) = y(i,jtx-7,k)
             y(i,jtx-2,k) = y(i,jtx-6,k)
             y(i,jtx-3,k) = y(i,jtx-5,k)
          enddo
        enddo
        do j=1,jtx
          do i=1,itx
c           *** Dirichlet
            y(i,j,1) = 2d0-y(i,j,7)
            y(i,j,2) = 2d0-y(i,j,6)
            y(i,j,3) = 2d0-y(i,j,5)
            y(i,j,ktx) = 0d0-y(i,j,ktx-6)
            y(i,j,ktx-1) = 0d0-y(i,j,ktx-5)
            y(i,j,ktx-2) = 0d0-y(i,j,ktx-4)
         enddo
        enddo
      else
         write(6,*) 'PERROR(res3d): norder is not 2,4,6,8'
         write(6,*) '                   norder = ',norder
         stop
      endif

c *************************************************************
c *   Calculate Y'
c *************************************************************
c     *** ORDER 2
      if (norder.eq.2) then
        do k=1,ktx
          kp = k-norder/2+1
          do j=1,jtx
            jp = j-norder/2
            do i=1,itx
               ip = i-norder/2
               if (k.eq.1.or.k.eq.ktx) then
                  yprime(i,j,k) = 0
               else if (j.eq.1.or.j.eq.jtx) then
                  yprime(i,j,k) = 0
               else if (i.eq.1.or.i.eq.itx) then
                  yprime(i,j,k) = 0
               else
                  d2tdx = y(i-1,j,k) - 2*y(i,j,k) + y(i+1,j,k)
                  d2tdx = d2tdx * oneh2x
                  d2tdy = y(i,j-1,k) - 2*y(i,j,k) + y(i,j+1,k)
                  d2tdy = d2tdy * oneh2y
                  d2tdz = y(i,j,k-1) - 2*y(i,j,k) + y(i,j,k+1)
                  d2tdz = d2tdz * oneh2z
                  dtdx  = y(i+1,j,k)-y(i-1,j,k)
                  dtdx  = halfhx*dtdx
                  dtdy  = y(i,j+1,k)-y(i,j-1,k)
                  dtdy  = halfhy*dtdy
                  dtdz  = y(i,j,k+1)-y(i,j,k-1)
                  dtdz  = halfhz*dtdz
                  tprime = d2tdx + d2tdy + d2tdz
     v               -u(ip,jp,kp)*dtdx-v(ip,jp,kp)*dtdy-w(ip,jp,kp)*dtdz 
                  yprime(i,j,k) = tprime
c                dtdxl = halfhx*(y(i+1,j,k)-y(i-1,j,k))
c                dtdyl = halfhy*(y(i,j+1,k)-y(i,j-1,k))
c                dtdzl = halfhz*(y(i,j,k+1)-y(i,j,k-1))
c                d2tdxl=oneh2x*(y(i+1,j,k)-2d0*y(i,j,k)+y(i-1,j,k))
c                d2tdyl=oneh2y*(y(i,j+1,k)-2d0*y(i,j,k)+y(i,j-1,k))
c                d2tdzl=oneh2z*(y(i,j,k+1)-2d0*y(i,j,k)+y(i,j,k-1))
c                write(6,100) i,j,k,dtdx*u(ip,jp,kp),dtdxl*u(ip,jp,kp),
c    v                            dtdy*v(ip,jp,kp),dtdyl*v(ip,jp,kp),
c    v                            dtdz*w(ip,jp,kp),dtdzl*w(ip,jp,kp),
c    v              d2tdx,d2tdxl,d2tdz,d2tdzl

               endif
            enddo
          enddo
        enddo
      else if (norder.eq.4) then
        do k=1,ktx
          kp = k-norder/2+1
          do j=1,jtx
            jp = j-norder/2
            do i=1,itx
               ip = i-norder/2
               if (k.le.2.or.k.ge.ktx-1) then
                  yprime(i,j,k) = 0
               else if (j.le.2.or.j.ge.jtx-1) then
                  yprime(i,j,k) = 0
               else if (i.le.2.or.i.ge.itx-1) then
                  yprime(i,j,k) = 0
               else
                 d2tdx = w242*y(i+2,j,k) + w241*y(i+1,j,k) +
     v                   w240*y(i,j,k) +
     v                   w241*y(i-1,j,k) + w242*y(i-2,j,k)
                 d2tdx = d2tdx * oneh2x
                 d2tdy = w242*y(i,j+2,k) + w241*y(i,j+1,k) +
     v                   w240*y(i,j,k) +
     v                   w241*y(i,j-1,k) + w242*y(i,j-2,k)
                 d2tdy = d2tdy * oneh2y
                 d2tdz = w242*y(i,j,k+2) + w241*y(i,j,k+1) +
     v                   w240*y(i,j,k) +
     v                   w241*y(i,j,k-1) + w242*y(i,j,k-2)
                 d2tdz = d2tdz * oneh2z

                 dtdx = w142*y(i+2,j,k)+w141*y(i+1,j,k)-w141*y(i-1,j,k)
     v                  -w142*y(i-2,j,k)
                 dtdx = dtdx * onehx
                 dtdy = w142*y(i,j+2,k)+w141*y(i,j+1,k)-w141*y(i,j-1,k)
     v                  -w142*y(i,j-2,k)
                 dtdy = dtdy * onehy
                 dtdz = w142*y(i,j,k+2)+w141*y(i,j,k+1)-w141*y(i,j,k-1)
     v                  -w142*y(i,j,k-2)
                 dtdz = dtdz * onehz
                 tprime = d2tdx + d2tdy + d2tdz
     v               -u(ip,jp,kp)*dtdx-v(ip,jp,kp)*dtdy-w(ip,jp,kp)*dtdz  
                 yprime(i,j,k) = tprime
c                dtdxl = halfhx*(y(i+1,j,k)-y(i-1,j,k))
c                dtdyl = halfhy*(y(i,j+1,k)-y(i,j-1,k))
c                dtdzl = halfhz*(y(i,j,k+1)-y(i,j,k-1))
c                d2tdxl=oneh2x*(y(i+1,j,k)-2d0*y(i,j,k)+y(i-1,j,k))
c                d2tdyl=oneh2y*(y(i,j+1,k)-2d0*y(i,j,k)+y(i,j-1,k))
c                d2tdzl=oneh2z*(y(i,j,k+1)-2d0*y(i,j,k)+y(i,j,k-1))
c                write(6,100) i,j,k,dtdx*u(ip,jp,kp),dtdxl*u(ip,jp,kp),
c    v                            dtdy*v(ip,jp,kp),dtdyl*v(ip,jp,kp),
c    v                            dtdz*w(ip,jp,kp),dtdzl*w(ip,jp,kp),
c    v              d2tdx,d2tdxl,d2tdz,d2tdzl

               endif
            enddo
          enddo
        enddo

c       *** ORDER 6
      else if (norder.eq.6) then
        do k=1,ktx
          kp = k-norder/2+1
          do j=1,jtx
            jp = j-norder/2
            do i=1,itx
               ip = i-norder/2
               if (k.le.3.or.k.ge.ktx-2) then
                  yprime(i,j,k) = 0
               else if (j.le.3.or.j.ge.jtx-2) then
                  yprime(i,j,k) = 0
               else if (i.le.3.or.i.ge.itx-2) then
                  yprime(i,j,k) = 0
               else
                 d2tdx = w263*y(i+3,j,k) + w262*y(i+2,j,k) + 
     v                   w261*y(i+1,j,k) + w260*y(i,j,k)   +
     v                   w261*y(i-1,j,k) + w262*y(i-2,j,k) + 
     v                   w263*y(i-3,j,k)
                 d2tdx = d2tdx * oneh2x
                 d2tdy = w263*y(i,j+3,k) + w262*y(i,j+2,k) + 
     v                   w261*y(i,j+1,k) + w260*y(i,j,k)   +
     v                   w261*y(i,j-1,k) + w262*y(i,j-2,k) + 
     v                   w263*y(i,j-3,k)
                 d2tdy = d2tdy * oneh2y
                 d2tdz = w263*y(i,j,k+3) + w262*y(i,j,k+2) + 
     v                   w261*y(i,j,k+1) + w260*y(i,j,k)   +
     v                   w261*y(i,j,k-1) + w262*y(i,j,k-2) +
     v                   w263*y(i,j,k-3)
                 d2tdz = d2tdz * oneh2z

                 dtdx=   w163*y(i+3,j,k) + w162*y(i+2,j,k) + 
     v                   w161*y(i+1,j,k) - w161*y(i-1,j,k) -
     v                   w162*y(i-2,j,k) - w163*y(i-3,j,k) 
                 dtdx = dtdx * onehx
                 dtdy=   w163*y(i,j+3,k) + w162*y(i,j+2,k) +
     v                   w161*y(i,j+1,k) - w161*y(i,j-1,k) -
     v                   w162*y(i,j-2,k) - w163*y(i,j-3,k) 
                 dtdy = dtdy * onehy
                 dtdz=   w163*y(i,j,k+3) + w162*y(i,j,k+2) +
     v                   w161*y(i,j,k+1) - w161*y(i,j,k-1) -
     v                   w162*y(i,j,k-2) - w163*y(i,j,k-3) 
                 dtdz = dtdz * onehz

c                dtdxl = halfhx*(y(i+1,j,k)-y(i-1,j,k))
c                dtdyl = halfhy*(y(i,j+1,k)-y(i,j-1,k))
c                dtdzl = halfhz*(y(i,j,k+1)-y(i,j,k-1))
c                d2tdxl=oneh2x*(y(i+1,j,k)-2d0*y(i,j,k)+y(i-1,j,k))
c                d2tdyl=oneh2y*(y(i,j+1,k)-2d0*y(i,j,k)+y(i,j-1,k))
c                d2tdzl=oneh2z*(y(i,j,k+1)-2d0*y(i,j,k)+y(i,j,k-1))
c                write(6,100) i,j,k,dtdx*u(ip,jp,kp),dtdxl*u(ip,jp,kp),
c    v                            dtdy*v(ip,jp,kp),dtdyl*v(ip,jp,kp),
c    v                            dtdz*w(ip,jp,kp),dtdzl*w(ip,jp,kp),
c    v              d2tdx,d2tdxl,d2tdz,d2tdzl


                 tprime = d2tdx + d2tdy + d2tdz
     v               -u(ip,jp,kp)*dtdx-v(ip,jp,kp)*dtdy-w(ip,jp,kp)*dtdz  
                 yprime(i,j,k) = tprime
              endif
            enddo
          enddo
        enddo

c     **** ORDER 8
      else if (norder.eq.8) then
        do k=1,ktx
          kp = k-norder/2+1
          do j=1,jtx
            jp = j-norder/2
            do i=1,itx
               ip = i-norder/2
               if (k.le.4.or.k.ge.ktx-3) then
                  yprime(i,j,k) = 0
               else if (j.le.4.or.j.ge.jtx-3) then
                  yprime(i,j,k) = 0
               else if (i.le.4.or.i.ge.itx-3) then
                  yprime(i,j,k) = 0
               else
                 d2tdx = w284*y(i+4,j,k) + w283*y(i+3,j,k)
     v                 + w282*y(i+2,j,k) + w281*y(i+1,j,k) 
     v                 + w280*y(i  ,j,k) + w281*y(i-1,j,k) 
     v                 + w282*y(i-2,j,k) + w283*y(i-3,j,k)
     v                 + w284*y(i-4,j,k)
                 d2tdx = d2tdx * oneh2x
                 d2tdy = w284*y(i,j+4,k) + w283*y(i,j+3,k)
     v                 + w282*y(i,j+2,k) + w281*y(i,j+1,k) 
     v                 + w280*y(i,j,  k) + w281*y(i,j-1,k) 
     v                 + w282*y(i,j-2,k) + w283*y(i,j-3,k)
     v                 + w284*y(i,j-4,k)
                 d2tdy = d2tdy * oneh2y
                 d2tdz = w284*y(i,j,k+4) + w283*y(i,j,k+3)
     v                 + w282*y(i,j,k+2) + w281*y(i,j,k+1) 
     v                 + w280*y(i,j,k)   + w281*y(i,j,k-1)
     v                 + w282*y(i,j,k-2) + w283*y(i,j,k-3) 
     v                 + w284*y(i,j,k-4)
                 d2tdz = d2tdz * oneh2z

                 dtdx = w184*y(i+4,j,k) + w183*y(i+3,j,k)
     v                + w182*y(i+2,j,k) + w181*y(i+1,j,k) 
     v                - w181*y(i-1,j,k) - w182*y(i-2,j,k)
     v                - w183*y(i-3,j,k) - w184*y(i-4,j,k)
                 dtdx = dtdx * onehx
                 dtdy = w184*y(i,j+4,k) + w183*y(i,j+3,k)
     v                + w182*y(i,j+2,k) + w181*y(i,j+1,k) 
     v                - w181*y(i,j-1,k) - w182*y(i,j-2,k)
     v                - w183*y(i,j-3,k) - w184*y(i,j-4,k)
                 dtdy = dtdy * onehy
                 dtdz = w184*y(i,j,k+4) + w183*y(i,j,k+3)
     v                + w182*y(i,j,k+2) + w181*y(i,j,k+1) 
     v                - w181*y(i,j,k-1) - w182*y(i,j,k-2)
     v                - w183*y(i,j,k-3) - w184*y(i,j,k-4)
                 dtdz = dtdz * onehz

                 tprime = d2tdx + d2tdy + d2tdz
     v               -u(ip,jp,kp)*dtdx-v(ip,jp,kp)*dtdy-w(ip,jp,kp)*dtdz  
                 yprime(i,j,k) = tprime
c                dtdxl = halfhx*(y(i+1,j,k)-y(i-1,j,k))
c                dtdyl = halfhy*(y(i,j+1,k)-y(i,j-1,k))
c                dtdzl = halfhz*(y(i,j,k+1)-y(i,j,k-1))
c                d2tdxl=oneh2x*(y(i+1,j,k)-2d0*y(i,j,k)+y(i-1,j,k))
c                d2tdyl=oneh2y*(y(i,j+1,k)-2d0*y(i,j,k)+y(i,j-1,k))
c                d2tdzl=oneh2z*(y(i,j,k+1)-2d0*y(i,j,k)+y(i,j,k-1))
c                write(6,100) i,j,k,dtdx*u(ip,jp,kp),dtdxl*u(ip,jp,kp),
c    v                            dtdy*v(ip,jp,kp),dtdyl*v(ip,jp,kp),
c    v                            dtdz*w(ip,jp,kp),dtdzl*w(ip,jp,kp),
c    v              d2tdx,d2tdxl,d2tdz,d2tdzl

              endif
            enddo
          enddo
        enddo


      endif
 
100   format(3i5,2x,3(2f12.7,2x),2x,4f12.7)
      return
      end

