c $Log:   /pvcs.config/mc3dvv/src/dosles.F_a  $
c
c   Rev 1.0   Mon Jul 14 14:21:12 1997   pvcs
c 
c *************************************************************
c *   DOSLES
c *
c *   Use SLES from the PETSc library of Argonne Natl lab to
c *   solve the steady state advection diffusion equation.
c *
c *   It's a little primitive yet: the velocity vectors are written
c *   out to ascii files, an external SLES program is called 
c *   and the created temperature solution file is retrieved.
c *
c *   Before this routine can be used the MPI and PETSc software
c *   have to be installed. See 
c *       http://www.mcs.anl.gov/petsc/petsc.html
c *   for more info.
c *
c *   PvK 951227
c *************************************************************
      subroutine dosles(y,yprime,itx,jtx,ktx,u,v,w,inx,jnx,knx,norder) 
      use module_precision
      use module_gia3
      use module_a3
      dimension y(itx,jtx,ktx),yprime(itx,jtx,ktx)
      dimension u(inx,jnx,knx),v(inx,jnx,knx),w(inx,jnx,knx)
      character*200 syscommand

      if (norder.ne.2) then
         write(6,*) 'PERROR(dosles): not yet adapted for norder<>2'
         stop
      endif
      if (jtx.ne.1) then
         write(6,*) 'PERROR(dosles): not yet adapted for 3-D'
         stop
      endif

c     **** First, write velocity to ASCII file
      open(198,file='u2d.asc')
      write(198,*) inx-1,knx-1
      do k=1,knx-1
         do i=1,inx-1
            write(198,*) u(i,1,k)
         enddo 
      enddo
      close(198)

      open(198,file='w2d.asc')
      write(198,*) inx-1,knx-1
      do k=1,knx-1
         do i=1,inx-1
            write(198,*) w(i,1,k)
         enddo 
      enddo
      close(198)
      open(198,file='sles.out')
      write(198,*) 1
      close(198)
      write(syscommand,'(''mpirun sles2d'',
     v          '' -nx '',i5,'' -nz '',i5,
     v        '' -xl '',f8.3,'' -pc_type lu -ksp_type preonly '',
     v        ''  -pc_lu_in_place '')') 
     v         inx-1,knx-1,xl
      write(6,*) syscommand
      call system(syscommand)
      
      open(198,file='sles.out')
      read(198,*) icorrect
      if (icorrect.ne.0) then
         write(6,*) 'PERROR(dosles): SLES indicates there is a problem'
         write(6,*) 'Aborting. Try using itimechoice>=0'
         stop
      endif
      close(198)

c     *** Read temperature from ASCII file
      ioff = norder/2
      koff = norder/2-1
      open(198,file='solution.asc')
      rewind(198)
      do k=1,knx-1
         do i=1,inx-1
            read(198,*) y(ioff+i,1,koff+k) 
         enddo
      enddo
      close(198)

      return
      end
