c $Log:   /pvcs.config/mc3dvv/src/flow.F_a  $
c
c   Rev 1.7   Fri Jul 30 15:51:26 1999   lowman
c  Added zwork to solvlu calls (this was required when the
c  array was made dynamic
c
c   Rev 1.6   Tue Mar 23 16:45:08 1999   lowman
cFixed pvcs log problem
c   
c   Rev 1.2   Tue Mar 23 14:00:00 1999   lowman
c Introduced flag imp_sym (implement symmetry). When activated
c this flag uses the symmetry of the abe matrix to save significant
c memory. A factor of approximately 4 in memory requirement is 
c now saved automatically for calculations with periodic boundary
c conditions. This is done by only saving the abe matrix once for 
c each L value.
c
c   Rev 1.1   Tue Oct 06 16:04:38 1998   lowman
c Removed imposed symmetry option (this option can now be applied
c in userftaccess).
c Added call to userout 'spectral'
c Added calls to chvarx and chvary to fix kinematic plate b.c.'s
c Changed hardwired subroutine arguments to variable names.
c E.g., 'ijob_pttouv' was introduced.
c
c   Rev 1.0   Mon Jul 14 14:21:16 1997   pvcs
c 
C *************************************************************
c *   FLOW
c *
c *   In case of variable viscosity, VFLOW is executed instead
c *   of this routine (PvK 960627)
c *************************************************************
c***begin prologue  flow
c***date written   890101   (yymmdd)
c***revision date  930802   (yymmdd)
c***category no.  ????
c***keywords
c***author   Carl W. Gable   (Los Alamos National Laboratory)
c***purpose Solve for velocity and stress given body forces.
c***description
c***references (none)
c***routines called bc, chvar, ft2nrm, initflow, inttauc, inttaup, 
c                   netflow, plflow, pttotau, pttouv, solvlu, surfv, 
c                   ttoftp, viscos,
c***modifications (yymmdd) modification
c***modifications (930803) [cwg] Change in arguments to solvlu.f
c***end prologue  flow
c
c     this routine computes the velocity field using various
c     methods selected by isolve
c
      subroutine flow
      use module_precision
      use module_param
      use module_giparam
      use module_a1
      use module_gia1
      use module_a2
      use module_gia3
      use module_a3
      use module_a22
      use module_afftw
c#include<a4.h>
      use module_a6
      use module_a7
      use module_a9
      use module_gia9
      use module_a11
      use module_a12
c     common /a12/ lpu, lpv, lpw, lpt13, lpt23, lpt33, lpft, nxu, nxv,
c    1 nxw, nxt13, nxt23, nxt33, nxft, nxr, mpu, mpv, mpw, mpt13, mpt23,
c    2 mpt33, mpft, nyu, nyv, nyw, nyt13, nyt23, nyt33, nyft, nyr
      use module_a13
      use module_a14
      use module_a20
      use module_a26
      use module_a30
      use module_viscom
c lu
      use module_flow
      use module_ftcoef
      use module_slvelu
      use module_mytime

      real l, l2
      common /start1/ restrt, initlt, istart
      common /a4/ vsij(700)
      common /a8/ l2, l, vps, vts
      common /a10/ jcycle
       dimension sscalv(2), iindexv(2)
       dimension scalv(4) 
       dimension indexv(4), sdktmp(4)

c JPL add
       common/cinttaus/iholdstill
       common/cplsave/isweep,iiidummy
#ifdef MPI
      include 'mpif.h'
       call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )
       call MPI_COMM_SIZE( MPI_COMM_WORLD, numprocs, ierr )
#else
       myid=0
       numprocs=1
#endif
#ifdef MEMORY_CHECK
c       call mmverify()
#endif

c      data itmax,    conv,   ne,   nb,   ml,   mu 
c     1      / 60,   1.e-6,    4,    2,    5,    5 /

c      btime=MPI_WTIME()

      if (ivistype.le.-1) then
         call vflow
         return
      endif

      nvisdim = ksx
      nvnudim = ksx
      nrdim   = kmx
      nzdim   = 514
      jcycle = icycle
      twopi = 2.*pimach(dumm)
      pi = pimach(dumm)
c      open(40,file = 'tape40')
c
      if ((isolve.gt.2).and.
     1 ((icycle.eq.1).or.(restrt.ne.0.0).or.(istart.eq.0))) then
c
c     initialize flow
c
        call initflow
c
      endif

      if ((ibct.eq.2).and.(isurfv.lt.0)) then
c        flow driven dynamic plates
c

         if ((icycle.eq.1).or.(restrt.ne.0.0)) then
            call plflow
            call inttaup
         endif
      endif

      if ((icycle.eq.1).or.(restrt.ne.0.0).or.(isurfv .eq. 500)) then
c*********************************************************
c*     Calculate surface velocity for kinematic b.c.     *
c*********************************************************
        iiidummy=1
        if(ibct.eq.3) call kinsurf(isurfv,iiidummy)
c*********************************************************
c*  End of calculation of kinematic surface velocity b.c.*
c*********************************************************
      endif

      isweep=1

      if (isolve.eq.3) then
100     iiidummy=-1
        if((ifsavepl.eq.0).and.(isweep.eq.2).and.(isurfv.lt.0))then 
        call kinsurf(isweep,iiidummy)
        endif
c
c    solve 4 1st order ode's for primitive variables
c
c*********************************************************
c*    Prepare calculation of buoyancy induced flow       * 
c*********************************************************

        if ((icycle.eq.1).or.(restrt.ne.0)) then
c MPI ALERT: changed loop indices
#ifdef MPI
      call mpiblock(nx,j3,numprocs,myid,
     &             ixstar,ixstop,iystar,iystop)
      if (ixstop .gt. nx) ixstop =  nx
      if (iystop .gt. j3) iystop =  j3
#else
      iystar=1
      iystop=j3
      ixstar=1
      ixstop=nx
#endif
          do 120 k = 1,nz
          do 120 j = iystar,iystop
          do 120 i = ixstar,ixstop
c         do 120 j = 1,j3
c         do 120 i = 1,nx
          vp  (i-ixstar+1,j-iystar+1,k) = zero
          v3  (i-ixstar+1,j-iystar+1,k) = zero
          taup(i-ixstar+1,j-iystar+1,k) = zero
          tau3(i-ixstar+1,j-iystar+1,k) = zero
          vtf (i-ixstar+1,j-iystar+1,k) = zero
  120     taut(i-ixstar+1,j-iystar+1,k) = zero
        endif
c
c     convert temperature at cell centers to cell corners
c
      if(initlt .eq. 16)then
         if((icycle.eq.1).or.(restrt.ne.0))then
            continue
         else
           call ttoft
         endif
      else
         call ttoft
      endif
c
c     fft of temperature
c
      call ft2nrm (1,lpft,nxft,mpft,nyft,nz,isx,jsx,ft)
c
      call userftaccess

      if (ivistype.lt.0) then
c        mean field variable viscosity
          call viscos(vis,vnu,ft)
      endif
c
c     change of variables (multiply by i = (-1)**.5)
      ijob = 1
      call chvar (ft,isx,nxr,lbd,jsx,nyr,mbd,nz,ijob)
#ifdef MPI
c      ftime=MPI_WTIME()
      flowtime = flowtime + ftime - btime
      impi = -1
      call mpicomm(impi,nx,j3,nz,ft,isx,jsx,ksx)
c      btime=MPI_WTIME()
#endif
c
      numeq = 0
      conv = 1.e-10
c     give it more iterations on restart
      if ((isolve.gt.2).and.
     1   ((icycle.eq.1).or.(restrt.ne.0.0).or.(istart.eq.0))) then
          itmax = 800
      else
          itmax = 50
      endif
      niterate = 10
      m = nz
      slowc    = one*rat
      scalv(1) = one*rat
      scalv(2) = one*rat
      scalv(3) = one*rat
      scalv(4) = one*rat
      scalft = 1.e-15
      ne  = 4
      nyk = m
      nyj = ne
      nb  = 2
      nci = ne
      ncj = ne-nb+1
      nck = m+1
      nsi = ne
      nsj = ne+ne+1
      
      ml = 5
      mu = 5
      lda = 4*kmx
      n = ne*m
      npt = ne
      
      if (ibcb.eq.2) then
         indexv(1) = 1
         indexv(2) = 2
         indexv(3) = 3
         indexv(4) = 4
      else
         indexv(1) = 3
         indexv(2) = 2
         indexv(3) = 1
         indexv(4) = 4
      endif

c

      if((istart.eq.0).and.
     1  ((ifssd.eq.1).or.(nabe.eq.(isx*jsx)+1).or.(ifabe.eq.-1)))then
c*********************************************************************
c*      calculate and store factored partial derivative matrix       *
c*      on first pass through flow                                   *
c*********************************************************************
       imp_sym=0
       if(((isx.eq.jsx).and.(xl.eq.yl)).and.
     1 ((lbd.eq.0).and.(mbd.eq.0)))then
#ifdef MPI
          write(6,*) 'PERROR(flow): ABE matrix reduction will not'
          write(6,*) ' work correctly with MPI. Sorry.'
          call MPI_FINALIZE(rc)
          stop
#endif
         write(*,*) '******************************************'
         write(*,*) '* REDUCING SIZE OF ABE BY USING SYMMETRY *'
         write(*,*) '******************************************'
         imp_sym=1 
       endif
        lmn = 0
c MPI ALERT: changed loop indices
#ifdef MPI
      call mpiblock(nx,j3,numprocs,myid,
     &             ixstar,ixstop,iystar,iystop)
      if (ixstop .gt. nxr) ixstop =  nxr
      if (iystop .gt. nyr) iystop =  nyr
#else
      iystar=1
      iystop=nyr
      ixstar=1
      ixstop=nxr
#endif
        do 135 j = iystar,iystop
         do 135 i = ixstar,ixstop
c       do 135 j = 1,nyr
c        do 135 i = 1,nxr
c
c       Factor of 4 reduction for 3D periodic runs
c
          if(((lbd.eq.0).and.(mbd.eq.0)).and.(j3.ne.1))then
           if(((i.eq.1).and.(mod(j,2).eq.0)).or.
     $      ((j.eq.1).and.(mod(i,2).eq.0)).or.
     $      ((mod(i,2).eq.0).and.(mod(j,2).eq.0)))then

            if(imp_sym.ne.1)then
              lmn = lmn+1
            elseif(i.ge.j)then
c
c       Additional factor of 2 reduction for cases where dx=dy
c       and xl=yl
c
              lmn = lmn+1
            endif
           endif
          else
c
c       Reflecting boundary conditions and 2D periodic
c
           lmn = lmn+1
          endif

           ijob=0
           labe = 1
           if(ifssd .eq. 1)then
              ijob = 4
           endif

           if(nabe .eq. (isx*jsx)+1)then

            ijob = 5
            if (((lbd.eq.0).and.(mbd.eq.0)).and.(j3.ne.1)) then
c
c     3D Periodic boundaries saving abe
c
c     calculate the indice corresponding to the abe matrix (labe)
c
             ifill=0
             ijobabe=1 
             call abeaddr(labe,i,j,nxr,nyr,ijobabe,ifill,imp_sym)

c     if labe has not changed from its previous value don't calculate
c     a new abe (i.e. go to 135)
c  
             if(.not.(((i.eq.1).and.(mod(j,2).eq.0)).or.
     $       ((j.eq.1).and.(mod(i,2).eq.0)).or.
     $       ((mod(i,2).eq.0).and.(mod(j,2).eq.0))))goto 135
            
             if(((imp_sym).eq.1).and.(i.lt.j)) goto 135

            else
c
c     reflecting boundaries and 2D
c
             labe=lmn + 1
            endif
           endif

           l = fl(i,j)
           l2 = l**2

           call solvlu(itmax,conv,slowc,scalv,indexv,ne,m,y,nyj,nyk
     1       ,abe(1,1,labe),lda,n,b,ipvt(1,labe),npt,ml,mu,lmn,ijob,
     2       l,l2,vps,vts,rr,nrdim,vis,nvisdim,vnu,nvnudim,
     3       z,nzdim,nz,ibcb,ibct,
     4       ivistype,r1,r2,r3,r4,ksl,i,j,icycle,niterate,ncabe,zwork)
  135   continue
      endif

c******** End of partial derivative matrix - store and factor - ********

c MPI ALERT: changed loop indicies
#ifdef MPI
        call mpiblock(nx,j3,numprocs,myid,
     &             ixstar,ixstop,iystar,iystop)
      if (ixstop .gt. nx) ixstop =  nx
      if (iystop .gt. j3) iystop =  j3
#else
      iystar=1
      iystop=j3
      ixstar=1
      ixstop=nx
#endif
      if (icycle.ne.1) then
         do 140 ll = 1,nz
           do 140 j = iystar,iystop
             do 140 i = ixstar,ixstop
c             find max value 
c             (in a way that converts from single to double precision)
              if(vp  (i-ixstar+1,j-iystar+1,ll)**2 .gt. scalv(1)**2)
     1               scalv(1)=sqrt(vp  (i-ixstar+1,j-iystar+1,ll)**2)
              if(v3  (i-ixstar+1,j-iystar+1,ll)**2 .gt. scalv(2)**2)
     1               scalv(2)=sqrt(v3  (i-ixstar+1,j-iystar+1,ll)**2)
              if(taup(i-ixstar+1,j-iystar+1,ll)**2 .gt. scalv(3)**2)
     1               scalv(3)=sqrt(taup(i-ixstar+1,j-iystar+1,ll)**2)
              if(tau3(i-ixstar+1,j-iystar+1,ll)**2 .gt. scalv(4)**2)
     1               scalv(4)=sqrt(tau3(i-ixstar+1,j-iystar+1,ll)**2)
              if(ft  (i,j,ll)**2 .gt. scalft  **2)
     1               scalft  =sqrt(ft  (i,j,ll)**2)
  140    continue
         slowc = zero
         do 141 nscale = 1,4
            if(scalv(nscale) .gt. slowc)slowc = scalv(nscale)
  141    continue
#ifdef MPI
      call MPI_ALLREDUCE(scalv,sdktmp,4,MPI_DOUBLE_PRECISION,MPI_MAX,
     &                MPI_COMM_WORLD,ierr)
      scalv(1) = sdktmp(1)
      scalv(2) = sdktmp(2)
      scalv(3) = sdktmp(3)
      scalv(4) = sdktmp(4)
#endif
      slowc = max(scalv(1),scalv(2),scalv(3),scalv(4))
      else
c        *** icycle = 1
         do 150 ll = 1,nz
           do 150 j = iystar,iystop
             do 150 i = ixstar,ixstop
              if(ft  (i,j,ll)**2 .gt. scalft  **2)
     1             scalft  =sqrt(ft  (i,j,ll)**2)
  150    continue
      endif
c MPI ALERT changed scalft to local scale and use mpi reduce
c     scalft = scalft*conv*.01
#ifdef MPI
      scalmy = scalft*conv*.01
      call MPI_ALLREDUCE(scalmy,scalft,1,MPI_DOUBLE_PRECISION,MPI_MAX,
     &                MPI_COMM_WORLD,ierr)
#else
      scalft = scalft*conv*.01
#endif
 
      lmn = 0

c  JPL add
      iskipcount=0
c
c MPI ALERT: changed loop indicies
#ifdef MPI
      call mpiblock(nx,j3,numprocs,myid,
     &             ixstar,ixstop,iystar,iystop)
      if (ixstop .gt. nxr) ixstop =  nxr
      if (iystop .gt. nyr) iystop =  nyr
#else
      ixstar=1   
      iystar=1
      ixstop=nxr   
      iystop=nyr  
#endif
      do 210 j = iystar,iystop
        do 200 i = ixstar,ixstop
c     do 210 j = 1,nyr
c       do 200 i = 1,nxr
          if(((lbd.eq.0).and.(mbd.eq.0)).and.(j3.ne.1))then
           if(((i.eq.1).and.(mod(j,2).eq.0)).or.
     $     ((j.eq.1).and.(mod(i,2).eq.0)).or.
     $     ((mod(i,2).eq.0).and.(mod(j,2).eq.0)))then
c
c      3D periodic
c
            if(imp_sym.ne.1)then
              lmn = lmn+1
            elseif(i.ge.j)then
c
c      3D periodic with dx=dy and xl=yl
c
              lmn = lmn+1
            endif
           endif
          else
c
c         Reflecting or 2D periodic
c
           lmn = lmn+1
          endif
          ijob = 0
          labe = 1
          if (ifssd .eq. 1)then
             ijob = 2
          endif
c
c      test if there is power in the temperature field
c      or the surface boundary conditions. Skip solution
c      of modes with no power.
c
          irtst = 0
          do 160 ll = 1,nz
             if ((lbd.eq.0).and.(mbd.eq.0)) then
               rr(ll) = ft(i,j,ll)
               if (abs(rr(ll)).gt.scalft) irtst = 1
c              **** if(abs(rr(ll)) .gt. 1.e-8)irtst = 1
             else if ((lbd.eq.1).and.(mbd.eq.1)) then
               rr(ll) = -ft(i,j,ll)
               if (abs(rr(ll)).gt.scalft) irtst = 1
c              ***** if(abs(rr(ll)) .gt. 1.e-8)irtst = 1
             else
               write(*,*) 'Not setup for mixed b.c.'
               write(*,*) 'stopped in flow'
               stop
c              error not set up for mixed b.c.
             endif
  160     continue
c
          l = fl(i,j)
          l2 = l**2
          vps = zero
          if ((ibct.eq.3).or.
     1    ((ifsavepl.eq.0).and.(isweep.eq.2).and.(isurfv.lt.0)))then
          vps = vpb(i,j)
          endif

          if (((irtst.eq.0).and.(abs(vps).lt..001*conv))
     1      .or.(l.eq.0.0)) then
            do 170 ll = 1, nz
              vp(i-ixstar+1,j-iystar+1,ll)   = zero
              v3(i-ixstar+1,j-iystar+1,ll)   = zero
              taup(i-ixstar+1,j-iystar+1,ll) = zero
              tau3(i-ixstar+1,j-iystar+1,ll) = zero
  170       continue
            iskipcount=iskipcount+1
          else
            if(nabe .eq. (isx*jsx)+1)then
              ijob = 3
              if(((lbd.eq.0).and.(mbd.eq.0)).and.(j3.ne.1))then

c
c   Retrieve the abe address label (labe) corresponding to nxr and nyr
c
               ijobabe=1
               ifill=1
               call abeaddr(labe,i,j,nxr,nyr,ijobabe,ifill,imp_sym)
              else
               labe=lmn+1
              endif
            endif

c           *** set initial guess to solution from last time step
            do 180 ll=1,nz
              y(1,ll) = vp  (i-ixstar+1,j-iystar+1,ll)
              y(2,ll) = v3  (i-ixstar+1,j-iystar+1,ll)
              y(3,ll) = taup(i-ixstar+1,j-iystar+1,ll)
180           y(4,ll) = tau3(i-ixstar+1,j-iystar+1,ll)
              npt=4
              numeq=numeq+1

              call solvlu(itmax,conv,slowc,scalv,indexv,ne,m,y,nyj,nyk,
     1           abe(1,1,labe),lda,n,b,ipvt(1,labe),npt,ml,mu,lmn,ijob,
     2           l,l2,vps,vts,rr,nrdim,vis,nvisdim,vnu,nvnudim,
     3           z,nzdim,nz,ibcb,ibct,ivistype,
     4           r1,r2,r3,r4,ksl,i,j,icycle,niterate,ncabe,zwork)
              do 190 ll=1,nz
c               write(6,197)i,j,ll,nz,y(1,ll),y(2,ll),y(3,ll),y(4,ll)
                vp  (i-ixstar+1,j-iystar+1,ll) = y(1,ll)
                v3  (i-ixstar+1,j-iystar+1,ll) = y(2,ll)
                taup(i-ixstar+1,j-iystar+1,ll) = y(3,ll)
190             tau3(i-ixstar+1,j-iystar+1,ll) = y(4,ll)

          endif
200     continue
210   continue

#ifdef MPI
      call MPI_ALLREDUCE(iskipcount,itotal,1,MPI_INTEGER,MPI_SUM,
     &                MPI_COMM_WORLD,ierr)
#else
      itotal = iskipcount
#endif
      if (myid .eq. 0) then
        write(*,*) itotal,'modes were skipped: poloidal solution'
      endif
      iskipcount=0
c
c     solve toroidal part on only first two iteration
c      and only when surface velocity is specified
c     OR when isurfv .lt. 0 and ifsavepl=0
c
        if (((ibct.eq.3).and.((icycle.lt.3).or.(restrt.ne.0.0)))
     1 .or.((isweep.eq.2).and.(ifsavepl.eq.0).and.(j3.gt.1))) then
          cconv = conv
          iitmax = itmax
          nniterate = niterate
          sslowc = 1.e6
          sscalv(1) = 1.*rat
          sscalv(2) = 1.*rat
          nn = nz
          nne = 2
          nnb = 1
          nnyk = nn
          nnyj = nne
          nnci = nne
          nncj = nne-nnb+1
          nnck = nn+1
          nnsi = nne
          nnsj = 2*nne+1
          
          mml = 2
          mmu = 2
c          lda = 2*mml+mmu+1
          lda = 4*kmx
          n = nne*nn
          npt = ne
          
          if (ibcb.eq.2) then
            iindexv(1) = 1
            iindexv(2) = 2
          else
            iindexv(1) = 2
            iindexv(2) = 1
          endif
           if ((icycle.gt.1).and.(ifsavepl.ne.0)) then  
            sscalv(1) = .01
            sscalv(2) = .01
c MPI ALERT: changed loop indices
#ifdef MPI
      call mpiblock(nx,j3,numprocs,myid,
     &             ixstar,ixstop,iystar,iystop)
      if (ixstop .gt. nxr) ixstop =  nxr
      if (iystop .gt. nyr) iystop =  nyr
#else
      iystar=1
      iystop=nyr
      ixstar=1
      ixstop=nxr
#endif
            do 220 ll = 1,nz
            do 220 j = iystar,iystop
            do 220 i = ixstar,ixstop
c           do 220 j = 1,nyr
c           do 220 i = 1,nxr
c
            if(vtf (i-ixstar+1,j-iystar+1,ll)**2 .gt. sscalv(1)**2)
     1             sscalv(1)=sqrt(vtf (i-ixstar+1,j-iystar+1,ll)**2)
            if(taut(i-ixstar+1,j-iystar+1,ll)**2 .gt. sscalv(2)**2)
     1             sscalv(2)=sqrt(taut(i-ixstar+1,j-iystar+1,ll)**2)
c            sscalv(1) = dmax1(abs(vtf (i,j,ll)),sscalv(1))
c            sscalv(2) = dmax1(abs(taut(i,j,ll)),sscalv(2))
  220       continue
c            sslowc = dmax1(sscalv(1),sscalv(2))
             if(sscalv(1) .gt. sscalv(2))then
               sslowc = sscalv(1)
              else
               sslowc = sscalv(2)
              endif
#ifdef MPI
            call MPI_ALLREDUCE(sscalv,sdktmp,2,MPI_DOUBLE_PRECISION,
     &                        MPI_MAX,MPI_COMM_WORLD,ierr)
            sscalv(1) = sdktmp(1)
            sscalv(2) = sdktmp(2)
#endif
            sslowc = max(sscalv(1),sscalv(2))
          endif
c
c MPI ALERT: changed loop indices
#ifdef MPI
      call mpiblock(nx,j3,numprocs,myid,
     &             ixstar,ixstop,iystar,iystop)
      if (ixstop .gt. nx) ixstop =  nx
      if (iystop .gt. j3) iystop =  j3
#else
      iystar=1
      iystop=j3
      ixstar=1
      ixstop=nx
#endif
         do 270 j = iystar,iystop
         do 260 i = ixstar,ixstop
c        do 270 j = 1,j3
c        do 260 i = 1,nx
          l = fl(i,j)
          l2 = l**2
          vts = 0.
          if ((ibct.eq.3)
     1    .or.((isweep.eq.2).and.(ifsavepl.eq.0))) vts = vtb(i,j) 

ccc          if ((ibct.ne.3).or.(abs(vts).lt..001*cconv)
ccc     1  .or.(l.eq.0.0)) then
ccc            do 230 ll = 1,nz
ccc            vtf (i,j,ll) = zero
ccc            taut(i,j,ll) = zero
ccc  230       continue
ccc          else
c
c     set initial guess to solution from last time step
c
            do 240 ll = 1,nz
            yy(1,ll) = vtf (i-ixstar+1,j-iystar+1,ll)
  240       yy(2,ll) = taut(i-ixstar+1,j-iystar+1,ll)
            npt = 2
            ijob = 0
          call solvlu(iitmax,cconv,sslowc,sscalv,iindexv,nne,nn,yy,nnyj,
     1    nnyk,abe(1,1,1),lda,n,b,ipvt(1,1),npt,mml,mmu,0,ijob,
     2    l,l2,vps,vts,rr,nrdim,vis,nvisdim,vnu,nvnudim,
     3    z,nzdim,nz,ibcb,ibct,
     4    ivistype,r1,r2,r3,r4,ksl,i,j,icycle,nniterate,ncabe,zwork)
            do 250 ll = 1,nz
c        write(6,*)ip,i,j,ll,lmn,l,l2,vts,yy(1,ll),yy(2,ll)
            vtf (i-ixstar+1,j-iystar+1,ll) = yy(1,ll)
  250       taut(i-ixstar+1,j-iystar+1,ll) = yy(2,ll)
ccc          endif
  260     continue
  270     continue
        endif

c
      if ((ibct.eq.2).and.(isurfv.lt.0)) then
      if(isweep.eq.1) call inttauc
      endif
c
c     The following is for dynamic plates with memory savings
c     
      isweep=isweep+1
      if((isurfv.lt.0).and.(ifsavepl.eq.0).and.(isweep.eq.2))goto 100
c
c     Change from poloidal and toroidal fields to
c     u and v and superimpose plate velocity when
c     plates are present. Also convert vertical w field.
c

c      ftime=MPI_WTIME()
      flowtime = flowtime + ftime - btime

       ijob_pttouv=0
       ipttype_pttouv=0
       call pttouv(ipttype_pttouv,ijob_pttouv)

c      btime=MPI_WTIME()
c
c      call userout("spectral")
c

       call ft2nrm (-1,lpu,nxu,mpu,nyu,nz,inx,jnx,u)
       call ft2nrm (-1,lpv,nxv,mpv,nyv,nz,inx,jnx,v)
       call ft2nrm (-1,lpw,nxw,mpw,nyw,nz,inx,jnx,w)
c
c      ftime=MPI_WTIME()
      ffttime = ffttime + ftime - btime
c
c      btime=MPI_WTIME()
c
c    assign values to end points for periodic boundary
c
      if(iholdstill.eq.1)then
       netflo=25
      else
       netflo=0
      endif
      call netflow(nx,ny,inx,jnx,u,v,vis,wk1d,netflo)
      call bc (u,v,w,inx,nxr,nx,lbd,jnx,nyr,j3,mbd,nz,1)
c
        if (time+dt.ge.prnt.or.time+dt.ge.pltime) then
          iskip = 1
          if (iskip.eq.1) go to 350
          ipttype = 0
          ipttotau  = 0 
          call pttotau(ipttype,ipttotau)
  350     continue
        endif
  
      endif

  197          format(4i5,4e14.5)
#ifdef MEMORY_CHECK
c       call mmverify()
#endif
c      ftime=MPI_WTIME()
      flowtime = flowtime + ftime - btime
 
      return
      end


