c $Log:   /pvcs.config/mc3dvv/src/inputs.F_a  $
c
c   Rev 1.4   Fri Jul 30 15:38:48 1999   lowman
c    Added itemplate to namelist variables
c
c   Rev 1.3   Tue Feb 09 14:58:06 1999   lowman
cMade lith an input variable on namelist card and set 
cdefault value to 1
c
c   Rev 1.2   Fri Aug 22 13:25:56 1997   hall
cCommented out some chemical tracking logic that was choking on thermal plumes
c
c   Rev 1.1   Fri Jul 18 17:08:46 1997   hall
cReintroduced old logic for npla, npssd
c
c   Rev 1.0   Mon Jul 14 14:21:32 1997   pvcs
c 
c *************************************************************
c *   INPUTS
c *
c *   Version 50 
c *   970718 [PSH]  Reintroduced logic for npla, npssd (dynamic
c *			plates)
c *   970822 [PSH]  Commented out some chemical tracking logic
c *   			that was choking on thermal plumes
c *************************************************************
c *   Version 49
c *   950810 [PvK]  Eliminated restart possibility
c *                 Made output parameters such as dplot obsolete
c *                    Replace all output variables by dtout.
c *************************************************************
c *
c * this routine initializes arrays
c *
c *   first line of input contains 2 values in (2f10.0) format.
c *   the first value indicates initial run if = 0, and a
c *   restart run if .ne. 0.  If .ne. 0, tape1 and tape9 must
c *   be supplied.  second variable indicates, if .ne. 0, that
c *   namelist will be read in to change some variables on a
c *   restart run.
c *
c *    1.     0.        old tape9 restart without reading namelist
c *
c *   -2.     0.        old tape9 restart without namelist; run
c *                      a single cycle and output u v w for ptracer
c *                      also call dprnt dplot plt3dv
c *
c *   -3.     0.         not a restart; needs tape7 startup and namelist
c *                      run a single cycle and output u v w for ptracer
c *                      setting pol and the tor portion of the
c *                      field to zero.
c *
c *   -1.     0.        new tape9 restart without reading namelist
c *
c *   -1.     1.        new tape9 restart and read namelist
c *
c *    if one wishes to start with a fresh tape9(output for plotting),
c *    then restart with time .lt. 0. This will rewind TAPE9 and
c *    set time to zero. This only works for restarts.
c *
c *   second line is a title line.
c *
c *   the rest of the input is contained in a namelist structure
c *
c *  namelist setup variables
c *
c *  xl = width of box in x direction, dimensionless multiple of depth
c *  yl = width of box in y direction, dimensionless multiple of depth
c *
c *  nx = number of cells in x direction   must be .gt. 3
c *  ny = number of cells in y direction   must be .gt. 3  or  =1 for 2d
c *  nz = number of cells in z direction
c *
c *  rat = Rayleigh number for bottom heating
c *  rah = Rayleigh number for internal heating
c *
c *  initlt = controls method of setting initial temperature
c *      = 1 for linear plus 2-d perturb. using alfx,alfy,amp and random
c *      = 2 for reading 2-d (x-z) temp field from tape7 projected to
c *        3d; plus random
c *      =-2 same as 2 except first record is title (i.e. tape7 was a
c *        restart file)
c *      = 3 for reading 3-d field from tape7 plus random
c *      = 4 for T=0.5*(1-z**2) plus perturb. using amp & ampr
c *      = 5 for reading 3-d field from tape7 where tape7 was tape1
c *          from another run.
c *      =-5 same as 5 but reset on boundary T > 1 = .999 and T < 0 =
c *              .001
c *      = 6 restart with tape1 from run with different nx,ny,nz &
c *        resolution old dimensions go in namelist /setup2/ nzold,
c *        nxold, nyold (note:nxold,nyold,nzold are nx+2, ny+2, nz+2
c *        based on values from input file of previous runs.
c *      = 7 read 2D tape1 and set T(i,j,k)=.5*(T(i,k)+T(j,k)
c *      = 8 read tape1 from a single precision run into a double
c *        precision run
c *      = 9 read tape93 written in subroutine bm3drs
c *      = 10 read tape7, flip over temperature; upwelling becomes
c *        downwelling
c *      = 11 read tape7, transpose temperature field; T(i,j,k) =>
c *        T(j,i,k)
c *      = 12 zero temperature, except single node set to one with 
c *         xpos = alfx
c *         ypos = alfy
c *         zpos = ampr
c *         amp  = amp
c *      = 13 use cos(alfx*x + alfy*y) as input perturbation function
c *      = 14 use cos(alfx*x) + cos(alfy*y) as input perturbation
c *        function  default uses cos(alfx*x)*cos(alfy*y) 
c *      = 15 call user defined function funcper(ichoice,t,x,y,z)
c *           user supplies function that defines temperature
c *           perturbation.
c *      = 16 Read input from Uli C. for 3D benchmark problems.
c *      = 17 transpose along y axis   T(i,j[1-n],k) => T(i,j[n-1],k)
c *      = 25 uniform temperature. Set T(x,y,z)=0.5
c *      = 26 uniform temperature. Set T(x,y,z)=0.5, bottom = 1.0
c *      = 27 uniform temperature. Set T(x,y,z)=0.5, top = 0.0
c *      = 28 uniform temperature. Set T(x,y,z)=0.5, top = 0.0, bottom
c *                                                              = 1.0
c *  ibcb = bottom boundary condition.
c *     = 1 for free slip
c *     = 2 for rigid
c *
c *  ibct = top boundary condition.
c *     = 1 for free slip
c *     = 2 for rigid
c *     = 3 for specified surface velocity
c *
c *  isurfv = mode of determining surface velocity
c *      <=-2 for plates and flow determines velocity
c *          *** ibct must =2 ***
c *          =-2   split down the center
c *          =-3   transform
c *          =-4   oblique boundary
c *          =-5   strike slip boundary
c *          =-6   one plate
c *          =-7   off center break
c *
c *       = 1 for table lookup in VSIJ
c *      >= 2 for user supplied function in sub SURFV
c *          *** ibct must =3 ***
c *          =2   divergent center line
c *          =3   divergent with transform
c *          =4   oblique boundary
c *          =5   strike slip boundary
c *          =6   sin function in i & j direction
c *         >=200 &  <300 read input file (input.mode) in flow and
c *               set magnitude of individual P and T modes
c *
c *   vsij = table of surface velocity components for ISURFV = 1
c *       format is i1,i2,j1,j2,u,v,...., where x component
c *       u and y component v are assigned to cells in columnsc
c *       i1 thru i2 and rows j1 thru j2.
c *
c *   ivistype = type of viscosity variation
c *        <= -2 temperature dependent viscosity (not fully implimented
c *            at this time)
c *         = -1 temperature dependent viscosity (mean field only)
c *         = 0 default, constant viscosity
c *         = 1
c *         = 2  two layer viscosity structure
c *         = 3  three layer viscosity structure
c *         = 4  four layer viscosity structure
c *         = 5  linear viscosity change v=b*z+c
c *         = 6  quadratic change        v=az**2+b*z+c
c *
c *        = 7  exp. change             v=a*exp(b*z)
c *
c *   tend = last time value
c *
c *   isolve = solution approach
c *       = 1 for free-slip side boundaries. two poisson eqns.
c *       = 2 for psuedo-stream function for any combination of side b.c
c *       = 3 for primitive variable approach
c *       = 4 for type 3 with variable vertical zoning
c *            for the last two cases, nx, ny must be
c *             a power of 2
c *       = 5 velocity field specified in subroutine vofxyz
c *   
c *   dtout = time interval between output time steps 
c *
c *   dwate = weight for anti-diffusion tensor in heat
c *        transport. should be about 1 or slightly more. default is 1
c *
c *   lbd = side boundary condition indicator, x direction.
c *      = 0 for periodic solution
c *      = 1 for reflecting (free slip) sides
c *      = 2 for no slip sides
c *
c *   mbd = side boundary condition indicator, y direction.
c *      = 0 for periodic solution
c *      = 1 for reflecting (free slip) sides
c *      = 2 for no slip sides
c *
c *   alfx = wave number for initial perturbation in x direction
c *         alfx .gt. 1 when lbd=0
c *
c *   alfy = wave number for initial perturbation in y direction
c *         alfy .gt. 1 when mbd=0
c *
c *      also used to force symmetric solution in flow for 2D
c *      calculations only.
c *      alfy = 33. set in FLOW
c *      ifsym=0
c *      if(alfy .eq. 33.)ifsym=1
c *
c *   amp = amplitude of initial temperature perturbation
c *
c *   ampr = amplitude of random perturbation for initial temperature
c *     if a random perturbation is added to a restart run
c *     then set (time .lt. 0 ) and (ampr .gt. 0.0)
c *
c *   time = 0.0 is default value
c *      = value from tape9 on restarts
c *      < 0. to restart and rewind tape9
c *
c *   curant = fraction of courant time step to take. default = 1.
c *
c *   istedy = control for writing onto tape21 for benchmarking
c *       = -2 write tape80 file for restarts at higher resolution
c *       = -1 (default) no action
c *       = 0 for time dependent info. onto tape21
c *       = 1 for stedy state info. onto tape21
c *
c *   ibot = indicator for insulated bottom boundary
c *     = 0 isothermal bottom boundary
c *     = 1 for insulated bottom boundary
c *     < 0 for specified basal temperature in sub tbase
c *       three digit number -xyz   where
c *       x=wave number of sinusoidal variation in x
c *       y=wave number of sinusoidal variation in y
c *       z=amplitude of variation amp=(z/10.)
c *
c *   itop = indicator for insulated top boundary
c *     = 0 isothermal top boundary
c *     = 1 for insulated top boundary
c *     < 0 for specified top temperature in sub tbase
c *       three digit number -xyz   where
c *       x=wave number of sinusoidal variation in x
c *       y=wave number of sinusoidal variation in y
c *       z=amplitude of variation amp=(z/10.)
c *
c *   netflo = control of input of net horizontal flow field (see sub.
c *        netflow)
c *      = 0 no action
c *      = 1 linear velocity gradient
c *      = 2 hyperbolic velocity gradient
c *      = 3 hyperbolic velocity gradient layered viscosity
c *      = 4 same as 3 with scaled velocity 5 times bigger
c *      = 5 same as 3 with scaled velocity 50 times bigger
c *      = 20 add constant horizontal velocity, same value to all depth
c *      > 1000000 add constant x horizontal velocity, same value to
c *        all depth vx = netflow - 1000000
c *      > 2000000 add constant y horizontal velocity, same value to
c *        all depth vy = netflow - 2000000
c *
c *   kchem  = 0 no action (default)
c *      = 1-nz  assigns an impermiable boundary (zero vertical
c *        velocity)
c *        at an internal layer where kchem is the Z level
c *        of boundary.
c *
c *   mcycle = flow is computed every mcycle time steps
c *         default is 1.
c *
c *   ifssd = 1 use (solid state) disk file for plate fields  (npla=1)
c *         = 0 store fields in memory (npla=2*nplate)
c *
c *   NOTE(for below): ifabe=0 or -1 requires less memory if nx=ny and xl=yl. 
c *
c *   ifabe = 0 (default) store factored abe matrix for every wavenumber
c *             (saves a lot of CPU).
c *         = 1 third dimension of abe is 1, calculate abe matrix
c *             at each call to solvlu (saves a lot of memory).
c *         =-1 (dynamic plate option only)
c *             Reduces memory by storing factored abe matrix for 
c *             every wavenumber. Storage is in a block of memory that  
c *             becomes available aftering deallocating the 
c *             memory which holds the tau?pl matrices during 
c *             the 'plflow' phase of dynamic plate calculations.
c *             Thus the run changes on the fly from ifabe=1 to ifabe=0 option. 
c *
c *   iptrac = switch for particle tracer code
c *          = 0 no particle tracer
c *          > 0 particle tracer code active
c *         At this time there is only one particle tracking
c *         method so the maximum number of particles is set
c *         equal to iptrac.
c *
c *   lith = nodal depth at which force balance is to be calculated
c *          Integrals are done at a depth of nz-lith=lithd.
c *************************************************************
c *   These variables have become obsolete (PvK 9508)
c *
c *   dplot = time interval between writes to plot tape (tape9)
c *   dprnt = time interval between print-outs
c *   ncplan = number of planes to be plotted. ncplan must be .le. 10
c *   icplan = specification of planes to be plotted.
c *         format is icplan = i1,j1,i2,j2,..., where i1 refers to the
c *         direction (1 for y-z plane, 2 for x-z , 3 for x-y ),
c *         and j2 refers to the plane number in the i1 direction.
c *         temperature and/or velocity fields are written to
c *         tape9 for postprocessing.  e.g., icplan=3,2, means plot
c *         horizontal plane variables through midpoint
c *         of second layer of cells from bottom boundary.
c *   nxpcon = control for temperature plots
c *       = 0 no action
c *       = 1 write temperature to tape9 for 2-d plots
c *   ivelp = control of velocity plots
c *      = 0 no action
c *      = 1 write velocity field onto tape9 for plots
c *   dplt3
c *   idot3 = control for temperature plots (dtout is time interval)
c *      = 0  no action
c *      = 1 write temperature field to tape11
c *      = 2 write temperature field to raster output and tape11
c *      = 3 write temperature field to raster output
c *   ivelp3 = control for 3-d veloctiy plots (dtout is time interval)
c *       = 0  no action
c *       = 1, write 3-d velocity field to tape12
c *       = 2, write 3-d velocity field to tape12 and vout
c *************************************************************
c *************************************************************
c *              >>>>>>>>G.Ito<<<<<<<<<<<
c *
c *   bigd= depth in km of box
c *   cv = criterium for convergence in flow for variable visc
c *   delt=temperature (deg C) contrast between top and bottom
c *   delta= across-ridge distance to consider melt production
c *          that accretes to the crust
c *   icycmax= max. time steps to do
c *   imelt- =1 if melting, =0 if no melt
c *         delt= temp. contrast between base and top in degC
c *         f0= amount of melt retained in matrix
c *   ithalf=1 if lower half of box t=1
c *   itvvmax= max. iterations in flow, if reached then no convergence
c *   nspecies= no. of chemicals to track  c(i,j,k,nspecies)=depletion field
c *   relax= relaxation factor for variable visc.
c *   sprate= input surface velocity in km/my to be non-dimensionized
c *           in surfv
c *   vcut= maximum viscosity
c *   visfact- total varation in viscosity (vismax/vismin)
c *   vsij(20)=ipatch=1 if adding hot cylinder in lower half
c *   xf1,xf2- for vv, lower and upper cut freq. for filtering visc,
c *         vnuc,tau13,tau23,u,v
c *************************************************************
c *    There are some parameters that are set internally.
c *
c *    ishift=used in surfgeo to specify transform
c *           geometry.
c *
c *    imeth=method of inversion of plate interaction
c *          coefficient matrix.
c *         =1   numerical inversion using SGEFS
c *         =2   analytic inverse of 4x4 matrix
c *
c *   resist=collisional resistance which reduces integrated shear
c *          stress on the base of the plate. Set in INTTAUC
c *         =1 no resistance
c *         =0 resistance equals shear stress
c *
c *   ireal=converts bits to words for allocating memory
c *         =8 for double precision
c *         =4 for single precision
c *         =1 when using LIBMMSL
c *
c *   integ=converts bits to words for allocating memory
c *         =4
c *         =1 when using LIBMMSL
c *
c *   nplai=input for npla
c *
c *   npssdi=input for npssd
c *
c *************************************************************
c *   I/O file names
c *
c *   tape1------restart files written in MANTLE
c *              (unformatted)
c *   tape7------2d temperature field that can be used as initial 
c *              temperature for startup
c *              (unformatted)
c *   tape8------fourier transform of temperature field
c *              (unformatted)
c *   tape9------2d slices of temperature and velocity for plotting
c *              (unformatted)
c *   tape11-----entire temperature field
c *              (unformatted)
c *   tape12-----entire velocity field
c *              (unformatted)
c *   tape13-----entire concentration field
c *              (unformatted)
c *   tape16-----horizontal slice of temperature spectrum
c *              (formatted)
c *   tape17-----poloidal and toroidal power as a function of depth
c *              (formatted)
c *   tape21-----x pos., y pos, x velocity, z velocity, temperature
c *              (formatted)
c *   tape30-----timeseries of velocity values in the area of a plume patch
c *              Used to calculate shear in boundary layer.
c *   tape32-----stress fields tau13, tau23, tau33, tau11
c *   tape33-----value and position of maximum stress for each plate
c *   tape34-----change in time, k.e., Nu, flux, ...
c *              (formatted)
c *   tape35-----time, k.e., Nu, flux, ...
c *              (formatted)
c *   tape39-----time, k.e., components of temp. spectrum time series
c *              (formatted)
c *   tape36-----subroutine patch output
c *              (formatted)
c *   tape37-----geoid anomaly output
c *   tape38-----geoid anomaly output
c *              (formatted)
c *   ssdfile---(unit=50) for storing superposition plate velocity
c *              fields on output device
c *   ssdlu-----(unit=51) store factored momentum solver matrix
c *              on output device
c *   tape80-----euro-benchmark output
c *              (formatted)
c *              (i)      icycle, time, tznu, energk
c *              (ii)     vertical velocity and temperature at specific points
c *              (iii)    surface heat flux, surface topography, geoid anomaly
c *              (v)      mean temperature
c *   tape90-----timing information
c *              (formatted)
c *   tape91-----crash protection written every hour, same format as
c *               tape1 (unformatted)
c *   tape92-----crash protection written every hour, same format as
c *               tape1 (unformatted)
c *   crust.out--- if imelt=1, crustal thickness profiles every time step
c *                unit=210 (G.Ito)
c **************************************************************
c *   3/8/93 CWG
c *    add input variable ifabe to allow user control of
c *    saving factored abe matrix.
c *
c *************************************************************
      subroutine inputs
      use module_precision
      use module_param
      use module_giparam
      use module_a1
      use module_gia1
      use module_a2
      use module_a3
      use module_gia3
      use module_a22
      use module_fnames
c      common /xxxxx/ restrt
c      common /xxxxxx/ initlt
c      common /xxxxxxx/ istart
      use module_ftcoef
      use module_spwork
      use module_intfac
      use module_prtcl
      use module_viscom
      use module_timeint
      use module_routput
      use module_craster
      use module_petracer
      common /a4/ vsij(700)
      parameter(ITVVMDEF=300)
      common/csym/ifsym
      common/cinttaus/iholdstill
      character dtitle*72
      common /start1/ restrt, initlt, istart
c      equivalence (ixa2,nx), (xa2,xl)
      dimension ixa2(1), xa2(1)
      common /cviscos/ vis1, vis2, vis3, vis4, vis5,
     1     kdepth1, kdepth2, kdepth3, kdepth4, kdepth5

#ifdef MPI
      include 'mpif.h'
       call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )
       call MPI_COMM_SIZE( MPI_COMM_WORLD, numprocs, ierr )
#else
       myid=0
       numprocs=1
#endif
      namelist /setup/ rat,rah,ibcb,ibct,isurfv,vsij,tend,xl,yl,nx,ny,nz
     1 ,isolve,dplot,dprnt,icplan,ncplan,idot3,ivelp3,ivelp,nxpcon,dwate
     2 ,lbd,mbd,alfx,alfy,amp,initlt,ampr,curant,mcycle,istedy,ibot
     3 ,dplt3,icycle,time,ivistype,ifssd,iptrac,ifabe
     4 ,vis1,vis2,vis3,vis4,vis5
     5 ,kdepth1,kdepth2,kdepth3,kdepth4,kdepth5,resist,itop,netflo
     6 ,kchem, nplai, npssdi,itimechoice,norder,mxord,maxl,mxml
     7 ,mxmu,atol,rtol,icycmax,itvvmax, cv,cvm,sprate, bigd
     8 ,xf1,xf2,visfact,vis0,imelt, delta, delt, f0, ithalf, rad
     9 ,ram, vcut, relax,ivisout, iftout,dtout,tstartname
     1 ,trast,dtrast,irast,x0tr,x1tr,y0tr,y1tr,z0tr,z1tr
     2 ,itracoption,ntrac,ndist,ntrsteps,lith,itemplate,ifsym
     3 ,iholdstill,ifsavepl,iftau
c     *** set defaults
       ivistype=0
       kdepth1=-999
       kdepth2=-999
       kdepth3=-999
       kdepth4=-999
       kdepth5=-999
       istedy=-1
       ibot=0
       itop=0
       lbd=1
       mbd=1
       initlt=1
       mcycle=1
       icycle=0
       ifssd=0
       iptrac=0
       isolve=3
       nplate=0
       isurfv=0
       lith=1
       itemplate=1
       imp_sym=0
       netflo=0
       kchem=0
       nplai=1
       npssdi=1
       ifabe=0
       iftau=0
       imelt=0
       ithalf=0
       ivisout=0
       iftout=0
       ifsavepl=1
       vis1=-9999
       vis2=-9999
       vis3=-9999
       vis4=-9999
       vis5=-9999
       resist=1.0
       curant=0.95    
       dwate=1.0
       dtout=1.0
       xl=1.0
       yl=1.0
       rat=1.e4     
       rah=0.0
       time=0.0
       sprate=50
       bigd=400
       xf1=0.4
       xf2=0.7
       visfact=1.0
       delt=1350.
       f0=0.02
       rad=0.0
       ram=0.0
       delta=100.
       vcut=50.
       cv=1.e-5
       cvm=1e-5
       relax=0.0
       vis0=0.0
c     *** Initialize variables for DASPK 
       itimechoice=0
       norder=2
       mxord=5
       max1=5
       mxm=1
       mxmu=1
       atol=1e-3
       rtol=0d0
c     *** initialize variable for input/output options
       tstartname='abcdefghijklmnopqrstuvwxyz'
       dout=1.e20
       trast=1.e20
       dtrast=1.e20
       irast=0
c     *** Initialize variables for tracer option
       itracoption=0
       ntrac(1)=0
       ndist=0

c     *** Initialize variables for variable viscosity
      icycmax=1e3 
      itvvmax=ITVVMDEF

c     *** Initialize variable for central plate stationarity
c     *** and specified symmetry
      ifsym=0 
      iholdstill=0 

c     *** These variables are now obsolete: initialized to -999
      ivelp3=-999   
      idot3=-999
      nxpcon(1)=-999
      nxpcon(2)=-999
      ivelp=-999
      icplan(1)=-999
      ncplan=-999
      dplot=-999
      dprnt=-999
      dplt3=-999
 
      do 2 ij=1,300
        vsij(ij)=zero
    2 continue
c *************************************************************
c *   Read info from input file 'input.in'
c *************************************************************
      open (unit=3,file='input.in',status='old',form='formatted')
      read (3,*) restrt,change
      if ((restrt.ne.0.0) .and. (restrt.ne. -3.0)) then
         write(6,*) 'PERROR(inputs): restart not supported'
         stop
      endif
      read (3,290) ptitle
      read (3,setup)
c     if (myid .eq. 0) write(6,setup)

c     write(6,*) 'itracoption: ',itracoption
c     write(6,*) 'ndist: ',ndist
c     write(6,*) 'ntrac: ',(ntrac(i),i=1,ndist)
c     write(6,*) 'x0tr: ',(x0tr(i),i=1,ndist)
c     write(6,*) 'x1tr: ',(x1tr(i),i=1,ndist)
c     write(6,*) 'y0tr: ',(y0tr(1),i=1,ndist)
c     write(6,*) 'y1tr: ',(y1tr(1),i=1,ndist)
c     write(6,*) 'z0tr: ',(z0tr(1),i=1,ndist)
c     write(6,*) 'z1tr: ',(z1tr(1),i=1,ndist)

c *************************************************************
c *   Initialize dimensions of arrays (actual declaration is
c *   in ALLOCMEM)
c *
c *   nx,ny,nz   - number of cells
c *   imx,jmx,kmx- number of velocity/pressure nodal points
c *   inx,jnx,knx- number of temperature nodal points (OLD)
c *   itx,jtx,ktx- number of temperature nodal points (NEW)
c *
c *   The velocity arrays are defined as (inx,jnx,knx) because
c *   ghost nodal points may be used in the implementation of
c *   the boundary conditions (see bc.F)
c *************************************************************

      if (ny.ge.3) then
c       *** 3-D problem
        imx = nx + 1
        jmx = ny + 1
        kmx = nz + 1
      else 
c       *** 2-D problem
        imx = nx + 1
        jmx = 1
        kmx = nz + 1
      end if
      ijkmx = imx * jmx * kmx


      if (ny .ge. 3) then
         inx = imx + 1
         jnx = jmx + 1
         knx = kmx + 1
      else
         inx = imx + 1
         jnx = 1
         knx = kmx + 1
      end if
      ijknx = inx * jnx * knx

      if (itimechoice.eq.1.or.itimechoice.eq.-1) then
c        *** higher order spatial FD: DASPK, or steady state version
         if (ibot.ne.0.or.itop.ne.0.or.mbd.ne.1.or.lbd.ne.1) then
           if (myid .eq. 0) then
           write(6,*) 'PERROR(inputs): DASPK can only handle isothermal'
           write(6,*) '  top and bottom and reflective side boundaries' 
           write(6,*) '  now. Change ibot/itop=0 and mbd/lbd=1'
           endif
#ifdef MPI
           call MPI_FINALIZE(rc)
#endif
           stop 
         endif
         if (ny.ge.3) then
            itx = imx + norder
            jtx = jmx + norder
            ktx = kmx + norder - 2
         else
            itx = imx + norder
            jtx = 1
            ktx = kmx + norder - 2
         endif
         ijktx = itx * jtx * ktx
         niwork = 40 + ijktx
         nrwork = 50+(mxord+5)*ijktx+(maxl+4)*ijktx+(maxl+3)*maxl+1
         nrwork = nrwork + ijktx*(2*mxml+mxmu+1) +
     v               2*ijktx/(mxml+mxmu+1)+1

      else
         itx = inx
         jtx = jnx
         ktx = knx
         ijktx = itx * jtx * ktx
      endif    

c     if (myid .eq. 0) write(6,'(''inx: '',4i10)') inx,jnx,knx,ijknx
c     if (myid .eq. 0) write(6,'(''imx: '',4i10)') imx,jmx,kmx,ijkmx
c     if (myid .eq. 0) write(6,'(''itx: '',4i10)') itx,jtx,ktx,ijktx
 
c     *** Only need arrays (iqx,jqx,kqx) when surface velocity is 
c     *** specified (isurfv>0) or when there are dynamic plates (isurfv<0)
      if (isurfv .eq. 0) then                    
        iqx = 1
        jqx = 1
        kqx = 1
      else
        iqx = imx
        jqx = jmx
        kqx = kmx
      endif
      ijkqx = iqx * jqx * kqx
      if (ifsavepl.eq.0) then
        ixpl = 1
        jypl = 1
        kzpl = 1
      else
        ixpl = imx
        jypl = jmx
        kzpl = kmx
      endif

c
c     *** When Cartesian tau arrays are to be saved for output
c

      if(iftau.eq.0)then
        ixtau = imx
        jytau = jmx
        kztau = 1 
      else
        ixtau = imx
        jytau = jmx
        kztau = kmx
      endif
 
c     *** These index will be used to dimensions arrays used
c     *** for variable viscosity calculations.  (G.Ito 8/95)
      if (ivistype .ge. 0) then
        isl = 1
        jsl = 1
        ksl = 1
      else
        isl = imx
        jsl = jmx
        ksl = kmx
c >>>>>>>> PvK 12/21
c >>>>>>> !!!!
c       isl = inx
c       jsl = jnx
c       ksl = knx
c >>>>>>>>>
      end if
      ijksl = isl * jsl * ksl

      nxyz = max(nx+1,ny+1,nz+1)

      isx = imx
      jsx = jmx
      ksx = kmx
      ijksx = isx * jsx * ksx
 
c     *** For tracking of fields other than temperature (chemical)
c     *** set these arrays to the same dimension as the temperature
c     *** array.  In allocmemf NSPECIES arrays of dimension (icx,jcx,kcx)
c     *** are declared. NSPECIES is defined below.
      if (imelt.eq.1.or.ipatch.eq.1) then
         icx = inx
         jcx = jnx
         kcx = knx
      else
         icx = 1
         jcx = 1
         kcx = 1
      endif
      ijkcx = icx*jcx*kcx
 
c     *** left over from when there was a streamfunction, vorticity
c     *** momentum solver. No longer used.
      imxsf = 1 
      jmxsf = 1 
      kmxsf = 1 
      inxsf = 1
      jnxsf = 1
      knxsf = 1
      ijksf = 1

      mxmy = imx * jmx
      nbdd = 1
      ncabe = 16
      nspecies = 1
      maxout = 6

c     *** Control number of iterations in VFLOW (G.Ito, 8/8/95)
      if (itvvmax.eq.ITVVMDEF.and.ivistype.ge.0) then
c        *** We need only one iteration in VFLOW:
c        *** SOLVLU checks for convergence of velocity correction
c        *** This is only changed if itvvmax has the default
c        *** value...
         itvvmax = 1
      endif
         

c------------------------------------------------------------
c     >>>>>>> Check if chemical tracking         G.Ito 7/1/95
c------------------------------------------------------------
      nspecies = 1
      ipatch = int(vsij(20))
c      if (ipatch.eq.1) then
c         write(6,*) 'PWARN(inputs): ipatch=0 not supported yet' 
c         stop
c      endif
      if (imelt.eq.1.and.ipatch.eq.1) nspecies=2

c------------------------------------------------------------
c  Assign buoyancy terms                G. Ito 7/1/95
c  alpha*rho*g/kappa=1.12e+6
c------------------------------------------------------------
      if (vis0.ne.0) then
        rat=1.12e+6*delt*((bigd*1e+3)**3)/vis0
      endif
      if (rad.ge.1.0.and.rad.le.10.) then
         if (myid .eq. 0) then
           write(6,*) 'PWARN(inputs) Depletion buoyancy not'
           write(6,*) '              supported yet'
           write(6,*) 'rat = ',rat
         endif
#ifdef MPI
         call MPI_FINALIZE(rc)
#endif
         stop
c        rad=1.48*rat
      endif
      if (ram.ge.1.0.and.ram.le.10.) then
         if (myid .eq. 0) then
           write(6,*) 'PWARN(inputs) Melt buoyancy not'
           write(6,*) '              supported yet'
           write(6,*) 'ram = ',ram
         endif
#ifdef MPI
         call MPI_FINALIZE(rc)
#endif
         stop
c        ram=3.0*rat
      endif

#ifdef MPI
      if (ifabe .eq. 0) then
        if (myid .eq. 0) then
          write(6,*) 'MPI WARNING ifabe = 0 option not supported'
          write(6,*) 'under MPI.  Exiting.  Modify input.in !'
        endif
        call MPI_FINALIZE(rc)
        stop
      endif
#endif
      if (ifabe.eq.0) then
         nabe = isx * jsx + 1
      else
         nabe = 1
      endif

      ireal = 1
      integ = 1

c     npla = nplai
c     npssd = npssdi
c
c ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
c ~  7/18/97	PSH
c ~	Reintroduced old logic for npla, npssd (dyn. plates)
c ~	? Should be: else ... npssd = (1/2)*npla ?
c ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      if(isurfv .ge. 0)then
c    nondynamic
        npla = 1
        npssd = 1
      elseif(ny .ge. 3)then
c    dynamic 3D
        npla =  2*nplai
        npssd = 2*nplai
      else
c    dynamic 2D
        npla  = 2*nplai
        npssd = 2*npla
      endif

      nps = 0
      if(iptrac .gt. 0)then
        nparticle = iptrac
      else
        nparticle = 1
      endif

      if (itracoption.ne.0) then
         nmark=0
         do idist=1,ndist
            nmark=nmark+ntrac(idist)
         enddo
         if (ny.le.2) then
           nmarkdim = 2*nmark
         else
           nmarkdim = 3*nmark
         endif
         if (myid .eq. 0) then
           write(6,*) 'nmark, nmarkdim: ',nmark,nmarkdim
         endif
      endif

#ifdef MPI
        knxx_max = 0
        knyy_max = 0
        do ichunk = 0, numprocs-1
          if (ny .eq. 1) then
            ntmp = 1
          else
            ntmp = ny+1
          endif
          call mpiblock(nx+1,ntmp,numprocs,ichunk,
     &         ixstart,ixstop,iystart,iystop)
          knxx = ixstop - ixstart + 1
          knyy = iystop - iystart + 1
          knxx_max = max(knxx_max,knxx)
          knyy_max = max(knyy_max,knyy)
        enddo
        ixxmpi = knxx_max
        iyympi = knyy_max
        ijkred = knxx_max*knyy_max*ksx
#else
        ixxmpi = isx
        iyympi = jsx
        ijkred = ijksx
#endif

290   format(a72)

      return
      end
