c $Log:   /pvcs.config/mc3dvv/src/mbmelt.F_a  $
c
c   Rev 1.0   Mon Jul 14 14:21:42 1997   pvcs
c 
c *************************************************************
c *   MBMELT
c *************************************************************
c=======================================================================
c     program MBMELT.F
c     Computes melt fraction, f, crustal thickness,
c     and reduces mantle temperature due to latent heat
c     using McKenzie and Bickle's (1988) formulation.
c     
c     Computes depletion in upper 1/4 of box (ie i=1,nx/2 & k=nx/2-nx)
c
c     f0= amount of melt that remains in matrix and does not contribute
c         to crust
c     delta= perpendicular distance from ridge axis from which melt
c            is extracted
c
c     G. Ito 4/95
c=======================================================================
	subroutine mbmelt
      use module_precision
      use module_param
      use module_giparam
      use module_a1		!t,visc
      use module_gia1
      use module_a2		!u,v,w
      use module_gia3
      use module_a3		!inputs,icycle,bigd,dt,nz1
      use module_ac1
      use module_a12
c     common /a12/ lpu, lpv, lpw, lpt13, lpt23, lpt33, lpft, nxu, nxv,
c    1 nxw, nxt13, nxt23, nxt33, nxft, nxr, mpu, mpv, mpw, mpt13, mpt23,
c    2 mpt33, mpft, nyu, nyv, nyw, nyt13, nyt23, nyt33, nyft, nyr
c-----------------------------------------------------------------------
      if (icycle.eq.1.and.sprate.gt.zero) then
        open(210,file='crust.out')
      endif
      dmin=1.e+5
      dmax=-1.e+5
      dfmax=-1.
      delz=bigd/(nz)	!thickness of each layer
      delx=bigd*xl/nx
      dtime=dt*bigd**2/31
c      fmark=1000.d0
      fmark=zero

      cp=1.0		!kJ/(kg K)
c      xlatent=600.	!kJ/(kg)						
c      do 100 k=1,knx	!>>> debugging
c      do 100 j=1,jnx
c      do 100 i=1,inx
c        c(i,j,k,2)=zero
c 100  continue		!>>>>>>>>>>>>>>>
c---------------------------------------------------------------------
c   Initialize for melt calculation
c---------------------------------------------------------------------
      ftot=0.
      dfmean=0.
      n2=0
      n=0
      crmean=0.
      nxmax=delta/delx

c      write(*,*)'Call to melt, time,icycle=',time,icycle
c1011  format(a5,e10.4,i8)

c----------------------------------------------------------------
c  Calculate melt fraction 
c----------------------------------------------------------------
      do 600 j=1,j3
        imax=0
        dftot=0.			!calculated crust along x=1
	canom=zero
	ncanom=0
      do 500 k=nz/2,nz			
        dep=bigd*(1.-dble(k-1)/dble(nz))	!7/18/95
	call tstl(dep,tl,ts)			!get M&B solidus and liquidus
c      do 500 i=1,nx/2
      do 500 i=1,nx
        tt=t(i,j,k)*delt
	forig=c(i,j,k,1)
	if (forig.gt.one) then
	   print *, 'Problems in mbmelt i,j,k,c=',i,j,k,forig
	   stop
        endif
        df=depl(tt,ts,tl,dep,forig,t0,cc,xlatent)	!calculate melt fraction
	if (df.lt.1.0e-8) then		!chk if depletion increased
	  df=zero
        else

          fnew=forig+df
          fnew=min(fnew,1.0)
	  c(i,j,k,1)=fnew+fmark
	  cmax(1)=dmax1(dble(fnew),cmax(1))
c	  print *, '>>Marking i,j,k,df,c=',i,j,k,df,c(i,j,k,1)
c          if (fnew.gt.cmax(1)) then  !>>>>>>>debugging
c            tfmax=tt
c            depmax=dep
c            cmax(1)=fnew
c            ccmax=cc
c            t0max=t0
c            forigmax=forig
c	    dfmax=dmax1(df,dfmax)
c          endif		     !>>>>>>>>>>>>>>>
c-------------------------------------------------------------
c   Average, min, max, of depletion, melt production, depths
c------------------------------------------------------------
          ftot=ftot+fnew
          n=n+1
          if (df.gt.dfmax) then
            dfmax=max1(df,dfmax)
            imax=i
c            depmax=dep
c            tsmax=ts
c            tlmax=tl
c            dfmax=df
c	    tmax=tt
c            forigmax=forig
c	    t0max=t0
c	    ccmax=cc
          endif
c          if ((forig.gt.f0.and.i.le.nxmax) .and.  !only contributes to crust if melted by atleast f0
c	   if (i.le.nxmax)then		   !and only within delta of ridge	
 	    dftot=df+dftot			
	    n2=n2+1
	    ncanom=ncanom+1
            imax=max0(i,imax)
c          endif
	  if (nspecies.gt.1) canom=canom+c(i,j,k,nspecies)  		!count how much comes from plume
 
          dmax=max1(dep,dmax)
	  dmin=min1(dep,dmin)
c---------------------------------------------------
c   Reduce mantle temperatures due to Latent heat
c--------------------------------------------------
          ddt=xlatent*df/cp
          t(i,j,k)=t(i,j,k)-ddt/delt
	  tts=ts/delt
	  t(i,j,k)=dmax1(t(i,j,k),dble(tts))
c	  print *, 'Xlatent=',xlatent
c	  write(*,'(a20,2i5,5e12.4)'), 'i,k,df,ddt,dep,ts,t=',
c     &         i,k,df,ddt,dep,ts,tt
	  if (df.gt.zero.and.j.eq.1) then
c             write(*,*) i,k,df,dftot,forig,f0,i,nxmax
	  endif
        endif
 500  continue
	if (sprate.gt.zero) then
          xmax=imax*delx
          rx=dtime*sprate
          crust=dftot*(delz*delx*3.3/2.9)/(rx)
	  if (ncanom.gt.0) then
             canom=canom/dble(ncanom)
	  else
	     canom=zero
	  endif
          if (j.eq.1) then
            write(210,1001)icycle, time, crust,canom, xmax
          else
            write(210,1003) crust,canom,xmax
          endif
	endif
        dfmean=dfmean+dftot
        crmean=crust+crmean
 600  continue
1001  format(i8,e12.4,3f11.3)
1003  format(20x,3f11.3)
1004  format(f11.3)
c-----------------------------------------------------------------
c     Done now calculate statistics
c-----------------------------------------------------------------
      if (n.gt.0)then
        ftot=ftot/dble(n)
      else
        ftot=zero
      endif
      if (n2.gt.0) then
	dftot=dfmean
        dfmean=dfmean/dble(n2)
      else
        dfmean=zero
      endif
      crmean=crmean/dble(j4)	
c      write(212,*) f0, crmean	!>>> sensitivity test <<<<
c      if (icycle.eq.1) isens=0
c      isens=isens+1
c      if (isens.eq.5) then 		
c	 f0=f0+.002
c	 isens=0
c      endif			!>>>>>>>>>>>>>>>>>>>>
      
c      open(211,file='crust.stat')
c      write(211,*) '==============Crust=',crmean,'=================='
c      write(211,*) 'DF max,ave,tot, n2',dfmax,dfmean,dftot,n2
c      write(211,*) 'dtime, rx=',dtime,rx
c      write(211,*) 'F max, ave, n =',cmax(1),ftot,n
c      write(211,*) 'tfmax,depmax=',tfmax,depmax		!debugging
c      write(211,*) 'Dep min, max',dmin,dmax
c      write(211,*) 'X extent (km)=', xmax
c      write(211,*) 'Max: dep,ts,tl,t,=',depmax,tsmax,tlmax,tmax
c      write(211,*) 'MAX: df,forig,t0,cc=',dfmax,forigmax,t0max,ccmax
c      close(211)
1000  format(2f8.2,a3,2f8.2,a3,i10)
1002  format(2e10.2,2x,i10)
3001  return
      end
  


c=======================================================================
      real*8 function depl(tt,ts,tl,dep,forig,t0,cc,xlatent)
c     Calculates the degree of depletion using MkKenzie and Bickle (1988) given
c     the temperature, solidus, liquidus, and depth
c     Required input:  tt,ts,tl,forig
c  	       output: df
c=======================================================================
      use module_precision

      a0=.4256
      a1=2.988
      dels=0.4		!kJ/(kg K)
      cp=1.0
c---------------------------------------------------
c     Calculate dependent variables
c----------------------------------------------------
        t0=(tt-.5*(tl+ts))/(tl-ts)		!T0 prime
        if (t0.gt.0.5) then			!if T too hot the error		
	  print *, '>>>>>> Temperature TOO HIGH in Mbmelt <<<<<<<<'
      	  print *, 'T=', tt
 	  stop
 	endif
	
        if (t0.lt.-0.5) then			!if T.lt.solidus don't do
	  depl=0.d0
	  goto 500
        endif

        f1=.5+t0+(t0**2-.25)*(a0+a1*t0)		!melt fraction w/o Lat. heat.

	xlatent=tt*dels
	cc=xlatent/(cp*(tl-ts))
	c2=-(a0+3*a1*t0)/(a1*cc)
	c1=1+(1-a1/4+2*t0*a0+3*a1*t0**2)/(a1*cc**2)
	c0=forig-f1
        if (c0.ge.0) then			!if previous melt% greater don't do
	  depl=0.d0
	  goto 500
        endif

c---------------------------------------------------
c Solution to Cubic equation
c--------------------------------------------------
	q=c1/3.d0-(c2**2)/9.d0
	r=((c1*c2)-(3*c0))/6.d0-(c2**3)/27.d0
	test=q**3+r**2
	aa=r+sqrt(test)
	if (aa.lt.0.) then
	  s1=-(-aa)**(1./3.)
	else
	  s1=(aa)**(1./3.)
	endif
	bb=r-sqrt(test)
	if (bb.lt.0.)then
	  s2=-(-bb)**(1./3.)
	else
	  s2=(bb)**(1./3.)
	endif

	depl=(s1+s2)-c2/3.d0

        chk=depl**3.+(c2*depl**2.)+c1*depl+c0
        if (chk.gt.1e-5) then
          print *, '>>>>>> Problems in DEPL <<<<<<<<'
          print *, 'chk,depl,dep,t=',chk,depl,dep,tt
	  print *, 's1,c0=',s1,c0
          print *, 'aa,r,test,q=',aa,r,test,q
          print *, 'forig,f1,t0=',forig,f1,t0
	  stop
        endif
 500    return
	end



c=================================================================
c Subroutine TSTL
c Computes McKenzie and Bickle solidus and liquidus for a 
c specified depth 
c================================================================
      subroutine tstl(z,tl,ts)
      use module_precision
      parameter (nf=5)
      real f(nf+1)
c-----------------------------------------------------------------------
c    Coefficients of best fit 5th order polygon to McKenzie and Bickle's
c    solidus curve
c
      data (f(i),i=1,nf+1)/-5.5502e-10, 5.9870e-07, -2.1582e-04,
     &   0.0214, 3.8245, 1.1047e+03/
c-----------------------------------------------------------------------

      gpa=z/30.
      ts=f(nf+1)
      do k=1,nf
	ts=f(k)*z**(nf-k+1)+ts
      enddo
      tl=1736.2+(4.343*gpa)+(180.*atan(gpa/2.2169))

      if (ts.eq.0) then
         print *, 'Something wrong, Ts=0'
	 print *, 'gpa=',gpa
	 stop
      endif

      return
      end



     
