c $Log:   /pvcs.config/mc3dvv/src/movetracers.F_a  $
c
c   Rev 1.0   Mon Jul 14 14:21:48 1997   pvcs
c 
c *************************************************************
c *   MOVETRACERS
c *
c *   Trace particles through velocity field generated by MC3D
c *   Particles are integrated by explicit Euler using 
c *   itcyc sub-steps.
c *
c *   PvK 9603 (with a close look at prtcl.F)
c *************************************************************
      subroutine movetracers
      use module_precision
      use module_petracer
      use module_param
      use module_giparam
      use module_a3
      use module_gia3


       if (ny.gt.2) then
          call trace3d
       else
          call trace2d
       endif

       return
       end

c *************************************************************
c *    TRACE3D
c *************************************************************
       subroutine trace3d
      use module_precision
      use module_petracer
      use module_param
      use module_giparam
      use module_a1
      use module_gia1
      use module_a2
      use module_a3
      use module_gia3

       dz = 1d0/(nz-1)
       dtp = dt/ntrsteps

c      write(6,*) 'trace3d: ndist',ndist,dz,dtp
       do itrstep=1,ntrsteps

       iptr = 0
       do idist=1,ndist
c         write(6,*) 'movetracers, idist=',idist,ntrac(idist)
          do i=iptr+1,iptr+ntrac(idist)
             xtr = coormark(3*i-2)
             ytr = coormark(3*i-1)
             ztr = coormark(3*i)
c            write(6,'(/''New point: '',i5,3f8.3)') i,xtr,ytr,ztr
c            *** cel in which the tracer is
             ip = min(int(xtr/dx + 1),nx-1)
             jp = min(int(ytr/dy + 1),ny-1)
             kp = min(int(ztr/dz + 1),nz-1)
c            write(6,'(''cell no: '',3i5)') ip,jp,kp
c            *** local coordinates
             xp = (xtr-(ip-1)*dx)/dx
             yp = (ytr-(jp-1)*dy)/dy
             zp = (ztr-(kp-1)*dz)/dz
c            write(6,'(''local coord: '',3f8.3)') xp,yp,zp
c            *** Linear shape functions
             chi1 = (1-xp)*(1-yp)*(1-zp)
             chi2 = xp*(1-yp)*(1-zp)
             chi3 = xp*yp*(1-zp)
             chi4 = (1-xp)*yp*(1-zp)
             chi5 = (1-xp)*(1-yp)*zp
             chi6 = xp*(1-yp)*zp
             chi7 = xp*yp*zp
             chi8 = (1-xp)*yp*zp
c            *** Interpolation of velocity
             uquad=u(ip,jp,kp)*chi1+u(ip+1,jp,kp)*chi2
     1      +u(ip+1,jp+1,kp)*chi3+u(ip,jp+1,kp)*chi4
     2      +u(ip,jp,kp+1)*chi5+u(ip+1,jp,kp+1)*chi6
     3      +u(ip+1,jp+1,kp+1)*chi7+u(ip,jp+1,kp+1)*chi8
             vquad=v(ip,jp,kp)*chi1+v(ip+1,jp,kp)*chi2
     1      +v(ip+1,jp+1,kp)*chi3+v(ip,jp+1,kp)*chi4
     2      +v(ip,jp,kp+1)*chi5+v(ip+1,jp,kp+1)*chi6
     3      +v(ip+1,jp+1,kp+1)*chi7+v(ip,jp+1,kp+1)*chi8
             wquad=w(ip,jp,kp)*chi1+w(ip+1,jp,kp)*chi2
     1      +w(ip+1,jp+1,kp)*chi3+w(ip,jp+1,kp)*chi4
     2      +w(ip,jp,kp+1)*chi5+w(ip+1,jp,kp+1)*chi6
     3      +w(ip+1,jp+1,kp+1)*chi7+w(ip,jp+1,kp+1)*chi8

             coormark(3*i-2) = xtr + dtp*uquad
             coormark(3*i-1) = ytr + dtp*vquad
             coormark(3*i) = ztr + dtp*wquad
c            write(6,'(i5,3f8.5)') 3*i-2,xtr,dtp*uquad,
c    v          coormark(3*i-2)
c            write(6,'(i5,3f8.5)') 3*i-1,ytr,dtp*vquad,
c    v          coormark(3*i-1)
c            write(6,'(i5,3f8.5)') 3*i,ztr,dtp*wquad,
c    v          coormark(3*i)
            
         enddo
         iptr = iptr + ntrac(idist)
      enddo

      enddo

      return
      end

      subroutine trace2d
      use module_precision
      use module_petracer
      use module_param
      use module_giparam
      use module_a1
      use module_gia1
      use module_a2
      use module_a3
      use module_gia3
       dz = 1d0/(nz-1)
       dtp = dt/ntrsteps

       do itrstep=1,ntrsteps

c      write(6,*) 'trace2d: ndist',ndist
       iptr = 0
       do idist=1,ndist
          write(6,*) 'movetracers, idist=',idist,ntrac(idist),iptr
          do i=iptr+1,iptr+ntrac(idist)
             xtr = coormark(2*i-1)
             ztr = coormark(2*i)
c            write(6,'(/''New point: '',i5,3f8.3)') i,xtr,ytr,ztr
c            *** cel in which the tracer is
             ip = min(int(xtr/dx + 1),nx-1)
             jp = 1
             kp = min(int(ztr/dz + 1),nz-1)
c            write(6,'(''cell no: '',3i5)') ip,jp,kp
c            *** local coordinates
             xp = (xtr-(ip-1)*dx)/dx
             zp = (ztr-(kp-1)*dz)/dz
c            write(6,'(''local coord: '',3f8.3)') xp,yp,zp
c            *** Linear shape functions
             chi1 = (1-xp)*(1-zp)
             chi2 = xp*(1-zp)
             chi3 = xp*zp
             chi4 = (1-xp)*zp
c            *** Interpolation of velocity
             uquad=u(ip,jp,kp)*chi1+u(ip+1,jp,kp)*chi2
     1      +u(ip+1,jp,kp+1)*chi3+u(ip,jp,kp+1)*chi4
             wquad=w(ip,jp,kp)*chi1+w(ip+1,jp,kp)*chi2
     1      +w(ip+1,jp,kp+1)*chi3+w(ip,jp,kp+1)*chi4

             coormark(2*i-1) = xtr + dtp*uquad
             coormark(2*i) = ztr + dtp*wquad
c            write(6,'(i5,3f8.5)') 2*i-1,xtr,dtp*uquad,
c    v          coormark(2*i-1)
c            write(6,'(i5,3f8.5)') 2*i,ztr,dtp*wquad,
c    v          coormark(2*i)
            
         enddo
         iptr = iptr + ntrac(idist)
      enddo

      enddo

      return 
      end
