c $Log:   /pvcs.config/mc3dvv/src/netflow.F_a  $
c
c   Rev 1.3   Fri Jul 30 15:42:30 1999   lowman
cFixed bug in option 25
c
c   Rev 1.1   Tue Feb 09 14:36:10 1999   lowman
cRemoved superfluous arguments from call list.
cAdded action for ijob=25 option
c
c   Rev 1.0   Mon Jul 14 14:21:50 1997   pvcs
c 
c *************************************************************
c *   NETFLOW
c *************************************************************
      subroutine netflow(nxu,nyu,ldim,mdim,u,v,vis,work,ijob)
c
c     add in a net horizontal component to the flow field
c     In general this should be called just before velocity
c     boundary conditions are established. This should not
c     be used with reflecting boundary conditions because
c     horizontal velocity must go to zero at the edges
c     and the velocity inposed here would be incompatible.
c
c ijob   = control of input of net horizontal flow field 
c          (see sub. netflow)
c        = 0 no action
c        = 1 linear velocity gradient
c        = 2 hyperbolic velocity gradient
c        = 3 hyperbolic velocity gradient layered viscosity
c        = 4 same as 3 with scaled velocity 5 times bigger
c        = 5 same as 3 with scaled velocity 50 times bigger
c        = 20 add constant horizontal velocity, same value to all depth
c        = 25 add constant horizontal velocity equal to average surface velocity
c        > 1000000 add constant x horizontal velocity, same value to
c          all depth vx = netflow - 1000000
c
c
      use module_precision
      use module_param
      use module_a3

      dimension u(ldim,mdim,*)
      dimension v(ldim,mdim,*)
c      dimension u(66,1,66)
      dimension vis(nz)
      dimension work(nz)
c      real*4 spw(33)
c      real*4 spw2(33)
c      real*4 spv(33)
      common /cviscos/ vis1, vis2, vis3, vis4, vis5,
     1     kdepth1, kdepth2, kdepth3, kdepth4, kdepth5
c
c------------------------------------------------------

c      call mmverify()

      if(ijob .eq. 0)return
      if(ijob.eq.25.and.icycle.eq.1)open(unit=84,file='netflow_ts')
c
      fnzm1 = float(nz-1)
      ijobsave = 0
      if(ijob .eq. 4)then
       ijob = 3
       ijobsave = 4
      endif
      if(ijob .eq. 5)then
       ijob = 3
       ijobsave = 5
      endif
c
c     find RMS surface velocity
c
      sum = 0.0
      do 10 j = 1,nyu
      do 10 i = 1,nxu
c      sum = u(i,j,nz) + sum
      sum = u(i,j,nz)**2 + sum
   10 continue
      if(sum .lt. 1.e-5)then
c
c     for the no slip top case, get the RMS velocity of the bottom
c
      sum = 0.0
      do 12 j = 1,nyu
      do 12 i = 1,nxu
      sum = u(i,j,1)**2 + sum
   12 continue
      endif

      sum = (sum/(float(nxu*nyu)))**(.5)
c      sum = (sum/(float(nxu*nyu)))
c------------------------------------------------------
c
c     linear velocity gradient    z
c
      if(ijob .eq. 1)then
      do 15 k = 1,nz
      do 18 j = 1,nyu
      do 18 i = 1,nxu
      u(i,j,k) = u(i,j,k) + (float(k-1)/float(nz-1))*sum
   18 continue
   15 continue
      endif
c------------------------------------------------------
c
c     hyperbolic velocity gradient   z**2
c     zero at surface -1 on the base
c
      if(ijob .eq. 2)then
      do 20 k = 1,nz
      fkm1 = float(k-1)
      do 25 j = 1,nyu
      do 25 i = 1,nxu
      u(i,j,k) = u(i,j,k) + (((fkm1/fnzm1)**2)-1.0)*0.2*sum
   25 continue
   20 continue
      write(6,29)sum
   29 format('RMS plate velocity in NETFLOW = ', e14.7)
      endif
c
c     net flow with layered viscosity
c
      if(ijob .eq. 3)then

      nvis = 1
      visf1 = vis(1)
      do 31 k = 2,nz
       if(vis(k) .ne. vis(k-1))then
          nvis = nvis + 1
          if(nvis .eq. 2)visf2 = vis(k)
          if(nvis .eq. 3)visf3 = vis(k)
       endif
   31 continue
c      write(6,*)visf1,visf2,visf3 
      if(nvis .gt. 3)then
         write(6,39)nvis
   39    format('Number of different viscosity layers is .gt. 3',/
     1          'STOP in NETFLOW       nvis = ',i5)
         stop
      endif

      lvis = 1
      work(1) = zero
c      spw(1)=work(1)
c      spv(1)=vis(1)
c      spw2(1)=sqrt(work(1))
      do 30 k = 2,nz
      fkm1 = float(k-1)
        if(lvis .eq. 1)then
          if(vis(k) .eq. vis(k-1))then
            work(k)  = ((fkm1/fnzm1)**2)/vis(k)
          else
            lswitch = 1
            lvis = lvis + 1
            zb1  = fkm1/fnzm1
          endif
        endif
        if(lvis .eq. 2)then
          if((vis(k) .eq. vis(k-1)) .or. (lswitch .eq. 1))then
            work(k)  = ((fkm1/fnzm1)**2)/vis(k)  +
     1              ((zb1**2)*(vis(k)-visf1))/(visf1*visf2)
            lswitch = 0
          else
            lswitch = 1
            lvis = lvis + 1
            zb2  = fkm1/fnzm1
          endif
        endif
        if(lvis .eq. 3)then
           if((vis(k) .eq. vis(k-1)) .or. (lswitch .eq. 1))then
             work(k)  = ((fkm1/fnzm1)**2)/vis(k)  +
     1              ((zb1**2)*(visf2 - visf1))/(visf1*visf2) +
     2              ((zb2**2)*(vis(k) - visf2))/(visf2*visf3)
            lswitch = 0
           else
             lvis = lvis + 1
             zb2  = fkm1/fnzm1
          endif
        endif
   30 continue
c
c     rescale so velocity at surface is 1
c
      do 32 k = 1, nz
         work(k) = work(k)/work(nz)
         work(k) = (work(k)-1.0)*0.2*sum
   32 continue
      do 35 k = 1,nz
      do 35 j = 1,nyu
      do 35 i = 1,nxu
      u(i,j,k) = u(i,j,k) + work(k)
   35 continue
      write(6,29)sum

      if(icycle .eq. 1)then
      open(unit = 85, file = 'tapenetflo2', form='formatted')
      call wheader(85,2)
      write (85,85)icycle
c   85 format (i8,e14.7,/,
   85 format (i8,14x,/,
     1"Viscosity
     2   z             z      depth(km)     vel_netflo    viscosity")

      do 75 k=1,nz
      fkm1 = float(k-1)
          write (85,105) fkm1/fnzm1,
     1    3000.*float(nz-k)/float(nz-1),(work(k)-1.0)*0.2*sum,vis(k)
     
  105 format (11x,5(e10.3,2x))
   75     continue
       close(85)
       endif
      endif
c------------------------------------------------------
       if(ijobsave .eq. 4)then
         do 45 k = 1,nz
         work(k) = (work(k)-1.0)*0.2*sum
         do 45 j = 1,nyu
         do 45 i = 1,nxu
           u(i,j,k) = u(i,j,k) + work(k)
   45   continue
        ijob = 4
       endif
c------------------------------------------------------
       if(ijobsave .eq. 5)then
         do 55 k = 1,nz
         work(k) = (work(k)-1.0)*10.0*sum
         do 55 j = 1,nyu
         do 55 i = 1,nxu
           u(i,j,k) = u(i,j,k) + work(k)
   55   continue
        ijob = 5
       endif
c------------------------------------------------------
c
c      constant offset
c
       if(ijob .eq. 20)then
         do 65 k = 1,nz
         work(k) = 100.0
         do 65 j = 1,nyu
         do 65 i = 1,nxu
           u(i,j,k) = u(i,j,k) + work(k)
   65   continue
       endif
c------------------------------------------------------
       if(ijob .eq. 25)then
       write(84,200) time,u(nxu/2,nyu/2,nz),v(nxu/2,nyu/2,nz)
         unetflow=u(nxu/2,nyu/2,nz)
         vnetflow=v(nxu/2,nyu/2,nz)
         do 84 k =1,nz
         do 84 j = 1,nyu
         do 84 i = 1,nxu
           u(i,j,k) = u(i,j,k) - unetflow 
   84   continue
         if(nyu.ge.3)then
         do 82 k =1,nz
         do 82 j = 1,nyu
         do 82 i = 1,nxu
           v(i,j,k) = v(i,j,k) - vnetflow
   82   continue
        endif

 200   format(1x,f9.6,f8.3,f8.3)
       endif
c------------------------------------------------------



c
c      constant offset x
c
       if((ijob .gt. 1000000) .and. (ijob .lt. 2000000))then
         vxvel = float(ijob) - 1000000.0
         do 70 k = 1,nz
         work(k) = vxvel
         do 70 j = 1,nyu
         do 70 i = 1,nxu
           u(i,j,k) = u(i,j,k) + work(k)
   70   continue
       endif
c------------------------------------------------------
c
c      constant offset y
c
c       if((ijob .gt. 2000000) .and. (ijob .lt. 3000000))then
c        vyvel = float(ijob) - 2000000.0
c         do 72 k = 1,nz
c         do 72 j = 1,nyu
c         do 72 i = 1,nxu
c           v(i,j,k) = v(i,j,k) + vyvel
c   72   continue
c       endif
      if(icycle .eq. 1)then
c      call mmverify()
      open(unit = 85, file = 'tapenetflo', form='formatted')
      call wheader(85,2)
      write (85,95)netflo
   95 format (i8,14x,/,
     1 " *             z       depth(km)  vel_netflo    u(1,1,k)")

      do 175 k=1,nz
      fkm1 = float(k-1)
          write (85,205) fkm1/fnzm1,
     1    3000.*float(nz-k)/float(nz-1),work(k),u(1,1,k)
     
  205 format (11x,5(e10.3,2x))
  175     continue
       close(85)
       endif
      return
      end
