c $Log:   /pvcs.config/mc3dvv/src/peras3.F_a  $
c
c   Rev 1.1   Tue Feb 09 14:34:16 1999   lowman
c
c   Rev 1.0   Mon Jul 14 14:21:58 1997   pvcs
c 
c *************************************************************
c *   PERAS3
c *
c *   Original name: raster3
c *   Version 7/28/93 (mc3d version 46) (Peter van Keken)
c *   Previous version needed a temporary character*1 array
c *   Scaling is now performed in a C-routine 
c *
c *   Version 49: some revision to allow for easier use with
c *       2-D and DASPK type solution vectors.
c *
c *   Call this subroutine with t(inx,jnx,knx) where 
c *   (inx,jnx,knx) are the declared dimensions, (nx,ny,nz)
c *   should be the actual dimensions (ny==1 for 2-D).
c *   For example:
c *   
c *   if (itimechoice.eq.0) then
c *      call peras3(icall,inx,jnx,knx,nx1,j4,nz1,t,ifscal,...)
c *   else
c *      call ttoft
c *      call peras3(icall,inx,jnx,knx,nx1,j4,nx1,ft,ifscal,..)
c *   endif
c *   
c *************************************************************
      subroutine peras3(icall,
     1                   inx,
     2                   jnx,
     3                   knx,
     4                   nx,
     5                   ny,
     6                   nz,
     7                   t,
     8                   ifscal,
     9                   iscal,
     1                   scal,
     1                   root)
c
c     Each call to RASTER will produce an output file.
c
c     icall= the number of times this routine has been called.
c            This value is used in putting a suffix on the output
c            file. The file names will have sequential suffixes so they can
c            be easily read by imagetool in animate mode. Suffix will
c            be between 101 and 999 if icall .gt. 0 and .lt. 899.
c            You are on your own if you want more than 900 files!
c
c     t=    three-dimensional array of real*8 numbers to be
c           sent to a raster file for input.
c
c     inx=  first dimension of array t
c     jnx=  second dimension of array t   
c     knx=  third dimension of array t   
c
c     nx= number of data in x direction
c     ny= number of data in x direction
c     nz= number of data in z direction
c
c     ifscal = 0 auto scale of input values
c            = 1 max min scale is set in 2D aray scal, values
c              outside of scale will clip
c
c     scal   = array of length 2 with REAL*8 min and max values
c
c     iscal   = array of length 2 with 1 BYTE INTEGER min and max values
c     iscal(1) .ge. 0
c     iscal(2) .le. 255
c     If user input is outside of 0-255 range, values will be
c     clipped to insure they fall within this range.
c
c
c     root=raster file's name root to which suffix will be added.
c          must be declared as character*72 on MAIN.
c
      implicit none
      integer icall, inx, jnx, knx, nx, ny, nz, ifscal
      real*8 t(inx,jnx,knx)
      real*8 scal(2),a,b,rmin,rmax, rmin_dat, rmax_dat, rmin_s, rmax_s
      integer iscal(2), myid, ierr
      integer i, j, k, imin, imax, namlen, ncall, nbyte, iwarn
      character*80 fname
      character*72 root
      character*5 ch5
      common/name0/fname

#ifdef MPI
      include 'mpif.h' 
#endif
c     common/csym/ifsym

c
c    Compute range of data
c

c MPI ALERT: one line added
#ifdef MPI
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )   
#else
      myid=0
#endif
         rmax_dat=-1.e20
         rmin_dat=1.e20     
         do 80 k=1,nz
          do 80 j=1,ny
           do 80 i=1,nx
              rmax_dat = max(rmax_dat,t(i,j,k))
              rmin_dat = min(rmin_dat,t(i,j,k))
   80    continue
c
      if (ifscal.eq.0) then
c
c     Set range to computed values
c
         rmin = rmin_dat
         rmax = rmax_dat
      else
c
c    User input will define data range. Anything
c    outside range will be clipped.
c
       if((rmin_dat .lt. scal(1)) .or. (rmax_dat .gt. scal(2)))then
        if(myid .eq. 0) then
        write(6,*)
     1 'WARNING peras3: Data range outside of user input range.'
        write(6,*)
     1 'WARNING peras3: scal(1) = ',scal(1), ' t_min = ',rmin_dat
        write(6,*)
     1 'WARNING peras3: scal(2) = ',scal(2), ' t_max = ',rmax_dat
        write(6,*)
     1 'WARNING peras3: Data outside range will be clipped.'
        endif
       endif
         
         rmin = scal(1)
         rmax = scal(2)
      endif
c
c     Reset byte integer values if input is invalid
c
      if(iscal(1) .ge. iscal(2))then
        imin = 0
        imax = 255
        if (myid .eq. 0) then
        write(6,*)
     1 'WARNING peras3: Invalid Input iscal(1) must be lt iscal(2)'
        write(6,*)
     1 'WARNING peras3: iscal(1) = ',iscal(1), 'iscal(2) = ', iscal(2)
        write(6,*)
     1 'WARNING peras3: imin set to 0, imax set to 255'
        endif
      endif
c
c     Reset byte integer values if input is outside 0-255 range
c
      iwarn = 0
      if(iscal(1) .lt. 0)then
        imin = 0
        iwarn = 1
      else
        imin = iscal(1)
      endif
      if(iscal(2) .gt. 255)then
        imax = 255
        iwarn = 2
      else
        imax = iscal(2)
      endif
      if (iwarn .ne. 0)then
      write(6,*)
     1 'WARNING peras3: Input iscal(1) iscal(2) range must be 0-255'
      write(6,*)
     1 'WARNING peras3: iscal(1)= ',iscal(1), ' iscal(2)= ', iscal(2)
      write(6,*)
     1 'WARNING peras3: Internal values reset to be within range.'
      endif
c
c     *** name output file
      fname='                                                       '
      do i=1,72
         if(root(i:i).eq.' ') goto 3
      enddo
3     namlen=i-1
      ncall=icall+10000
      write(ch5,5) ncall
5     format(i5)
      do i=1,namlen
         fname(i:i)=root(i:i)
      enddo
      do i=1,5
         fname(namlen+i:namlen+i)=ch5(i:i)
      enddo
c     *** write to output file
      nbyte = nx*ny*nz
      a=float(imax-imin)/(rmax-rmin)
      b=(rmax*float(imin)-rmin*float(imax))/(rmax-rmin)
      rmax_s = a*rmax + b
      rmin_s = a*rmin + b
c     write(6,*)'peras3:data range   rmin  =', rmin,  ' rmax  =',rmax
c     write(6,*)'peras3:data rescale rmin_s=', rmin_s,' rmax_s=',rmax_s
c     write(6,*)'peras3:byte range   imin  =', imin,  ' imax  =',imax
c     write(6,*)'peras3:', inx,jnx,knx,nx,ny,nz, nbyte
c     write(6,*)'peras3:output file ',fname

      call peras3out(t,inx,jnx,knx,nx,ny,nz,a,b,rmin,rmax,ifscal,fname)

      return
      end
