c $Log:   /pvcs.config/mc3dvv/src/solvde.F_a  $
c
c   Rev 1.0   Mon Jul 14 14:22:18 1997   pvcs
c 
c *************************************************************
c *   SOLVDE
c *************************************************************
*deck,relax
      subroutine solvde(itmax,conv,slowc,scalv,indexv,ne,nb,m,
     .    y,nyj,nyk,c,nci,ncj,nck,s,nsi,nsj,npt)
      use module_precision
c
c    driver routine for solution of two point boundary value problems by
c    relaxation.  itmax is the maximum number of iterations. conv
c    is the convergence criterion.  slowc controls the fraction of
c    corrections actually used after each iteration. scalv contains
c    typical sizes for each dependent variable, used to weight errors.
c    indexv lists the column ordering of variables used to construct
c    the matrix s of derivatives.  (the nb boundary conditions at the
c    first mesh point must contain some dependence on the first nb
c    variables listed in indexv.) the problem involves ne equations
c    for ne adjustable dependent variables at each point. at the first
c    mesh point there are nb boundary conditions.  there are a total of
c    m mesh points.  y is the two-dimensional array of size (nyj,nyk)
c    that contains the initial guess for all the dependent variables
c    at each mesh point.  on each iteration, it is updated by the
c    calculated correction. the arrays c(nci,ncj,nck), s(nsi,nsj)
c    supply dummy storage used by the relaxation code; the minimum
c    dimensions must satisfy: nci=ne, ncj=ne-nb+1,
c    nck=m+1, nsi=ne, nsj=2*ne+1.
c
      parameter (nmax=4)
c
c    nmax=largest expected value for ne
c
      dimension y(nyj,nyk),c(nci,ncj,nck),s(nsi,nsj),scalv(nyj),
     . indexv(nyj)
      dimension ermax(nmax),kmax(nmax)
c
c     set up row and column markers
c
      k1=1
      k2=m
      nvars=ne*m
      j1=1
      j2=nb
      j3=nb+1
      j4=ne
      j5=j4+j1
      j6=j4+j2
      j7=j4+j3
      j8=j4+j4
      j9=j8+j1
      ic1=1
      ic2=ne-nb
      ic3=ic2+1
      ic4=ne
      jc1=1
      jcf=ic3
      iextra=0
c
c     primary iteration loop
c
      do 16 it=1,itmax
c
c    boundary conditions at first point.
c
        k=k1
        if(npt .eq. 4)then
        call difeq4(k,k1,k2,j9,ic3,ic4,indexv,ne,s,nsi,nsj,y,nyj,nyk)
        elseif(npt .eq. 2)then
        call difeq2(k,k1,k2,j9,ic3,ic4,indexv,ne,s,nsi,nsj,y,nyj,nyk)
        endif
        call pinvs(ic3,ic4,j5,j9,jc1,k1,c,nci,ncj,nck,s,nsi,nsj)
c
c     finite difference equations at all point pairs.
c
        do 11 k=k1+1,k2
          kp=k-1
          if(npt .eq. 4)then
          call difeq4(k,k1,k2,j9,ic1,ic4,indexv,ne,s,nsi,nsj,y,nyj,nyk)
          call red4i(ic1,ic4,j1,j2,j3,j4,j9,ic3,jc1,jcf,kp,
     .        c,nci,ncj,nck,s,nsi,nsj)
c          call pinvs4i(ic1,ic4,j3,j9,jc1,k,c,nci,ncj,nck,s,nsi,nsj)
          call pinvs(ic1,ic4,j3,j9,jc1,k,c,nci,ncj,nck,s,nsi,nsj)
          elseif(npt .eq. 2)then
          call difeq2(k,k1,k2,j9,ic1,ic4,indexv,ne,s,nsi,nsj,y,nyj,nyk)
          call red(ic1,ic4,j1,j2,j3,j4,j9,ic3,jc1,jcf,kp,
     .        c,nci,ncj,nck,s,nsi,nsj)
          call pinvs(ic1,ic4,j3,j9,jc1,k,c,nci,ncj,nck,s,nsi,nsj)
          endif
   11    continue
       k=k2+1
c
c     final boundary conditions
c
        if(npt .eq. 4)then
        call difeq4(k,k1,k2,j9,ic1,ic2,indexv,ne,s,nsi,nsj,y,nyj,nyk)
        elseif(npt .eq. 2)then
        call difeq2(k,k1,k2,j9,ic1,ic2,indexv,ne,s,nsi,nsj,y,nyj,nyk)
        endif
        call red(ic1,ic2,j5,j6,j7,j8,j9,ic3,jc1,jcf,k2,
     .           c,nci,ncj,nck,s,nsi,nsj)
        call pinvs(ic1,ic2,j7,j9,jcf,k2+1,c,nci,ncj,nck,s,nsi,nsj)
c
c     back substitution
c
      if(npt .eq. 4)then
          call bksb4i(ne,nb,jcf,k1,k2,c,nci,ncj,nck)
      elseif(npt .eq. 2)then
          call bksub(ne,nb,jcf,k1,k2,c,nci,ncj,nck)
      endif
          err=0.
c
c     convergence check, accumulate average error
c
          do 13 j=1,ne
            jv=indexv(j)
c cg            ermax(j)=0.
            errj=0.
c cg            kmax(j)=0.
c cg            vmax=0.
c
c     find point with largest error, for each dependent variable
c
c cg            do 12 k=k1,k2
c cg              vz=abs(c(j,1,k))
c cg              if(vz.gt.vmax) then
c cg                vmax=vz
c cg                km=k
c cg              endif
c cg              errj=errj+vz
c cg   12       continue
          do 12 k=k1,k2
             errj=errj+abs(c(j,1,k))
   12     continue
c
c     note weighting for each dependent variable
c
          err=err+errj/scalv(jv)
c cg          ermax(j)=c(j,1,km)/scalv(jv)
c cg          kmax(j)=km
   13   continue
        err=err/nvars
c
c     reduce correction applied when error is large
c
        fac=slowc/max(slowc,err)
c
c     apply corrections
c
        do 15 jv=1,ne
          j=indexv(jv)
          do 14 k=k1,k2
            y(j,k)=y(j,k)-fac*c(jv,1,k)
   14     continue
   15   continue
c
c     summary of corrections for this step
c
c      if(npt .eq. 2)then
c       write(6,100) it,err,fac,(kmax(j),ermax(j),j=1,ne)
c      endif
      if(err.lt.conv) then
c      write(6,100) it,err,fac,(kmax(j),ermax(j),j=1,ne)
      iextra=iextra+1
c      if(iextra .gt. 4)then
c      write(6,88)it
c   88 format(' number of iterations= ',i5)
      return
c      endif
      endif
   16 continue
c
c    convergence failed
c
      if(npt .eq. 2)then
      print 98
   98 format('  itmax exceeded toroidal solution')
      elseif(npt .eq. 4)then
      print 99
   99 format('  itmax exceeded poloidal solution')
      endif
  100 format(1x,i4,1x,e12.5,1x,e12.5,(/5x,i5,2x,e12.6))
      return
      end
