c $Log:   /pvcs.config/mc3dvv/src/userout.F_a  $
c
c   Rev 1.2   Fri Jul 30 11:48:26 1999   lowman
c  Added options for geoid, topography, viscosity and mean temperature
c  field output
c
c   Rev 1.1   Tue Feb 09 14:49:54 1999   lowman
cIntroduced option for printing out poloidal and toroidal powers
cIntroduced control of output of AVS fields
c
c   Rev 1.0   Mon Jul 14 14:22:34 1997   pvcs
c 
c *************************************************************
c *   USEROUT
c *
c *   General output routine, which may be adapted at the user level
c *   Provides windows into the computational part of MC3D by
c *   reverse communication.
c *
c *   Parameters:
c *       KEYWORD i Character string, describes at which point
c *                 USEROUT is called.
c *                 Currently following calls are implemented
c *                 'init'   After initialization 
c *                 'flow'   After solution for velocity/stress
c *                          (subroutine flow)
c *               'spectral' During flow (following call to pttouv)
c *                          That is, after solution for Spectral
c *                          coefficients for u and v but prior to
c *                          conversion of u and v to Cartesian space 
c *                 'geoid'  Output geoid at the same time as T-field
c *                 'topo'   Output surface topography at the same time
c *                          as the T-field
c *                 'heat'   After one step of the time-integrator
c *                 'visco'  At the end of each viscosity calculation
c *                          (subroutine viscos)
c *                 'pttouv-1' Called from pttouv for calculation of
c *                          the power spectrum of P/T
c *                 'pttouv-2' Called from pttouv for calculation of
c *                          the power spectrum of P/T
c *                 'tau'    Follows "flow" call and produces output of tau_ij
c *                 'end'    Just before the program finishes
c *   ---->>>> Not all of these words need to be referenced in this subroutine
c *
c *************************************************************
c *   Replaces OUTPUT and related routines 
c *   (PvK 950811)
c *  
c *   Library version (PvK/SDK 093099)
c *   Rudimentary output of time-series, temperature and velocity files.
c *   This routine can be replaced with your favorite routine
c *   that creates AVS output, Explorer output, plays Pacman,
c *   whatever, but PLEASE keep personalized versions out of the 
c *   library.
c *************************************************************
      subroutine userout(keyword)
      use module_precision
      use module_param
      use module_giparam
      use module_timeint
      use module_a1
      use module_a11
      use module_a12
      use module_gia1
      use module_a2
      use module_a22
      use module_a3
      use module_gia3
      use module_ftcoef
      use module_craster
      use module_pttouv
      use module_userout

#ifdef MPI
      include 'mpif.h'
#endif
c     common /ptouserout/ pfac,tfac,prnt_last
      common /ptouserout/ pfac,tfac

      character*10 keyword
      character*80 rasroot,filename
      character*20 fieldname
      save tcrash,tinc
      data tcrash,tinc/3600.,3600./
cJPL insert
      io_ft=0 
      io_vx=0
      io_vy=0
      io_vz=0
      io_v3=0 
      io_vis=0
      io_power_vx=0
      io_power_vy=0
      io_power_vz=0 
      io_xz=0 
      io_yz=0
      io_geoid=0 
      io_topo=0       
      itstats=0  
c
#ifdef MPI
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )
#else
      myid=0
#endif
      if (myid .ne. 0) return

      if (keyword(1:4).eq.'init') then

c        *** Write temperature to file
         if (itimechoice.eq.0) then
            open (unit=7,file='tape7_000',form='unformatted')
            rewind (7)
            write (7) nx1,j4,nz1
            write (7) (((t(i,j,k),i=1,nx1),j=1,j4),k=1,nz1)
            close (7)

c           *** Output a cell-centered version as well.
            write(filename,'(''temp_000'')')
            open (unit=7,file=filename,form='unformatted')
            rewind (7)

            call ttoft

            write (7) nx,j3,nz
            write (7) (((ft(i,j,k)/rat,i=1,nx),j=1,j3),k=1,nz)
            close (7)

         else

            call ttoft
            open (unit=7,file='tape7_000',form='unformatted')
            rewind (7)
            write (7) nx1,j4,nz1
            write (7) (((ft(i,j,k)/rat,i=1,nx1),j=1,j4),k=1,nz1)
            close (7)
            rasroot='rast'
            ifscal=0

            call peras3(irast,isx,jsx,ksx,nx1,j4,nz1,ft,ifscal,
     v              iscal,scal,rasroot)
         endif
         irast = irast+1
         trast = trast + dtrast
  
      else if (keyword(1:4).eq.'heat') then
c        *** After each step of the time-integrator
         call output(7)

c        *** Time-series information
         call output(9)

      else if (keyword(1:3).eq.'end') then

c        *** Finishing up
         call output(10)

      endif

      return
      end
