c $Log:   /pvcs.config/mc3dvv/src/vsolvlu.F_a  $
c
c   Rev 1.1   Fri Jul 30 15:22:14 1999   lowman
c  Made zwork allocatable and added slvelu.h includes
c
c   Rev 1.0   Mon Jul 14 14:22:42 1997   pvcs
c 
c *************************************************************
c *   VSOLVLU
c *
c *   950810 Added NCABE to the parameter list
c *************************************************************
      subroutine vsolvlu (itmax,conv,slowc,scalv,indexv,ne,m,y,nyj,nyk
     1 ,abe,lda,n,b,ipvt,npt,ml,mu,lmn,ijob,
     2       l,l2,vps,vts,r,nrdim,vis,nvisdim,vnu,nvnudim,
     3    z,nzdim,nz,ibcb,ibct,ivistype,
     4    r1,r2,r3,r4,ksl,ivalue,jvalue,icycle,niterate,ncabe,zwork) 
c***begin prologue  solvlu
c***date written   900214   (yymmdd)
c***revision date  930802   (yymmdd)
c***category no.  ????
c***keywords
c***author   Carl W. Gable   (Los Alamos National Laboratory)
c***purpose
c***description
c
c     ijob=0 don't use ssd, don't store factored abe & ipvt 
c            calculate abe & b
c            factor
c            solve
c     ijob=1 abe & ipvt will be stored to ssd and solution calculated
c            calculate abe & b
c            factor
c            solve
c            write abe and ipvt to ssd
c     ijob=2 abe & ipvt are already stored in ssd
c            calclate b
c            read factored abe and ipvt
c            solve 
c     ijob=3 abe & ipvt are already set (used on second and greater iteration)
c            calclate b
c            solve 
c     ijob=4 abe & ipvt will be stored to ssd, no solution calculated
c            calculate abe 
c            factor
c            write abe and ipvt to ssd
c     ijob=5 abe & ipvt are calculated and array is kept in memory
c            calculate abe
c            factor
c            return
c
c     niterate = If > 1, the solver only checks for convergence every
c                niterate cycles. 
c
c     if_skip_check = During initial spin up, convergence should be
c                     checked every cycle. This parameter switches
c                     on the niterate option after if_skip_check
c                     cycles. ~100 is a reasonable choice for this
c                     parameter. Note: This means the code will run
c                     faster only after if_skip_check cycles.
c   
c***references (none)
c***routines called  vdifeqlu, dnbco, dnbfa, dnbsl, rwlussd, 
c                             snbco, snbfa, snbsl
c***modifications (yymmdd) modification
c***modifications (930802) [cwg] Only pass through solver for
c                          a single iteration, check for convergence
c                          only every niterate cycles.
c***end prologue  solvlu
      use module_precision
c      use module_slvelu
c
      real l, l2
      dimension y(nyj,nyk),  abe(lda,16), b(n), scalv(nyj),
     1 ipvt(n), indexv(nyj)
      dimension zwork(nz+1)
      dimension ermax(4)
      dimension kmax(4)
      dimension r(nrdim)
      dimension r1(ksl),r2(ksl),r3(ksl),r4(ksl)
      dimension vis(nvisdim), vnu(nvnudim)
      dimension z(nzdim)
      data iextra / 0 /
      data ifdebug / 0 /
 
      kz1=1                                   
      kz2=m

c     *** Control convergence check of velocity corrections (G.Ito 8/8/95).
c     *** See INPUTS: itvvmax. For variable viscosity (ivistype<0)
c     *** the accuracy is checked in vflow. For constant viscosity
c     *** it is performed here, similar to FLOW/SOLVLU.
      if (ivistype.ge.-1) then
         write(6,*) 'PERROR(vsolvlu): This routine should not be called'
         write(6,*) 'in case of time-independent viscosity'
         stop
      endif

      if_skip_check = 100
      irec=lmn
 
c     *** primary iteration loop
 
      do 60 it=1,itmax

c     *** Double precision
        if (precis.eq.2.) then
          if (ijob.eq.0) then
             call vdifeqlu(npt,indexv,abe,lda,n,b,y,nyj,nyk,0,
     1         l,l2,vps,vts,r,nrdim,vis,nvisdim,vnu,nvnudim,
     3         z,nzdim,nz,ibcb,ibct,ivistype,r1,r2,r3,r4,ksl,ncabe)
             call dnbfa(abe,lda,n,ml,mu,ipvt,info)
             call dnbsl(abe,lda,n,ml,mu,ipvt,b,0)
             ijob=3
          else if (ijob.eq.1) then
             call vdifeqlu(npt,indexv,abe,lda,n,b,y,nyj,nyk,0,
     1         l,l2,vps,vts,r,nrdim,vis,nvisdim,vnu,nvnudim,
     3         z,nzdim,nz,ibcb,ibct,ivistype,r1,r2,r3,r4,ksl,ncabe)
             call dnbco(abe,lda,n,ml,mu,ipvt,rcond,zwork)
             call dnbsl(abe,lda,n,ml,mu,ipvt,b,0)
             call rwlussd(abe,lda,ncabe,n,ipvt,1,irec)
             ijob=3
          else if (ijob.eq.2) then
             call vdifeqlu(npt,indexv,abe,lda,n,b,y,nyj,nyk,1,
     1         l,l2,vps,vts,r,nrdim,vis,nvisdim,vnu,nvnudim,
     3         z,nzdim,nz,ibcb,ibct,ivistype,r1,r2,r3,r4,ksl,ncabe)
             call rwlussd(abe,lda,ncabe,n,ipvt,2,irec)
             call dnbsl(abe,lda,n,ml,mu,ipvt,b,0)
             ijob=3
          else if (ijob.eq.3) then
             call vdifeqlu(npt,indexv,abe,lda,n,b,y,nyj,nyk,1,
     1           l,l2,vps,vts,r,nrdim,vis,nvisdim,vnu,nvnudim,
     3           z,nzdim,nz,ibcb,ibct,ivistype,r1,r2,r3,r4,ksl,ncabe)
             call dnbsl(abe,lda,n,ml,mu,ipvt,b,0)
          else if (ijob.eq.4) then
             call vdifeqlu(npt,indexv,abe,lda,n,b,y,nyj,nyk,4,
     1         l,l2,vps,vts,r,nrdim,vis,nvisdim,vnu,nvnudim,
     3         z,nzdim,nz,ibcb,ibct,ivistype,r1,r2,r3,r4,ksl,ncabe)
             call dnbco(abe,lda,n,ml,mu,ipvt,rcond,zwork)
             write(6,*)lmn,rcond
             call rwlussd(abe,lda,ncabe,n,ipvt,1,irec)
             return
          else if (ijob.eq.5) then
             call vdifeqlu(npt,indexv,abe,lda,n,b,y,nyj,nyk,4,
     1         l,l2,vps,vts,r,nrdim,vis,nvisdim,vnu,nvnudim,
     3         z,nzdim,nz,ibcb,ibct,ivistype,r1,r2,r3,r4,ksl,ncabe)
             call dnbco(abe,lda,n,ml,mu,ipvt,rcond,zwork)
             return
          endif
 
 
        else if (precis.eq.1.) then
c         *** Single precision
          if (ijob.eq.0) then
             call vdifeqlu(npt,indexv,abe,lda,n,b,y,nyj,nyk,0,
     1         l,l2,vps,vts,r,nrdim,vis,nvisdim,vnu,nvnudim,
     3         z,nzdim,nz,ibcb,ibct,ivistype,r1,r2,r3,r4,ksl,ncabe)
             call snbfa(abe,lda,n,ml,mu,ipvt,info)
             call snbsl(abe,lda,n,ml,mu,ipvt,b,0)
             ijob=3
          else if (ijob.eq.1) then
             call vdifeqlu(npt,indexv,abe,lda,n,b,y,nyj,nyk,0,
     1         l,l2,vps,vts,r,nrdim,vis,nvisdim,vnu,nvnudim,
     3         z,nzdim,nz,ibcb,ibct,ivistype,r1,r2,r3,r4,ksl,ncabe)
             call snbco(abe,lda,n,ml,mu,ipvt,rcond,zwork)
             call snbsl(abe,lda,n,ml,mu,ipvt,b,0)
             call rwlussd(abe,lda,ncabe,n,ipvt,1,irec)
             ijob=3
          else if (ijob.eq.2) then
             call vdifeqlu(npt,indexv,abe,lda,n,b,y,nyj,nyk,1,
     1         l,l2,vps,vts,r,nrdim,vis,nvisdim,vnu,nvnudim,
     3         z,nzdim,nz,ibcb,ibct,ivistype,r1,r2,r3,r4,ksl,ncabe)
             call rwlussd(abe,lda,ncabe,n,ipvt,2,irec)
             call snbsl(abe,lda,n,ml,mu,ipvt,b,0)
             ijob=3
          else if (ijob.eq.3) then
             call vdifeqlu(npt,indexv,abe,lda,n,b,y,nyj,nyk,1,
     1         l,l2,vps,vts,r,nrdim,vis,nvisdim,vnu,nvnudim,
     3         z,nzdim,nz,ibcb,ibct,ivistype,r1,r2,r3,r4,ksl,ncabe)
             call snbsl(abe,lda,n,ml,mu,ipvt,b,0)
          else if (ijob.eq.4) then
             call vdifeqlu(npt,indexv,abe,lda,n,b,y,nyj,nyk,4,
     1         l,l2,vps,vts,r,nrdim,vis,nvisdim,vnu,nvnudim,
     3         z,nzdim,nz,ibcb,ibct,ivistype,r1,r2,r3,r4,ksl,ncabe)
             call snbco(abe,lda,n,ml,mu,ipvt,rcond,zwork)
             write(6,*)lmn,rcond
             call rwlussd(abe,lda,ncabe,n,ipvt,1,irec)
             return
          else if (ijob.eq.5) then
             call vdifeqlu(npt,indexv,abe,lda,n,b,y,nyj,nyk,4,
     1         l,l2,vps,vts,r,nrdim,vis,nvisdim,vnu,nvnudim,
     3         z,nzdim,nz,ibcb,ibct,ivistype,r1,r2,r3,r4,ksl,ncabe)
             call snbco(abe,lda,n,ml,mu,ipvt,rcond,zwork)
             return
          endif
        endif
 
 
        if ((icycle .le. if_skip_check).or.
     1     (mod(icycle,niterate).eq.0)) then
c          *** Only check convergence every niterate cycles.
           errsave = err
           err=0.
 
c          *** Convergence check, accumulate average error
           do 30 j=1,ne
             icount=j
             jv=indexv(j)
             ermax(j)=0.
             errj=0.
             kmax(j)=0
             vmax=0.
 
c            *** find point with largest error, for each dependent variable
 
             do k=kz1,kz2
               vz=abs(b(icount))
               if (vz.gt.vmax) then
                  vmax=vz
                  km=icount
               endif
               icount=icount+ne
               errj=errj+vz
             enddo
 
c            *** note weighting for each dependent variable
             err=err+errj/scalv(jv)
             ermax(j)=b(km)/scalv(jv)
             kmax(j)=km
   30      continue
           err=err/((kz2-kz1)*ne)
 
c          *** reduce correction applied when error is large
c
           fac=slowc/max(slowc,err)
        endif

 
c       *** apply corrections
c
        fac = 1.
        if (fac.eq.1.) then
          do 51 jv=1,ne
             icount=jv
             j=indexv(jv)
             do 41 k=kz1,kz2
                y(j,k)=y(j,k)-b(icount)
                icount=icount+ne
   41        continue
   51     continue
        else
          do 50 jv=1,ne
             icount=jv
             j=indexv(jv)
             do 40 k=kz1,kz2
                y(j,k)=y(j,k)-fac*b(icount)
                icount=icount+ne
   40        continue
   50     continue
        endif
c       write(6,90)lmn, it,err,fac,(kmax(j),ermax(j),j=1,ne)

 
c       *** Only check convergence every niterate cycles.
        if (icycle.gt.if_skip_check) then
           if (mod(icycle,niterate).ne.0) then
              return
           endif
        endif
 
c       *** summary of corrections for this step
        if (ifdebug.ne.0) then
           write(6,90) lmn, it,err,fac,(kmax(j),ermax(j),j=1,ne)
        endif
        if (err.lt.conv) then
           if ((it.gt.2).and.
     1        (niterate.ne.1).and.
     2        (icycle .ge. if_skip_check)) then
              write(6,*)'*****************************************'
              write(6,*)'WARNING: SOLVLU Solution may be incorrect.'
              write(6,*)'More than 2 iterations required to converge.'
              write(6,*)'Convergence is only checked every'
              write(6,*)'niterate = ',niterate,' cycles.'
              write(6,*)'it = ',it,' i = ',ivalue,' j = ',jvalue
              write(6,*)'icycle = ',icycle,' convergence limit ',conv
              write(6,*)'error it-1 ',errsave,' error it ',err
           endif
           return
        endif

60    continue
c     ***** END OF LARGE LOOP
 
c     *****  Convergence failed
      if (npt.eq.2) then
         write (6,70) err,itmax,ivalue,jvalue
      else if (npt.eq.4) then
         write (6,80) err,itmax,ivalue,jvalue
      endif
      return
 
   70 format (e14.7,3i5,'  itmax exceeded toroidal solution')
   80 format (e14.7,3i5,'  itmax exceeded poloidal solution')
   90 format ('VSOLV: ',2i5,1x,2(e10.4,1x),6(i5,1x,e10.4))
  100 format (i5,1x,2(e10.4,1x),6(i5,1x,e10.4))
   88 format(' number of iterations= ',i5)
      end
