c $Log:   /pvcs.config/mc3dvv/src/calvrms.F_a  $
c
c   Rev 1.0   Mon Jul 14 14:20:54 1997   pvcs
c 
c *************************************************************
c *   CALVRMS
c *
c *   Calculate rms-velocity in 2-D or 3-D
c *   Grid should be equidistant in x,y and z.
c *
c *   PvK 9508
c *************************************************************
      function calvrms(ichois)
      use module_precision
      use module_a2
      use module_gia3
      use module_a3
      use module_param
      use module_giparam
      integer ichois

      calvrms = 0
      if (ny.lt.3) then
         calvrms = vrms2d(ichois)
      else
         calvrms = vrms3d(ichois)
      endif

      return
      end
c *************************************************************
c *   VRMS2D
c *************************************************************
      function vrms2d(ichois)
      use module_precision
      use module_a2
      use module_a3
      use module_gia3
      use module_param
      use module_giparam
      use module_cvrms
      integer ichois

      dx = xl/(nx-1)
      dz = 1d0/(nz-1)

      if (ichois.eq.1) then
c        *** Corner points (weight: 0.25*dx*dy)
         vrms2d = 0.25*(u(1,1,1)*u(1,1,1)+w(1,1,1)*w(1,1,1))
         vrms2d = vrms2d + 
     v      0.25*(u(1,1,nz)*u(1,1,nz)+w(1,1,nz)*w(1,1,nz))
         vrms2d = vrms2d + 
     v      0.25*(u(nx,1,nz)*u(nx,1,nz) + 
     v            w(nx,1,nz)*w(nx,1,nz))
         vrms2d =  vrms2d + 
     v     0.25*(u(nx,1,1)*u(nx,1,1)+w(nx,1,1)*w(nx,1,1))

c     *** Side points (weight: 0.5*dx*dy)
         do k=2,nz-1
            vrms2d = vrms2d + 
     v        0.5*(u(1,1,k)*u(1,1,k)+w(1,1,k)*w(1,1,k))
            vrms2d = vrms2d + 
     v        0.5*(u(nx,1,k)*u(nx,1,k)+w(nx,1,k)*w(nx,1,k))
         enddo
         do i=2,nx-1
            vrms2d = vrms2d + 
     v           0.5*(u(i,1,1)*u(i,1,1)+w(i,1,1)*w(i,1,1))
            vrms2d = vrms2d + 
     v           0.5*(u(i,1,nz)*u(i,1,nz)+w(i,1,nz)*w(i,1,nz))
         enddo
        
c        *** Internal nodal points
         do k=2,nz-1
            do i=2,nx-1
               vrms2d = vrms2d + 
     v               u(i,1,k)*u(i,1,k) + w(i,1,k)*w(i,1,k)
            enddo
         enddo
         vrms2d = vrms2d*dx*dz
         vrms2d = sqrt(vrms2d/xl)
     
      else
 
c        *** Integrate line-by-line in the z-direction first
c        *** and store the results in the 1-D array hvrms
         do k=1,nz
            call trap2vrms(u(1,1,k),w(1,1,k),nx,hvrms(k))
         enddo

c        *** Now, integrate hvrms in the k-direction
         call trapnorm(hvrms(1),nz,vrms2)
         vrms2d = vrms2*dx*dz
         vrms2d = sqrt(vrms2d/xl)

      endif

      return
      end

c *************************************************************
c *   VRMS3D
c *************************************************************
      function vrms3d(ichois)
      use module_precision
      use module_a2
      use module_a3
      use module_gia3
      use module_param
      use module_giparam
      use module_cvrms
      integer ichois

      dx = xl/(nx-1)
      dy = yl/(ny-1)
      dz = 1d0/(nz-1)

c     *** Do first the integration in the x-direction.
c     *** Store the results in the 2-D array h2vrms
      do k=1,nz
         do j=1,ny
            call trap3vrms(u(1,j,k),v(1,j,k),w(1,j,k),
     v            nx,h2vrms(j,k))
         enddo
      enddo

c     *** Now, integrate h2vrms in the j-direction
      do k=1,nz
         call trapnorm(h2vrms(1,k),ny,hvrms(k))
      enddo

c     *** Now, integrate hvrms in the k-direction
      call trapnorm(hvrms(1),nz,vrms2)

      vrms3d = vrms2*dx*dy*dz
      vrms3d = sqrt(vrms3d/(xl*yl))

      return
      end

c *************************************************************
c *   TRAP3VRMS
c *  
c *   Integrate / u1d*u1d + v1d*v1d + w1d*w1d dl using
c *   a 1D trapezoid rule.c *   a 1-D trapezoid rule.
c *   PvK 9508
c *************************************************************
      subroutine trap3vrms(u1d,v1d,w1d,nx,h2vrms)
      use module_precision
      dimension u1d(nx),v1d(nx),w1d(nx)

      vr = 
     v     0.5*(u1d(1)*u1d(1)+v1d(1)*v1d(1)+w1d(1)*w1d(1)) 
      vr = vr + 
     v     0.5*(u1d(nx)*u1d(nx)+v1d(nx)*v1d(nx)+w1d(nx)*w1d(nx))  
      do i=2,nx-1
         vr = vr + 
     v     (u1d(i)*u1d(i)+v1d(i)*v1d(i)+w1d(i)*w1d(i)) 
      enddo
 
      h2vrms = vr

      return
      end

c *************************************************************
c *   TRAPNORM
c *
c *   Integrate f using an 1-D trapezoid rule
c *   PvK 9508
c *************************************************************
      subroutine trapnorm(f,n,fint)
      use module_precision
      dimension f(n)

      fint = 0.5*f(1) + 0.5*f(n)
      do i=2,n-1
         fint = fint + f(i)
      enddo
 
      return
      end

c *************************************************************
c *   TRAP2VRMS
c *  
c *   Integrate / u1d*u1d + v1d*v1d + w1d*w1d dl using
c *   a 1D trapezoid rule.c *   
c *   PvK 9508
c *************************************************************
      subroutine trap2vrms(u1d,w1d,nx,hvrms)
      use module_precision
      dimension u1d(nx),w1d(nx)

      vr = 
     v     0.5*(u1d(1)*u1d(1)+w1d(1)*w1d(1)) 
      vr = vr + 
     v     0.5*(u1d(nx)*u1d(nx)+w1d(nx)*w1d(nx))  
      do i=2,nx-1
         vr = vr + 
     v     (u1d(i)*u1d(i)+w1d(i)*w1d(i)) 
      enddo
 
      hvrms = vr

      return
      end

