c $Log:   /pvcs.config/mc3dvv/src/ctensr.F_a  $
c
c   Rev 1.0   Mon Jul 14 14:21:00 1997   pvcs
c 
c *************************************************************
c *   CTENSR
c *
c *   calculate terms to cancel numerical tensor diffusion
c *   this routine integrates div.(uQ) dvol by using quadratic
c *   representation of velocity over cell face and quadratic
c *   representation of temperature over each quadrant of cell
c *   face.
c *   (Array Q is called DTENSR in FLUX)
c *
c *   MPI version  Scott King and Peter van Keken Summer 1999
c *   Los Alamos/Michigan/Purdue
c *
c *   For main modifications: see flux_mpi.F
c *
c *   2D:
c *      fluxv(1,k)  <-    q(k) u(k) u(k-1)
c *     difshn(1,k)  <-    u(k+1) u(k) u(k-1) q(k+1) q(k) q(k-1)
c *      fluxv(3,k)  <-    q(k+1) q(k) w(k)
c *     difshn(3,k)  <-    q(k+1) q(k) w(k)
c *      fluxd(1,k)  <-    fluxv(1,k) difshn(1,k)
c *      fluxd(3,k)  <-    fluxv(3,k) difshn(3,k)
c *      q(1,k)      <-    fluxd(1,k) fluxd(3,k)   fluxd(3,k-1)
c *                  <-  q(k+1) q(k) q(k-1) q(k-2) u(k+1) u(k) u(k-1) u(k-2)
c *   Loops for (fluxv,difshn,fluxd) should be from
c *       innerstart-1 to innerstop
c *   Final loop for q should be from innerstart to innerstop
c *   At input, Q has to be specified from innerstart-2 to innerstop+1,
c *   except for bottom layer (innerstart-1 to innerstop+1)
c *************************************************************
      subroutine ctensr(t,q,idx,idy)
      use module_precision
      use module_param
      use module_giparam
c      use module_a1
      use module_gia1
      use module_a2
      use module_gia3
      use module_a3
      use module_a77
      use module_intfac
      use module_ctensr
#ifdef MPI
      use module_mpibuf
      include 'mpif.h'
#endif
      dimension t(idx,idy,*), q(idx,idy,*)
      parameter (oneth=1.0D0/3.0D0,twoth=2.0D0/3.0D0,oneni=1.0D0/9.0D0)
      logical bottom_layer,top_layer
      integer ilow,ihigh
#ifdef MPI
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )
      call MPI_COMM_SIZE( MPI_COMM_WORLD, numprocs, ierr )
      call mpilayer(nz,numprocs,myid,izstart,izstop)

      bottom_layer=.false.
      top_layer=.false.
      izstart = izstart - 2
      izstop  = izstop + 2
      if (izstart .lt. 2) then
         bottom_layer=.true.
         izstart = 2
      endif
      if (izstop .gt. nz) then
         top_layer=.true.
         izstop = nz
      endif
      ioffset = 0
#else
      izstart = 2
      izstop = nz
      myid = 0
      ioffset = 0
      top_layer=.true.
      bottom_layer=.true.
#endif
c
c     ensure side boundary conditions are correct for q
c
c *    *** MPI caution. If you uncomment this make sure idz is known
c *    *** PvK 6/26/99.
c      call meshbc(q,idx,idy,idz,nx,ny,nz,j1,j2,j3,j4,lbd,mbd)
c
 
      if (ny.ne.2) then
        if (mbd.eq.0) then
c         do 20 k=2,nz
          do k=izstart,izstop
            do i=2,nx
              q(i,1,k)=q(i,j3,k)
              q(i,j4,k)=q(i,j2,k)
            enddo
          enddo
        else
c         do 40 k=2,nz
          do k=izstart,izstop
            do i=2,nx
              q(i,1,k)=q(i,j2,k)
              q(i,j4,k)=q(i,j3,k)
            enddo
          enddo
        endif
      endif
      if (lbd.eq.0) then
c       do 60 k=2,nz
        do k=izstart,izstop
          do j=1,j3
            q(1,j,k)=q(nx,j,k)
            q(nx1,j,k)=q(2,j,k)
          enddo
        enddo
      else
c       do 80 k=2,nz
        do k=izstart,izstop
          do j=1,j3
            q(1,j,k)=q(2,j,k)
            q(nx1,j,k)=q(nx,j,k)
          enddo
        enddo
      endif

      if ((lbd.eq.1).and.(mbd.eq.1)) then
        if (ny.ne.2) then
c         do 90 k=2,nz
          do k=izstart,izstop
            q(1,1,k)=q(2,j2,k)
            q(nx1,j4,k)=q(nx,j3,k)
            q(1,j4,k)=q(2,j3,k)
            q(nx1,1,k)=q(nx,j2,k)
          enddo
        else
c         do 100 k=2,nz
          do k=izstart,izstop
            q(1,1,k)=q(2,1,k)
            q(nx1,1,k)=q(nx,1,k)
            q(1,1,k)=q(2,1,k)
            q(nx1,1,k)=q(nx,1,k)
          enddo
        endif
      endif

      if ((lbd.eq.0).and.(mbd.eq.0)) then
        if (ny.ne.2) then
c         do 110 k=2,nz
          do k=izstart,izstop
            q(1,1,k)=q(nx,j3,k)
            q(nx1,j4,k)=q(2,j2,k)
            q(1,j4,k)=q(nx,j2,k)
            q(nx1,1,k)=q(2,j3,k)
          enddo
        else
c         do 120 k=2,nz
          do k=izstart,izstop
            q(1,1,k)=q(nx,1,k)
            q(nx1,1,k)=q(2,1,k)
            q(1,1,k)=q(nx,1,k)
            q(nx1,1,k)=q(2,1,k)
          enddo
        endif
      endif
      if ((lbd.eq.0).and.(mbd.eq.1)) then
         write (6,380)
         stop
      endif
380   format ("input error: mixed b.c. not tested in ctensr")

c *************************************************************
c     for 2-D calculations
c *************************************************************
      if (ny.eq.2) then
        q11=xrb(2,1,1)+3.*xrb(2,2,1)
        q12=xlb(2,1,1)+3.*xlb(2,2,1)+2.
c       do 160 k=2,nz
        do k=izstart,izstop
          kglob = k+ioffset
          t1=one/(ytb(kglob,3,1)*xrb(2,3,1))
          t2=one/(xlb(2,3,1)*ytb(kglob,3,1))
          t3=one/(xrb(2,3,1)*ybb(kglob,3,1))
          t4=one/(xlb(2,3,1)*ybb(kglob,3,1))
          q13=t1*q11+t2*q12
          q14=t3*q11+t4*q12
          f1=3.*ytb(kglob,2,1)*q13+ybb(kglob,1,1)*q14
          f2=ytb(kglob,1,1)*q13+3.*ybb(kglob,2,1)*q14

c DaN---------------------------------------------------------------DaN
          kw = k-izstart+1
c          if (izstart.eq.2) kw = kw + 1
          if (bottom_layer) kw = kw + 1
c DaN---------------------------------------------------------------DaN

          do i=2,nx
            fluxv(i,1,1,k)=(q(i+1,1,k)+q(i,1,k))*
     v              (u(i,1,kw)*f1+u(i,1,kw-1)*f2) 
          enddo
          df1=2.*ybb(kglob,1,1)*t3+6.*ytb(kglob,2,1)*t1
          df2=6.*ybb(kglob,2,1)*t3+2.*ytb(kglob,1,1)*t1
          df3=5.*(((oneth*xrb(2,1,1)+xrb(2,2,1))*t1)+((oneth*xlb(2,1,1)
     1        +xlb(2,2,1))*t2))+(30./9.)*(t1+t2)
          df4=((oneth*xrb(2,1,1)+xrb(2,2,1))*t1)+((oneth*xlb(2,1,1)+
     v        xlb(2,2,1))*t2)+(6./9.)*(t1+t2)
          df5=((oneth*xlb(2,1,1)+xlb(2,2,1))*t4)+((oneth*xrb(2,1,1)+
     v        xrb(2,2,1))*t3)+(6./9.)*(t3+t4)
          df6=5.*(((oneth*xlb(2,1,1)+xlb(2,2,1))*t4)+((oneth*xrb(2,1,1)
     v       +xrb(2,2,1))*t3))+(30./9.)*(t3+t4)
          do i=2,nx
        difshn(i,1,1,k)=(q(i+1,1,k)+q(i,1,k))*(u(i,1,kw)*df1+u(i,1,kw-1)
     1  *df2)+(q(i+1,1,k+1)+q(i,1,k+1))*(u(i,1,kw)*df3+u(i,1,kw-1)*df4)
     2  +(q(i+1,1,k-1)+q(i,1,k-1))*(u(i,1,kw)*df5+u(i,1,kw-1)*df6)
          enddo
        enddo

        do k=izstart,izstop
          do i=2,nx
            fluxv(i,1,2,k)  = zero
            difshn(i,1,2,k) = zero
          enddo
        enddo

        q31=ytb(2,1,3)+3.*ytb(2,2,3)
        q32=3.*ybb(2,2,3)+ybb(2,1,3)
        q33=oneth*ytb(2,1,3)+ytb(2,2,3)
        q34=oneth*ybb(2,1,3)+ybb(2,2,3)
        do 210 i=2,nx
        t1=one/(xrb(i,3,3)*ybb(2,3,3))
        t2=one/(ytb(2,3,3)*xrb(i,3,3))
        t3=one/(xlb(i,3,3)*ytb(2,3,3))
        t4=one/(xlb(i,3,3)*ybb(2,3,3))
        f1=(xrb(i,1,3)*t2+3.*xlb(i,2,3)*t3)*q31+(xrb(i,1,3)*t1+3.*xlb(i,
     1  2,3)*t4)*q32
        f2=(3.*xrb(i,2,3)*t2+xlb(i,2,3)*t3)*q31+(3.*xrb(i,2,3)*t1+xlb(i,
     1  1,3)*t4)*q32
        f3=q33*t3+q34*t4
        df1=(ybb(2,2,3)+oneth*ybb(2,1,3))*t1+(oneth*ytb(2,1,3)+ytb(2,2,3
     1  ))*t2+twoth*(t1+t2)
        df2=2.*xrb(i,1,3)*(t1+t2)+6.*xlb(i,2,3)*(t3+t4)
        df3=6.*xrb(i,2,3)*(t1+t2)+2.*xlb(i,1,3)*(t3+t4)
        df4=2.*oneth*(t3+t4)
c       do 190 k=2,nz
        do 190 k=izstart,izstop
c DaN---------------------------------------------------------------DaN
          kw = k-izstart+1
c          if (izstart.eq.2) kw = kw + 1
          if (bottom_layer) kw = kw + 1
c DaN---------------------------------------------------------------DaN
        fluxv(i,1,3,k)=(q(i,1,k+1)+q(i,1,k))*(((w(i-1,1,kw)*xrb(i,1,3)+
     1  3.*w(i,1,kw)*xrb(i,2,3))*ytb(2,1,3)+ytb(2,2,3)*3.*(w(i-1,1,kw)*
     2  xrb(i,1,3)+3.*w(i,1,kw)*xrb(i,2,3)))/(ytb(2,3,3)*xrb(i,3,3))+
     3  (ytb(2,1,3)*(3.*w(i-1,1,kw)*xlb(i,2,3)+w(i,1,kw)*xlb(i,1,3))+
     4  3.*ytb(2,2,3)*(3.*w(i-1,1,kw)*xlb(i,2,3)+w(i,1,kw)*xlb(i,1,3)))/
     5  (xlb(i,3,3)*ytb(2,3,3))+(3.*ybb(2,2,3)*(w(i-1,1,kw)*xrb(i,1,3)+
     6  3.*w(i,1,kw)*xrb(i,2,3))+ybb(2,1,3)*(w(i-1,1,kw)*xrb(i,1,3)+3.*
     7  w(i,1,kw)*xrb(i,2,3)))/(xrb(i,3,3)*ybb(2,3,3))+(3.*ybb(2,2,3)*
     8  (3.*w(i-1,1,kw)*xlb(i,2,3)+w(i,1,kw)*xlb(i,1,3))+ybb(2,1,3)*(3.*
     9  w(i-1,1,kw)*xlb(i,2,3)+w(i,1,kw)*xlb(i,1,3)))/(xlb(i,3,3)*
     $  ybb(2,3,3)))+(q(i-1,1,k+1)+q(i-1,1,k))*((ybb(2,1,3)*(w(i,1,kw)+
     $  5.*w(i-1,1,kw))/3.+(5.*w(i-1,1,kw)+w(i,1,kw))*ybb(2,2,3))/
     $  (xlb(i,3,3)*ybb(2,3,3))+(ytb(2,1,3)*(5.*w(i-1,1,kw)+w(i,1,kw))/
     $  3.+ytb(2,2,3)*(5.*w(i-1,1,kw)+w(i,1,kw)))/(xlb(i,3,3)*
     $  ytb(2,3,3)))
  190   continue
c       do 200 k=2,nz
        do 200 k=izstart,izstop
c DaN---------------------------------------------------------------DaN
          kw = k-izstart+1
c          if (izstart.eq.2) kw = kw + 1
          if (bottom_layer) kw = kw + 1
c DaN---------------------------------------------------------------DaN
        difshn(i,1,3,k)=(q(i+1,1,k+1)+q(i+1,1,k))*(w(i-1,1,kw)+5.*
     1  w(i,1,kw))*df1+(q(i,1,k+1)+q(i,1,k))*(w(i-1,1,kw)*df2+w(i,1,kw)*
     2  df3)+(q(i-1,1,k+1)+q(i-1,1,k))*(5.*w(i-1,1,kw)+w(i,1,kw))*df4
  200   continue
  210   continue

        do l=1,3,2
          t1=0.5*dxdyc(l)
c         do 220 k=2,nz
          do k=izstart,izstop
            do i=2,nx
              fluxd(i,1,l,k)=t1*(fluxv(i,1,l,k)+difshn(i,1,l,k))
            enddo
          enddo
        enddo

        if (lbd.eq.0) then
c         do 230 k=2,nz
          do k=izstart,izstop
             fluxd(1,1,1,k)=fluxd(nx,1,1,k)
          enddo
        else
c          do 235 k=2,nz
           do k=izstart,izstop
              fluxd(1,1,1,k)= zero
           enddo
        endif

        do k=izstart,izstop
           do i=2,nx
             q(i,1,k)=(fluxd(i,1,1,k)-fluxd(i-1,1,1,k)+fluxd(i,1,3,k)
     v                -fluxd(i,1,3,k-1))
           enddo
        enddo


      else

c ************************************************************
c     3-D problems
c ************************************************************

c       ****** x-direction *****
        l=1
        ttus = 0.5*dxdyc(l)
c       do 270 k=2,nz
        do 270 k=izstart,izstop
c DaN---------------------------------------------------------------DaN
          kw = k-izstart+1
c          if (izstart.eq.2) kw = kw + 1
          if (bottom_layer) kw = kw + 1
c DaN---------------------------------------------------------------DaN
          do 270 j=2,ny
            t1=one/(ytb(k,3,1)*xrb(j,3,1))
            t2=one/(xlb(j,3,1)*ytb(k,3,1))
            t3=one/(xrb(j,3,1)*ybb(k,3,1))
            t4=one/(xlb(j,3,1)*ybb(k,3,1))
c           **** boundary conditions
            if (lbd.eq.0) then
c              *** periodic b.c.
               i = nx
        vtus        =(q(i+1,j,k)+q(i,j,k))*(((u(i,j-1,kw-1)*xrb(j,1,1)+
     1  3.*u(i,j,kw-1)*xrb(j,2,1))*ytb(k,1,1)+ytb(k,2,1)*3.*(u(i,j-1,kw)
     2  *xrb(j,1,1)+3.*u(i,j,kw)*xrb(j,2,1)))*t1+(ytb(k,1,1)*(3.*u(i,j-1
     3  ,kw-1)*xlb(j,2,1)+u(i,j,kw-1)*xlb(j,1,1))+3.*ytb(k,2,1)*(3.*
     4  u(i,j-1,kw)*xlb(j,2,1)+u(i,j,kw)*xlb(j,1,1)))*t2+(3.*ybb(k,2,1)*
     5  (u(i,j-1,kw-1)*xrb(j,1,1)+3.*u(i,j,kw-1)*xrb(j,2,1))+ybb(k,1,1)*
     6  (u(i,j-1,kw)*xrb(j,1,1)+3.*u(i,j,kw)*xrb(j,2,1)))*t3+(3.*
     7  ybb(k,2,1)*(3.*u(i,j-1,kw-1)*xlb(j,2,1)+u(i,j,kw-1)*xlb(j,1,1))+
     8  ybb(k,1,1)*(3.*u(i,j-1,kw)*xlb(j,2,1)+u(i,j,kw)*xlb(j,1,1)))*
     9  t4)+(q(i+1,j-1,k)+q(i,j-1,k))*((ybb(k,1,1)*(u(i,j,kw)+5.*
     $  u(i,j-1,kw))/3.+(5.*u(i,j-1,kw-1)+u(i,j,kw-1))*ybb(k,2,1))*t4+
     $  (ytb(k,1,1)*(5.*u(i,j-1,kw-1)+u(i,j,kw-1))/3.+ytb(k,2,1)*(5.*
     $  u(i,j-1,kw)+u(i,j,kw)))*t2)
        diftus =(q(i+1,j+1,k)+q(i,j+1,k))*(((u(i,j-1,kw-1)+5.*
     1  u(i,j,kw-1))*ybb(k,2,1)+(u(i,j-1,kw)+5.*u(i,j,kw))*ybb(k,1,1)/
     2  3.)*t3+(ytb(k,1,1)*(u(i,j-1,kw-1)+5.*u(i,j,kw-1))/3.+
     3  (u(i,j-1,kw)+5.*u(i,j,kw))*ytb(k,2,1))*t1)+(q(i+1,j,k+1)+
     4  q(i,j,k+1))*((xrb(j,1,1)*(u(i,j-1,kw-1)+5.*u(i,j-1,kw))/3.+
     5  (u(i,j,kw-1)+5.*u(i,j,kw))*xrb(j,2,1))*t1+(xlb(j,2,1)*
     6  (u(i,j-1,kw-1)+5.*u(i,j-1,kw))+xlb(j,1,1)*(u(i,j,kw-1)+5.*
     7  u(i,j,kw))/3.)*t2)+(q(i+1,j,k-1)+q(i,j,k-1))*((xlb(j,1,1)*
     8  (u(i,j,kw)+5.*u(i,j,kw-1))/3.+(u(i,j-1,kw)+5.*u(i,j-1,kw-1))*
     9  xlb(j,2,1))*t4+(xrb(j,1,1)*(u(i,j-1,kw)+5.*u(i,j-1,kw-1))/3.+
     $  (u(i,j,kw)+5.*u(i,j,kw-1))*xrb(j,2,1))*t3)+(q(i+1,j-1,k-1)+
     $  q(i,j-1,k-1))*(u(i,j,kw)+5.*(u(i,j-1,kw)+u(i,j,kw-1))+25.*
     $  u(i,j-1,kw-1))*(t4/9.)+(q(i+1,j+1,k-1)+q(i,j+1,k-1))*
     $  (u(i,j-1,kw)+5.*(u(i,j,kw)+u(i,j-1,kw-1))+25.*u(i,j,kw-1))*
     $  (t3/9.)+(q(i+1,j-1,k+1)+q(i,j-1,k+1))*(u(i,j,kw-1)+5.*
     $  (u(i,j-1,kw-1)+u(i,j,kw))+25.*u(i,j-1,kw))*(t2/9.)+
     $  (q(i+1,j+1,k+1)+q(i,j+1,k+1))*(u(i,j-1,kw-1)+5.*(u(i,j,kw-1)+
     $  u(i,j-1,kw))+25.*u(i,j,kw))*(t1/9.)
               vtus  = ttus*(vtus+diftus)
            else
c              *** reflecting b.c.
               vtus = 0
            endif
            do 250 i=2,nx
               vprev = vtus
        vtus        =(q(i+1,j,k)+q(i,j,k))*(((u(i,j-1,kw-1)*xrb(j,1,1)+
     1  3.*u(i,j,kw-1)*xrb(j,2,1))*ytb(k,1,1)+ytb(k,2,1)*3.*(u(i,j-1,kw)
     2  *xrb(j,1,1)+3.*u(i,j,kw)*xrb(j,2,1)))*t1+(ytb(k,1,1)*(3.*
     3  u(i,j-1,kw-1)*xlb(j,2,1)+u(i,j,kw-1)*xlb(j,1,1))+3.*ytb(k,2,1)*
     4  (3.*u(i,j-1,kw)*xlb(j,2,1)+u(i,j,kw)*xlb(j,1,1)))*t2+(3.*
     5  ybb(k,2,1)*(u(i,j-1,kw-1)*xrb(j,1,1)+3.*u(i,j,kw-1)*xrb(j,2,1))+
     6  ybb(k,1,1)*(u(i,j-1,kw)*xrb(j,1,1)+3.*u(i,j,kw)*xrb(j,2,1)))*t3+
     7  (3.*ybb(k,2,1)*(3.*u(i,j-1,kw-1)*xlb(j,2,1)+u(i,j,kw-1)*
     8  xlb(j,1,1))+ybb(k,1,1)*(3.*u(i,j-1,kw)*xlb(j,2,1)+u(i,j,kw)*
     9  xlb(j,1,1)))*t4)+(q(i+1,j-1,k)+q(i,j-1,k))*((ybb(k,1,1)*
     $  (u(i,j,kw)+5.*u(i,j-1,kw))/3.+(5.*u(i,j-1,kw-1)+u(i,j,kw-1))*
     $  ybb(k,2,1))*t4+(ytb(k,1,1)*(5.*u(i,j-1,kw-1)+u(i,j,kw-1))/3.+
     $  ytb(k,2,1)*(5.*u(i,j-1,kw)+u(i,j,kw)))*t2)
        diftus =(q(i+1,j+1,k)+q(i,j+1,k))*(((u(i,j-1,kw-1)+5.*
     1  u(i,j,kw-1))*ybb(k,2,1)+(u(i,j-1,kw)+5.*u(i,j,kw))*ybb(k,1,1)/
     2  3.)*t3+(ytb(k,1,1)*(u(i,j-1,kw-1)+5.*u(i,j,kw-1))/3.+
     3  (u(i,j-1,kw)+5.*u(i,j,kw))*ytb(k,2,1))*t1)+(q(i+1,j,k+1)+
     4  q(i,j,k+1))*((xrb(j,1,1)*(u(i,j-1,kw-1)+5.*u(i,j-1,kw))/3.+
     5  (u(i,j,kw-1)+5.*u(i,j,kw))*xrb(j,2,1))*t1+(xlb(j,2,1)*
     6  (u(i,j-1,kw-1)+5.*u(i,j-1,kw))+xlb(j,1,1)*(u(i,j,kw-1)+5.*
     7  u(i,j,kw))/3.)*t2)+(q(i+1,j,k-1)+q(i,j,k-1))*((xlb(j,1,1)*
     8  (u(i,j,kw)+5.*u(i,j,kw-1))/3.+(u(i,j-1,kw)+5.*u(i,j-1,kw-1))*
     9  xlb(j,2,1))*t4+(xrb(j,1,1)*(u(i,j-1,kw)+5.*u(i,j-1,kw-1))/3.+
     $  (u(i,j,kw)+5.*u(i,j,kw-1))*xrb(j,2,1))*t3)+(q(i+1,j-1,k-1)+
     $  q(i,j-1,k-1))*(u(i,j,kw)+5.*(u(i,j-1,kw)+u(i,j,kw-1))+25.*
     $  u(i,j-1,kw-1))*(t4/9.)+(q(i+1,j+1,k-1)+q(i,j+1,k-1))*
     $  (u(i,j-1,kw)+5.*(u(i,j,kw)+u(i,j-1,kw-1))+25.*u(i,j,kw-1))*
     $  (t3/9.)+(q(i+1,j-1,k+1)+q(i,j-1,k+1))*(u(i,j,kw-1)+5.*
     $  (u(i,j-1,kw-1)+u(i,j,kw))+25.*u(i,j-1,kw))*(t2/9.)+
     $  (q(i+1,j+1,k+1)+q(i,j+1,k+1))*(u(i,j-1,kw-1)+5.*(u(i,j,kw-1)+
     $  u(i,j-1,kw))+25.*u(i,j,kw))*(t1/9.)
              vtus = ttus*(vtus+diftus)
              qshad(i,j,k-izstart+1) = vtus - vprev
  250       continue
  270   continue

c       ****** y-direction *****
        l=2
        ttus = 0.5*dxdyc(l)
c       do 300 k=2,nz
        do 300 k=izstart,izstop
c DaN---------------------------------------------------------------DaN
          kw = k-izstart+1
c          if (izstart.eq.2) kw = kw + 1
          if (bottom_layer) kw = kw + 1
c DaN---------------------------------------------------------------DaN
          do 300 i=2,nx
            t1=one/(ytb(k,3,2)*xrb(i,3,2))
            t2=one/(xlb(i,3,2)*ytb(k,3,2))
            t3=one/(xrb(i,3,2)*ybb(k,3,2))
            t4=one/(xlb(i,3,2)*ybb(k,3,2))
c           **** boundary conditions
            if (mbd.eq.0) then
c              *** periodic b.c.
               j=ny
        vtus = (q(i,j+1,k)+q(i,j,k))*(((v(i,j,kw-1)*xrb(i,1,2)+3.
     1  *v(i-1,j,kw-1)*xrb(i,2,2))*ytb(k,1,2)+ytb(k,2,2)*3.*(v(i,j,kw)*x
     2  rb(i,1,2)+3.*v(i-1,j,kw)*xrb(i,2,2)))*t1+(ytb(k,1,2)*(3.*v(i,j,k
     3  w-1)*xlb(i,2,2)+v(i-1,j,kw-1)*xlb(i,1,2))+3.*ytb(k,2,2)*(3.*v(i,
     4  j,kw)*xlb(i,2,2)+v(i-1,j,kw)*xlb(i,1,2)))*t2+(3.*ybb(k,2,2)*(v(i
     5  ,j,kw-1)*xrb(i,1,2)+3.*v(i-1,j,kw-1)*xrb(i,2,2))+ybb(k,1,2)*(v(i
     6  ,j,kw)*xrb(i,1,2)+3.*v(i-1,j,kw)*xrb(i,2,2)))*t3+(3.*ybb(k,2,2)*
     7  (3.*v(i,j,kw-1)*xlb(i,2,2)+v(i-1,j,kw-1)*xlb(i,1,2))+ybb(k,1,2)*
     8  (3.*v(i,j,kw)*xlb(i,2,2)+v(i-1,j,kw)*xlb(i,1,2)))*t4)+(q(i+1,j+1
     9  ,k)+q(i+1,j,k))*((ybb(k,1,2)*(v(i-1,j,kw)+5.*v(i,j,kw))/3.+(5.*v
     $  (i,j,kw-1)+v(i-1,j,kw-1))*ybb(k,2,2))*t4+(ytb(k,1,2)*(5.*v(i,j,k
     $  w-1)+v(i-1,j,kw-1))/3.+ytb(k,2,2)*(5.*v(i,j,kw)+v(i-1,j,kw)))*t2
     $  )
        diftus=(q(i-1,j+1,k)+q(i-1,j,k))*(((v(i,j,kw-1)+5.*v(i-1
     1  ,j,kw-1))*ybb(k,2,2)+(v(i,j,kw)+5.*v(i-1,j,kw))*ybb(k,1,2)/3.)*t
     2  3+(ytb(k,1,2)*(v(i,j,kw-1)+5.*v(i-1,j,kw-1))/3.+(v(i,j,kw)+5.*v(
     3  i-1,j,kw))*ytb(k,2,2))*t1)+(q(i,j+1,k+1)+q(i,j,k+1))*((xrb(i,1,2
     4  )*(v(i,j,kw-1)+5.*v(i,j,kw))/3.+(v(i-1,j,kw-1)+5.*v(i-1,j,kw))*x
     5  rb(i,2,2))*t1+(xlb(i,2,2)*(v(i,j,kw-1)+5.*v(i,j,kw))+xlb(i,1,2)*
     6  (v(i-1,j,kw-1)+5.*v(i-1,j,kw))/3.)*t2)+(q(i,j+1,k-1)+q(i,j,k-1))
     7  *((xlb(i,1,2)*(v(i-1,j,kw)+5.*v(i-1,j,kw-1))/3.+(v(i,j,kw)+5.*v(
     8  i,j,kw-1))*xlb(i,2,2))*t4+(xrb(i,1,2)*(v(i,j,kw)+5.*v(i,j,kw-1))
     9  /3.+(v(i-1,j,kw)+5.*v(i-1,j,kw-1))*xrb(i,2,2))*t3)+(q(i+1,j+1,k-
     $  1)+q(i+1,j,k-1))*(v(i-1,j,kw)+5.*(v(i,j,kw)+v(i-1,j,kw-1))+25.*v
     $  (i,j,kw-1))*(t4/9.)+(q(i-1,j+1,k-1)+q(i-1,j,k-1))*(v(i,j,kw)+5.*
     $  (v(i-1,j,kw)+v(i,j,kw-1))+25.*v(i-1,j,kw-1))*(t3/9.)+(q(i+1,j+1,
     $  k+1)+q(i+1,j,k+1))*(v(i-1,j,kw-1)+5.*(v(i,j,kw-1)+v(i-1,j,kw))+
     $  25.*v(i,j,kw))*(t2/9.)+(q(i-1,j+1,k+1)+q(i-1,j,k+1))*(v(i,j,kw-1
     $  )+5.*(v(i-1,j,kw-1)+v(i,j,kw))+25.*v(i-1,j,kw))*(t1/9.)
               vtus = ttus*(vtus+diftus)
            else
               vtus = 0
            endif
            do 280 j=2,ny
               vprev = vtus
        vtus = (q(i,j+1,k)+q(i,j,k))*(((v(i,j,kw-1)*xrb(i,1,2)+3.
     1  *v(i-1,j,kw-1)*xrb(i,2,2))*ytb(k,1,2)+ytb(k,2,2)*3.*(v(i,j,kw)*x
     2  rb(i,1,2)+3.*v(i-1,j,kw)*xrb(i,2,2)))*t1+(ytb(k,1,2)*(3.*v(i,j,k
     3  w-1)*xlb(i,2,2)+v(i-1,j,kw-1)*xlb(i,1,2))+3.*ytb(k,2,2)*(3.*v(i,
     4  j,kw)*xlb(i,2,2)+v(i-1,j,kw)*xlb(i,1,2)))*t2+(3.*ybb(k,2,2)*(v(i
     5  ,j,kw-1)*xrb(i,1,2)+3.*v(i-1,j,kw-1)*xrb(i,2,2))+ybb(k,1,2)*(v(i
     6  ,j,kw)*xrb(i,1,2)+3.*v(i-1,j,kw)*xrb(i,2,2)))*t3+(3.*ybb(k,2,2)*
     7  (3.*v(i,j,kw-1)*xlb(i,2,2)+v(i-1,j,kw-1)*xlb(i,1,2))+ybb(k,1,2)*
     8  (3.*v(i,j,kw)*xlb(i,2,2)+v(i-1,j,kw)*xlb(i,1,2)))*t4)+(q(i+1,j+1
     9  ,k)+q(i+1,j,k))*((ybb(k,1,2)*(v(i-1,j,kw)+5.*v(i,j,kw))/3.+(5.*v
     $  (i,j,kw-1)+v(i-1,j,kw-1))*ybb(k,2,2))*t4+(ytb(k,1,2)*(5.*v(i,j,k
     $  w-1)+v(i-1,j,kw-1))/3.+ytb(k,2,2)*(5.*v(i,j,kw)+v(i-1,j,kw)))*t2
     $  )
        diftus=(q(i-1,j+1,k)+q(i-1,j,k))*(((v(i,j,kw-1)+5.*v(i-1
     1  ,j,kw-1))*ybb(k,2,2)+(v(i,j,kw)+5.*v(i-1,j,kw))*ybb(k,1,2)/3.)*t
     2  3+(ytb(k,1,2)*(v(i,j,kw-1)+5.*v(i-1,j,kw-1))/3.+(v(i,j,kw)+5.*v(
     3  i-1,j,kw))*ytb(k,2,2))*t1)+(q(i,j+1,k+1)+q(i,j,k+1))*((xrb(i,1,2
     4  )*(v(i,j,kw-1)+5.*v(i,j,kw))/3.+(v(i-1,j,kw-1)+5.*v(i-1,j,kw))*x
     5  rb(i,2,2))*t1+(xlb(i,2,2)*(v(i,j,kw-1)+5.*v(i,j,kw))+xlb(i,1,2)*
     6  (v(i-1,j,kw-1)+5.*v(i-1,j,kw))/3.)*t2)+(q(i,j+1,k-1)+q(i,j,k-1))
     7  *((xlb(i,1,2)*(v(i-1,j,kw)+5.*v(i-1,j,kw-1))/3.+(v(i,j,kw)+5.*v(
     8  i,j,kw-1))*xlb(i,2,2))*t4+(xrb(i,1,2)*(v(i,j,kw)+5.*v(i,j,kw-1))
     9  /3.+(v(i-1,j,kw)+5.*v(i-1,j,kw-1))*xrb(i,2,2))*t3)+(q(i+1,j+1,k-
     $  1)+q(i+1,j,k-1))*(v(i-1,j,kw)+5.*(v(i,j,kw)+v(i-1,j,kw-1))+25.*v
     $  (i,j,kw-1))*(t4/9.)+(q(i-1,j+1,k-1)+q(i-1,j,k-1))*(v(i,j,kw)+5.*
     $  (v(i-1,j,kw)+v(i,j,kw-1))+25.*v(i-1,j,kw-1))*(t3/9.)+(q(i+1,j+1,
     $  k+1)+q(i+1,j,k+1))*(v(i-1,j,kw-1)+5.*(v(i,j,kw-1)+v(i-1,j,kw))+
     $  25.*v(i,j,kw))*(t2/9.)+(q(i-1,j+1,k+1)+q(i-1,j,k+1))*(v(i,j,kw-1
     $  )+5.*(v(i-1,j,kw-1)+v(i,j,kw))+25.*v(i-1,j,kw))*(t1/9.)
              vtus=ttus*(vtus+diftus)
              qshad(i,j,k-izstart+1) = qshad(i,j,k-izstart+1)
     &         + vtus - vprev
  280       continue
  300   continue

c       ****** z-direction *****
        l=3
        ttus = 0.5*dxdyc(l)
        do 330 i=2,nx
          do 330 j=2,ny
            t1=one/(ytb(j,3,3)*xrb(i,3,3))
            t2=one/(xlb(i,3,3)*ytb(j,3,3))
            t3=one/(xrb(i,3,3)*ybb(j,3,3))
            t4=one/(xlb(i,3,3)*ybb(j,3,3))
            vtus=0
            do 310 k=izstart,izstop
c DaN---------------------------------------------------------------DaN
              kw = k-izstart+1
c              if (izstart.eq.2) kw = kw + 1
              if (bottom_layer) kw = kw + 1
c DaN---------------------------------------------------------------DaN
               vprev = vtus
        vtus = (q(i,j,k+1)+q(i,j,k))*(((w(i-1,j-1,kw)*xrb(i,1,3)+
     1  3.*w(i,j-1,kw)*xrb(i,2,3))*ytb(j,1,3)+ytb(j,2,3)*3.*(w(i-1,j,kw)
     2  *xrb(i,1,3)+3.*w(i,j,kw)*xrb(i,2,3)))*t1+(ytb(j,1,3)*(3.*w(i-1,j
     3  -1,kw)*xlb(i,2,3)+w(i,j-1,kw)*xlb(i,1,3))+3.*ytb(j,2,3)*(3.*w(i-
     4  1,j,kw)*xlb(i,2,3)+w(i,j,kw)*xlb(i,1,3)))*t2+(3.*ybb(j,2,3)*(w(i
     5  -1,j-1,kw)*xrb(i,1,3)+3.*w(i,j-1,kw)*xrb(i,2,3))+ybb(j,1,3)*(w(i
     6  -1,j,kw)*xrb(i,1,3)+3.*w(i,j,kw)*xrb(i,2,3)))*t3+(3.*ybb(j,2,3)*
     7  (3.*w(i-1,j-1,kw)*xlb(i,2,3)+w(i,j-1,kw)*xlb(i,1,3))+ybb(j,1,3)*
     8  (3.*w(i-1,j,kw)*xlb(i,2,3)+w(i,j,kw)*xlb(i,1,3)))*t4)+(q(i-1,j,k
     9  +1)+q(i-1,j,k))*((ybb(j,1,3)*(w(i,j,kw)+5.*w(i-1,j,kw))/3.+(5.*w
     $  (i-1,j-1,kw)+w(i,j-1,kw))*ybb(j,2,3))*t4+(ytb(j,1,3)*(5.*w(i-1,j
     $  -1,kw)+w(i,j-1,kw))/3.+ytb(j,2,3)*(5.*w(i-1,j,kw)+w(i,j,kw)))*t2
     $  )
        diftus=(q(i+1,j,k+1)+q(i+1,j,k))*(((w(i-1,j-1,kw)+5.*w(i
     1  ,j-1,kw))*ybb(j,2,3)+(w(i-1,j,kw)+5.*w(i,j,kw))*ybb(j,1,3)/3.)*t
     2  3+(ytb(j,1,3)*(w(i-1,j-1,kw)+5.*w(i,j-1,kw))/3.+(w(i-1,j,kw)+5.*
     3  w(i,j,kw))*ytb(j,2,3))*t1)+(q(i,j+1,k+1)+q(i,j+1,k))*((xrb(i,1,3
     4  )*(w(i-1,j-1,kw)+5.*w(i-1,j,kw))/3.+(w(i,j-1,kw)+5.*w(i,j,kw))*x
     5  rb(i,2,3))*t1+(xlb(i,2,3)*(w(i-1,j-1,kw)+5.*w(i-1,j,kw))+xlb(i,1
     6  ,3)*(w(i,j-1,kw)+5.*w(i,j,kw))/3.)*t2)+(q(i,j-1,k+1)+q(i,j-1,k))
     7  *((xlb(i,1,3)*(w(i,j,kw)+5.*w(i,j-1,kw))/3.+(w(i-1,j,kw)+5.*w(i-
     8  1,j-1,kw))*xlb(i,2,3))*t4+(xrb(i,1,3)*(w(i-1,j,kw)+5.*w(i-1,j-1,
     9  kw))/3.+(w(i,j,kw)+5.*w(i,j-1,kw))*xrb(i,2,3))*t3)+(q(i-1,j-1,k+
     $  1)+q(i-1,j-1,k))*(w(i,j,kw)+5.*(w(i-1,j,kw)+w(i,j-1,kw))+25.*w(i
     $  -1,j-1,kw))*(t4/9.)+(q(i+1,j-1,k+1)+q(i+1,j-1,k))*(w(i-1,j,kw)+5
     $  .*(w(i,j,kw)+w(i-1,j-1,kw))+25.*w(i,j-1,kw))*(t3/9.)+(q(i-1,j+1,
     $  k+1)+q(i-1,j+1,k))*(w(i,j-1,kw)+5.*(w(i-1,j-1,kw)+w(i,j,kw))+25.
     $  *w(i-1,j,kw))*(t2/9.)+(q(i+1,j+1,k+1)+q(i+1,j+1,k))*(w(i-1,j-1,k
     $  w)+5.*(w(i,j-1,kw)+w(i-1,j,kw))+25.*w(i,j,kw))*(t1/9.)
              vtus=ttus*(vtus+diftus)
              qshad(i,j,k-izstart+1) = qshad(i,j,k-izstart+1) 
     &           + vtus - vprev
  310       continue
  330   continue

        do k=izstart,izstop
          do j=2,ny
            do i=2,nx
              q(i,j,k)= qshad(i,j,k-izstart+1)
            enddo
          enddo
        enddo

      endif
 
      return
      end
