c $Log:   /pvcs.config/mc3dvv/src/flux.F_a  $
c
c   Rev 1.1   Fri Jul 30 15:05:10 1999   lowman
c  Add Nu.h includes
c
c   Rev 1.0   Mon Jul 14 14:21:20 1997   pvcs
c 
c *************************************************************
c *   FLUX
c *
c *   this routine integrates flux terms div(uT-grad(T))dvol by
c *   analytic evaluation of integral of (uT-grad(T))dA over each
c *   cell face.  over each face, u=u1+b.p+c.q+d.p.q
c *   where p=x-x1, q=y-y1 and numbering of nodes is lower left=1,
c *   lower right=2, upper left=3, upper right=4. definition
c *   here for x and y depends on direction of normal to a face.
c *   a similar expression for temperature over each face quadrant
c *   is used. temperatures are known at cell centers and are
c *   averaged to find temperature at face centers.
c *
c *   MPI version  Scott King and Peter van Keken Summer 1999
c *   Los Alamos/Michigan/Purdue
c *
c *   PvK 6/26/99  Modified parameter list for MPI implementation
c *   PvK 6/26/99  Introduced local and globaling depth numbering
c *                (izstart/izstop)
c *   PvK 6/26/99  Made loop over depth levels (k=1,nz) local
c *                (k=izstart,izstop)
c *   PvK 7/15/99  Changed numbering of ytb/ybb/d3 array to global
c *   PvK 7/15/99  Changed 'six' to 6d0 and 'nine' to 9d0.
c *                (integer nine intended as real*8 ....)
c *   PvK 7/15/99  Changed order of dimensions of difshn/fluxv
c *************************************************************
      subroutine flux(t,dtensr,idx,idy)
      use module_precision
      use module_param
      use module_giparam
      use module_gia1
      use module_a2
      use module_gia3
      use module_a3
      use module_a77
      use module_nu
      use module_intfac
      use module_slvelu
      use module_mytime
#ifdef MPI
      use module_mpibuf
#endif
      parameter (oneth=1.0D0/3.0D0,twoth=2.0D0/3.0D0,oneni=1.0D0/9.0D0)
      dimension t(idx,idy,*), dtensr(idx,idy,*)
      logical bottom_layer,top_layer
#ifdef MPI
      include 'mpif.h'
      integer stat(MPI_STATUS_SIZE)

      call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )
      call MPI_COMM_SIZE( MPI_COMM_WORLD, numprocs, ierr )
      call mpilayer(nz,numprocs,myid,izstart,izstop)
      bottom_layer=.false.
      top_layer=.false.
      izstart = izstart - 2
      if (izstart .lt. 2) izstart = 2
      izstop = izstop + 2
      if (izstop .gt. nz) izstop = nz
      if (izstart.eq.2) bottom_layer=.true.
      if (izstop.eq.nz) top_layer=.true.
      ioffset = 0
#else
      myid = 0
      izstart = 2
      izstop = nz
      ioffset = 0
      bottom_layer=.true.
      top_layer=.true.
#endif

      if (ny.eq.2) then
c *************************************************************
c     for 2-D calculations
c *************************************************************
        lim1=nx
        lim2=1
        lim3=nz
        lim123  = inx*jnx*knx
        lim1233 = inx*jnx*knx*3
        do l=1,3
          do k=1,kdeclared
            do j=1,jnx
              do i=1,inx
                difshn(i,j,l,k) = zero
                fluxv(i,j,l,k) = zero
              enddo
            enddo
          enddo
        enddo
        do k=1,knx
          do j=1,jnx
            do i=1,inx
             dtensr(i,j,k)  = zero
c             advctd(i,j,k) = zero
            enddo
          enddo
        enddo

c *************************************************************
c *     ADVECTIVE PART : X-direction
c *************************************************************
        q11=xrb(2,1,1)+three*xrb(2,2,1)
        q12=xlb(2,1,1)+three*xlb(2,2,1)+2d0
        do 40 k=izstart,izstop
          kglob=k+ioffset
          t1=1d0/(ytb(kglob,3,1)*xrb(2,3,1))
          t2=1d0/(xlb(2,3,1)*ytb(kglob,3,1))
          t3=1d0/(xrb(2,3,1)*ybb(kglob,3,1))
          t4=1d0/(xlb(2,3,1)*ybb(kglob,3,1))
          q13=t1*q11+t2*q12
          q14=t3*q11+t4*q12
          f1=three*ytb(kglob,2,1)*q13+ybb(kglob,1,1)*q14
          f2=ytb(kglob,1,1)*q13+three*ybb(kglob,2,1)*q14

c DaN---------------------------------------------------------------DaN
          kw = k-izstart+1
          if (bottom_layer) kw = k-izstart+2
c DaN---------------------------------------------------------------DaN

          do 20 i=2,nx
            fluxv(i,1,1,k)=  (t(i+1,1,k)+t(i,1,k)) *
     1                       (u(i,1,kw)*f1+u(i,1,kw-1)*f2)
   20     continue
          df1=2d0*ybb(kglob,1,1)*t3+6d0*ytb(kglob,2,1)*t1
          df2=6d0*ybb(kglob,2,1)*t3+2d0*ytb(kglob,1,1)*t1
          df3= 5.*(((oneth*xrb(2,1,1)+xrb(2,2,1))*t1) +
     1         ((oneth*xlb(2,1,1) +xlb(2,2,1))*t2))+(30d0/9d0)*(t1+t2)
          df4= ((oneth*xrb(2,1,1)+xrb(2,2,1))*t1)+
     1         ((oneth*xlb(2,1,1)+xlb(2,2,1))*t2)+(6d0/9d0)*(t1+t2)
          df5= ((oneth*xlb(2,1,1)+xlb(2,2,1))*t4)+
     1         ((oneth*xrb(2,1,1)+xrb(2,2,1))*t3)+(6d0/9d0)*(t3+t4)
          df6=five*(((oneth*xlb(2,1,1)+xlb(2,2,1))*t4)+
     1         ((oneth*xrb(2,1,1)+xrb(2,2,1))*t3))+(30d0/9d0)*(t3+t4)
          do 30 i=2,nx
            difshn(i,1,1,k)=(t(i+1,1,k)+t(i,1,k))*(u(i,1,kw)*df1 +
     1           u(i,1,kw-1)*df2)+(t(i+1,1,k+1)+t(i,1,k+1))*
     2          (u(i,1,kw)*df3+u(i,1,kw-1)*df4)+(t(i+1,1,k-1)+
     3          t(i,1,k-1))*(u(i,1,kw)*df5+u(i,1,kw-1)*df6)
   30     continue
   40   continue

c *************************************************************
c *     ADVECTIVE PART : Y-direction
c *************************************************************
        do k=1,kdeclared
          do i=2,nx
            fluxv (i,1,2,k)  = zero
            difshn(i,1,2,k)  = zero
          enddo
        enddo

c *************************************************************
c *     ADVECTIVE PART : Z-direction
c *************************************************************
        q31=ytb(2,1,3)+3.*ytb(2,2,3)
        q32=3.*ybb(2,2,3)+ybb(2,1,3)
        q33=oneth*ytb(2,1,3)+ytb(2,2,3)
        q34=oneth*ybb(2,1,3)+ybb(2,2,3)
        do 90 i=2,nx
          t1=1d0/(xrb(i,3,3)*ybb(2,3,3))
          t2=1d0/(ytb(2,3,3)*xrb(i,3,3))
          t3=1d0/(xlb(i,3,3)*ytb(2,3,3))
          t4=1d0/(xlb(i,3,3)*ybb(2,3,3))
          f1=(xrb(i,1,3)*t2+3.*xlb(i,2,3)*t3)*q31+
     1       (xrb(i,1,3)*t1+3.*xlb(i,2,3)*t4)*q32
          f2=(3.*xrb(i,2,3)*t2+xlb(i,2,3)*t3)*q31+
     1       (3.*xrb(i,2,3)*t1+xlb(i,1,3)*t4)*q32
          f3=q33*t3+q34*t4
          df1=(ybb(2,2,3)+oneth*ybb(2,1,3))*t1+(oneth*ytb(2,1,3)+
     1        ytb(2,2,3))*t2+twoth*(t1+t2)
          df2=2.*xrb(i,1,3)*(t1+t2)+6.*xlb(i,2,3)*(t3+t4)
          df3=6.*xrb(i,2,3)*(t1+t2)+2.*xlb(i,1,3)*(t3+t4)
          df4=2.*oneth*(t3+t4)
c
          do 70 k=izstart,izstop
c DaN---------------------------------------------------------------DaN
          kw = k-izstart+1
          if (bottom_layer) kw = k-izstart+2
c DaN---------------------------------------------------------------DaN
        fluxv(i,1,3,k)=(t(i,1,k+1)+t(i,1,k))*(((w(i-1,1,kw)*xrb(i,1,3)+3
     1  .*w(i,1,kw)*xrb(i,2,3))*ytb(2,1,3)+ytb(2,2,3)*3.*(w(i-1,1,kw)*xr
     2  b(i,1,3)+3.*w(i,1,kw)*xrb(i,2,3)))/(ytb(2,3,3)*xrb(i,3,3))+(ytb(
     3  2,1,3)*(3.*w(i-1,1,kw)*xlb(i,2,3)+w(i,1,kw)*xlb(i,1,3))+3.*ytb(2
     4  ,2,3)*(3.*w(i-1,1,kw)*xlb(i,2,3)+w(i,1,kw)*xlb(i,1,3)))/(xlb(i,3
     5  ,3)*ytb(2,3,3))+(3.*ybb(2,2,3)*(w(i-1,1,kw)*xrb(i,1,3)+3.*w(i,1,
     6  kw)*xrb(i,2,3))+ybb(2,1,3)*(w(i-1,1,kw)*xrb(i,1,3)+3.*w(i,1,kw)*
     7  xrb(i,2,3)))/(xrb(i,3,3)*ybb(2,3,3))+(3.*ybb(2,2,3)*(3.*w(i-1,1,
     8  kw)*xlb(i,2,3)+w(i,1,kw)*xlb(i,1,3))+ybb(2,1,3)*(3.*w(i-1,1,kw)*
     9  xlb(i,2,3)+w(i,1,kw)*xlb(i,1,3)))/(xlb(i,3,3)*ybb(2,3,3)))+(t(i-
     $  1,1,k+1)+t(i-1,1,k))*((ybb(2,1,3)*(w(i,1,kw)+5.*w(i-1,1,kw))/3.+
     $  (5.*w(i-1,1,kw)+w(i,1,kw))*ybb(2,2,3))/(xlb(i,3,3)*ybb(2,3,3))+(
     $  ytb(2,1,3)*(5.*w(i-1,1,kw)+w(i,1,kw))/3.+ytb(2,2,3)*(5.*w(i-1,1,
     $  kw)+w(i,1,kw)))/(xlb(i,3,3)*ytb(2,3,3)))
   70     continue

         do 80 k=izstart,izstop
c DaN---------------------------------------------------------------DaN
           kw = k-izstart+1
           if (bottom_layer) kw = k-izstart+2
c DaN---------------------------------------------------------------DaN
        difshn(i,1,3,k)=(t(i+1,1,k+1)+t(i+1,1,k))*(w(i-1,1,kw)+5.*w(i,1,
     1  kw))*df1+(t(i,1,k+1)+t(i,1,k))*(w(i-1,1,kw)*df2+w(i,1,kw)*df3)+(
     2  t(i-1,1,k+1)+t(i-1,1,k))*(5.*w(i-1,1,kw)+w(i,1,kw))*df4
   80     continue
   90   continue

        l=1
        t1=0.5*dxdyc(l)
        do k=izstart,izstop
           do i=2,nx
              fluxv(i,1,l,k)=t1*(fluxv(i,1,l,k)+difshn(i,1,l,k))
              difshn(i,1,l,k)=0.
           enddo
        enddo
        l=3
        t1=0.5*dxdyc(l)
        do k=izstart,izstop
           do i=2,nx
              fluxv(i,1,l,k)=t1*(fluxv(i,1,l,k)+difshn(i,1,l,k))
              difshn(i,1,l,k)=0.
           enddo
        enddo

c *************************************************************
c *     DIFFUSIVE PART : X-direction
c *************************************************************

        do 120 k=izstart,izstop
          m=k+ioffset
          t1=1d0/(xlb(2,3,1)*ybb(m,3,1))
          t2=1d0/(ybb(m,3,1)*xrb(2,3,1))
          t3=1d0/(xlb(2,3,1)*ytb(m,3,1))
          t4=1d0/(ytb(m,3,1)*xrb(2,3,1))
          do 110 i=2,nx
        difshn(i,1,1,k)=4.*dxdyc(1)/d3(i,1)*((t(i+1,1,k)-t(i,1,k))*(ybb
     1  (m,4,1)+ytb(m,4,1))*(xrb(2,4,1)+xlb(2,4,1))+((t(i+1,1,k)-t(i,1,k
     2  ))/xlb(2,3,1)+(t(i+1,1,k)-t(i,1,k))/xrb(2,3,1))*(ybb(m,4,1)+ytb
     3  (m,4,1))+((t(i+1,1,k+1)-t(i,1,k+1))/ytb(m,3,1)+(t(i+1,1,k-1)-t(i
     4  ,1,k-1))/ybb(m,3,1))*(xlb(2,4,1)+xrb(2,4,1))+(t(i+1,1,k-1)-t(i,1
     5  ,k-1))*t1+(t(i+1,1,k-1)-t(i,1,k-1))*t2+(t(i+1,1,k+1)-t(i,1,k+1))
     6  *t3+(t(i+1,1,k+1)-t(i,1,k+1))*t4)
  110     continue
  120   continue

c *************************************************************
c *     DIFFUSIVE PART : Y-direction
c *************************************************************
        do k=2,kdeclared-1
          do i=2,nx
            difshn(i,1,2,k)=zero
          enddo
        enddo

c *************************************************************
c *     DIFFUSIVE PART : Z-direction
c *************************************************************
        do 160 i=2,nx
          t1=1d0/(xlb(i,3,3)*ybb(2,3,3))
          t2=1d0/(ybb(2,3,3)*xrb(i,3,3))
          t3=1d0/(xlb(i,3,3)*ytb(2,3,3))
          t4=1d0/(ytb(2,3,3)*xrb(i,3,3))

          do 150 k=izstart,izstop
             kglob=k+ioffset
        difshn(i,1,3,k)=4.*dxdyc(3)/d3(kglob,3)*
     v         ((t(i,1,k+1)-t(i,1,k))*(ybb(2,4,3)+
     1  ytb(2,4,3))*(xrb(i,4,3)+xlb(i,4,3))+((t(i-1,1,k+1)-t(i-1,
     2  1,k))/xlb(i,3,3)+(t(i+1,1,k+1)-t(i+1,1,k))/xrb(i,3,3))*(ybb(2,4,
     3  3)+ytb(2,4,3))+((t(i,1,k+1)-t(i,1,k))/ytb(2,3,3)+(t(i,1,k+1)-t(i
     4  ,1,k))/ybb(2,3,3))*(xlb(i,4,3)+xrb(i,4,3))+(t(i-1,1,k+1)-t(i-1,1
     5  ,k))*t1+(t(i+1,1,k+1)-t(i+1,1,k))*t2+(t(i-1,1,k+1)-t(i-1,1,k))
     6  *t3+(t(i+1,1,k+1)-t(i+1,1,k))*t4)
  150     continue
  160   continue

c *************************************************************
c *     BOUNDARY CONDITIONS
c *************************************************************
        if (lbd.eq.0) then

          do 170 k=izstart,izstop
            difshn(1,1,1,k)=difshn(nx,1,1,k)
  170       fluxv(1,1,1,k)=fluxv(nx,1,1,k)
        else

          do 175 k=izstart,izstop
            difshn(1,1,1,k)=zero
  175       fluxv(1,1,1,k)=zero
        endif

        do 180 i=2,nx
        difshn(i,1,3,1)=4.*dxdyc(3)/d3(1,3)*((t(i,1,2)-t(i,1,1))*(ybb(2,
     1  4,3)+ytb(2,4,3))*(xrb(i,4,3)+xlb(i,4,3))+((t(i-1,1,2)-t(i-1,1,1)
     2  )/xlb(i,3,3)+(t(i+1,1,2)-t(i+1,1,1))/xrb(i,3,3))*(ybb(2,4,3)+ytb
     3  (2,4,3))+((t(i,1,2)-t(i,1,1))/ytb(2,3,3)+(t(i,1,2)-t(i,1,1))/ybb
     4  (2,3,3))*(xlb(i,4,3)+xrb(i,4,3))+(t(i-1,1,2)-t(i-1,1,1))/(xlb(i,
     5  3,3)*ybb(2,3,3))+(t(i+1,1,2)-t(i+1,1,1))/(ybb(2,3,3)*xrb(i,3,3))
     6  +(t(i-1,1,2)-t(i-1,1,1))/(xlb(i,3,3)*ytb(2,3,3))+(t(i+1,1,2)-t(i
     7  +1,1,1))/(ytb(2,3,3)*xrb(i,3,3)))
  180   continue

c *************************************************************
c *     GATHER ADVECTIVE AND DIFFUSIVE PARTS
c *************************************************************
        dxdy=dx*dy

        do 190 k=izstart,izstop
          m=k+ioffset
          voli=0.5*dt/(dxdy*(z(m)-z(m-1)))
          do 189 i=2,nx
          dtensr(i,1,k)=voli*(fluxv(i,1,1,k)-fluxv(i-1,1,1,k)
     1           +fluxv(i,1,3,k)-fluxv(i,1,3,k-1))

          advctd(i,1,k-izstart+1)=
     1           -(fluxv(i,1,1,k)-fluxv(i-1,1,1,k)+fluxv(i,1,3,k)
     2            -fluxv(i,1,3,k-1))+difshn(i,1,1,k)-difshn(i-1,1,1,k)
     3            +difshn(i,1,3,k)-difshn(i,1,3,k-1)
  189   continue
c       write(myid+80,*)"izstart,iztop,k,index",izstart,izstop,k,
c    &        k-izstart+1
  190   continue

c *************************************************************
c *   calculate nusselt number
c *************************************************************
        tznu=0.
c       *** SDK: Parallel version over this processor's layer
c       *** PvK 6/99: modified for local numbering. Keep znu array global.
c       ***           If izstart < 2: we're dealing with lower layer
c       ***           if izstart > nz: we're dealing with top layer
c       do 210 k=1,nz

        do 210 klocal=izstart-1,izstop
          k = ioffset + klocal
          znu(k)=0.
          do 200 i=2,nx
              znu(k)=znu(k)+fluxv(i,1,3,klocal)-difshn(i,1,3,klocal)
  200     continue
  210   continue

      else

c ************************************************************* 
c     3-D calculations
c ************************************************************* 

        l=1
        ttus = 0.5*dxdyc(l)
        index = 0
#ifdef MPI
        b1time = MPI_WTIME()
#endif
c       do 240 k=izstart,izstop
        do 240 k=izstart,izstop
c DaN---------------------------------------------------------------DaN
          kw = k-izstart+1
          if (bottom_layer) kw = k-izstart+2
c DaN---------------------------------------------------------------DaN
          do 239 j=2,ny
          m = k+ioffset
            ta=1d0/(ytb(m,3,1)*xrb(j,3,1))
            tb=1d0/(xlb(j,3,1)*ytb(m,3,1))
            tc=1d0/(xrb(j,3,1)*ybb(m,3,1))
            td=1d0/(xlb(j,3,1)*ybb(m,3,1))
            te=1d0/(xlb(j,3,1)*ybb(m,3,1))
            tf=1d0/(ybb(m,3,1)*xrb(j,3,1))
            tg=1d0/(xlb(j,3,1)*ytb(m,3,1))
            th=1d0/(ytb(m,3,1)*xrb(j,3,1))
c           **** BOUNDARY CONDITIONS (X)
            if (lbd.eq.0) then
c              *** periodic b.c.: calculate ftus,diftus for i=nx
               i=nx
        ftus         =(t(i+1,j,k)+t(i,j,k))*(((u(i,j-1,kw-1)*xrb(j,1,1)+
     1  3.*u(i,j,kw-1)*xrb(j,2,1))*ytb(m,1,1)+ytb(m,2,1)*3.*(u(i,j-1,kw)
     2  *xrb(j,1,1)+3.*u(i,j,kw)*xrb(j,2,1)))*ta+(ytb(m,1,1)*(3.*u(i,j-1
     3  ,kw-1)*xlb(j,2,1)+u(i,j,kw-1)*xlb(j,1,1))+3.*ytb(m,2,1)*(3.*u(i,
     3  j-1,kw)*xlb(j,2,1)+u(i,j,kw)*xlb(j,1,1)))*tb+(3.*ybb(m,2,1)*(u(i
     4  ,j-1,kw-1)*xrb(j,1,1)+3.*u(i,j,kw-1)*xrb(j,2,1))+ybb(m,1,1)*(u(i
     5  ,j-1,kw)*xrb(j,1,1)+3.*u(i,j,kw)*xrb(j,2,1)))*tc+(3.*ybb(m,2,1)*
     6  (3.*u(i,j-1,kw-1)*xlb(j,2,1)+u(i,j,kw-1)*xlb(j,1,1))+ybb(m,1,1)*
     7  (3.*u(i,j-1,kw)*xlb(j,2,1)+u(i,j,kw)*xlb(j,1,1)))*td)+(t(i+1,j-1
     8  ,k)+t(i,j-1,k))*((ybb(m,1,1)*(u(i,j,kw)+5.*u(i,j-1,kw))/3.+(5.*u
     9  (i,j-1,kw-1)+u(i,j,kw-1))*ybb(m,2,1))*td+(ytb(m,1,1)*(5.*u(i,j-1
     $  ,kw-1)+u(i,j,kw-1))/3.+ytb(m,2,1)*(5.*u(i,j-1,kw)+u(i,j,kw)))*tb
     $  )
        diftus        =(t(i+1,j+1,k)+t(i,j+1,k))*(((u(i,j-1,kw-1)+5.*u(i
     1  ,j,kw-1))*ybb(m,2,1)+(u(i,j-1,kw)+5.*u(i,j,kw))*ybb(m,1,1)/3.)*t
     2  c+(ytb(m,1,1)*(u(i,j-1,kw-1)+5.*u(i,j,kw-1))/3.+(u(i,j-1,kw)+5.*
     3  u(i,j,kw))*ytb(m,2,1))*ta)+(t(i+1,j,k+1)+t(i,j,k+1))*((xrb(j,1,1
     4  )*(u(i,j-1,kw-1)+5.*u(i,j-1,kw))/3.+(u(i,j,kw-1)+5.*u(i,j,kw))*x
     5  rb(j,2,1))*ta+(xlb(j,2,1)*(u(i,j-1,kw-1)+5.*u(i,j-1,kw))+xlb(j,1
     6  ,1)*(u(i,j,kw-1)+5.*u(i,j,kw))/3.)*tb)+(t(i+1,j,k-1)+t(i,j,k-1))
     7  *((xlb(j,1,1)*(u(i,j,kw)+5.*u(i,j,kw-1))/3.+(u(i,j-1,kw)+5.*u(i,
     8  j-1,kw-1))*xlb(j,2,1))*td+(xrb(j,1,1)*(u(i,j-1,kw)+5.*u(i,j-1,kw
     9  -1))/3.+(u(i,j,kw)+5.*u(i,j,kw-1))*xrb(j,2,1))*tc)+(t(i+1,j-1,k-
     $  1)+t(i,j-1,k-1))*(u(i,j,kw)+5.*(u(i,j-1,kw)+u(i,j,kw-1))+25.*u(i
     $  ,j-1,kw-1))*(td/9.)+(t(i+1,j+1,k-1)+t(i,j+1,k-1))*(u(i,j-1,kw)+5
     $  .*(u(i,j,kw)+u(i,j-1,kw-1))+25.*u(i,j,kw-1))*(tc/9.)+(t(i+1,j-1,
     $  k+1)+t(i,j-1,k+1))*(u(i,j,kw-1)+5.*(u(i,j-1,kw-1)+u(i,j,kw))+25.
     $  *u(i,j-1,kw))*(tb/9.)+(t(i+1,j+1,k+1)+t(i,j+1,k+1))*(u(i,j-1,kw-
     $  1)+5.*(u(i,j,kw-1)+u(i,j-1,kw))+25.*u(i,j,kw))*(ta/9.)
        ftus  = ttus*(ftus+diftus)
        diftus         =4.*dxdyc(1)/d3(i,1)*((t(i+1,j,k)-t(i,j,k))*(ybb
     1  (m,4,1)+ytb(m,4,1))*(xrb(j,4,1)+xlb(j,4,1))+((t(i+1,j-1,k)-t(i,j
     2  -1,k))/xlb(j,3,1)+(t(i+1,j+1,k)-t(i,j+1,k))/xrb(j,3,1))*(ybb(m,4
     3  ,1)+ytb(m,4,1))+((t(i+1,j,k+1)-t(i,j,k+1))/ytb(m,3,1)+(t(i+1,j,k
     4  -1)-t(i,j,k-1))/ybb(m,3,1))*(xlb(j,4,1)+xrb(j,4,1))+(t(i+1,j-1,k
     5  -1)-t(i,j-1,k-1))*te+(t(i+1,j+1,k-1)-t(i,j+1,k-1))*tf+(t(i+1,j-1
     6  ,k+1)-t(i,j-1,k+1))*tg+(t(i+1,j+1,k+1)-t(i,j+1,k+1))*th)
            else
                ftus    = zero
                diftus  = zero
            endif

c           *** Now the real work (for x)
            do 220 i=2,nx
                fprev   = ftus
                difprev = diftus
        ftus         =(t(i+1,j,k)+t(i,j,k))*(((u(i,j-1,kw-1)*xrb(j,1,1)+
     1  3.*u(i,j,kw-1)*xrb(j,2,1))*ytb(m,1,1)+ytb(m,2,1)*3.*(u(i,j-1,kw)
     2  *xrb(j,1,1)+3.*u(i,j,kw)*xrb(j,2,1)))*ta+(ytb(m,1,1)*(3.*u(i,j-1
     3  ,kw-1)*xlb(j,2,1)+u(i,j,kw-1)*xlb(j,1,1))+3.*ytb(m,2,1)*(3.*u(i,
     4  j-1,kw)*xlb(j,2,1)+u(i,j,kw)*xlb(j,1,1)))*tb+(3.*ybb(m,2,1)*(u(i
     5  ,j-1,kw-1)*xrb(j,1,1)+3.*u(i,j,kw-1)*xrb(j,2,1))+ybb(m,1,1)*(u(i
     6  ,j-1,kw)*xrb(j,1,1)+3.*u(i,j,kw)*xrb(j,2,1)))*tc+(3.*ybb(m,2,1)*
     7  (3.*u(i,j-1,kw-1)*xlb(j,2,1)+u(i,j,kw-1)*xlb(j,1,1))+ybb(m,1,1)*
     8  (3.*u(i,j-1,kw)*xlb(j,2,1)+u(i,j,kw)*xlb(j,1,1)))*td)+(t(i+1,j-1
     9  ,k)+t(i,j-1,k))*((ybb(m,1,1)*(u(i,j,kw)+5.*u(i,j-1,kw))/3.+(5.*u
     $  (i,j-1,kw-1)+u(i,j,kw-1))*ybb(m,2,1))*td+(ytb(m,1,1)*(5.*u(i,j-1
     $  ,kw-1)+u(i,j,kw-1))/3.+ytb(m,2,1)*(5.*u(i,j-1,kw)+u(i,j,kw)))*tb
     $  )
        diftus        =(t(i+1,j+1,k)+t(i,j+1,k))*(((u(i,j-1,kw-1)+5.*u(i
     1  ,j,kw-1))*ybb(m,2,1)+(u(i,j-1,kw)+5.*u(i,j,kw))*ybb(m,1,1)/3.)*t
     2  c+(ytb(m,1,1)*(u(i,j-1,kw-1)+5.*u(i,j,kw-1))/3.+(u(i,j-1,kw)+5.*
     3  u(i,j,kw))*ytb(m,2,1))*ta)+(t(i+1,j,k+1)+t(i,j,k+1))*((xrb(j,1,1
     4  )*(u(i,j-1,kw-1)+5.*u(i,j-1,kw))/3.+(u(i,j,kw-1)+5.*u(i,j,kw))*x
     5  rb(j,2,1))*ta+(xlb(j,2,1)*(u(i,j-1,kw-1)+5.*u(i,j-1,kw))+xlb(j,1
     6  ,1)*(u(i,j,kw-1)+5.*u(i,j,kw))/3.)*tb)+(t(i+1,j,k-1)+t(i,j,k-1))
     7  *((xlb(j,1,1)*(u(i,j,kw)+5.*u(i,j,kw-1))/3.+(u(i,j-1,kw)+5.*u(i,
     8  j-1,kw-1))*xlb(j,2,1))*td+(xrb(j,1,1)*(u(i,j-1,kw)+5.*u(i,j-1,kw
     9  -1))/3.+(u(i,j,kw)+5.*u(i,j,kw-1))*xrb(j,2,1))*tc)+(t(i+1,j-1,k-
     $  1)+t(i,j-1,k-1))*(u(i,j,kw)+5.*(u(i,j-1,kw)+u(i,j,kw-1))+25.*u(i
     $  ,j-1,kw-1))*(td/9.)+(t(i+1,j+1,k-1)+t(i,j+1,k-1))*(u(i,j-1,kw)+5
     $  .*(u(i,j,kw)+u(i,j-1,kw-1))+25.*u(i,j,kw-1))*(tc/9.)+(t(i+1,j-1,
     $  k+1)+t(i,j-1,k+1))*(u(i,j,kw-1)+5.*(u(i,j-1,kw-1)+u(i,j,kw))+25.
     $  *u(i,j-1,kw))*(tb/9.)+(t(i+1,j+1,k+1)+t(i,j+1,k+1))*(u(i,j-1,kw-
     $  1)+5.*(u(i,j,kw-1)+u(i,j-1,kw))+25.*u(i,j,kw))*(ta/9.)
        ftus  = ttus*(ftus+diftus)
        diftus         =4.*dxdyc(1)/d3(i,1)*((t(i+1,j,k)-t(i,j,k))*(ybb
     1  (m,4,1)+ytb(m,4,1))*(xrb(j,4,1)+xlb(j,4,1))+((t(i+1,j-1,k)-t(i,j
     2  -1,k))/xlb(j,3,1)+(t(i+1,j+1,k)-t(i,j+1,k))/xrb(j,3,1))*(ybb(m,4
     3  ,1)+ytb(m,4,1))+((t(i+1,j,k+1)-t(i,j,k+1))/ytb(m,3,1)+(t(i+1,j,k
     4  -1)-t(i,j,k-1))/ybb(m,3,1))*(xlb(j,4,1)+xrb(j,4,1))+(t(i+1,j-1,k
     5  -1)-t(i,j-1,k-1))*te+(t(i+1,j+1,k-1)-t(i,j+1,k-1))*tf+(t(i+1,j-1
     6  ,k+1)-t(i,j-1,k+1))*tg+(t(i+1,j+1,k+1)-t(i,j+1,k+1))*th)
c            if (k .ne. izstart-1) then
c            if (k .ne. izstop+1) then
              dtensr(i,j,k) =  ftus - fprev
              advctd(i,j,k-izstart+1) = -(ftus-fprev)+diftus-difprev
c             write(80+myid,*) "in 240 loop",i,j,index,advctd(i,j,index)
c            endif
c            endif
  220     continue
  239   continue 
          index = index+1
  240   continue
#ifdef MPI
        f1time = MPI_WTIME()
        time240 = time240 + f1time -b1time
#endif

c       *********************  Y-Direction ********************
        l=2
        ttus = 0.5*dxdyc(l)
#ifdef MPI
        b1time = MPI_WTIME()
#endif
        index = 0
        do 270 k=izstart,izstop
          m = k+ioffset
c DaN---------------------------------------------------------------DaN
          kw = k-izstart+1
          if (bottom_layer) kw = k-izstart+2
c DaN---------------------------------------------------------------DaN
          do 269 i=2,nx
            ta=1d0/(ytb(m,3,2)*xrb(i,3,2))
            tb=1d0/(xlb(i,3,2)*ytb(m,3,2))
            tc=1d0/(xrb(i,3,2)*ybb(m,3,2))
            td=1d0/(xlb(i,3,2)*ybb(m,3,2))
            te=1d0/(xlb(i,3,2)*ybb(m,3,2))
            tf=1d0/(ybb(m,3,2)*xrb(i,3,2))
            tg=1d0/(xlb(i,3,2)*ytb(m,3,2))
            th=1d0/(ytb(m,3,2)*xrb(i,3,2))
c           *** BOUNDARY CONDITIONS (Y)
            if (mbd.eq.0) then
c              *** periodic b.c.: calculate ftus,diftus for j=ny
               j=ny
        ftus         =(t(i,j+1,k)+t(i,j,k))*(((v(i,j,kw-1)*xrb(i,1,2)+3.
     1  *v(i-1,j,kw-1)*xrb(i,2,2))*ytb(m,1,2)+ytb(m,2,2)*3.*(v(i,j,kw)*x
     2  rb(i,1,2)+3.*v(i-1,j,kw)*xrb(i,2,2)))*ta+(ytb(m,1,2)*(3.*v(i,j,k
     3  w-1)*xlb(i,2,2)+v(i-1,j,kw-1)*xlb(i,1,2))+3.*ytb(m,2,2)*(3.*v(i,
     4  j,kw)*xlb(i,2,2)+v(i-1,j,kw)*xlb(i,1,2)))*tb+(3.*ybb(m,2,2)*(v(i
     5  ,j,kw-1)*xrb(i,1,2)+3.*v(i-1,j,kw-1)*xrb(i,2,2))+ybb(m,1,2)*(v(i
     6  ,j,kw)*xrb(i,1,2)+3.*v(i-1,j,kw)*xrb(i,2,2)))*tc+(3.*ybb(m,2,2)*
     7  (3.*v(i,j,kw-1)*xlb(i,2,2)+v(i-1,j,kw-1)*xlb(i,1,2))+ybb(m,1,2)*
     8  (3.*v(i,j,kw)*xlb(i,2,2)+v(i-1,j,kw)*xlb(i,1,2)))*td)+(t(i+1,j+1
     9  ,k)+t(i+1,j,k))*((ybb(m,1,2)*(v(i-1,j,kw)+5.*v(i,j,kw))/3.+(5.*v
     $  (i,j,kw-1)+v(i-1,j,kw-1))*ybb(m,2,2))*td+(ytb(m,1,2)*(5.*v(i,j,k
     $  w-1)+v(i-1,j,kw-1))/3.+ytb(m,2,2)*(5.*v(i,j,kw)+v(i-1,j,kw)))*tb
     $  )
        diftus        =(t(i-1,j+1,k)+t(i-1,j,k))*(((v(i,j,kw-1)+5.*v(i-1
     1  ,j,kw-1))*ybb(m,2,2)+(v(i,j,kw)+5.*v(i-1,j,kw))*ybb(m,1,2)/3.)*t
     2  c+(ytb(m,1,2)*(v(i,j,kw-1)+5.*v(i-1,j,kw-1))/3.+(v(i,j,kw)+5.*v(
     3  i-1,j,kw))*ytb(m,2,2))*ta)+(t(i,j+1,k+1)+t(i,j,k+1))*((xrb(i,1,2
     4  )*(v(i,j,kw-1)+5.*v(i,j,kw))/3.+(v(i-1,j,kw-1)+5.*v(i-1,j,kw))*x
     5  rb(i,2,2))*ta+(xlb(i,2,2)*(v(i,j,kw-1)+5.*v(i,j,kw))+xlb(i,1,2)*
     6  (v(i-1,j,kw-1)+5.*v(i-1,j,kw))/3.)*tb)+(t(i,j+1,k-1)+t(i,j,k-1))
     7  *((xlb(i,1,2)*(v(i-1,j,kw)+5.*v(i-1,j,kw-1))/3.+(v(i,j,kw)+5.*v(
     8  i,j,kw-1))*xlb(i,2,2))*td+(xrb(i,1,2)*(v(i,j,kw)+5.*v(i,j,kw-1))
     9  /3.+(v(i-1,j,kw)+5.*v(i-1,j,kw-1))*xrb(i,2,2))*tc)+(t(i+1,j+1,k-
     $  1)+t(i+1,j,k-1))*(v(i-1,j,kw)+5.*(v(i,j,kw)+v(i-1,j,kw-1))+25.*v
     $  (i,j,kw-1))*(td/9.)+(t(i-1,j+1,k-1)+t(i-1,j,k-1))*(v(i,j,kw)+5.*
     $  (v(i-1,j,kw)+v(i,j,kw-1))+25.*v(i-1,j,kw-1))*(tc/9.)+(t(i+1,j+1,
     $  k+1)+t(i+1,j,k+1))*(v(i-1,j,kw-1)+5.*(v(i,j,kw-1)+v(i-1,j,kw))+
     $  25.*v(i,j,kw))*(tb/9.)+(t(i-1,j+1,k+1)+t(i-1,j,k+1))*(v(i,j,kw-1
     $  )+5.*(v(i-1,j,kw-1)+v(i,j,kw))+25.*v(i-1,j,kw))*(ta/9.)
        ftus  = ttus*(ftus+diftus)
        diftus         =4.*dxdyc(2)/d3(j,2)*((t(i,j+1,k)-t(i,j,k))*(ybb
     1  (k,4,2)+ytb(m,4,2))*(xrb(i,4,2)+xlb(i,4,2))+((t(i+1,j+1,k)-t(i+1
     2  ,j,k))/xlb(i,3,2)+(t(i-1,j+1,k)-t(i-1,j,k))/xrb(i,3,2))*(ybb(m,4
     3  ,2)+ytb(m,4,2))+((t(i,j+1,k+1)-t(i,j,k+1))/ytb(m,3,2)+(t(i,j+1,k
     4  -1)-t(i,j,k-1))/ybb(m,3,2))*(xlb(i,4,2)+xrb(i,4,2))+(t(i+1,j+1,k
     5  -1)-t(i+1,j,k-1))*te+(t(i-1,j+1,k-1)-t(i-1,j,k-1))*tf+(t(i+1,j+1
     6  ,k+1)-t(i+1,j,k+1))*tg+(t(i-1,j+1,k+1)-t(i-1,j,k+1))*th)
            else
              diftus=0
              ftus=0
            endif
c           *** Now the real work (for y)
            do 250 j=2,ny
              difprev = diftus
                fprev =   ftus
        ftus         =(t(i,j+1,k)+t(i,j,k))*(((v(i,j,kw-1)*xrb(i,1,2)+3.
     1  *v(i-1,j,kw-1)*xrb(i,2,2))*ytb(m,1,2)+ytb(m,2,2)*3.*(v(i,j,kw)*x
     2  rb(i,1,2)+3.*v(i-1,j,kw)*xrb(i,2,2)))*ta+(ytb(m,1,2)*(3.*v(i,j,k
     3  w-1)*xlb(i,2,2)+v(i-1,j,kw-1)*xlb(i,1,2))+3.*ytb(m,2,2)*(3.*v(i,
     4  j,kw)*xlb(i,2,2)+v(i-1,j,kw)*xlb(i,1,2)))*tb+(3.*ybb(m,2,2)*(v(i
     5  ,j,kw-1)*xrb(i,1,2)+3.*v(i-1,j,kw-1)*xrb(i,2,2))+ybb(m,1,2)*(v(i
     6  ,j,kw)*xrb(i,1,2)+3.*v(i-1,j,kw)*xrb(i,2,2)))*tc+(3.*ybb(m,2,2)*
     7  (3.*v(i,j,kw-1)*xlb(i,2,2)+v(i-1,j,kw-1)*xlb(i,1,2))+ybb(m,1,2)*
     8  (3.*v(i,j,kw)*xlb(i,2,2)+v(i-1,j,kw)*xlb(i,1,2)))*td)+(t(i+1,j+1
     9  ,k)+t(i+1,j,k))*((ybb(m,1,2)*(v(i-1,j,kw)+5.*v(i,j,kw))/3.+(5.*v
     $  (i,j,kw-1)+v(i-1,j,kw-1))*ybb(m,2,2))*td+(ytb(m,1,2)*(5.*v(i,j,k
     $  w-1)+v(i-1,j,kw-1))/3.+ytb(m,2,2)*(5.*v(i,j,kw)+v(i-1,j,kw)))*tb
     $  )
        diftus        =(t(i-1,j+1,k)+t(i-1,j,k))*(((v(i,j,kw-1)+5.*v(i-1
     1  ,j,kw-1))*ybb(m,2,2)+(v(i,j,kw)+5.*v(i-1,j,kw))*ybb(m,1,2)/3.)*t
     2  c+(ytb(m,1,2)*(v(i,j,kw-1)+5.*v(i-1,j,kw-1))/3.+(v(i,j,kw)+5.*v(
     3  i-1,j,kw))*ytb(m,2,2))*ta)+(t(i,j+1,k+1)+t(i,j,k+1))*((xrb(i,1,2
     4  )*(v(i,j,kw-1)+5.*v(i,j,kw))/3.+(v(i-1,j,kw-1)+5.*v(i-1,j,kw))*x
     5  rb(i,2,2))*ta+(xlb(i,2,2)*(v(i,j,kw-1)+5.*v(i,j,kw))+xlb(i,1,2)*
     6  (v(i-1,j,kw-1)+5.*v(i-1,j,kw))/3.)*tb)+(t(i,j+1,k-1)+t(i,j,k-1))
     7  *((xlb(i,1,2)*(v(i-1,j,kw)+5.*v(i-1,j,kw-1))/3.+(v(i,j,kw)+5.*v(
     8  i,j,kw-1))*xlb(i,2,2))*td+(xrb(i,1,2)*(v(i,j,kw)+5.*v(i,j,kw-1))
     9  /3.+(v(i-1,j,kw)+5.*v(i-1,j,kw-1))*xrb(i,2,2))*tc)+(t(i+1,j+1,k-
     $  1)+t(i+1,j,k-1))*(v(i-1,j,kw)+5.*(v(i,j,kw)+v(i-1,j,kw-1))+25.*v
     $  (i,j,kw-1))*(td/9.)+(t(i-1,j+1,k-1)+t(i-1,j,k-1))*(v(i,j,kw)+5.*
     $  (v(i-1,j,kw)+v(i,j,kw-1))+25.*v(i-1,j,kw-1))*(tc/9.)+(t(i+1,j+1,
     $  k+1)+t(i+1,j,k+1))*(v(i-1,j,kw-1)+5.*(v(i,j,kw-1)+v(i-1,j,kw))+
     $  25.*v(i,j,kw))*(tb/9.)+(t(i-1,j+1,k+1)+t(i-1,j,k+1))*(v(i,j,kw-1
     $  )+5.*(v(i-1,j,kw-1)+v(i,j,kw))+25.*v(i-1,j,kw))*(ta/9.)
        ftus  = ttus*(ftus+diftus)
        diftus         =4.*dxdyc(2)/d3(j,2)*((t(i,j+1,k)-t(i,j,k))*(ybb
     1  (k,4,2)+ytb(m,4,2))*(xrb(i,4,2)+xlb(i,4,2))+((t(i+1,j+1,k)-t(i+1
     2  ,j,k))/xlb(i,3,2)+(t(i-1,j+1,k)-t(i-1,j,k))/xrb(i,3,2))*(ybb(m,4
     3  ,2)+ytb(m,4,2))+((t(i,j+1,k+1)-t(i,j,k+1))/ytb(m,3,2)+(t(i,j+1,k
     4  -1)-t(i,j,k-1))/ybb(m,3,2))*(xlb(i,4,2)+xrb(i,4,2))+(t(i+1,j+1,k
     5  -1)-t(i+1,j,k-1))*te+(t(i-1,j+1,k-1)-t(i-1,j,k-1))*tf+(t(i+1,j+1
     6  ,k+1)-t(i+1,j,k+1))*tg+(t(i-1,j+1,k+1)-t(i-1,j,k+1))*th)
c            if (k .ne. izstart-1) then
c            if (k .ne. izstop+1) then
              dtensr(i,j,k) =  dtensr(i,j,k) + ftus-fprev
              advctd(i,j,k-izstart+1) = advctd(i,j,k-izstart+1)
     &           -(ftus-fprev)+diftus-difprev
c            write(80+myid,*) "in 270 loop",i,j,index,advctd(i,j,index),
c    &         ftus,fprev,diftus,difprev
c            endif
c            endif
250       continue
269     continue
        index = index + 1
270     continue
#ifdef MPI
        f1time = MPI_WTIME()
        time270 = time270 + f1time-b1time
#endif

c       ******************** Z ********************
     
        l=3
        ttus = 0.5*dxdyc(l)
c       *** Initialize znu for Nusselt number calculation

        do 410 k=1,nz
410        znu(k) = 0.

#ifdef MPI
        b1time=MPI_WTIME()
#endif
        do 300 i=2,nx
          do 300 j=2,ny
            t1=1d0/(ytb(j,3,3)*xrb(i,3,3))
            t2=1d0/(xlb(i,3,3)*ytb(j,3,3))
            t3=1d0/(xrb(i,3,3)*ybb(j,3,3))
            t4=1d0/(xlb(i,3,3)*ybb(j,3,3))
            te=1d0/(xlb(i,3,3)*ybb(j,3,3))
            tf=1d0/(ybb(j,3,3)*xrb(i,3,3))
            tg=1d0/(xlb(i,3,3)*ytb(j,3,3))
            th=1d0/(ytb(j,3,3)*xrb(i,3,3))
c           ***** BOUNDARY CONDITION (Z)
        if (bottom_layer) then
            ftus=0
        diftus         =4.*dxdyc(3)/d3(1,3)*((t(i,j,2)-t(i,j,1))*(ybb(j,
     1  4,3)+ytb(j,4,3))*(xrb(i,4,3)+xlb(i,4,3))+((t(i-1,j,2)-t(i-1,j,1)
     2  )/xlb(i,3,3)+(t(i+1,j,2)-t(i+1,j,1))/xrb(i,3,3))*(ybb(j,4,3)+ytb
     3  (j,4,3))+((t(i,j+1,2)-t(i,j+1,1))/ytb(j,3,3)+(t(i,j-1,2)-t(i,j-1
     4  ,1))/ybb(j,3,3))*(xlb(i,4,3)+xrb(i,4,3))+(t(i-1,j-1,2)-t(i-1,j-1
     5  ,1))/(xlb(i,3,3)*ybb(j,3,3))+(t(i+1,j-1,2)-t(i+1,j-1,1))/(ybb(j,
     6  3,3)*xrb(i,3,3))+(t(i-1,j+1,2)-t(i-1,j+1,1))/(xlb(i,3,3)*ytb(j,3
     7  ,3))+(t(i+1,j+1,2)-t(i+1,j+1,1))/(ytb(j,3,3)*xrb(i,3,3)))
            znu(1) = znu(1) - diftus
        else
c         *** First do the underlying ghost layer to get difprev/fprev!
          k=2
          m=k+ioffset
c        ftus = 0 
c DaN...instead of above line, sdk's version has...

        ftus =(t(i,j,k+1)+t(i,j,k))*(((w(i-1,j-1,k)*xrb(i,1,3)+
     1  3.*w(i,j-1,k)*xrb(i,2,3))*ytb(j,1,3)+ytb(j,2,3)*3.*(w(i-1,j,k)
     2  *xrb(i,1,3)+3.*w(i,j,k)*xrb(i,2,3)))*t1+(ytb(j,1,3)*(3.*w(i-1,j-
     3  1,k)*xlb(i,2,3)+w(i,j-1,k)*xlb(i,1,3))+3.*ytb(j,2,3)*(3.*w(i-1,j
     4  ,k)*xlb(i,2,3)+w(i,j,k)*xlb(i,1,3)))*t2+(3.*ybb(j,2,3)*(w(i-1,j-
     5  1,k)*xrb(i,1,3)+3.*w(i,j-1,k)*xrb(i,2,3))+ybb(j,1,3)*(w(i-1,j,k)
     6  *xrb(i,1,3)+3.*w(i,j,k)*xrb(i,2,3)))*t3+(3.*ybb(j,2,3)*(3.*w(i-1
     7  ,j-1,k)*xlb(i,2,3)+w(i,j-1,k)*xlb(i,1,3))+ybb(j,1,3)*(3.*w(i-1,j
     8  ,k)*xlb(i,2,3)+w(i,j,k)*xlb(i,1,3)))*t4)+(t(i-1,j,k+1)+t(i-1,j,k
     9  ))*((ybb(j,1,3)*(w(i,j,k)+5.*w(i-1,j,k))/3.+(5.*w(i-1,j-1,k)+w(i
     $  ,j-1,k))*ybb(j,2,3))*t4+(ytb(j,1,3)*(5.*w(i-1,j-1,k)+w(i,j-1,k))
     $  /3.+ytb(j,2,3)*(5.*w(i-1,j,k)+w(i,j,k)))*t2)
        diftus =(t(i+1,j,k+1)+t(i+1,j,k))*(((w(i-1,j-1,k)+5.*w(i
     1  ,j-1,k))*ybb(j,2,3)+(w(i-1,j,k)+5.*w(i,j,k))*ybb(j,1,3)/3.)*t3+
     2  (ytb(j,1,3)*(w(i-1,j-1,k)+5.*w(i,j-1,k))/3.+(w(i-1,j,k)+5.*w(i,j
     3  ,k))*ytb(j,2,3))*t1)+(t(i,j+1,k+1)+t(i,j+1,k))*((xrb(i,1,3)*(w(i
     4  -1,j-1,k)+5.*w(i-1,j,k))/3.+(w(i,j-1,k)+5.*w(i,j,k))*xrb(i,2,3))
     5  *t1+(xlb(i,2,3)*(w(i-1,j-1,k)+5.*w(i-1,j,k))+xlb(i,1,3)*(w(i,j-1
     6  ,k)+5.*w(i,j,k))/3.)*t2)+(t(i,j-1,k+1)+t(i,j-1,k))*((xlb(i,1,3)*
     7  (w(i,j,k)+5.*w(i,j-1,k))/3.+(w(i-1,j,k)+5.*w(i-1,j-1,k))*xlb(i,2
     8  ,3))*t4+(xrb(i,1,3)*(w(i-1,j,k)+5.*w(i-1,j-1,k))/3.+(w(i,j,k)+5.
     9  *w(i,j-1,k))*xrb(i,2,3))*t3)+(t(i-1,j-1,k+1)+t(i-1,j-1,k))*(w(i
     $  ,j,k)+5.*(w(i-1,j,k)+w(i,j-1,k))+25.*w(i-1,j-1,k))*(t4/9.)+(t(i+
     $  1,j-1,k+1)+t(i+1,j-1,k))*(w(i-1,j,k)+5.*(w(i,j,k)+w(i-1,j-1,k))+
     $  25.*w(i,j-1,k))*(t3/9.)+(t(i-1,j+1,k+1)+t(i-1,j+1,k))*(w(i,j-1,k
     $  )+5.*(w(i-1,j-1,k)+w(i,j,k))+25.*w(i-1,j,k))*(t2/9.)+(t(i+1,j+1
     $  ,k+1)+t(i+1,j+1,k))*(w(i-1,j-1,k)+5.*(w(i,j-1,k)+w(i-1,j,k))+25.
     $  *w(i,j,k))*(t1/9.)
        ftus = ttus*(ftus +diftus )

c ...wonder if it'll work now
        diftus =4.*dxdyc(3)/d3(m,3)*((t(i,j,k+1)-t(i,j,k))*(ybb
     1  (j,4,3)+ytb(j,4,3))*(xrb(i,4,3)+xlb(i,4,3))+((t(i-1,j,k+1)-t(i-1
     2  ,j,k))/xlb(i,3,3)+(t(i+1,j,k+1)-t(i+1,j,k))/xrb(i,3,3))*(ybb(j,4
     3  ,3)+ytb(j,4,3))+((t(i,j+1,k+1)-t(i,j+1,k))/ytb(j,3,3)+(t(i,j-1,k
     4  +1)-t(i,j-1,k))/ybb(j,3,3))*(xlb(i,4,3)+xrb(i,4,3))+(t(i-1,j-1,k
     5  +1)-t(i-1,j-1,k))*te+(t(i+1,j-1,k+1)-t(i+1,j-1,k))*tf+(t(i-1,j+1
     6  ,k+1)-t(i-1,j+1,k))*tg+(t(i+1,j+1,k+1)-t(i+1,j+1,k))*th)
        endif

c       *** Main loop
            index = 0
            do 280 k=izstart,izstop
c DaN---------------------------------------------------------------DaN
              kw = k-izstart+1
              if (bottom_layer) kw = k-izstart+2             
c DaN---------------------------------------------------------------DaN
               m=k+ioffset
               difprev = diftus
                 fprev =   ftus
        ftus          =(t(i,j,k+1)+t(i,j,k))*(((w(i-1,j-1,kw)*xrb(i,1,3)
     1  +3.*w(i,j-1,kw)*xrb(i,2,3))*ytb(j,1,3)+ytb(j,2,3)*3.*(w(i-1,j,kw
     2  )*xrb(i,1,3)+3.*w(i,j,kw)*xrb(i,2,3)))*t1+(ytb(j,1,3)*(3.*w(i-1,
     3  j-1,kw)*xlb(i,2,3)+w(i,j-1,kw)*xlb(i,1,3))+3.*ytb(j,2,3)*(3.*w(i
     4  -1,j,kw)*xlb(i,2,3)+w(i,j,kw)*xlb(i,1,3)))*t2+(3.*ybb(j,2,3)*(w(
     5  i-1,j-1,kw)*xrb(i,1,3)+3.*w(i,j-1,kw)*xrb(i,2,3))+ybb(j,1,3)*(w(
     6  i-1,j,kw)*xrb(i,1,3)+3.*w(i,j,kw)*xrb(i,2,3)))*t3+(3.*ybb(j,2,3)
     7  *(3.*w(i-1,j-1,kw)*xlb(i,2,3)+w(i,j-1,kw)*xlb(i,1,3))+ybb(j,1,3)
     8  *(3.*w(i-1,j,kw)*xlb(i,2,3)+w(i,j,kw)*xlb(i,1,3)))*t4)+(t(i-1,j,
     9  k+1)+t(i-1,j,k))*((ybb(j,1,3)*(w(i,j,kw)+5.*w(i-1,j,kw))/3.+(5.*
     $  w(i-1,j-1,kw)+w(i,j-1,kw))*ybb(j,2,3))*t4+(ytb(j,1,3)*(5.*w(i-1,
     $  j-1,kw)+w(i,j-1,kw))/3.+ytb(j,2,3)*(5.*w(i-1,j,kw)+w(i,j,kw)))*t
     $  2)
        diftus         =(t(i+1,j,k+1)+t(i+1,j,k))*(((w(i-1,j-1,kw)+5.*w(
     1  i,j-1,kw))*ybb(j,2,3)+(w(i-1,j,kw)+5.*w(i,j,kw))*ybb(j,1,3)/3.)*
     3  t3+(ytb(j,1,3)*(w(i-1,j-1,kw)+5.*w(i,j-1,kw))/3.+(w(i-1,j,kw)+5.
     3  *w(i,j,kw))*ytb(j,2,3))*t1)+(t(i,j+1,k+1)+t(i,j+1,k))*((xrb(i,1,
     4  3)*(w(i-1,j-1,kw)+5.*w(i-1,j,kw))/3.+(w(i,j-1,kw)+5.*w(i,j,kw))*
     5  xrb(i,2,3))*t1+(xlb(i,2,3)*(w(i-1,j-1,kw)+5.*w(i-1,j,kw))+xlb(i,
     6 1,3)*(w(i,j-1,kw)+5.*w(i,j,kw))/3.)*t2)+(t(i,j-1,k+1)+t(i,j-1,k))
     7  *((xlb(i,1,3)*(w(i,j,kw)+5.*w(i,j-1,kw))/3.+(w(i-1,j,kw)+5.*w(i-
     8  1,j-1,kw))*xlb(i,2,3))*t4+(xrb(i,1,3)*(w(i-1,j,kw)+5.*w(i-1,j-1,
     9  kw))/3.+(w(i,j,kw)+5.*w(i,j-1,kw))*xrb(i,2,3))*t3)+(t(i-1,j-1,k+
     $  1)+t(i-1,j-1,k))*(w(i,j,kw)+5.*(w(i-1,j,kw)+w(i,j-1,kw))+25.*w(i
     $  -1,j-1,kw))*(t4/9.)+(t(i+1,j-1,k+1)+t(i+1,j-1,k))*(w(i-1,j,kw)+5
     $  .*(w(i,j,kw)+w(i-1,j-1,kw))+25.*w(i,j-1,kw))*(t3/9.)+(t(i-1,j+1,
     $  k+1)+t(i-1,j+1,k))*(w(i,j-1,kw)+5.*(w(i-1,j-1,kw)+w(i,j,kw))+25.
     $  *w(i-1,j,kw))*(t2/9.)+(t(i+1,j+1,k+1)+t(i+1,j+1,k))*(w(i-1,j-1,k
     $  w)+5.*(w(i,j-1,kw)+w(i-1,j,kw))+25.*w(i,j,kw))*(t1/9.)
        ftus = ttus*(ftus+diftus)
        diftus =4.*dxdyc(3)/d3(m,3)*((t(i,j,k+1)-t(i,j,k))*(ybb
     1  (j,4,3)+ytb(j,4,3))*(xrb(i,4,3)+xlb(i,4,3))+((t(i-1,j,k+1)-t(i-1
     2  ,j,k))/xlb(i,3,3)+(t(i+1,j,k+1)-t(i+1,j,k))/xrb(i,3,3))*(ybb(j,4
     3  ,3)+ytb(j,4,3))+((t(i,j+1,k+1)-t(i,j+1,k))/ytb(j,3,3)+(t(i,j-1,k
     4  +1)-t(i,j-1,k))/ybb(j,3,3))*(xlb(i,4,3)+xrb(i,4,3))+(t(i-1,j-1,k
     5  +1)-t(i-1,j-1,k))*te+(t(i+1,j-1,k+1)-t(i+1,j-1,k))*tf+(t(i-1,j+1
     6  ,k+1)-t(i-1,j+1,k))*tg+(t(i+1,j+1,k+1)-t(i+1,j+1,k))*th)
c         if (k .ne. izstart-1) then
c         if (k .ne. izstop+1) then
             dtensr(i,j,k) =  dtensr(i,j,k) + ftus-fprev
             advctd(i,j,k-izstart+1) = advctd(i,j,k-izstart+1)
     &          -(ftus-fprev)+diftus-difprev
c            write(80+myid,*) "in 280 loop",i,j,index,advctd(i,j,index)
             znu(m) = znu(m) + ftus - diftus
c         endif
c         endif
c             *** znu is a global array
         index = index + 1
  280    continue
  300   continue
#ifdef MPI
        f1time=MPI_WTIME()
        time300 = time300 + f1time-b1time
#endif 

c       *** Now scale DTENSR with the cell volumes
        dxdy=dx*dy
        do k=izstart,izstop
          kglob=k+ioffset
          voli=0.5*dt/(dxdy*(z(kglob)-z(kglob-1)))
          do j=2,ny
            do i=2,nx
               dtensr(i,j,k)= voli*dtensr(i,j,k)
            enddo
          enddo
        enddo
      endif
 
c       *** Calculate Nusselt number
        tznu=0.
#ifdef MPI
        call mpilayer(nz,numprocs,myid,izstart,izstop)
        do k=1,izstart-1
          znu(k) = 0.0d0
        enddo
        do k=izstop+1,nz
          znu(k) = 0.0d0
        enddo
        call MPI_ALLREDUCE(znu,zwork,nz,MPI_DOUBLE_PRECISION,MPI_SUM,
     &        MPI_COMM_WORLD,ierr)
        do k=1,nz
          znu(k) = zwork(k)
        enddo
#endif
        do k=1,nz
          m=k+ioffset
          ff=1.
          if (m.eq.1.or.m.eq.nz) ff=0.5
          znu(k)=znu(k)/(xl*yl)
          tznu=tznu+znu(k)*ff
        enddo
        tznu=tznu/float(nz-1)

#ifdef MEMORY_CHECK
c      write(80+myid,*)"about to call mmverify in flux"
c       call mmverify()
#endif

      return
      end
