c $Log:   /pvcs.config/mc3dvv/src/funcvis.F_a  $
c
c   Rev 1.0   Mon Jul 14 14:21:24 1997   pvcs
c 
c *************************************************************
c *   FUNCVIS
c *
c *   Function to define viscosity in nodal point (i,j,k) with
c *   coordinates (x,y,z), temperature T and second invariant secinv
c *   If IVISTYPE>0, viscosity is dependent on the z-coordinate
c *   only, and this function is called for k=1,nz only.
c *   If IVISTYPE<-1, viscosity is dependent on all three coordinates
c *   and may be used to define viscosity that is dependent on
c *   temperature, pressure, strain rate and composition.
c *   To include composition dependent viscosity, information on the
c *   composition should be retrieved from the common blocks describing
c *   it. 
c *
c *   For any modifications: it is strongly recommended that any 
c *   parameters describing the rheological law are passed through
c *   user defined commons, and that the variables in MC3D commons
c *   (such as those in a3.h) are NOT used.
c * 
c *   PvK 950811
c *************************************************************
      function funcvis(ivistype,i,j,k,x,y,z,T,secinv) 
      use module_precision
      use module_giparam

      integer ivistype,i,j,k
      real*8 x,y,z,T,secinv
c     *** Some constants for Busse et al. rheology
      parameter(rr=2.9957323,qq=225d0/rr-1d0/4d0*rr,gg=15d0/rr-1d0/2d0) 

      funcvis = 1d0

      if (ivistype.eq.-20) then
c        *** Temperature dependent viscosity for Busse et al. benchmark
c        *** Garrett Ito, 5/95\
          v1=qq/(T+gg)
          v0=qq/(1d0/2d0+gg)
          funcvis = exp(v1-v0)
      endif

      return
      end
 
 
