c $Log:   /pvcs.config/mc3dvv/src/geoid.F_a  $
c
c   Rev 1.1   Fri Jul 30 15:01:34 1999   lowman
c  Adopted changes made to routine by Scott King.
c  Added control of output to different sequential output 
c  files for each time a geoid or topography calculation
c  is specified.
c
c   Rev 1.0   Mon Jul 14 14:21:26 1997   pvcs
c 
c *************************************************************
c *   GEOID
c *************************************************************
      subroutine geoid(ijob)
c
c     compute geoid given temperature (density)
c     and vertical normal stress (topography)
c
c     ijob = 0 total geoid
c          = 1 neglect topography contribution
c          = 2 only calculate contribution from topography
c          = 3 calculate dimensional surface topography
c
      use module_precision
      use module_param
      use module_giparam
      use module_gia3
      use module_a3
      use module_ftcoef
      use module_cgeoid
      use module_a11
      use module_a12
c     common /a12/ lpu, lpv, lpw, lpt13, lpt23, lpt33, lpft, nxu, nxv,
c    1 nxw, nxt13, nxt23, nxt33, nxft, nxr, mpu, mpv, mpw, mpt13, mpt23,
c    2 mpt33, mpft, nyu, nyv, nyw, nyt13, nyt23, nyt33, nyft, nyr
      use module_a20

      character*25 fileroot

      real l
      common /cgdtopo/icall_geoid,icall_topo

c these values are for the Blankenbach Benchmark

      if(j3.eq.1)then

      if(ijob.eq.0)then
      if(icycle.eq.1)icall_geoid=0
      icall_geoid=icall_geoid+1
      lunum=37
      iformat=2
      fileroot='geoid_'
      itail=0
      call namefile1(icall_geoid,lunum,iformat,fileroot,itail,fname)
      endif

      if(ijob.eq.3)then
      if(icycle.eq.1)icall_topo=0
      icall_topo=icall_topo+1
      lunum=37
      iformat=2
      fileroot='topo_'
      itail=0
      call namefile1(icall_topo,lunum,iformat,fileroot,itail,fname)
      endif

      endif

      pi=pimach(dumm)
      dz=one/float(nz-1)
c
c     z coordinate where geoid is calculated 
      zref = one
c
c     conversion factor to topography to surface density.
c
      trhos =-1.0
      trhocmb = -1.0

c     transform temperature field to cell corners
c NB: This multiplies ft by rat (we don't want that)
c
      call ttoft
c
c    convert ft to nondimensional density perturbation 
c    (alpha deltaT density/rat) - happens to be same as topography units
c
      idim = 3
      ijo = 18
      call units(idim,ijo,rat,rah,ft,isx,jsx,ksx)
c
c     convert to dimensional density
c
      idim = 3
      ijo = 3
      call units (idim,ijo,rat,rah,ft,isx,jsx,ksx)
c
c     Fourier transform density field
c   
      call ft2nrm (1,lpft,nxft,mpft,nyft,nz,isx,jsx,ft)
c     
      do 10 j=1,nyr
      do 10 i=1,nxr
      gd(i,j)=zero
  10  continue

      if((ijob .eq. 0) .or. (ijob .eq. 1))then
c
c     calculate contribution due to internal density field
c     in transform domain
c
       do 40 k = 1, nz-1
         do 30 j = 1,nyft
           do 20 i = 1,nxft
              l = fl(i,j)
              if(l .ne. 0.0)then
                gd(i,j)=gd(i,j)-
     &         0.5*( (ft(i,j,k  )/l)*exp(-l*(zref-(k-1)*dz))+
     &               (ft(i,j,k+1)/l)*exp(-l*(zref-(k  )*dz)) )*dz
              endif
c             print *,i,j,k,l,ft(i,j,k),gd(i,j)
   20      continue
   30    continue
   40  continue
      
      endif
      if((ijob .eq. 0) .or. (ijob .eq. 2))then
c
c     calculate contribution due to surface topography (vertical normal
c     stress)
c
c     fill tauz array with fourier components of vertical stress

      ipttype = 0
      ipttotau  = 6 
      call pttotau(ipttype,ipttotau)
c
c     convert to topography
c
      idim = 3
      ijo = 18
      call units(idim,ijo,rat,rah,tauz,isx,jsx,ksx)
c
c     convert to surface density anomalies (sigma)
c
      idim = 3
      ijo = 3
      call units(idim,ijo,rat,rah,tauz,isx,jsx,ksx)
c
      ztop = 1.0
      zbot = 0.0
      zdist = zref-ztop
      zdist1 = zref-bot
      do 60 j = 1,nyft
         do 50 i = 1,nxft
           l = fl(i,j)
           if(l .ne. 0.0)then
              gd(i,j)=gd(i,j)+trhos*(tauz(i,j,nz)/l)*exp(-l*zdist)
     &                     -trhocmb*(tauz(i,j,1 )/l)*exp(-l*zdist1)
           endif
c           print *,i,j,l,l2,tauz(i,j,nz),gd(i,j)
   50    continue
   60 continue
      endif
c      
c     convert to dimensional geoid
c
      idim = 2
      ijo = 20
      call units (idim,ijo,rat,rah,gd,isx,jsx,ksx)
c
c     if we want dimensional suface topography, do this
c
      if (ijob .eq. 3) then
c
c     fill tauz array with fourier components of vertical stress
c
        ipttype = 0
        ipttotau  = 6
        call pttotau(ipttype,ipttotau)
c
c     convert to topography
c
        idim = 3
        ijo = 18
        call units(idim,ijo,rat,rah,tauz,isx,jsx,ksx)
c
        do 160 j = 1,nyft
         do 150 i = 1,nxft
            gd(i,j)=trhos*tauz(i,j,nz)
  150    continue
  160   continue
      end if

c      write(38,100)1,icycle,time
c      write(38,100)ijob
c      write(38,110)((gd(i,j),i=1,nx),j=1,j3)
c
c     transform to space domain
      call ft2nrm (-1,lpft,nxft,mpft,nyft,1,isx,jsx,gd)

c     Fix last point for periodic boundary condition

       if((lbd.eq.0).and.(mbd.eq.0))then
        do 180 j =1,j3
         gd(nx,j)=gd(1,j3)
180     continue
       endif

c
c      write(37,100)1,icycle,time
c      write(37,100)ijob
      if(j3.eq.1)then
      x=-dx
      do 200 i=1,nx
      x=x+dx
  200 write(37,120) x,gd(i,1)
      endif
  100 format(2i8,e14.7)
  110 format(13000(e15.7,/))
  120 format(1x,e14.7,1x,e14.7)
      close(37)

      return
      end
