c $Log:   /pvcs.config/mc3dvv/src/heatflo.F_a  $
c
c   Rev 1.1   Fri Jul 30 15:15:58 1999   lowman
c  Added Nu.h and a4.h includes
c
c   Rev 1.0   Mon Jul 14 14:21:26 1997   pvcs
c 
c *************************************************************
c *   HEATFLO
c *
c *   Integrate the heat equation for one timestep.
c *
c *   MPI implementation: Scott King & Peter van Keken
c *   Los Alamos/Michigan/Purdue Summer 1999.
c *
c *   Each processor contains one layer of the grid. For each
c *   processor the corresponding grid points are calculated in
c *   MPILAYER. Each processor (except bottom layer) needs
c *   two ghostpoints at the bottom (see flux_mpi.F) and one
c *   ghostpoint at the top.
c *
c *   Note changes in allocmem_mpi.F/flux_mpi.F/ctensr_mpi.F
c *
c *   PvK 6/26/99 Changed calls to ctensr/flux for MPI implementation
c *************************************************************
      subroutine heatflo
      use module_precision
c
c     calculate heat flow using weighted donor explicit scheme
c
c     rearranged to make 2-D version only need a single layer
c     in the y direction. The value of parameter value jnx
c     can now be set to 1 for 2-D problems.
c     Also moved invarient calculations outside
c     of loops in 3-D version. Changes made to heatflo, flux,
c     ctensr.                                    C.G. 8/26/89
c
      use module_param
      use module_giparam
      use module_a1
      use module_gia1
      use module_a2
      use module_gia3
      use module_a3
      use module_nu
      use module_a77
      use module_slvelu
      use module_mytime
      use module_depthdep !+DaN
      common /a4/ vsij(700)
#ifdef MPI
      include 'mpif.h'
      logical bottom_layer, top_layer

      call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )
      call MPI_COMM_SIZE( MPI_COMM_WORLD, numprocs, ierr )
      call mpilayer(nz,numprocs,myid,izstart,izstop)
      if (izstart .lt. 2) izstart = 2
c      btime = MPI_WTIME()
#else
      myid    = 0
      izstart = 2
      izstop  = nz
#endif

#ifdef MEMORY_CHECK
c       call mmverify()
#endif
 
      energk = zero
      dx2    = dx**2
      dy2    = dy**2
      dx2i   = one/dx2
      dy2i   = one/dy2
      dxi    = one/dx
      dyi    = one/dy
      dtn    = 1.e30
      dxdy   = dx*dy
      rrat   = rah/rat
c >>>>>>>>>>> PETER (BEGIN)
c     if (rah.gt.0) rrat   = 1
c >>>>>>>>>> PETER(END)
      
c *************************************************************
c *   Calculate kinetic energy, time step and vrms
c *************************************************************
      do k = izstart,izstop

c DaN---------------------------------------------------------------DaN
        kw = k-izstart+3                                             !
        if (izstart.eq.2) kw = kw-1                                  !
c DaN---------------------------------------------------------------DaN

        dzz  = z(k)-z(k-1)
        dz2i = one/dzz**2
        dzi  = one/dzz
 
        if (ny.eq.2) then
c         *** 2-D problem
          j = 1
          do i = 2,nx
c            *** calculate total kinetic energy
c            *** Note: this uses trapezoidal rule over each element
             uu1 = u(i,  1,kw  ) + u(i  ,1,kw-1)                  !DaN
             uu2 = u(i-1,1,kw  ) + u(i-1,1,kw-1)                  !DaN
             ww1 = w(i  ,1,kw  ) + w(i-1,1,kw  )                  !DaN
             ww2 = w(i  ,1,kw-1) + w(i-1,1,kw-1)                  !DaN
             uu =  quarter*(uu1+uu2)
             ww =  quarter*(ww1+ww2)

             energk = energk+half*(uu**2+ww**2)*dx*dzz
 
c            *** calculate time step size
             uu = quarter*(abs(uu1)+abs(uu2))
             ww = quarter*(abs(ww1)+abs(ww2))
             dtt = one/(1.732*(uu*dxi+ww*dzi)+two*(dx2i+dy2i+dz2i)) 
             if(dtt .lt. dtn) dtn = dtt
          enddo
 
        else
c       *** 3-D problem
          do j = 2,ny
            do i = 2,nx
              uu1= u(i  ,j  ,kw  ) + u(i  ,j-1,kw  )                
     &           + u(i  ,j  ,kw-1) + u(i  ,j-1,kw-1)                !DaN
              uu2= u(i-1,j  ,kw  ) + u(i-1,j-1,kw  )              
     &           + u(i-1,j  ,kw-1) + u(i-1,j-1,kw-1)                !DaN
              vv1= v(i  ,j  ,kw  ) + v(i-1,j  ,kw  )             
     &           + v(i  ,j  ,kw-1) + v(i-1,j  ,kw-1)                !DaN
              vv2= v(i  ,j-1,kw  ) + v(i-1,j-1,kw  )            
     &           + v(i  ,j-1,kw-1) + v(i-1,j-1,kw-1)                !DaN
              ww1= w(i  ,j  ,kw  ) + w(i-1,j  ,kw  )           
     &           + w(i  ,j-1,kw  ) + w(i-1,j-1,kw  )                !DaN
              ww2= w(i  ,j  ,kw-1) + w(i-1,j  ,kw-1)         
     &           + w(i  ,j-1,kw-1) + w(i-1,j-1,kw-1)                !DaN
              uu = eighth*(uu1+uu2)
              vv = eighth*(vv1+vv2)
              ww = eighth*(ww1+ww2)
c             *** calculate total kinetic energy
c             *** Note:  uses 3D trapezoidal rule
              energk = energk+half*(uu**2+vv**2+ww**2)*dx*dy*dzz
c             ***  calculate time step size
              uu = eighth*(abs(uu1)+abs(uu2))
              vv = eighth*(abs(vv1)+abs(vv2))
              ww = eighth*(abs(ww1)+abs(ww2))
              dtt = 1./(1.732d0*(uu*dxi+vv*dyi+ww*dzi)+
     1           2.0d0*(dx2i+dy2i+dz2i))
              if (dtt .lt. dtn) dtn = dtt
            enddo
          enddo
        endif              
      enddo

#ifdef MPI
      call MPI_ALLREDUCE(dtn,dt,1,MPI_DOUBLE_PRECISION,MPI_MIN,
     &                MPI_COMM_WORLD,ierr)
      call MPI_ALLREDUCE(energk,tmpeng,1,MPI_DOUBLE_PRECISION,MPI_SUM,
     &                MPI_COMM_WORLD,ierr)
      energk=tmpeng
      vrms   = sqrt(2.0*energk/(xl*yl))
      call MPI_BCAST(vrms,1,MPI_DOUBLE_PRECISION,0,
     &               MPI_COMM_WORLD,ierr)
      dt = 0.95*dt*curant
#else
      dt = 0.95*dtn*curant
      vrms = sqrt(2.0*energk/(xl*yl))
#endif

c     *** Main computational part.
c     *** Calculate dtensr and advctd
      call flux(t,dtensr,itx,jtx)
      if (dwate.ne.0.0) call ctensr(t,dtensr,itx,jtx)

c     *** Correct for overdiffusion

#ifdef MPI
c     *** Put all the information together in tn and copy to t
c     *** Work only over internal points.
         izstart = izstart -2
         izstop = izstop + 2
      if (izstart .lt. 2) izstart = 2
      if (izstop .gt. nz) izstop = nz
#endif

c DaN---------------------------------------------------------------DaN
      if (icycle.eq.1)   call iheat
c   Returns vratio(nz) (= 1 if iheatype=0) to multiply H (rrat) in 
c   heat eqn below

c      if (myid.eq.0) then
c        open(unit=62, file='k_Vs_vratio')
c        do i=1,knx-2
c          write(62,27)i,vratio(i)
c27        format(i3,2x,f24.22)
c        enddo
c      close(62)
c      endif
c DaN---------------------------------------------------------------DaN
      if (ny.eq.2) then
c        *** 2-D problem
         do k = izstart,izstop
            dz = z(k)-z(k-1)
            voli = one/(dz*dx)
            do i = 2,nx
               t(i,1,k) =  t(i,1,k)+
     v            dt*((advctd(i,1,k-izstart+1)+
     v            dtensr(i,1,k)*dwate)*voli+rrat*vratio(k))
c    v            dt*((advctd(i,1,k)+
            enddo
         enddo
      else
c        *** 3-D problem
         do k = izstart,izstop
            dz = z(k)-z(k-1)
            voli = one/(dz*dxdy)
            do j = 2,ny
              do i = 2,nx
               t(i,j,k) =  t(i,j,k)+
     v            dt*((advctd(i,j,k-izstart+1)+
     v            dtensr(i,j,k)*dwate)*voli+rrat*vratio(k))
c    v            dt*((advctd(i,j,k)+
c              write(myid+80,1012) i,j,k,advctd(i,j,k-izstart+1),
c    &            dtensr(i,j,k),t(i,j,k)
 1012          format(3i4,3f12.5)
             enddo
           enddo
        enddo
      endif
      if (lbd.eq.0) then
        do k = izstart,izstop
           do j = j2,j3
              t(1,j,k) = t(nx,j,k)
              t(nx1,j,k) = t(2,j,k)
           enddo
        enddo
      else
        do k = izstart,izstop
           do j = j2,j3
              t(1,j,k) = t(2,j,k)
              t(nx1,j,k) = t(nx,j,k)
           enddo
        enddo
      endif

      if (ny.ne.2) then
        if (mbd.eq.0) then
          do k = izstart,izstop
            do  i = 1,nx1
               t(i,1,k) = t(i,ny,k)
               t(i,ny1,k) = t(i,2,k)
            enddo
          enddo
        else
          do k = izstart,izstop
            do i = 1,nx1
              t(i,1,k) = t(i,2,k)
              t(i,ny1,k) = t(i,ny,k)
            enddo
          enddo
        endif
      endif
#ifdef MPI
      ftime = MPI_WTIME()
      heattime = heattime + ftime-btime
      call mpiexchange(nx1,j4,nz,t,itx,jtx)
      btime = MPI_WTIME()
#endif

c *************************************************************
c *   From here on all processors have the same full T array 
c *************************************************************

c     *** Boundary conditions for insulated top/boundary
      if (ibot.eq.1) then
        do j = 1,j4
           do i = 1,nx1
              t(i,j,1) = t(i,j,2)
           enddo
        enddo
      endif
      if (itop.eq.1) then
        do j = 1,j4
           do i = 1,nx1
              t(i,j,nz1) = t(i,j,nz1-1)
           enddo
        enddo
      endif

c     *** calculate avg. vertical temperature
      tbar = zero
      div = float((nx-1)*(ny-1))
#ifdef MPI
      call mpilayer(nz,numprocs,myid,izstart,izstop)
      if (izstop .eq. nz) izstop=nz1
#else
      izstart=1
      izstop=nz1
#endif
      do k=1,nz1
          tavrg(k) = 0.0d0
      enddo
      do k = izstart,izstop
         tavrg(k) = zero
         do j = j2,j3
            do i = 2,nx
               tavrg(k) = tavrg(k)+t(i,j,k)
            enddo
         enddo
         tavrg(k) = tavrg(k)/div
         tbar = tbar+tavrg(k)
      enddo
      tbar = tbar/float(nz1)

#ifdef MPI
        call MPI_ALLREDUCE(tavrg,zwork,nz1,MPI_DOUBLE_PRECISION,MPI_SUM,
     &        MPI_COMM_WORLD,ierr)
        do k=1,nz1
          tavrg(k) = zwork(k)
        enddo
#endif
 
c     *** divide nusselt number by temp. grad. across layer
      delti = one/(tavrg(1)-tavrg(nz1))
      if (delti.ne.1.) then
        do k = 1,nz
           znu(k) = znu(k)*delti
        enddo
        tznu = tznu*delti
      endif

#ifdef MEMORY_CHECK
c       call mmverify()
#endif
      ftime = MPI_WTIME()
      heattime = heattime + ftime-btime
      return
      end

