c $Log:   /pvcs.config/mc3dvv/src/mig_ridge.F_a  $
c
c   Rev 1.1   Fri Jul 16 13:48:50 1999   lowman
cAdded param and a3 includes
c
c   Rev 1.0   Mon Jul 14 14:21:44 1997   pvcs
c 
c *************************************************************
c *   MIGRATE_RIDGE
c *************************************************************
      subroutine migrate_ridge
     1   (iflag,idim,jdim,vp1,vp2,xridge,vridge,wplate,vsurf)
C************BEGIN PROLOGUE********************************
C   PROGRAM NAME  migrate_ridge
C   DATE WRITTEN   930312   (YYMMDD)
C   REVISION DATE  xxxxxx   (YYMMDD)
C   AUTHOR  Carl W. Gable   (Los Alamos National Laboratory)
C   PURPOSE  Return kinematic surface velocity with migrating ridge.
C   DESCRIPTION/COMMENTS
c       inx   = first dimension of vsurf array
c       jnx   = second dimension of vsurf array
c       nx    = number of nodes in x direction (.le. inx)
c       ny    = number of nodes in y direction (.le. jnx)
c       time  = integration time (assume calculation begins at t=0)
c       vp1   = velocity of firsts plate
c       vp2   = velocity of second plate
c       xridge= initial position of ridge (as a fraction of xl; e.g.
c               xridge = 0.5 is the center regardless of the box
c               dimensions. Ridge migrates according
c               to xr = xridge + vridge*time
c       vridge= ridge velocity; can be input or calculated.
c               If vridge .ge. 1.e20, vridge is calculated
c               so that vp1*p1l(length of plate one) - vp2*p2l = 0
c               This condition insures there is no net horizontal 
c               velocity imposed by the plate. This is important
c               with perodic b.c. but does not matter with reflecting
c               b.c.
c       wplate= weight factor that control how abrupt velocity changes
c               from one plate to the other. Values around 200
c               give sharp plate boundaries. e.g
c               wplate = 20 nx=128, transition zone = 35 nodes
c               wplate = 60 nx=128, transition zone = 12 nodes
c               wplate =100 nx=128, transition zone =  7 nodes
c               wplate =140 nx=128, transition zone =  5 nodes
c               wplate =200 nx=128, transition zone =  3 nodes
c               wplate =360 nx=128, transition zone =  3 nodes
c               
C   MODIFICATIONS
C**********END PROLOGUE************************************
      use module_precision
      use module_gia3
      use module_param
      use module_a3
      dimension vsurf(idim,jdim,2)

      if(vridge .lt. 1.e20)then
         xr = xridge + vridge*time
         xtrench = zero
      else
        write(6,*) 'Stopped in mig_ridge'
        stop
      endif

        write(6,*)idim,jdim,nx,ny,time,vp1,vp2,xridge,vridge,wplate
        write(6,*)xr
        if(ny .eq. 2)then
           nj = 1
        else
           nj = ny
        endif

      dx = 1./(nx-1)
      do j = 1, nj
      x  = 0.0
      do i = 1, nx
        vsurf(i,j,1) = 
     1    vp1 + ((vp2 - vp1)/2.)*(1.0+tanh(wplate*(x - xr)))
c        write(6,*)wplate,i,x,vsurf(i,j,1)
        x = x + dx
        vsurf(i,j,2) = 0.0
      enddo
      enddo
      iridge = 999
      itrench = 999
      do i = 1, nx-1
        if((vsurf(i,1,1) .lt. zero).and.(vsurf(i+1,1,1).gt. zero))then
          iridge = i
        endif
        if((vsurf(i,1,1) .gt. zero).and.(vsurf(i+1,1,1).lt. zero))then
          itrench = i
        endif
      enddo
      write (31,80) 
     1  icycle,time,xr,xtrench,vridge,float(iridge),float(itrench)
   80 format (i8,5e14.7)

      return
      end
