      subroutine mpicomm_spwk(iswitch,nx,ny,nz,ublock,ii,jj,kk)
c
c  iswitch = 1: 
c    Move data which is stored in x,y blocks in array ublock from
c    Solvlu parallelism to a z-oriented (plane layers) storage
c    so that the ffts and the energy equation and be distributed in
c    the vertical direction.
c
c  iswitch = -1: 
c    Move data which is stored in z-oriented (plane layers) storage
c    in array ublock to x,y blocks needed for Solvlu parallelism.
c
c  This elliminates the need for global communication
c
c  Written S.D.K 6/3/99
c
c  Tested: S.D.K.  Sun cluster (mpich) - works on test problems!!
c
      use module_precision
      use module_param     !+DaN
      use module_spwork    !+DaN
      use module_mpibuf
#ifdef MPI
      include 'mpif.h'
#endif
c
      dimension ublock(ii,jj,*)     
      real*4 endtime, tarray(2)
c     real*4 btime
#ifdef MEMORY_CHECK
c       call mmverify()
#endif
#ifdef MPI
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )
      call MPI_COMM_SIZE( MPI_COMM_WORLD, numprocs, ierr )
      if (numprocs .eq. 1) return
c      btime = etime(tarray)
c     do 1 i=1,ibuff
c       sbuff(ibuff) = 0.0d0
c       rbuff(ibuff) = 0.0d0
c1    continue
c
c This section handles communication from
c the f.d. (z-chunk) data to spectial (x,y-chunk) 
c data pattern
c
      if (iswitch .eq. 1) then 
c
c loop over all subdomains (globally) to find the 
c biggest chunk, while doing this store the
c displacements (may not be needed)
c
        isize = 0
        knxx_max = 0
        kny_max = 0
        knz_max = 0
        do ichunk = 0, numprocs-1
          call mpiblock(nx,ny,numprocs,ichunk,
     &         ixstart,ixstop,iystart,iystop)
          ixstop = ixstop + 1
          call mpilayer(nz,numprocs,ichunk,izstart,izstop)
          izstart = izstart - 2
          if (izstart .lt. 1) izstart = 1
          izstop = izstop + 2
          if (izstop .gt. nz) izstop = nz
          knxx = ixstop - ixstart + 1
          kny = iystop - iystart + 1
          knz = izstop - izstart + 1
          knxx_max = max(knxx_max,knxx)
          kny_max = max(kny_max,kny)
          knz_max = max(knz_max,knz)
        enddo
        isize = max(isize,knxx_max*kny_max*knz_max)
c
c loop over all the chunks in the 'z' discritization
c for the processors x,y chunk and pack these into
c the send_buffer with uniform spacing
c
         do ichunk = 0, numprocs-1
           call mpiblock(nx,ny,numprocs,myid,
     &                   ixstart,ixstop,iystart,iystop)
           ixstop = ixstop + 1
           call mpilayer(nz,numprocs,ichunk,izstart,izstop)
           izstart = izstart - 2
           if (izstart .lt. 1) izstart = 1
           izstop = izstop + 2
           if (izstop .gt. nz) izstop = nz
           knxx = ixstop - ixstart + 1
           kny = iystop - iystart + 1
           knz = izstop - izstart + 1
           do k=1,knz
             do j=1,kny
               do i=1,knxx
                 sbuff(ichunk*isize + knxx_max*(j-1+kny_max*(k-1))+i) 
     &             = ublock(i+ixstart-1,j+iystart-1,k+izstart-1)
               enddo
             enddo
           enddo
         enddo
         if (isize*numprocs .gt. ibuff) then
         write(6,*) "WARNING: POSSIBLE PROBLEM WITH MPI BUFFERS"
         write(6,*) "mpicomm 1", isize*numprocs, ibuff,myid
         endif
         call MPI_ALLTOALL(
     &         sbuff,isize,MPI_DOUBLE_PRECISION,
     &         rbuff,isize,MPI_DOUBLE_PRECISION,
     &         MPI_COMM_WORLD,ierr)
c
c here the x,y chunk size is determined by where the
c data came from while the z chunk size is determined
c by the local processor 
c
         do ichunk = 0, numprocs-1
           call mpiblock(nx,ny,numprocs,ichunk,
     &                   ixstart,ixstop,iystart,iystop)
           ixstop = ixstop + 1
           call mpilayer(nz,numprocs,myid,izstart,izstop)
           izstart = izstart - 2
           if (izstart .lt. 1) izstart = 1
           izstop = izstop + 2
           if (izstop .gt. nz) izstop = nz
           knxx = ixstop - ixstart + 1
           kny = iystop - iystart + 1
           knz = izstop - izstart + 1
           do k=1,knz
             do j=1,kny
               do i=1,knxx
                 ublock(i+ixstart-1,j+iystart-1,k+izstart-1) =
     &           rbuff(ichunk*isize + knxx_max*(j-1+kny_max*(k-1))+i) 
               enddo
             enddo
           enddo
         enddo
c
c end of block to layer communication
c
      elseif (iswitch .eq. -1) then
c
c This section handles communication from
c the spectial (x,y-chunk) data to f.d. (z-chunk) 
c data pattern
c
c loop over all subdomains (globally) to find the 
c biggest chunk, while doing this store the
c displacements (may not be needed)
c
        isize = 0
        knxx_max = 0
        kny_max = 0
        knz_max = 0
        do 400 ichunk = 0, numprocs-1
          call mpiblock(nx,ny,numprocs,ichunk,
     &         ixstart,ixstop,iystart,iystop)
          call mpilayer(nz,numprocs,ichunk,izstart,izstop)
          knxx = ixstop - ixstart + 1
          kny = iystop - iystart + 1
          knz = izstop - izstart + 1
          knxx_max = max(knxx_max,knxx)
          kny_max = max(kny_max,kny)
          knz_max = max(knz_max,knz)
 400    continue
        isize = max(isize,knxx_max*kny_max*knz_max)
c
c loop over all the chunks in the 'z' discritization
c for the processors x,y chunk and pack these into
c the send_buffer with uniform spacing
c
        do 450 ichunk = 0, numprocs-1
          call mpiblock(nx,ny,numprocs,ichunk,
     &         ixstart,ixstop,iystart,iystop)
          call mpilayer(nz,numprocs,myid,izstart,izstop)
          knxx = ixstop - ixstart + 1
          kny = iystop - iystart + 1
          knz = izstop - izstart + 1
          do 425 k=1,knz
          do 425 j=1,kny
          do 425 i=1,knxx
           sbuff(ichunk*isize + knxx_max*(j-1+kny_max*(k-1))+i) 
     &           = spwk(i+ixstart-1,j+iystart-1,k+izstart-1)    !DaN
 425      continue
 450    continue
        if (isize*numprocs .gt. ibuff) then
        write(6,*) "WARNING: POSSIBLE PROBLEM WITH MPI BUFFERS"
        write(6,*) "mpicomm -1", isize*numprocs, ibuff,myid
        endif
        call MPI_ALLTOALL(
     &        sbuff,isize,MPI_DOUBLE_PRECISION,
     &        rbuff,isize,MPI_DOUBLE_PRECISION,
     &        MPI_COMM_WORLD,ierr)
c
c here the x,y chunk size is determined by where the
c data came from while the z chunk size is determined
c by the local processor 
c
        do 550 ichunk = 0, numprocs-1
          call mpiblock(nx,ny,numprocs,myid,
     &         ixstart,ixstop,iystart,iystop)
          call mpilayer(nz,numprocs,ichunk,izstart,izstop)
          knxx = ixstop - ixstart + 1
          kny = iystop - iystart + 1
          knz = izstop - izstart + 1
          do 525 k=1,knz
          do 525 j=1,kny
          do 525 i=1,knxx
           spwk(i+ixstart-1,j+iystart-1,k+izstart-1) =
     &     rbuff(ichunk*isize + knxx_max*(j-1+kny_max*(k-1))+i) 
 525      continue
 550    continue
      else
c
c warn user they set an incorrect option
c
        if (myid .eq. 0) then
          write(6,*) "Error in calling mpicomm:"
          write(6,*) "iswitch must be 1 or -1"
          write(6,*) "aborting program (not very gracefully)"
          stop
        endif
      endif
c
c update timing and return
c
c      endtime = etime(tarray)
c      commtime = commtime + endtime-btime
c
#endif

#ifdef MEMORY_CHECK
c       call mmverify()
#endif
      return
      end
