c $Log:   /pvcs.config/mc3dvv/src/pttouv2.F_a  $
c
c   Rev 1.0   Mon Jul 14 14:22:08 1997   pvcs
c 
c *************************************************************
c *   PTTOUV2
c *************************************************************
      subroutine pttouv2(ipttype,ijob)
c***begin prologue  pttouv
c***date written   880101   (yymmdd)
c***revision date  930826   (yymmdd)
c***category no.  ????
c***keywords
c             type=???? precision
c***author   Carl W. Gable   (Los Alamos National Laboratory)
c***purpose
c***description
c
c      transform from poloidal/toroidal field
c      to velocity in u and v.  Superimpose
c      plate velocity. Final output is still in spectral
c      space. Must still transform with ft2nrm.f
c
c
c  ipttype  =  0   combine toroidal and poloidal fields
c
c  ipttype  =  1   set toroidal part of solution to zero
c      
c  ipttype  =  2   set poloidal part of solution to zero
c
c  ijob = 0 superimpose plate and body force stress fields (when present)
c
c  ijob = 1 set plate field to 0, only transform density driven field
c
c  ijob = 2 set density driven field to 0, only transform plate field
c
c***references (none)
c***routines called  ????
c***modifications (yymmdd) modification
c***modifications (930826) [cwg] a20.h not needed
c***modifications (931118) [cwg] Reorder 3d periodic modes.
c***end prologue  pttouv
c  Identical to PTTOUV except does not calculate output and
c   write to tape17
c  G. Ito 2/95
c=======================================================================

      use module_precision
      use module_param
      use module_giparam
      use module_a2
      use module_gia3
      use module_a3
      use module_a22
      use module_a11
      use module_a13
c      use module_a20
      use module_a24
      use module_a25
      use module_a26
      use module_a30
      use module_viscom
      use module_pttouv
      use module_a12
      real l, l2
      common /start1/ restrt, initlt, istart
      common /a8/ l2, l, vps, vts

c     common /a12/ lpu, lpv, lpw, lpt13, lpt23, lpt33, lpft, nxu, nxv,
c    1 nxw, nxt13, nxt23, nxt33, nxft, nxr, mpu, mpv, mpw, mpt13, mpt23,
c    2 mpt33, mpft, nyu, nyv, nyw, nyt13, nyt23, nyt33, nyft, nyr
c
#ifdef MPI
      include 'mpif.h'
       call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )
       call MPI_COMM_SIZE( MPI_COMM_WORLD, numprocs, ierr )
#else
       myid=0
       numprocs=1
#endif
c MPI ALERT: changed loop indices
#ifdef MPI
      call mpiblock(nx,j3,numprocs,myid,
     &             ixstar,ixstop,iystar,iystop)
      if (ixstop .gt. nxr) ixstop =  nxr
      if (iystop .gt. nyr) iystop =  nyr
#else
      iystar=1
      iystop=nyr
      ixstar=1
      ixstop=nxr
#endif
c      sfact scales the kinetic energy to eliminate
c      dependence on box dimension or number of terms
c      in the Fourier expansion.
      sfact = (xl*yl)/float(nxr*nyr)
      if(ipttype .eq. 0)then
        pfac = one
        tfac = one
      elseif(ipttype .eq. 1)then
        pfac = one
        tfac = zero
      elseif(ipttype .eq. 2)then
        pfac = zero
        tfac = one
      endif

c************For plates**************************
      if ((ibct.eq.2).and.(isurfv.lt.0)) then  
c
        if(ijob .eq. 2)then
          ppfac = zero
          ttfac = zero
        else
          ppfac = pfac
          ttfac = tfac
        endif
c
        do 30 ll = 1,nz
        do 30 j = iystar,iystop
        do 30 i = ixstar,ixstop
   30   u(i,j,ll) = vp(i-ixstar+1,j-iystar+1,ll)*ppfac
        do 32 ll = 1,nz
        do 32 j = iystar,iystop
        do 32 i = ixstar,ixstop
   32   v(i,j,ll) = vtf(i-ixstar+1,j-iystar+1,ll)*ttfac
        do 280 ll = 1,nz
        do 280 j = iystar,iystop
        do 280 i = iystar,iystop
  280   w(i,j,ll) = v3(i-ixstar+1,j-iystar+1,ll)*ppfac
c
c cwg 931117
c The following change of variables reorders the modes of the periodic
c  Fourier spectrum. It only has to be done for the 3D modes. The
c  2D (cos(0)) modes along the edge are already correct due the
c  call to chvar(ft) in flow.
c
c
c  Temp    U      V      W
c
c  CxCy   SxCy   CxSy   CxCy
c
c  SxSy   CxSy   SxCy   SxSy
c
c  CxSy   SxSy   CxCy   CxSy
c
c  SxCy   CxCy   SxSy   SxCy
c
      if ((lbd.eq.0).and.(mbd.eq.0))then
        do ll = 1,nz
        do  j = 2,nyr,2
        do  i = 2,nxr,2
         ut11=u(i,j,ll)
         ut12=u(i,j+1,ll)
         ut21=u(i+1,j,ll)
         ut22=u(i+1,j+1,ll)

         vt11=v(i,j,ll)
         vt12=v(i,j+1,ll)
         vt21=v(i+1,j,ll)
         vt22=v(i+1,j+1,ll)

         u(i,j,ll)     = ut12
         u(i,j+1,ll)   = ut11
         u(i+1,j,ll)   = ut22
         u(i+1,j+1,ll) = ut21

         v(i,j,ll)     = vt21
         v(i,j+1,ll)   = vt22
         v(i+1,j,ll)   = vt11
         v(i+1,j+1,ll) = vt12

         w(i,j,ll)     = -w(i,j,ll)     
         w(i,j+1,ll)   = -w(i,j+1,ll)
         w(i+1,j,ll)   = -w(i+1,j,ll)
         w(i+1,j+1,ll) = -w(i+1,j+1,ll)
        enddo
        enddo
        enddo
      endif
c cwg 931117
 
      if(ijob .ne. 1)then 

        if(ifssd .eq. 1)then
           rewind(50)
        endif
 
        ifield = 0
        do 45 np = 1,nplate
c cwg        do 45 ixy = 1,2
        do 45 ixy = 1,npvel
        if(ixy .eq. 1)then
           im1 = 1
        else
           im1 = 0
        endif
c cwg 931123        ifield = 2*np-im1
        ifield = ifield + 1
c
c       read from ssd here
c       with ssd irec is the record to read
c       without ssd it is the index of the superposition array
      if(ifssd .eq. 1)then
        irec = 1
        call rwssd(-2,ifield,iqx,jqx,kqx,npssd,
     1                 vppl,v3pl,vtfpl,tautpl,tauppl,tau3pl)
      elseif(ifssd .eq. 0)then
        irec = ifield
      endif
      write(6,*)'pttouv',
     1  nplate,npvel,im1, np,ixy,ifield,irec,vpl(ixy,np)
c
        do 40 ll = 1,nz
        do 40 j = 1,nyr
        do 40 i = 1,nxr
  40    u(i,j,ll) = u(i,j,ll)+vpl(ixy,np)*vppl(i,j,ll,irec)*pfac
        do 42 ll = 1,nz
        do 42 j = 1,nyr
        do 42 i = 1,nxr
  42    v(i,j,ll) = v(i,j,ll)+vpl(ixy,np)*vtfpl(i,j,ll,irec)*tfac
        do 290 ll = 1,nz
        do 290 j = 1,nyr
        do 290 i = 1,nxr
  290   w(i,j,ll) = w(i,j,ll)+vpl(ixy,np)*v3pl(i,j,ll,irec)*pfac
  45    continue

      endif

        ichvar = 1  
        call chvar (w,inx,nxr,lbd,jnx,nyr,mbd,nz,ichvar)
c
c      calculate power spectrum for output
c >>>>>> Commented out PvK 951224: Deal with this in USEROUT
c     if (time+dt.ge.pltime) then
c     write (17,80)icycle,time
c     do 48 ll = 1,nz
c       akep(ll)   = zero
c       aket(ll)   = zero
c       akephv(ll) = zero
c       do 47 j = 1,nyr
c       do 47 i = 1,nxr
c       t1 = ((fli(i,j)*u(i,j,ll))**2+
c    1        (flj(i,j)*u(i,j,ll))**2)
c       akep(ll)   = t1+akep(ll)
c       akephv(ll) = t1+(w(i,j,ll))**2+akephv(ll)
c       aket(ll)   = ((fli(i,j)*v(i,j,ll))**2+
c    1                (flj(i,j)*v(i,j,ll))**2)+aket(ll)
c  47  continue
c       akep(ll) = sqrt(akep(ll))*sfact
c       aket(ll) = sqrt(aket(ll))*sfact
c       akephv(ll) = sqrt(akephv(ll))*sfact
c       if (akephv(ll).eq.0.) then
c         ake(ll) = 9.99e99
c       else
c         ake(ll) = aket(ll)/akephv(ll)
c       endif
c       write (17,90) z(ll),aket(ll),akep(ll),akephv(ll),ake(ll)
c  48 continue
c     endif
c >>>>>>
  
        do 50 ll = 1,nz
        do 50 j = 1,nyr
        do 50 i = 1,nxr
        t1 = u(i,j,ll)
        t2 = v(i,j,ll)
        u(i,j,ll) = fli(i,j)*t1+flj(i,j)*t2
  50    v(i,j,ll) = flj(i,j)*t1-fli(i,j)*t2
      else			
c
c***********not dynamic plate motion ************
c
c >>>>>> Commented out PvK 951224: Deal with this in USEROUT
c        if (time+dt.ge.pltime) then
c          write (17,80)icycle,time
c        endif
        do 58 ll = 1,nz
        do 58 j = iystar,iystop
        do 58 i = ixstar,ixstop
   58   w(i,j,ll) = v3(i-ixstar+1,j-iystar+1,ll)
        call chvar (w,inx,nxr,lbd,jnx,nyr,mbd,nz,1)
        do 60 ll = 1,nz
        do 55 j = iystar,iystop
        do 55 i = ixstar,ixstop
        u(i,j,ll) = pfac*fli(i,j)*vp(i-ixstar+1,j-iystar+1,ll)+
     1              tfac*flj(i,j)*vtf(i-ixstar+1,j-iystar+1,ll)
        v(i,j,ll) = pfac*flj(i,j)*vp(i-ixstar+1,j-iystar+1,ll)-
     1              tfac*fli(i,j)*vtf(i-ixstar+1,j-iystar+1,ll)
   55   continue
   60   continue
      endif
c
c      With periodic b.c. one must account for the
c      l = 0 term in the fourier expansion. This
c      term is the average value of the velocity
c      and only occurs when the imposed
c      surface velocity has a non-zero
c      average.
c
c   this is not right for layered viscosity
c
      if (ibct.eq.3) then
        if (((lbd.eq.0).and.(mbd.eq.0)).and.((fftvx(1,1).ne.0).or.(fftvy
     1  (1,1).ne.0))) then
          if (ivistype.ne.0) then
            write (6,100)
      stop
          endif
          if (ibcb.eq.1) then
            do 65 ll = 1,nz
            u(1,1,ll) = fftvx(1,1)
            v(1,1,ll) = fftvy(1,1)
   65       continue
          elseif (ibcb.eq.2) then
            gradu = fftvx(1,1)/float(nz-1)
            gradv = fftvy(1,1)/float(nz-1)
            do 70 ll = 1,nz
            fll = float(ll-1)
            u(1,1,ll) = fll*gradu
            v(1,1,ll) = fll*gradv
   70       continue
          endif
        endif
      endif
      return
c
   80 format (i8,e14.7,/,
     1"PT coupling fac.",/,
     2 "             z",12x,"aket         akep        akephv     ake")
   90 format (11x,5(e10.3,2x))
  100 format 
     1 ("input error in pttouv: variable viscosity not set for plates")
      end
