c $Log:   /pvcs.config/mc3dvv/src/smantle.F_a  $
c
c   Rev 1.2   Fri Jul 30 15:35:10 1999   lowman
c  Added template2 argument to surfgeo call
c
c   Rev 1.0   Mon Jul 14 14:22:18 1997   pvcs
c 
c *************************************************************
c *   SMANTLE
c *
c *   Subroutine MANTLE adapted for Picard iteration 
c *   towards steady state
c *
c *   Just as an experiment: use SLES from Argonne's PETSc package
c *
c *   See http://www.mcs.anl.gov/petsc/petsc.html 
c *************************************************************
      subroutine smantle
      use module_precision
      use module_param
      use module_giparam
      use module_a1
      use module_gia1
      use module_ac1
      use module_a2
      use module_gia3
      use module_a3
      use module_a22
      use module_nu
c      common /xxx/ tavrg(514), znu(514), tbar
      use module_ftcoef
      use module_spwork
      use module_work1d
c      common /work2d/ wk3d(inx,knx)
c      common /work3d/ wk3d(inx,jnx,knx)
      use module_temp4
      use module_viscom
      use module_a6
      use module_a12
c      common /a12/ lpu, lpv, lpw, lpt13, lpt23, lpt33, lpft, nxu, nxv,
c     1 nxw, nxt13, nxt23, nxt33, nxft, nxr, mpu, mpv, mpw, mpt13, mpt23,
c     2 mpt33, mpft, nyu, nyv, nyw, nyt13, nyt23, nyt33, nyft, nyr
      use module_a30
      use module_a77
      use module_timeint
      use module_routput

      common /start1/ restrt, initlt, istart
      external res,djack,dpsol
      common /ijc/ ijcycle

      data nrestrt / 1 /
      data tcrash, tinc / 3600., 3600. /

c     write(6,*) '*********************************************'
c     write(6,*) '*  PERROR(smantle): Due to modifications'
c     write(6,*) '*  in the SLES(Petsc) package, this routine'
c     write(6,*) '*  yields incorrect results. Please contact'
c     write(6,*) '*  Peter van Keken (keken@umich.edu) for an'
c     write(6,*) '*  update on the status of this error.'
c     write(6,*) '*********************************************'
c     stop
      

c *************************************************************
c *   INITIALIZE
c *       Define initial temperature field
c *       Define plate geometry
c *       Check input for consistency
c *       Calculate instantaneous velocity
c *************************************************************
      istart=0
      ijcycle = 0
c     *** Initialize temperature and such
      call init
c     *** Determine plate geometry
      call surfgeo (template,iplgeo,template2)
c     *** Check for inconsistent input parameters
      call incheck(imx, jmx, kmx, nxyz, isx, jsx, ksx, nbdd,
     1  ijksx, iqx, jqx, kqx, ijkqx, inx, jnx, knx, mxmy, 
     2  ijknx, ijkmx,  ijksf, inxsf, jnxsf, knxsf, imxsf, jmxsf,
     3  kmxsf, npla, npssd, ncabe, nabe,icx,jcx,kcx,nspecies)
      call userout('init')

      ijcycle = ijcycle

      qfs1 = calnus(norder,1)
      vrms1 = calvrms(1)
      ipicard = 0
      maxpicard = 50
      epspicard = 1d-4
      relaxpicard = 0.5
      
      write(6,'(''I(Picard) '',i5,3f15.7)') ipicard,qfs1,vrms1,1e0

c     *** Start of Picard iteration
      call gsrelax(tn,t,ijktx,zero)
100   continue
        ipicard = ipicard + 1

c *************************************************************
c *     VELOCITY
c *************************************************************
        icycle = icycle+1
        if (isolve.eq.5) then
c          *** Define velocity using analytical expression
           call vofxyz
        else
           if (kchem.eq.0) then
             call flow
           else
              write(6,*) 'PWARN(smantle): flow_layer not yet adapted' 
              write(6,*) '                for use with DASPK'
              stop
c             call flow_layer
           endif
        endif
c       *** Average velocities to force divergence on cell faces to zero
c       *** (use advctd as a temporary work array)
c       call velprm (u,v,advctd,inx,jnx,knx)
        call userout('flow')

c *************************************************************
c *     TEMPERATURE
c *************************************************************
c       gseps = 1d-10
c       ngsmax = 10000
c       call gs2d(tn,yprime,itx,jtx,ktx,u,v,w,inx,jnx,knx,norder,
c    v         gseps,ngsmax)  
        call dosles(tn,yprime,itx,jtx,ktx,u,v,w,inx,jnx,knx,norder)
        call gsrelax(t,tn,ijktx,relaxpicard)

        qfs = calnus(norder,1)
        vrms = calvrms(1)
        dqfs = abs(qfs-qfs1)/max(abs(qfs),abs(qfs1))
        dvrms = abs(vrms-vrms1)/max(abs(vrms),abs(vrms1))
        write(6,'(''I(Picard) '',i5,3f15.7)') ipicard,qfs,vrms,
     v                   max(dqfs,dvrms)
        if (dqfs.gt.epspicard.or.dvrms.gt.epspicard) then
           if (ipicard.lt.maxpicard) then
              vrms1 = vrms
              qfs1  = qfs
              goto 100
           else
              write(6,*) 'PERROR(smantle): Picard iteration did'
              write(6,*) '   not converge within ',maxpicard, 'iters'
           endif
        endif
 
c     *** The final one: 
      call userout('end')

      return
      end

      subroutine gsrelax(t,tn,ijktx,alpha)
      use module_precision
      dimension t(ijktx),tn(ijktx)
   
      do i=1,ijktx
         t(i) = alpha*t(i) + (1d0-alpha)*tn(i)
      enddo

      return
      end
