c $Log:   /pvcs.config/mc3dvv/src/units.F_a  $
c
c   Rev 1.1   Fri Jul 30 15:48:20 1999   lowman
c  Units for topography and geoid calculations
c
c   Rev 1.0   Mon Jul 14 14:22:32 1997   pvcs
c 
c *************************************************************
c *   UNITS
c *************************************************************
      subroutine units(
     1 ijob,ivar,rat,rah,array,idim1,idim2,idim3)
c
c     convert from dimensional to nondimensional variables 
c
c     ijob = 1 non-dimensional => dimensional variable or vector
c     ijob = -1 dimensional => non-dimensional variable or vector
c
c     ijob = 2 non-dimensional => dimensional 2d array
c     ijob = -2 dimensional => non-dimensional 2d array
c
c     ijob = 3 non-dimensional => dimensional 3d array
c     ijob = -3 dimensional => non-dimensional 3d array
c
c  1    length  = 	 length scale 
c     			[m]
c  2    deltat  = 	 temperature difference across the layer 
c     			[K]
c
c  3    density = 	 reference density at zero pressure, temperature 
c     			[kg/m**3]
c 
c  4    kappa   =	 thermal diffusivity 
c     			[m**2/s]
c  5    lambda  =	 thermal conductivity 
c     			[kg m K / s**3]
c  6    gravity =	 gravitational acceleration 
c     			[m / s**2]
c  7    bigg    =	 gravitational constant 
c     			[m**3 / s**2 kg]
c  8    csubp   =	 heat capacity 
c     			[kg m**2 / s**2 K mol]
c  9    qheat   =	 internal heat generation
c 10    alpha   =	 thermal expansion coefficient 
c     			[1/K]
c   DERIVED QUANTITIES
c 11    time   =		 time    (length**2/kappa ) 
c     			[s]
c 12   viskin =		 kinematic viscosity 
c     			[m**2/s]
c 13    visdyn =		 dynamic viscosity 
c     			[kg / m s]
c 14    speed  =		 velocity ( kappa /length)
c     			[m / s] 
c 15    force  =		 force 
c     			[m kg / s]
c 16    stress =		 stress 
c     			[kg / m s]
c 17   gradt  =		 thermal gradient (deltat/length) 
c     			[K / m] 
c 18    topography =	 from stress_zz 
c     			[m]
c 19    energy   =	  energy 
c     			[kg m**2 / s**2]
c 20    geoid    =        geoid (pi G length /ggravity)
c                       [ ]
c
      use module_precision
      real * 8 kappa, length
      dimension array(idim1,idim2,idim3)
c
      data ncall / 1 /
c these values are for the Blankenbach Benchmark
c      data length,   deltat,   density,  kappa ,   gravity,  bigg, 
c     1      csubp,    alpha /
c     2      1.0e6,    1000.0,   4000.0,    1.e-6,     10.0,   6.673e-11,
c     3      1250.0,   2.5e-5 /
      data length,   deltat,   density,  kappa ,   gravity,  bigg, 
     1      csubp,    alpha /
     2      2.9e6,    2900.0,   4700.0,    1.e-6,     10.0,   6.673e-11,
     3      1080.0,   1.4e-5 /

        visdyn = alpha*gravity*density*deltat*(length**3)/(kappa*rat)

        force  = alpha*gravity*density*deltat*(length**3)/rat
        
        time   = length**2/kappa
        
        speed  = kappa/length
        
        gradt  = deltat/length
        
        stress = alpha*gravity*density*deltat*length/rat
        
        topography = alpha*deltat*length/rat

        pi=pimach(dumm)

      if(ivar .eq. 1)then
        sfac=length
      elseif(ivar .eq. 2)then
        sfac=deltat
      elseif(ivar .eq. 3)then
        sfac=density
      elseif(ivar .eq. 4)then
        sfac=kappa
      elseif(ivar .eq. 5)then
        sfac=lambda
      elseif(ivar .eq. 6)then
        sfac=gravity
      elseif(ivar .eq. 7)then
        sfac=bigg
      elseif(ivar .eq. 8)then
        sfac=csubp
      elseif(ivar .eq. 9)then
        sfac=qheat
      elseif(ivar .eq. 10)then
        sfac=alpha
      elseif(ivar .eq. 11)then
        sfac=time
      elseif(ivar .eq. 12)then
        sfac=viskin
      elseif(ivar .eq. 13)then
        sfac=visdyn
      elseif(ivar .eq. 14)then
        sfac=speed
      elseif(ivar .eq. 15)then
        sfac=force
      elseif(ivar .eq. 16)then
        sfac=stress
      elseif(ivar .eq. 17)then
        sfac=gradt
      elseif(ivar .eq. 18)then
        sfac=topography
      elseif(ivar .eq. 19)then
        sfac=energy
      elseif(ivar .eq. 20)then
        sfac=2.0*pi*bigg*length/gravity
      endif
      
      if(ijob .eq. 1)then
      do 10 i=1,idim1
        array(i,1,1)=sfac*array(i,1,1)
   10 continue
      elseif(ijob .eq. 2)then
      do 20 j=1,idim2
      do 20 i=1,idim1
        array(i,j,1)=sfac*array(i,j,1)
   20 continue
      elseif(ijob .eq. 3)then
      do 30 k=1,idim3
      do 30 j=1,idim2
      do 30 i=1,idim1
        array(i,j,k)=sfac*array(i,j,k)
   30 continue
      endif

      if(ncall .eq. 1)then
      write(6,800)length, deltat, density, kappa, gravity, bigg,
     1            csubp, alpha
  800 format(/,'Quantities use for non-dimensional variables ',/,
        
     1         'Length scale (m)                ',e14.7,/,
     2         'Temperature scale (K)           ',e14.7,/,
     3         'Reference density (kg/m**3)     ',e14.7,/,
     4         'Thermal diffusivity (m**2/s)    ',e14.7,/,
     5         'Gravity (m/s**2)                ',e14.7,/,
     6         'Big G   (m**3/s**2 kg)          ',e14.7,/,
     7         'Heat capacity (kgm**2/s**2Kmol) ',e14.7,/,
     8         'Thermal expansivity (1/K)       ',e14.7,/)
      endif
      ncall = ncall + 1
      return
      end
