c $Log:   /pvcs.config/mc3dvv/src/vflow.F_a  $
c
c   Rev 1.2   Fri Jul 30 15:52:50 1999   lowman
c  Added zwork to solvlu argument list: required for dynamic allocation
c
c   Rev 1.1   Tue Oct 06 16:57:18 1998   lowman
cGot rid of call to mmverify
cGot rid of hardwired arguments in pttouv
c
c   Rev 1.0   Mon Jul 14 14:22:36 1997   pvcs
c 
c *************************************************************
c *   VFLOW
c *
c *   this routine computes the velocity field using various
c *   methods selected by isolve
c *
c * begin prologue  flow
c * date written   890101   (yymmdd)
c * revision date  930802   (yymmdd)
c * category no.  ????
c * keywords
c * author   Carl W. Gable   (Los Alamos National Laboratory)
c * purpose Solve for velocity and stress given body forces.
c * description
c * references (none)
c * routines called bc, chvar, ft2nrm, vinitflow, inttauc, inttaup, 
c *                 netflow, plflow, pttotau, pttouv, solvlu, surfv, 
c *                 ttoft, vviscos,
c * modifications (yymmdd) modification
c * modifications (930803) [cwg] Change in arguments to solvlu.f
c * modifications (931119) [cwg] Added debugging output option (ifdebug ne 0).
c * modifications (931119) [cwg] Added debugging output option.
c * Varabiable viscosity capabilities. G. Ito
c *    Relaxation factor, relax	G. Ito 6/95
c *      If convergence fails then increase relax to 0.3
c *
c *
c * end prologue  flow
c *************************************************************
c * An Attempt at describing some of the variables (PvK)
c *  
c *    icycle    Main loop counter (controlled in mantle)
c *    itvv      Counter for iteration loop for variable viscosity.
c *              (Loop 300)
c *    
c *************************************************************
      subroutine vflow
      use module_precision
      use module_param
      use module_giparam
      use module_a1
      use module_gia1
      use module_ac1		!debugging
      use module_a2
      use module_gia3
      use module_a3
      use module_afftw
      use module_a6
      use module_a7
      use module_a9
      use module_gia9
c      common /a10/ jcycle
      use module_a11
      use module_a12
      use module_a13
      use module_a14
      use module_a20
      use module_a22
      use module_a26
      use module_a30
      use module_alam
      use module_viscom
      use module_flow
      use module_ftcoef
      use module_slvelu
      parameter(pi=3.14159265358979323844)
      real l, l2
      common /start1/ restrt, initlt, istart
c#include<a4.h>
      common /a4/ vsij(700)
      common /a8/ l2, l, vps, vts
      dimension sscalv(2), iindexv(2)
      dimension scalv(4), indexv(4), scalft

      common /peprinterror/ error,errmx,itvv

c----------------------------------------------------------
c   Set convergence value
c----------------------------------------------------------
      errmx1 = 1e-2
c     *** (note : cv1 cv2 replaced by cv!)
c     *** Convergence parameter CONV for variable viscosity iteration
      conv = cv
c     *** Convergence parameter convsol for use in SOLVLU
      convsol = 1e-8
      itoroid=0			!G.Ito
      nvisdim = ksx
      nvnudim = ksx
      nrdim   = kmx
      nzdim   = 514
      jcycle = icycle
      twopi = 2.*pi

c *************************************************************
c *   initialize flow
c ************************************************************
      if ((isolve.gt.2).and.
     v     ((icycle.eq.1).or.(restrt.ne.0.0).or.(istart.eq.0))) then  
         call vinitflow
         nnp=nxr*nyr*nz		!total number of nodes for convergence check
         open(200,file='tape200')
         vprms1=zero		!Initialize for convergence check
         v3rms1=zero
         tprms1=zero
         t3rms1=zero
         ttrms1=zero
         vtfrms1=zero
      endif

c *************************************************************
c *   calculate surface velocity induced flow
c ************************************************************
      if ((icycle.eq.1).or.(restrt.ne.0.0).or.(isurfv .eq. 500)) then
        if (ibct.eq.3.and.isurfv.gt.0) then
          do j = 1,j3
            do i = 1,nx
               vsurf(i,j,1) = zero
               vsurf(i,j,2) = zero
            enddo
          enddo
          if (isurfv.ne.0) then
             if (isurfv.eq.1) then
c *             *** get surface velocity from input table vsij
                ll = 1
20              continue
                  i1 = int(vsij(ll))
                  i2 = int(vsij(ll+1))
                  jj1 = int(vsij(ll+2))
                  jj2 = int(vsij(ll+3))
                  uxs = vsij(ll+4)
                  vys = vsij(ll+5)
                  if (i1.eq.0) go to 110
                  do j = jj1,jj2
                    do i = i1,i2
                      vsurf(i,j,1) = uxs
                      vsurf(i,j,2) = vys
                    enddo
                  enddo
                  ll = ll+6
                  if (ll+6.lt.700) then
                     go to 20
                  else
                     write(6,*) 'PWARN(vflow): vsij(ll) in endless loop'  
                  endif
             else
c               *** get surface velocity from user supplied function in
c               *** subroutine SURFV
                call surfv (vsurf,nx,j3,template,iplgeo) 
             endif

c            *** insure that imposed velocity agrees with side b.c.
c            *** Only in 3D calculations

             if (ny.ne.2) then
                if (lbd.eq.0) then
                   do i = 1,nxr+1
                      vsurf(i,nyr+1,1) = vsurf(i,1,1)
                      vsurf(i,nyr+1,2) = vsurf(i,1,2)
                   enddo
                else if ((lbd.eq.1).or.(lbd.eq.2)) then
                   do j = 1,ny
                      vsurf(1,j,1) = 0.
                      vsurf(nx,j,1) = 0.
                   enddo
                endif
                if (mbd.eq.0) then
                   do j = 1,nyr+1
                      vsurf(nxr+1,j,1) = vsurf(1,j,1)
                      vsurf(nxr+1,j,2) = vsurf(1,j,2)
                   enddo
                elseif ((mbd.eq.1).or.(mbd.eq.2)) then
                   do i = 1,nx
                      vsurf(i,1,2) = 0.
                      vsurf(i,ny,2) = 0.
                   enddo
                endif
             endif

             if (isolve.gt.2) then
c              *** calculate fft of surface velocity and calculate  vp, vt
               do j = 1,j3
                  do i = 1,nx
                     fftvx(i,j) = vsurf(i,j,1)
                  enddo
               enddo
               call ft2nrm (1,lpu,nxu,mpu,nyu,1,iqx,jqx,fftvx)
               do j = 1,j3
                  do i = 1,nx
                     fftvy(i,j) = vsurf(i,j,2)
                  enddo
               enddo
               call ft2nrm (1,lpv,nxv,mpv,nyv,1,iqx,jqx,fftvy)
               do j = 1,nyr
                  do i = 1,nxr
                     vpb(i,j) = fli(i,j)*fftvx(i,j)+flj(i,j)*fftvy(i,j)   
                     vtb(i,j) = flj(i,j)*fftvx(i,j)-fli(i,j)*fftvy(i,j)
                  enddo
               enddo
             endif
 
             if ((isurfv.ge.200) .and. (isurfv .lt. 300)) then  
c               ***  set individual modes to non-zero value
c               *** this could be changed to use namelist vsij as
c               *** input for ipt, imode, jmode, mag
                do j = 1,nyr
                   do i = 1,nxr
                      vpb(i,j) = zero
                      vtb(i,j) = zero
                   enddo
                enddo
                open (unit = 46, file='input.modes')
                read (46,*) nmodes
                do nm = 1,nmodes
                   read (46,*) ipt, imode, jmode, mag
                   if (ipt.eq.1) vpb(imode,jmode)=mag
                   if (ipt.eq.2) vtb(imode,jmode)=mag
                enddo
                close (46)
             endif

  110       continue
          endif
        endif
      endif
c *************************************************************
c *   END of calculation of surface induced flow
c *************************************************************

      if (isolve.ne.3) return

c *************************************************************
c *   Prepare calculations of flow induced by buoyancy forces
c *************************************************************
      if ((icycle.eq.1.or.restrt.ne.0).and.initlt.ne.0) then   
         do k = 1,nz
            do j = 1,j3
               do i = 1,nx
                  vp  (i,j,k) = zero
                  v3  (i,j,k) = zero
                  taup(i,j,k) = zero
                  tau3(i,j,k) = zero
                  vtf (i,j,k) = zero
                  taut(i,j,k) = zero
               enddo
            enddo
         enddo
      endif
c *   *** convert temperature at cell centers to cell corners
      call ttoft

c *************************************************************
c *   MAIN LOOP (300)
c *   solve 4 1st order ode's for primitive variables
c *************************************************************
      do 300 itvv=1,itvvmax
c        *** Initialize RMS values
         vprms=0.		
         v3rms=0.
         tprms=0.
         t3rms=0.
         vtfrms=0.
         ttrms=0.
         dvpmx=0.
         dv3mx=0.
         dtpmx=0.
         dt3mx=0.
         dvtfmx=0.
         dttmx=0.
         ilam=0
         if (itvv.eq.1) then	!Relaxation factor
            rel1 = one
            rel2 = zero
         else
            rel1 = one-relax2
            rel2 = relax2
         endif

c *************************************************************
c *      Mean field and variable vviscosity.  
c *      Layered visc. set in
c *      viniflow.  G. Ito 5/95
c *************************************************************

         if (itvv.eq.1) call vviscos
         if (icycle.gt.1.or.itvv.gt.1.or.initlt.eq.3) ilam=1
         if (ilam.eq.1) call xlambda(itvv)

c *************************************************************
c        If depletion raleigh # (rad) or melt rayleigh # (ram) are
c        set then add buoyancy forces to ft.  Must do it here
c        because need just temperature for vviscos above. (GI 5/95)
c *************************************************************
         if (itvv.eq.1) then
            if ((rad.gt.0.or.ram.gt.0).and.imelt.eq.1) then
               call chem2ft
            endif
            call ft2nrm (1,lpft,nxft,mpft,nyft,nz,isx,jsx,ft)
         endif

         call userftaccess

c        Change of variables (multiply by i=(-1)**.5

         ijob = 1
         if (itvv.eq.1) call chvar(ft,isx,nxr,lbd,jsx,nyr,mbd,nz,ijob)

         numeq = 0
c        *** give it more iterations on restart
         if ((icycle.eq.1).or.(restrt.ne.0.0).or.(istart.eq.0).or.
     2        (ivistype.lt.0)) then			
            itmax = 1000			
            niterate = 10		
         else
            itmax = 50
            niterate = 10
         endif
         m = nz
         slowc    = max(one*rat,one)
         scalv(1) = one
         scalv(2) = one
         scalv(3) = one
         scalv(4) = one
         scalft = 1.e-15
         ne  = 4
         nyk = m
         nyj = ne
         nb  = 2
         nci = ne
         ncj = ne-nb+1
         nck = m+1
         nsi = ne
         nsj = ne+ne+1
         ml = 5
         mu = 5
         lda = 4*kmx
         n = ne*m
         npt = ne
       
         if (ibcb.eq.2) then
            indexv(1) = 1
            indexv(2) = 2
            indexv(3) = 3
            indexv(4) = 4
         else
            indexv(1) = 3
            indexv(2) = 2
            indexv(3) = 1
            indexv(4) = 4
         endif

c *************************************************************
c *      *** calculate and store factored partial derivative matrix
c *      *** on first pass through flow (done on each time step)        
c *************************************************************
         if ((ivistype.lt.0.and.itvv.eq.1).and. 
     1       ((ifssd.eq.1).or.(nabe.eq.(isx*jsx)+1))) then
       imp_sym=0
       if((isx.eq.jsx).and.(xl.eq.yl))then
         if(istart.eq.0)then
         write(*,*) '******************************************'
         write(*,*) '* REDUCING SIZE OF ABE BY USING SYMMETRY *'
         write(*,*) '******************************************'
         endif
         imp_sym=1
       endif
            lmn = 0
            do j = 1,nyr
              do i = 1,nxr
               if((lbd.eq.0).and.(mbd.eq.0))then

                 if(((i.eq.1).and.(mod(j,2).eq.0)).or.
     $      ((j.eq.1).and.(mod(i,2).eq.0)).or.
     $      ((mod(i,2).eq.0).and.(mod(j,2).eq.0)))then

                  if(imp_sym.ne.1)then
                    lmn = lmn+1
                  elseif(i.ge.j)then
                    lmn = lmn+1
                  endif

                 endif
               else
                 lmn = lmn+1
               endif
                 ijob=0
                 labe = 1
                 if (ifssd.eq.1) then
c                   *** calculate,factor,& write abe to ssd
                    ijob = 4			
                 endif
           if (nabe.eq.(isx*jsx)+1) then
c        *** calculate,factor,& store abe in memory
            ijob = 5	

            if ((lbd.eq.0).and.(mbd.eq.0)) then
c
c     Periodic boundaries
c
c     calculate the indice corresponding to the abe matrix (labe)
c
             ifill=0
             ijobabe=1
             call abeaddr(labe,i,j,nxr,nyr,ijobabe,ifill,imp_sym)

c     if labe has not changed from its previous value don't calculate
c     a new abe (i.e. go to 135)
c 
             if(.not.(((i.eq.1).and.(mod(j,2).eq.0)).or.
     $       ((j.eq.1).and.(mod(i,2).eq.0)).or.
     $       ((mod(i,2).eq.0).and.(mod(j,2).eq.0))))goto 135
           
             if(((imp_sym).eq.1).and.(i.lt.j)) goto 135

            else
c
c     reflecting boundaries
c
             labe=lmn + 1
            endif

           endif
           l = fl(i,j)
           l2 = l**2

            call vsolvlu(
     #         itmax,convsol,slowc,scalv,indexv,ne,m,y,nyj,nyk,
     1       abe(1,1,labe),lda,n,b,ipvt(1,labe),npt,ml,mu,lmn,ijob,   
     2       l,l2,vps,vts,rr,nrdim,vis,nvisdim,vnu,nvnudim,
     3       z,nzdim,nz,ibcb,ibct,ivistype,
     4       r1,r2,r3,r4,ksl,i,j,icycle,niterate,ncabe,zwork)

135          continue
               enddo
            enddo
         endif
 
c *************************************************************
c *      Find max value in a way that converts from single to double precision
c *************************************************************
         if (icycle.gt.1.or.itvv.gt.1) then
            do ll = 1,nz
              do j  = 1,j3
                do i  = 1,nx
                   scalv(1) = max(dabs(vp(i,j,ll)),scalv(1))
                   scalv(2) = max(dabs(v3(i,j,ll)),scalv(2))
                   scalv(3) = max(dabs(taup(i,j,ll)),scalv(3))
                   scalv(4) = max(dabs(tau3(i,j,ll)),scalv(4))
                   scalft   = max(dabs(ft(i,j,ll)),scalft)
                enddo
              enddo
            enddo
            slowc = max(scalv(1),scalv(2),scalv(3),scalv(4))
         else
c           *** icycle == 1
            do ll = 1,nz
               do j = 1,j3
                  do i = 1,nx
                     scalft   = max(dabs(ft(i,j,ll)),scalft)
                  enddo
               enddo
            enddo
         endif
         scalft = scalft*conv*.01

c *************************************************************
c *      POLOIDAL PART
c *      Solve nx*ny equations for each depth point.  
c *************************************************************

         lmn = 0
         do j = 1,nyr
            do i = 1,nxr
              if((lbd.eq.0).and.(mbd.eq.0))then

                if(((i.eq.1).and.(mod(j,2).eq.0)).or.
     $            ((j.eq.1).and.(mod(i,2).eq.0)).or.
     $            ((mod(i,2).eq.0).and.(mod(j,2).eq.0)))then

                  if(imp_sym.ne.1)then
                    lmn = lmn+1
                  elseif(i.ge.j)then
                    lmn = lmn+1
                  endif

                endif
              else

c Reflecting boundaries case
                lmn = lmn+1
              endif

               ijob = 0
               labe = 1
               if (ifssd.eq.1) then
                  ijob = 2
               endif
               if (nabe.eq.(isx*jsx)+1) then
                 ijob = 3
                 if((lbd.eq.0).and.(mbd.eq.0))then
c
c   Retrieve the abe address label (labe) corresponding to nxr and nyr
c
                   ijobabe=1
                   ifill=1
                   call abeaddr(labe,i,j,nxr,nyr,ijobabe,ifill,imp_sym)
                 else
                   labe=lmn+1
                 endif
               endif
 
c *            *** test if there is power in the temperature field
c *            *** or the surface boundary conditions. Skip solution
c *            *** of modes with no power.
               irtst = 0
               do ll=1,nz
                  if ((lbd.eq.0).and.(mbd.eq.0)) then
                     rr(ll) =  ft(i,j,ll)
                     if (abs(rr(ll)).gt.scalft) irtst = 1
                  else if ((lbd.eq.1).and.(mbd.eq.1)) then
                     rr(ll) = -ft(i,j,ll)
                     if (abs(rr(ll)).gt.scalft) irtst = 1
                  else
c                 ***  error not set up for mixed b.c.
                  endif
               enddo
               l = fl(i,j)
               l2 = l**2
               vps = zero
c-----------------------------------------------------------------
c   If no body forces and no surf. velocity, or if at zeroeth mode
c   then everything is zero.  ***debug** analytic solution for
c   net flow for perodic bc goes here??
c------------------------------------------------------------------

               if (ibct.eq.3) vps = vpb(i,j)
               if (((irtst.eq.0).and.(abs(vps).lt.0.001*conv))
     1             .or.(l.eq.0.0)) then
                  do ll = 1, nz
                     vp(i,j,ll)   = zero
                     v3(i,j,ll)   = zero
                     taup(i,j,ll) = zero
                     tau3(i,j,ll) = zero
                  enddo
               else
c                 *** set initial guess to solution from last iteration
                  do ll=1,nz
                     y(1,ll) = vp  (i,j,ll)
                     y(2,ll) = v3  (i,j,ll)
                     y(3,ll) = taup(i,j,ll)
                     y(4,ll) = tau3(i,j,ll)
                     if (ilam.eq.1) then 
                        r1(ll)=xlam1(i,j,ll)
                        r2(ll)=xlam2(i,j,ll)
                     endif
                  enddo
                  npt=4
                  numeq=numeq+1
           call vsolvlu(
     #            itmax,convsol,slowc,scalv,indexv,ne,m,y,nyj,nyk, 
     1            abe(1,1,labe),lda,n,b,ipvt(1,labe),npt,ml,mu,lmn,ijob,
     2            l,l2,vps,vts,rr,nrdim,vis,nvisdim,vnu,nvnudim,
     3            z,nzdim,nz,ibcb,ibct,ivistype,
     4            r1,r2,r3,r4,ksl,i,j,icycle,niterate,ncabe,zwork)     

c---------------------------------------------------------------------
c           Convergence stuff
c---------------------------------------------------------------------

                  do ll=1,nz
                     dvp=abs(y(1,ll)-vp(i,j,ll))/scalv(1)			
                     dv3=abs(y(2,ll)-v3(i,j,ll))/scalv(2)
                     dtp=abs(y(3,ll)-taup(i,j,ll))/scalv(3)
                     dt3=abs(y(4,ll)-tau3(i,j,ll))/scalv(4)
c                    *** Maximum changes since last iteration.
                     dvpmx = max(dvpmx,dvp)
                     dv3mx = max(dv3mx,dv3)
                     dtpmx = max(dtpmx,dtp)
                     dt3mx = max(dt3mx,dt3)
c                    *** Update velocity and stress with new values
                     vp  (i,j,ll) = rel1*y(1,ll)+rel2*vp(i,j,ll)
                     v3  (i,j,ll) = rel1*y(2,ll)+rel2*v3(i,j,ll)
                     taup(i,j,ll) = rel1*y(3,ll)+rel2*taup(i,j,ll)
                     tau3(i,j,ll) = rel1*y(4,ll)+rel2*tau3(i,j,ll)
c                    *** Find new rms values
                     vprms=vprms+vp(i,j,ll)**2
                     v3rms=v3rms+v3(i,j,ll)**2
                     tprms=tprms+taup(i,j,ll)**2
                     t3rms=t3rms+tau3(i,j,ll)**2
                 enddo
               endif
            enddo  ! i=1,nxr
         enddo   ! j=1,nyr

c *************************************************************
c *      *** RMS Values of Poloidal part to check convergence
c ************************************************************
         vprms=sqrt(vprms/nnp)
         v3rms=sqrt(v3rms/nnp)		
         tprms=sqrt(tprms/nnp)		
         t3rms=sqrt(t3rms/nnp)		

c *************************************************************
c *      END of POLOIDAL part
c *************************************************************

c *************************************************************
c *      TOROIDAL part
c *************************************************************
c *      solve toroidal part on only first two iteration
c *      and only when surface velocity is specified
c *      or when variable vviscosity G. Ito 1/12/95
c *************************************************************
         if (((ibct.eq.3).and.((icycle.lt.3).or.(restrt.ne.0.0))).or. 
     &       (ivistype.lt.0.).and.(ny.gt.2)) then
  	    itoroid=1				!G. Ito 5/95
            cconv = conv
            iitmax = itmax
            nniterate = niterate
            sslowc = 1.e6
            sscalv(1) = max(one*rat,one)	!added max option to handle 
            sscalv(2) = max(one*rat,one)	!small rat G. Ito 9/94
            nn = nz
            nne = 2
            nnb = 1
            nnyk = nn
            nnyj = nne
            nnci = nne
            nncj = nne-nnb+1
            nnck = nn+1
            nnsi = nne
            nnsj = 2*nne+1
            mml = 2
            mmu = 2
            lda = 4*kmx
            n = nne*nn
            npt = ne
            if (ibcb.eq.2) then
               iindexv(1) = 1
               iindexv(2) = 2
            else
               iindexv(1) = 2
               iindexv(2) = 1
            endif
            if (icycle.ne.1) then		
               sscalv(1) = .01
               sscalv(2) = .01
               do ll = 1,nz
                  do j = 1,nyr
                     do i = 1,nxr
                        sscalv(1) = max(abs(vtf(i,j,ll)),sscalv(1))    
                        sscalv(2) = max(abs(taut(i,j,ll)),sscalv(2))
                     enddo
                  enddo
               enddo
               if (sscalv(1) .gt. sscalv(2))then
                  sslowc = sscalv(1)
               else
                  sslowc = sscalv(2)
               endif
            endif
            do j = 1,j3
               do i = 1,nx
                  l = fl(i,j)
                  l2 = l**2
                  vts = 0.
                  if (ibct.eq.3) vts = vtb(i,j)
c                 *** set initial guess to solution from last time step
                  do ll = 1,nz
                     if (ilam.eq.1)then 	
                        r3(ll)=xlam3(i,j,ll)
                        r4(ll)=xlam4(i,j,ll)
                     endif				!added G. Ito 1/12/95
                     yy(1,ll) = vtf (i,j,ll)
                     yy(2,ll) = taut(i,j,ll)
                  enddo
 
                  npt = 2
                  ijob = 0		!calculate abe and b
                  call vsolvlu(
     &             iitmax,convsol,sslowc,sscalv,iindexv,nne,nn,yy,nnyj,        
     1             nnyk,abe(1,1,1),lda,n,b,ipvt(1,1),npt,mml,mmu,0,ijob,   
     2             l,l2,vps,vts,rr,nrdim,vis,nvisdim,vnu,nvnudim,
     3             z,nzdim,nz,ibcb,ibct,ivistype,
     4             r1,r2,r3,r4,ksl,i,j,icycle,nniterate,ncabe,zwork)

                  do ll = 1,nz
		     dvtf=abs(yy(1,ll)-vtf(i,j,ll))/sscalv(1)
                     dtt=abs(yy(2,ll)-taut(i,j,ll))/sscalv(2)
                     if (dvtf.gt.dvtfmx) dvtfmx=dvtf
                     if (dtt.gt.dttmx) dttmx=dtt
                     vtf (i,j,ll) = rel1*yy(1,ll)+rel2*yy(1,ll)
                     taut(i,j,ll) = rel1*yy(2,ll)+rel2*yy(1,ll)
                     vtfrms=vtfrms+vtf(i,j,ll)**2
                     ttrms=ttrms+taut(i,j,ll)**2
                  enddo
               enddo ! i=1,nx
            enddo  ! j=1,j3
         endif

c *************************************************************
c *      Convergence check for variable viscosity calculation
c *************************************************************
c        if (itvv.eq.1) then
c     write(*,1000) icycle,time
c     write(*,1001) 'itvv','error','errmx'
c        endif
         dvp=abs(vprms-vprms1)/scalv(1)
         dv3=abs(v3rms-v3rms1)/scalv(2)
         dtp=abs(tprms-tprms1)/scalv(3)
         dt3=abs(t3rms-t3rms1)/scalv(4)

         if (itoroid.eq.1) then          !Toroidal part
            vtfrms=sqrt(vtfrms/nnp)
            ttrms=sqrt(ttrms/nnp)
            dtt=abs(ttrms-ttrms1)/sscalv(1)
            dvtf=abs(vtfrms-vtfrms1)/sscalv(2)
            error=(dvp+dv3+dtp+dt3+dtt+dvtf)/6.
	    errmx=(dvpmx+dv3mx+dvtfmx)/3.0
         else
	    dtt=0.
	    dvtf=0.
            error=(dvp+dv3+dtp+dt3)/4.
	    errmx=(dvpmx+dv3mx)/2.0
         endif

         vprms1=vprms
	 v3rms1=v3rms
	 tprms1=tprms
	 t3rms1=t3rms
	 ttrms1=ttrms
	 vtfrms1=vtfrms

	 if (errmx.le.conv) then
            goto 310
	 endif

c *************************************************************
c *      Find local minumum in errmx <<<<< G.Ito
c *************************************************************
         if (itvv.gt.1) then
            esum=error+error0
            emxsum=errmx+errmx0
         else
            esum=zero
            emxsum=zero
            ibad1=0
            ibad2=0
         endif
         if (itvv.gt.2) then
            dd=esum-esum0
            dd2=emxsum-emxsum0
            if (dd.gt.zero) then
               ibad1=ibad1+1
            else
               ibad1=0
            endif
            if (dd2.gt.zero) then
               ibad2=ibad2+1
            else
               ibad2=0
            endif
            if (ibad1.gt.5.and.ibad2.gt.5) then
               print *, '>>>Local minimum; errmx=', errmx
               relax2=0.3			!G.Ito 7/21/95		
               if (errmx.gt.errmx1) then
                  icycle=1e+6
                  print *, '!!!!!!!!  Solution is unreliable   !!!!!' 
                  return
               endif
               goto 310
            endif
         endif
         error0=error
         errmx0=errmx
         esum0=esum
         emxsum0=emxsum
c---------------------------------------------------------
	 if (itvv.eq.itvvmax.and.itvv.gt.1) then
            write(6,*) '***********************************************'
	    write(6,*) '*   PWARN(vflow): itvv = itvvmax              *'
            write(6,*) '*   Current precision errmax = ',errmx
            write(6,*) '*   Variable viscosity iteration may be wrong *'
            write(6,*) '***********************************************'
            if (errmx.gt.errmx1) then
               icycle=1e+6
               print *,'!!! Solution is unreliable !!!'
               return
            endif
         endif

300   continue
c *************************************************************
c *   END OF MAIN LOOP
c *************************************************************
        

310   continue
      vrms = calvrms(1)
      if (ivistype.lt.0) then
c        write(*,1012)'icyc,time,iters,err,errmx=',icycle,time,
c    &      itvv,error,errmx,vrms,tznu
         write(200,1012)'icyc,time,iters,err,errmx=',icycle,time,
     &      itvv,error,errmx,vrms,tznu
c        write(*,*)'_________________________'
         call flush(6)
      endif
      if (ibad1.le.5.and.ibad2.le.5) relax2=relax	!G.Ito 7/21/95
      if ((ibct.eq.2).and.(isurfv.lt.0)) then
         if ((icycle.eq.1).or.(restrt.ne.0.0)) then	!flow driven plates
            call plflow
            call inttaup
         endif
c        *** calculate integrated shear stress on plate due to
c        *** convection and scale velocity of surface plates.
         call inttauc
      endif

c *************************************************************
c *   Change from poloidal and toroidal fields to
c *   u and v and superimpose plate velocity when
c *   plates are present. Also convert vertical w field.
c *************************************************************
      ijob_pttouv=0
      ipttype_pttouv=0
      call pttouv(ipttype_pttouv,ijob_pttouv)

c     ***  reset value of ifdebug in debugger for more output
      ifdebug = 0
      if (ifdebug .ne. 0) then
         write(6,*) icycle,time
         do j = 1,9
            write(6,876)(fl(i,j),i=1,9)
         enddo
         write(6,*)icycle,time
         do j = 1,9
            write(6,876)(ft(i,j,8),i=1,9)
         enddo
         write(6,*)icycle,time
         do j = 1,9
            write(6,876)(u(i,j,8),i=1,9)
         enddo
         write(6,*)icycle,time
         do j = 1,9
            write(6,876)(v(i,j,8),i=1,9)
         enddo
         write(6,*)icycle,time
         do j = 1,9
            write(6,876)(w(i,j,8),i=1,9)
         enddo
      endif

        
      call ft2nrm (-1,lpu,nxu,mpu,nyu,nz,inx,jnx,u)
      call ft2nrm (-1,lpv,nxv,mpv,nyv,nz,inx,jnx,v)
      call ft2nrm (-1,lpw,nxw,mpw,nyw,nz,inx,jnx,w)

c     *** assign values to end points for periodic boundary
      call netflow(nx,ny,inx,jnx,u,v,vis,wk1d,netflo)
      call bc (u,v,w,inx,nxr,nx,lbd,jnx,nyr,j3,mbd,nz,1)

      if (ifdebug .ne. 0) then
         write(6,*)icycle,time
         do j = 1,9
            write(6,876)(u(i,j,8),i=1,9)
         enddo
         write(6,*)icycle,time
         do j = 1,9
            write(6,876)(v(i,j,8),i=1,9)
         enddo
         write(6,*)icycle,time
         do j = 1,9
            write(6,876)(w(i,j,8),i=1,9)
         enddo
         write(6,*)icycle,time
         write(6,876)u(1,1,8),v(1,1,8),w(1,1,8)
         write(6,876)u(3,1,8),v(3,1,8),w(3,1,8)
         write(6,876)u(1,3,8),v(1,3,8),w(1,3,8)
         write(6,876)u(3,3,8),v(3,3,8),w(3,3,8)
         write(6,876)u(2,4,8),v(2,4,8),w(2,4,8)
         write(6,876)u(nx,ny,8),v(nx,ny,8),w(nx,ny,8)
      endif

      if (time+dt.ge.prnt.or.time+dt.ge.pltime) then
         iskip = 1
         if (iskip.ne.1) then
            itype = 0
            itotau  = 0 
            call pttotau(itype,itotau)
         endif
      endif
  

197   format(4i5,4e14.5)
876   format(9(e12.5,2x))
1000  format ("icycle,time=",i5,e15.5)
1001  format(a4,6a14)
1002  format(i4,6e14.6)
1012  format(a31,i6,e12.5,i6,4e12.5)
1016  format(9e13.5)
      return
      end
