c $Log:   /pvcs.config/mc3dvv/src/viscos.F_a  $
c
c   Rev 1.2   Fri Jul 16 13:55:00 1999   lowman
c
c   Rev 1.1   Tue Feb 09 14:47:02 1999   lowman
cAdded iviscos=8 option
c
c   Rev 1.0   Mon Jul 14 14:22:40 1997   pvcs
c 
c *************************************************************
c *   VISCOS
c *************************************************************
      subroutine viscos(vis,vnu,ft)
      use module_precision
      use module_param
      use module_giparam
      use module_gia3
      use module_a3
      use module_a12
      dimension vis(ksx), vnu(ksx)
      dimension ft(isx,jsx,ksx)
      common /start1/ restrt, initlt, istart
c     common /a12/ lpu, lpv, lpw, lpt13, lpt23, lpt33, lpft, nxu, nxv,
c    1 nxw, nxt13, nxt23, nxt33, nxft, nxr, mpu, mpv, mpw, mpt13, mpt23,
c    2 mpt33, mpft, nyu, nyv, nyw, nyt13, nyt23, nyt33, nyft, nyr
      common /cviscos/ vis1, vis2, vis3, vis4, vis5,
     1     kdepth1, kdepth2, kdepth3, kdepth4, kdepth5
#ifdef MPI
      include 'mpif.h'
#endif
c
c     routine for user definition of variation
c     of viscosity with depth.
c
c     ivistype = type of viscosity variation
c          <= 1 temperature dependent viscosity
c           = 0 default, constant viscosity
c           = 1
c           = 2  two layer viscosity structure
c           = 3  three layer viscosity structure
c           = 4  four layer viscosity structure
c           = 5  linear viscosity change v=b*z+c
c           = 6  quadratic change        v=az**2+b*z+c
c           = 7  log. change beginning at 670 km depth           
c  (JPL)    = 8  viscosity structure based on Pysklywec
c           fit of 1997 data with hyperbolic tan functions
c           = 10 set in input with kdepth() and vis()
c           = 11 set in input with kdepth() and vis()
c
c
c     kdepth = z index where viscosity changes
c
c     vis1 = viscosity of bottom layer
c     vis2 = viscosity of layer above vis1
c     vis3 = viscosity of layer above vis2; etc.
c
#ifdef MPI
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )
#else
      myid=0
#endif
c     check for valid input
      if ((ivistype.lt.-1).or.(ivistype.gt.7)) then
c        invalid input ivistype
      endif
c
c     mean field calculation
c
      if (ivistype.eq.-1) then
        do 10 k=1,nz
        vis(k)=ft(1,1,k)
   10   continue
c
c     normalize transformed temperature field
c
        call tempvis (lpft,nxft,mpft,nyft,nz,vis,rat)
c
c     determine viscosity from temperature
c     for this case T=0 vis=1; T=1 vis=.1
c
        do 20 k=1,nz
        vis(k)=1.-(.9*vis(k))
        vnu(k)=1./vis(k)
   20   continue
      endif
      if (ivistype.eq.0) then
        do 30 k=1,nz
        vis(k)=1.
        vnu(k)=1.
        vis1 = 1.
   30   continue
      elseif (ivistype.eq.2) then
        vis1=1.0
        vis2=0.01
        vis3=-9999.
        kdepth1=3*(nz/4)
        kdepth2=-99
c      kdepth1=nz-1
        do 40 k=1,nz
        if (k.lt.kdepth1) then
          vis(k)=vis1
          vnu(k)=1./vis(k)
        else
          vis(k)=vis2
          vnu(k)=1./vis(k)
        endif
   40   continue
c
      elseif (ivistype.eq.3) then
        vis1=1.0
        vis2=0.10
        vis3=10.0
        kdepth1=3*(nz/4)
        kdepth2=nz-1
        do 50 k=1,nz
        if (k.lt.kdepth1) then
          vis(k)=vis1
          vnu(k)=1./vis(k)
        elseif (k.lt.kdepth2) then
          vis(k)=vis2
          vnu(k)=1./vis(k)
        else
          vis(k)=vis3
          vnu(k)=1./vis(k)
        endif
   50   continue
      elseif (ivistype.eq.4) then
        vis1=0.001
        vis2=0.010
        vis3=0.100
        vis4=1.000
        do 60 k=1,nz
        if (k.lt.kdepth1) then
          vis(k)=vis1
          vnu(k)=1./vis(k)
        elseif (k.lt.kdepth2) then
          vis(k)=vis2
          vnu(k)=1./vis(k)
        elseif (k.lt.kdepth3) then
          vis(k)=vis3
          vnu(k)=1./vis(k)
        else
          vis(k)=vis4
          vnu(k)=1./vis(k)
        endif
   60   continue
      elseif(ivistype .eq. 5) then
      acoef=vis1
      bcoef=vis2
      dz=1./(nz-1)
      zpos=0.
      do 65 k=1,nz
         vis(k)=acoef+bcoef*zpos
         vnu(k)=1./vis(k)
         zpos=zpos+dz
   65 continue
      elseif(ivistype .eq. 6) then
      acoef = vis1
      bcoef = vis2
      ccoef = vis3
      dz=1./(nz-1)
      zpos=0.
      do 67 k=1,nz
         vis(k)=acoef+bcoef*zpos+ccoef*zpos**2
         vnu(k)=1./vis(k)
         zpos=zpos+dz
   67 continue
      elseif(ivistype .eq. 7) then
      acoef = vis1-1.0d0
      dz=1./(nz-1)
      zpos=0.
      do 68 k=1,nz
      if (zpos.ge.(nz-lith)*dz) then
         vis(k)=1000.d0
      elseif (zpos.ge. 0.769d0) then
         vis(k)=1.d0
      else
         vis(k)=1.0d0+acoef/dlog(1000.d0)*
     $dlog(1.d0+999.d0*(.769d0-zpos)/.769d0)
      endif
         vnu(k)=1./vis(k)
         zpos=zpos+dz
   68 continue
c
c JPL add This option is based on the viscosity profiles provided
c         by Russ Pysklywec and Jerry Mitrovica. N.B., zeroth order.
c
 
      elseif(ivistype .eq. 8) then
       a0_lm=40.0d0
       a0_tz=4.0d0
       a0_um=one
       a1_lm=zero
       a2_lm=zero
       a3_lm=zero
       a1_tz=zero
       a1_um=zero
       slant2=7.d0
       slant1=50.d0
       dep2=1.789d0
       dep1=1.9727d0

        r=2.20415d0
        gtz=half*(one-dtanh((dep2-r)*slant2))
        gum=half*(one-dtanh((dep1-r)*slant1))

        flm=a0_lm+a1_lm*r+a2_lm*r**2+a3_lm*r**3
        ftz=a0_tz+a1_tz*r
        fum=a0_um+a1_um*r

        vis(nz)=(flm+(ftz-flm)*gtz+(fum-ftz)*gum)

       do k=1,nz
        r=float(k-1)/float(nz-1)+1.20415d0
        gtz=half*(one-dtanh((dep2-r)*slant2))
        gum=half*(one-dtanh((dep1-r)*slant1))
 
        flm=a0_lm+a1_lm*r+a2_lm*r**2+a3_lm*r**3
        ftz=a0_tz+a1_tz*r
        fum=a0_um+a1_um*r

        if(k.le.(nz-lith))then
         vis(k)=(flm+(ftz-flm)*gtz+(fum-ftz)*gum)/vis(nz)
        else
         vis(k)=1000.
        endif

        vnu(k)=1./vis(k)

       enddo
      elseif(ivistype .eq. 10) then
c    e.g.
c    vis1 =  30., kdepth1 = 15,
c    vis3 =  0.1, kdepth3 = 31,
c    vis4 = 100., kdepth4 = 33,
c
c      if((kdepth1.le.kdepth2).and.(kdepth2.le.kdepth3).and.
c     1   (kdepth3.le.kdepth4).and.(kdepth4.le.kdepth5))then
      do 70 k=1,nz
      if (k .le. kdepth1) then
          vis(k)=vis1
          vnu(k)=1./vis(k)
        elseif (k.le.kdepth2) then
          vis(k)=vis2
          vnu(k)=1./vis(k)
        elseif (k.le.kdepth3) then
          vis(k)=vis3
          vnu(k)=1./vis(k)
        elseif (k .le. kdepth4) then
          vis(k)=vis4
          vnu(k)=1./vis(k)
        elseif (k .le. kdepth5) then
         vis(k)=vis5
         vnu(k)=1./vis(k)
        endif
   70   continue
c        else
c          write(6,970)kdepth1,kdepth2,kdepth3,kdepth4,kdepth5
c          stop
c        endif
      elseif(ivistype .eq. 11) then
c
c     e.g. kdepth1 = 1200, kdepth2 = 670, kdepth3 = 300, kdepth4 = 100
c
      if((kdepth1.ge.kdepth2).and.(kdepth2.ge.kdepth3).and.
     1   (kdepth3.ge.kdepth4).and.(kdepth4.ge.kdepth5))then
      do 82 k=1,nz
        if     (float(k)/float(nz) .le. float(3000-kdepth1)/3000.) then
          vis(k)=vis1
          vnu(k)=one/vis(k)
        elseif (float(k)/float(nz) .le. float(3000-kdepth2)/3000.) then
          vis(k)=vis2
          vnu(k)=one/vis(k)
        elseif (float(k)/float(nz) .le. float(3000-kdepth3)/3000.) then
          vis(k)=vis3
          vnu(k)=one/vis(k)
        elseif (float(k)/float(nz) .le. float(3000-kdepth4)/3000.) then
          vis(k)=vis4
          vnu(k)=one/vis(k)
        elseif (float(k)/float(nz) .le. float(3000-kdepth5)/3000.) then
         vis(k)=vis5
         vnu(k)=one/vis(k)
        endif
   82   continue
        else
          write(6,970)kdepth1,kdepth2,kdepth3,kdepth4,kdepth5
          stop
        endif
      endif
c
c     write viscosity type to output files
      if ((icycle.eq.1).or.(restrt.ne.0.0)) then
c       write (6,80) ivistype
c       write (6,90) kdepth1,kdepth2,kdepth3,kdepth4,kdepth5
c       write (6,92) vis1, vis2, vis3, vis4, vis5
        if(myid .eq. 0) then
          if (ivistype.ne.0) then
          open(unit = 85, file = 'tapeviscos', form='formatted')
          call wheader(85,2)
          write (85,85)icycle,time
          do 75 k=1,nz
          write (6,100) k,float(k-1)/float(nz-1),
     1    3000.*float(nz-k)/float(nz-1),vis(k),vnu(k)
          write (85,105) z(k),float(k-1)/float(nz-1),
     1    3000.*float(nz-k)/float(nz-1),vis(k),vnu(k)
   75     continue
          
   85 format (i8,e14.7,/,
     1"Viscosity
     2   z(k)      z        depth     viscos    viscos**-1")
  105 format (11x,5(e10.3,2x))
          close(85)
          endif
        endif
      endif
c
      return
c
   80 format ("viscosity type is ivistype=  ",i4)
   90 format ("kdepth= ",5i7)
   92 format ("vis1,2,3 4 5= ",5e14.4)
  100 format (i5,2x,4(e10.4,2x))
  970 format ("**************Input error in viscos ************",/, 
     1        " kdepth= ",5i7)
      end
