c $Log:   /pvcs.config/mc3dvv/src/xlambda.F_a  $
c
c   Rev 1.0   Mon Jul 14 14:22:44 1997   pvcs
c 
c *************************************************************
c *   XLAMBDA
c *************************************************************
	subroutine xlambda(itvv)
c
c  called from subroutine flow
c
c  this subroutine calculates the four terms in the momentum
c    equation due to temperature-dependent viscosity (Gable, 1989)
c
c  to calculate these terms, we need:
c    -the viscosity variation eta_bar(x,y,z) (eta(x,y,z) less the
c       horizontal average eta_0(z) ),
c    -the variation in the inverse viscosity (1/eta), again the
c       inverse less the horizontal average (z),
c    -and the velocity field
c    -and stress tensor.
c
c  since we will be taking horizontal derivatives in
c    spectral (real fft) space, we want fft(v) and fft(eta)
c
c  before this routine is called the variations in eta and its inverse
c    should already have been calculated (say, in the subroutine viscos)
c
c  Arrays are:
c
c  Recalculated every call to xlambda:
c  WORK1-4:  4 components of xlam1-4
c  WORKT:   Temporary buffer
c	   
c  Calculated only at itvv=1:
c  WORKVISX: stored x-deriviative of viscosity    
c  WORKVISY: "    " y-derivative of visc    
c********* Set arrays *************************
c    -calculate the fft of visc   WORK                        
c            call ft2nrm(1,visc(1,1,k), WORK)
c    -derivatives of visc(1,1,k), WORKVISX, WORKVISY
c       ifft 
c
c    -convert vp,vt to u,v
c
c    -set WORK3,WORK4 to zero
c  
c    ****** xlam1 ************
c
c    -convert taup,taut to taux,tauy 
c    -multiply the inverse viscosity variation by TAUxz to xlam1
c    -fft the product in to make WORK1
c
c    ****** xlam2 ************
c
c    -convert taup,taut to taux,tauy i
c    -multiply the inverse viscosity variation by TAUyz to xlam2
c    -fft the product to make WORK2
c
c    ****** xlam3 ************
c
c    -for 3 terms (Vx,xx ; Vy,xy ; Vx,yy)                               
c      -get the required derivatives of vx,vy in spectral space WORK
c            call diffxy(WORK,WORKU)
c      -inverse fft these derivatives (in work2)                       
c            call ft2nrm(-1,WORK)
c      -multiply by eta_bar and add the term to xlam3
c            WORK3=WORK3+visc*WORK
c
c    -for 2 terms (Vx,x ; Vy,y)
c      -get the required derivatives of vx,vy in spectral space (WORK) W3
c            call diffxy(v(1,1,k)x,WORK)
c      -inverse fft these derivatives (in work3)                        W3
c            call ft2nrm(-1,WORK)
c      -multiply by the deriv of eta_bar in work2 and add the term to xlam3
c            xlam3=xlam3+VISCX*WORK
c
c
c    - for 2 terms (Vx,y ; Vy,x)
c      -get the required derivatives of vx,vy in spectral space in WORK
c            call diffxy(v(1,1,k)x,WORK)
c      -inverse fft these derivatives (WORK)                         W3
c            call ft2nrm(-1,WORK)
c      -multiply by the deriv of eta_bar in work2 and add the term to xlam3
c            xlam3=xlam3+WORKVISY*WORK
c
c    - fft xlam3
c
c    ****** xlam4 ************
c
c    -for 3 terms (Vy,yy ; Vx,xy ; Vy,xx)                                
c      -get the required derivatives of vx,vy in spectral space (WORK) W2
c      -inverse fft these derivatives (WORK)                        W2
c      -multiply by visc and add the term to xlam4
c
c    -for 2 terms (Vx,x ; Vy,y)
c      -get the required derivatives of vx,vy in spectral space (WORK) W3
c      -inverse fft these derivatives (WORK)                        W3
c      -xlam4=xlam4+WORKVISY*WORK
c    
c    - for 2 terms (Vx,y ; Vy,x)
c      -get the required derivatives of vx,vy in spectral space WORK
c      -inverse fft these derivatives (WORK)                        W3
c      -xlam4=xlam4+WORKVISX*WORK
c
c    - fft xlam4
c
c  Modified arithmetic, arrays, and added filtering.  G. Ito 2/15/95
c  Calculates each layer at a time so only need to save horizontal
c   arrays and use much less memory.		      G. Ito 6/27/95
c=========================================================================
      use module_precision
      use module_param
      use module_giparam
      use module_a1
      use module_gia1
      use module_a2
      use module_gia3
      use module_a3
      use module_a22
      use module_afftw
      use module_alam
      use module_a11
      use module_a20
      use module_spwork

      use module_a12
c       common /a12/ lpu, lpv, lpw, lpt13, lpt23, lpt33, lpft, nxu, nxv,
c    1 nxw, nxt13, nxt23, nxt33, nxft, nxr, mpu, mpv, mpw, mpt13, mpt23,
c    2 mpt33, mpft, nyu, nyv, nyw, nyt13, nyt23, nyt33, nyft, nyr
   				
c*************************  Set up arrays *************************************
	call pttotau(0,0)					!tau13,tau23
	call pttouv2(0,0)
	call filt(u,inx,jnx,knx,nxu,nyu,nz,xf1,xf2)	!FILTER
	call filt(v,inx,jnx,knx,nxv,nyv,nz,xf1,xf2)	!FILTER
        ijsl=isl*jsl
c       call printy(visc,isl,ksl,'viscosity               ')
c       call printy(vnuc,isl,ksl,'1d0/viscosity           ')
c       call printy(tau13,isl,ksl,'tau13               ')
c       call printy(u,isl,ksl,'u                       ')
c       call printy(v,isl,ksl,'v                       ')

        do 800 k=1,ksl
           km=k
c-------------------------------------------------------------------
c    Viscosity and derivatives
c-------------------------------------------------------------------
c        if (itvv.eq.1.or.(icycle.eq.1.and.itvv.eq.2)) then

	call setm(visc(1,1,k),isl,jsl,1,ijsl			!WORKT=visc
     #		,workt,isl,jsl,1,ijsl)
c       write(6,*) 'workt',k
c       write(6,'(6e15.7)') (workt(i,1),i=1,isl)
   
	call ft2nrm(1,lpft,nxft,mpft,nyft,1,isl,jsl,workt)	!WORKT=visc (spect.)
c       write(6,*) 'workt-F',k
c       write(6,'(6e15.7)') (workt(i,1),i=1,isl)
	idifx=1
	idify=0
	call diffxy(workt,workvisx,isl,nxft,jsl,nyft,idifx,idify 	!workvisx=visc(1,1,k),x
     #       ,lpft,lpw6,mpft,mpw6,nxw6,nyw6,lbd,mbd)
c       write(6,*) 'workvisx-F',k
c       write(6,'(6e15.7)') (workvisx(i,1),i=1,isl)

	call ft2nrm(-1,lpw6,nxw6,mpw6,nyw6,1,isl,jsl,workvisx)		!workvisx=visc(1,1,k),x (space)
c       write(6,*) 'workvisx-before bc2',k
c       write(6,'(6e15.7)') (workvisx(i,1),i=1,isl)
	call bc2(workvisx,isl,jsl,1,lpw6,mpw6,1,lbd,mbd)
 
        if (ny.ne.2) then
	  idifx=0
	  idify=1
	  call diffxy(workt,workvisy,isl,nxft,jsl,nyft,idifx,idify  !workvisy=visc(1,1,k),y
     #      ,lpft,lpw6,mpft,mpw6,nxw6,nyw6,lbd,mbd)			         
	  call ft2nrm(-1,lpw6,nxw6,mpw6,nyw6,1,isl,jsl,workvisy)
	  call bc2(workvisy,isl,jsl,1,lpw6,mpw6,1,lbd,mbd)
        endif
c        endif
c       call printy2(workvisx,k,isl,'workvisx         ')
c       write(6,*) 'workvisx'
c       write(6,'(6e15.7)') (workvisx(i,1),i=1,isl)

 

c***************************  lam 1 ***************************************
	scalar=one
	call multadd(work1,vnuc(1,1,k),tau13(1,1,k),scalar,isl,jsl,1,0)		!work1=vnuc*tau13
c        if (k.eq.12) then
c        print *, '----- VNUC------'
c        write(*,1016)((vnuc(i,j,k),i=1,isl),j=1,jsl)			!xlamdrive
c        print *, '----- TAU13------'
c        write(*,1016)((tau13(i,j,k),i=1,isl),j=1,jsl)			!xlamdrive
c        print *, '----- WORK1------'
c        write(*,1016)((work1(i,j),i=1,isl),j=1,jsl)			!xlamdrive
c	endif

c------------------------------------------------------------------
c  set flags for the product
c
c  lpt13,mpt13,lpft,mpft=flags for side bc's
c  nxt13,nyt13,nxft,nyft=appropiate dimensions 
c  lpl1,nxl1,mpl1,nyl1= output
c  and fft xlam1=work1
c------------------------------------------------------------------
	call flagset(lpt13,nxt13,lpft,nxft,lpl1,nxl1)
	call flagset(mpt13,nyt13,mpft,nyft,mpl1,nyl1)
	call ft2nrm(1,lpl1,nxl1,mpl1,nyl1,1,isl,jsl,work1)	!work1=tau13*vnu (spect.)
c***************************  lam 2 ***************************************
	if(ny.ne.2) then		!<----3-D Only
	  scalar=one
	  call multadd(work2,vnuc(1,1,k),tau23(1,1,k),scalar,isl,jsl,1,0)  !work2=tau23*vnuc

c------------------------------------------------------------------
c  set flags for the product, based on ft flags for vnuc
c  and fft xlam2=work2
c------------------------------------------------------------------
	  call flagset(lpt23,nxt23,lpft,nxft,lpl2,nxl2) !
	  call flagset(mpt23,nyt23,mpft,nyft,mpl2,nyl2)
	  call ft2nrm(1,lpl2,nxl2,mpl2,nyl2,1,isl,jsl,work2)	!work2=tau23*vnuc (spect.)
	endif

c*************************** 1st term of lam3  *************************************
c---------------------------------------------------------------------
c  3 terms each for xlam3 and xlam4 with visc - 
c    1) differentiate in spectral space
c    2) transform field to real space
c    3) multiply by visc(1,1,k), scalar and add result to lambda
c---------------------------------------------------------------------
	idifx=2
	idify=0
	call diffxy(u(1,1,k),workt,inx,nxu,jnx,nyu,idifx,idify  	!workt=u,xx
     #	,lpu,lpw4,mpu,mpw4,nxw4,nyw4,lbd,mbd)
	call ft2nrm(-1,lpw4,nxw4,mpw4,nyw4,1,isl,jsl,workt)	!workt=u,xx (space)
	call bc2(workt,isl,jsl,1,lpw4,mpw4,1,lbd,mbd) 
	scalar=oneneg*four
	call multadd(work3,visc(1,1,k),workt,scalar,isl,jsl,1,0)	!work3=-visc(4u,xx)	
	call flagset(lpw4,nxw4,lpft,nxft,lpl3,nxl3)
	call flagset(mpw4,nyw4,mpft,nyft,mpl3,nyl3)

	if(ny.ne.2) then		!<----3-D Only
	  idifx=0
	  idify=2
	  call diffxy(u(1,1,k),workt,inx,nxu,jnx,nyu,idifx,idify     !workt=u,yy
     #      ,lpu,lpw4,mpu,mpw4,nxw4,nyw4,lbd,mbd)
	  call ft2nrm (-1,lpw4,nxw4,mpw4,nyw4,1,isl,jsl,workt)		!workt=u,yy (space)
	  call bc2(workt,isl,jsl,1,lpw4,mpw4,1,lbd,mbd)
	  scalar=oneneg
	  call multadd(work3,visc(1,1,k),workt,scalar,isl,jsl,1,1)	!work3=-visc(4u,xx+u,yy)

	  idifx=1
	  idify=1
	  call diffxy(v(1,1,k),workt,inx,nxv,jnx,nyv,idifx,idify   	!workt=v,xy
     #       ,lpv,lpw4,mpv,mpw4,nxw4,nyw4,lbd,mbd)
	  call ft2nrm (-1,lpw4,nxw4,mpw4,nyw4,1,isl,jsl,workt)		!workt=v,xy (space)
	  call bc2(workt,isl,jsl,1,lpw4,mpw4,1,lbd,mbd)
	  scalar=three*oneneg
	  call multadd(work3,visc(1,1,k),workt,scalar,isl,jsl,1,1)	!work3=-visc(4u,xx+u,yy+3v,xy) 
**********************  1st term of lam 4*****************************************
	  idifx=0
	  idify=2
	  call diffxy(v(1,1,k),workt,inx,nxv,jnx,nyv,idifx,idify    !workt=v,yy
     #       ,lpv,lpw4,mpv,mpw4,nxw4,nyw4,lbd,mbd)
	  call ft2nrm (-1,lpw4,nxw4,mpw4,nyw4,1,isl,jsl,workt)		!workt=v,yy (space)
	  call bc2(workt,isl,jsl,1,lpw4,mpw4,1,lbd,mbd)
	  scalar=four*oneneg
	  call multadd(work4,visc(1,1,k),workt,scalar,isl,jsl,1,0)	!work4=-visc(4v,yy)
			
          call flagset(lpw4,nxw4,lpft,nxft,lpl4,nxl4)		
          call flagset(mpw4,nyw4,mpft,nyft,mpl4,nyl4)

	
	  idifx=2
	  idify=0
	  call diffxy(v(1,1,k),workt,inx,nxv,jnx,nyv,idifx,idify	!workt=v,xx
     #        ,lpv,lpw4,mpv,mpw4,nxw4,nyw4,lbd,mbd)
	  call ft2nrm (-1,lpw4,nxw4,mpw4,nyw4,1,isl,jsl,workt)		!workt=v,xx (space)
	  call bc2(workt,isl,jsl,1,lpw4,mpw4,1,lbd,mbd)
	  scalar=oneneg	
	  call multadd(work4,visc(1,1,k),workt,scalar,isl,jsl,1,1)		!work4=-visc(4v,yy+v,xx)

	  idifx=1
	  idify=1
	  call diffxy(u(1,1,k),workt,inx,nxu,jnx,nyu,idifx,idify	!workt=u,xy
     #        ,lpu,lpw4,mpu,mpw4,nxw4,nyw4,lbd,mbd)
	  call ft2nrm (-1,lpw4,nxw4,mpw4,nyw4,1,isl,jsl,workt)		!workt=u,xy (space)
	  call bc2(workt,isl,jsl,1,lpw4,mpw4,1,lbd,mbd)
	  scalar=three*oneneg
	  call multadd(work4,visc(1,1,k),workt,scalar,isl,jsl,1,1)		!work4=-visc(4v,yy+v,xx+3u,xy)
	endif			!<----------- End 3-D only

**********************  2nd term of lam3 and lam4 *****************************************
c------------------------------------------------------------------
c  2 terms each for xlam3 and xlam4 with u_x & v_y - 
c    differentiate in spectral space
c    transform field to real space
c    multiply by visc
c    add result to lamda
c------------------------------------------------------------------
	idifx=1
	idify=0

	call diffxy(u(1,1,k),workt,inx,nxu,jnx,nyu,idifx,idify 	!workt=u,x
     #       ,lpu,lpw4,mpu,mpw4,nxw4,nyw4,lbd,mbd)
	call ft2nrm(-1,lpw4,nxw4,mpw4,nyw4,1,isl,jsl,workt)		!workt=u,x (space)
	call bc2(workt,isl,jsl,1,lpw4,mpw4,1,lbd,mbd)
       
	scalar=oneneg*four
	call multadd(work3,workvisx,workt,scalar,isl,jsl,1,1)		!work3=1st-visc,x(4u,x)

	if(ny.ne.2) then					!<------ 3D
	  scalar=oneneg*two
	  call multadd(work4,workvisy,workt,scalar,isl,jsl,1,1)		!work4=1st-visc,y(2u,x)

	  idifx=0
	  idify=1
	  call diffxy(v(1,1,k),workt,inx,nxv,jnx,nyv,idifx,idify 	!workt=v,y
     #         ,lpv,lpw5,mpv,mpw5,nxw5,nyw5,lbd,mbd)
	  call ft2nrm(-1,lpw5,nxw5,mpw5,nyw5,1,isl,jsl,workt)		!workt=v,y (space)
	  call bc2(workt,isl,jsl,1,lpw5,mpw5,1,lbd,mbd)

	  scalar=oneneg*two
	  call multadd(work3,workvisx,workt,scalar,isl,jsl,1,1)	!work3=1st-visc,x(4u,x+2v,y)

	  scalar=oneneg*four
	  call multadd(work4,workvisy,workt,scalar,isl,jsl,1,1)	!work4=1st-visc,y(2u,x+4v,y)
c
c**********************  3rd term of lam3 and lam4 ************************************
	  idifx=0
	  idify=1
	  call diffxy(u(1,1,k),workt,inx,nxu,jnx,nyu,idifx,idify  	!workt=u,y 
     #        ,lpu,lpw4,mpu,mpw4,nxw4,nyw4,lbd,mbd)
	  call ft2nrm(-1,lpw4,nxw4,mpw4,nyw4,1,isl,jsl,workt)		!workt=u,y (space)
	  call bc2(workt,isl,jsl,1,lpw4,mpw4,1,lbd,mbd)

	  scalar=oneneg
	  call multadd(work3,workvisy,workt,scalar,isl,jsl,1,1)	!work3=1st+2nd-visc,y(u,y)
	  call multadd(work4,workvisx,workt,scalar,isl,jsl,1,1)	!work4=1st+2nd-visc,x(u,y)
	  idifx=1
	  idify=0
	  call diffxy(v(1,1,k),workt,inx,nxv,jnx,nyv,idifx,idify	!workt=v,x
     #         ,lpv,lpw5,mpv,mpw5,nxw5,nyw5,lbd,mbd)
	  call ft2nrm(-1,lpw5,nxw5,mpw5,nyw5,1,isl,jsl,workt)		!workt=v,x (space)
	  call bc2(workt,isl,jsl,1,lpw5,mpw5,1,lbd,mbd)

	  call multadd(work3,workvisy,workt,scalar,isl,jsl,1,1)	!work3=1st+2nd-visc,y(u,y+v,x)
	  call multadd(work4,workvisx,workt,scalar,isl,jsl,1,1)	!work4=1st+2nd-visc,x(u,y+v,x)

	  call ft2nrm(1,lpl4,nxl4,mpl4,nyl4,1,isl,jsl,work4)	!work4=lam4 (spect.)

	endif
	call ft2nrm(1,lpl3,nxl3,mpl3,nyl3,1,isl,jsl,work3)	!work3=lam3 (spect.)
c-----------------------------------------------------------------------------------------
c    Generate Lambda arrays
c---------------------------------------------------------------------------------------

c       write(6,*) 'work1'
c         write(6,'(6e15.7)') (work1(i,1),i=1,isl)
c       write(6,*) 'work2'
c         write(6,'(6e15.7)') (work2(i,1),i=1,isl)
c       write(6,*) 'work3'
c         write(6,'(6e15.7)') (work3(i,1),i=1,isl)
c       write(6,*) 'work4'
c         write(6,'(6e15.7)') (work4(i,1),i=1,isl)

c	do 200 k  =1,ksl			
	  do 200 j = 1,jsl
	    do 200 i = 1,isl
	      xbuff1 = fli(i,j)*work1(i,j) +flj(i,j)*work2(i,j)
	      xbuff2 = fli(i,j)*work3(i,j)+flj(i,j)*work4(i,j)
	      xbuff3 = flj(i,j)*work1(i,j) -fli(i,j)*work2(i,j)
	      xbuff4 = flj(i,j)*work3(i,j)-fli(i,j)*work4(i,j)
c	      if (itvv.gt.2.and.itvv.lt.itvnosm) then
c                xlam1(i,j,k)=(xbuff1+slam1(i,j,k))/two
c                xlam2(i,j,k)=(xbuff2+slam2(i,j,k))/two
c                xlam3(i,j,k)=(xbuff3+slam3(i,j,k))/two
c                xlam4(i,j,k)=(xbuff4+slam4(i,j,k))/two
c	        slam1(i,j,k) = xbuff1
c	        slam2(i,j,k) = xbuff2
c	        slam3(i,j,k) = xbuff3
c	        slam4(i,j,k) = xbuff4
c	      else
                xlam1(i,j,k)=xbuff1
                xlam2(i,j,k)=xbuff2
                xlam3(i,j,k)=xbuff3
                xlam4(i,j,k)=xbuff4
c	      endif		
 200     continue
 800    continue

c       call printy(xlam1,isl,ksl,'xlam1              ')
c       call printy(xlam2,isl,ksl,'xlam2              ')
c       do k=1,ksl
c         write(6,'(6e15.7,:)') (xlam2(i,1,k),i=1,isl)
c       enddo



c        do k=8,4,-1
c        print *, '------------ FFT XLAM1--------------'		!xlamdrive
c        write(*,1016)((xlam1(i,j,12),i=1,isl),j=1,jsl)			!xlamdrive
c        print *, '------------ FFT XLAM2--------------'		!xlamdrive
c        write(*,1016)(((xlam2(i,j,k),i=1,isl),j=1,jsl),k=5,6)			!xlamdrive
c        print *, '------------ FFT XLAM3--------------'		!xlamdrive
c       write(*,1016)(((xlam3(i,j,k),i=1,isl),j=1,jsl),k=5,6)			!xlamdrive
c       print *, '------------ FFT XLAM4--------------'		!xlamdrive
c        write(*,1016)(((xlam4(i,j,k),i=1,isl),j=1,jsl),k=5,6)			!xlamdrive



c	stop
1016   format(10e11.3)
1000    format(10f10.2)
1110   format(i5,17f10.0)
1111   format(a5,17f10.0)
1001	format(i8,4e15.5)	        
	return
	end
c
