c $Log:   /pvcs.config/mc3dvv/src/userout.F_a  $
c
c   Rev 1.2   Fri Jul 30 11:48:26 1999   lowman
c  Added options for geoid, topography, viscosity and mean temperature
c  field output
c
c   Rev 1.1   Tue Feb 09 14:49:54 1999   lowman
cIntroduced option for printing out poloidal and toroidal powers
cIntroduced control of output of AVS fields
c
c   Rev 1.0   Mon Jul 14 14:22:34 1997   pvcs
c 
c *************************************************************
c *   USEROUT
c *
c *   General output routine, which may be adapted at the user level
c *   Provides windows into the computational part of MC3D by
c *   reverse communication.
c *
c *   Parameters:
c *       KEYWORD i Character string, describes at which point
c *                 USEROUT is called:
c *                 'init'   After initialization 
c *                 'flow'   After solution for velocity/stress
c *                          (subroutine flow)
c *               'spectral' During flow (following call to pttouv)
c *                          That is, after solution for Spectral
c *                          coefficients for u and v but prior to
c *                          conversion of u and v to Cartesian space 
c *                 'geoid'  Output geoid at the same time as T-field
c *                 'topo'   Output surface topography at the same time
c *                          as the T-field
c *                 'heat'   After one step of the time-integrator
c *                 'visco'  At the end of each viscosity calculation
c *                          (subroutine viscos)
c *                 'pttouv-1' Called from pttouv for calculation of
c *                          the power spectrum of P/T
c *                 'pttouv-2' Called from pttouv for calculation of
c *                          the power spectrum of P/T
c *                 'tau'    Follows "flow" call and produces output of tau_ij
c *                 'end'    Just before the program finishes
c *
c *************************************************************
c *   Replaces OUTPUT and related routines (PvK 950811)
c *
c *   PvK 950811
c *************************************************************
c *************************************************************
c *
c *   Each of these variables may be specified (left justified) on a
c *   separate line of an input file named "input.io". A _value_ for the
c *   following parameter must appear first.
c *   prntflds = specifies time interval between sequential field
c *              dumps. The output file name has a prefix describing
c *              the contained field. The output files are also assigned
c *              a numerical designation (e.g., vx10004.fld holds the
c *              fourth sequetial dump of the x-component of the velocity field).
c *              A suffix is assigned to the field indicating the
c *              format of the data (e.g., '.fld' is in format for
c *              avs "read field" module).
c *   A list of those fields which are to be output should then follow.
c *   The keyword "end" should be used to end the list.
c *   The following fields may be specified for output, presently:
c *   temperature
c *   velocity (producing dumps of u,v and w)
c *   spectral (producing dumps of power in u,v and w spectral components) 
c *   tau (producing dumps of tau13, tau23)
c *   tstats (produces dumps of temperature profiles for '98 AGU meeting).
c *   
c *
c *************************************************************


      subroutine userout(keyword)
      use module_precision
      use module_param
      use module_giparam
      use module_timeint
      use module_a1
      use module_a11
      use module_a12
      use module_gia1
      use module_a2
      use module_a22
      use module_a3
      use module_gia3
      use module_ftcoef
      use module_craster
      use module_pttouv
      use module_userout
#ifdef MPI
      include 'mpif.h'
#endif
c     common /ptouserout/ pfac,tfac,prnt_last
      common /ptouserout/ pfac,tfac

      character*10 keyword
      character*80 rasroot,naampje
      character*20 fieldname
      save tcrash,tinc
      tcrash=3600.
      tinc=3600.

      if(icycle.eq.0)then
      io_ft=0
      io_vx=0
      io_vy=0
      io_vz=0
      io_v3=0
      io_vis=0
      io_power_vx=0
      io_power_vy=0
      io_power_vz=0
      io_xz=0
      io_yz=0
      io_geoid=0
      io_topo=0
      itstats=0
      endif
c

#ifdef MPI
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )
#else
      myid=0
#endif
      if (myid .eq. 0) then
        if(icycle.eq.1)open(unit=17,file='tape17',form='formatted')
      endif

      if (keyword(1:4).eq.'init') then

        if (myid .eq. 0) then
c         call output(2)
        endif
c *************************************************************
c *      Read info from input file 'input.io'
c *************************************************************
        open (unit=33,file='input.io',status='old',
     1    form='formatted',iostat=ioerr)
         if(ioerr.ne.0)then
           if (myid .eq. 0) then
             write(*,*) '***********************'
             write(*,*) ' input.io not specified'
             write(*,*) 'no AVS output requested'
             write(*,*) '***********************'
           endif
           prntflds=1000.0
           goto 10
         endif
         read (33,*) prntflds
         if (myid .eq. 0) then
            write(*,*) '***************************'
            write(*,*) 'The following fields will'
            write(*,*) 'be output in AVS format at'
            write(*,*) 'intervals of:',prntflds
         endif
5        read (33,'(a)') fieldname

         if(fieldname(1:3).eq.'end')then
           goto 10
         elseif(fieldname(1:3).eq.'tem')then
           if(fieldname(1:16).eq.'temperature_byte')then
             io_ft = 2
           else
             io_ft=1
           endif
           if (myid .eq. 0) then
             if(io_ft .eq. 1)then
                write(*,*) 'temperature'
             elseif(io_ft .eq. 2)then
                write(*,*) 'temperature_byte'
             endif
           endif
         elseif(fieldname(1:3).eq.'vel')then
           io_vx=1
           if (myid .eq. 0) then
             write(*,*) 'vx velocity magnitudes'
            endif
            io_vy=1
            if (myid .eq. 0) then
              write(*,*) 'vy velocity magnitudes'
            endif
            io_vz=1
            if (myid .eq. 0) then
              write(*,*) 'vz velocity magnitudes'
            endif
            io_v3=1
            if (myid .eq. 0) then
              write(*,*) 'velocity vectors'
            endif
         elseif(fieldname(1:3).eq.'spe')then
           io_power_vx=1
           if (myid .eq. 0) then
             write(*,*) 'vx power spectrum'
           endif
           io_power_vy=1
           if (myid .eq. 0) then
             write(*,*) 'vy power spectrum'
           endif
           io_power_vz=1
           if (myid .eq. 0) then
             write(*,*) 'vz power spectrum'
           endif
         elseif(fieldname(1:3).eq.'geo')then
           io_geoid=1
           if (myid .eq. 0) then
             write(*,*) 'geoid'
           endif
         elseif(fieldname(1:3).eq.'top')then
           io_topo=1
           if (myid .eq. 0) then
             write(*,*) 'topography'
           endif
         elseif(fieldname(1:3).eq.'tau') then
           if(iftau.eq.0)then
             stop 'CANNOT OUTPUT tau fileds with iftau=0'
           endif
           io_xz=1
           if (myid .eq. 0) then
             write(*,*) 'tau_xz magnitudes'
           endif
           io_yz=1
           if (myid .eq. 0) then
             write(*,*) 'tau_yz magnitudes'
           endif
           stop 'ERROR: Output of stresses (tau) has been disabled'
         elseif(fieldname(1:3).eq.'tst') then
           itstats=1
           if (myid .eq. 0) then
             write(*,*) 'T-field stats will be written by tempstats'
           endif
         elseif(fieldname(1:3).eq.'vis') then
           if(ivistype.ge.0) then
             if (myid .eq. 0) then
               write(*,*) 'Cannot output viscosity field when ivistype'
               write(*,*) 'is non-negative (see userout.F)'
             endif
             stop
           endif
           io_vis=1
           if (myid .eq. 0) then
             write(*,*) 'viscosity field'
           endif
         else
           if (myid .eq. 0) then
             write(*,*) 'Requested AVS field output of'
             write(*,*) fieldname(1:20), 'not recognized' 
             write(*,*) 'the program will stop.'
           endif
           stop
         endif

         goto 5
10       continue
         if (myid .eq. 0) then
           write(*,*) '***************************' 
         endif
         close(33)
c JPL add 052298
         if (myid .eq. 0) then
            if(io_ft .eq. 1)then
              call wavsfield(1)
            elseif(io_ft .eq. 2)then
              call wavsfield(-1)
            endif
           write(6,*) 'dx= ',dx
           write(6,*) 'dy= ',dy
           write(6,*) 'dz= ',1.0/real(nz-1)
           write(6,*) 'dt= ',dt
         endif
         prnt_last=0.0
c JPL end
c         if (itimechoice.eq.0) then
c           if (myid .eq. 0) then
c            open (unit=7,file='tape7_000',form='unformatted')
c            rewind (7)
c            write (7) nx1,j4,nz1
c            write (7) (((t(i,j,k),i=1,nx1),j=1,j4),k=1,nz1)
c            close (7)
c           endif

c           *** Output a cell-centered version as well.
c           if (myid .eq. 0) then
c            write(naampje,'(''temp_000'')')
c            open (unit=7,file=naampje,form='unformatted')
c            rewind (7)

c            call ttoft

c            write (7) nx,j3,nz
c            write (7) (((ft(i,j,k)/rat,i=1,nx),j=1,j3),k=1,nz)
c            close (7)

c          endif

c            rasroot='rast'
c            ifscal=0


c            call peras3(irast,nx-1,j3-1,nz-1,nx,j3,nz,ft,ifscal,
c     v              iscal,scal,rasroot)

    

c         else
c           if (myid .eq. 0) then

c            call ttoft
c            open (unit=7,file='tape7_000',form='unformatted')
c            rewind (7)
c            write (7) nx1,j4,nz1
c            write (7) (((ft(i,j,k)/rat,i=1,nx1),j=1,j4),k=1,nz1)
c            close (7)
c            rasroot='rast'
c            ifscal=0


c            call peras3(irast,isx,jsx,ksx,nx1,j4,nz1,ft,ifscal,
c     v              iscal,scal,rasroot)
c          endif
c         endif
         irast = irast+1
         trast = trast + dtrast
  
      else if (keyword(1:8).eq.'spectral') then
        if (myid .eq. 0) then
          if((icycle.eq.1).and.(io_power_vx.eq.1))call wavsfield(12)
          if((icycle.eq.1).and.(io_power_vy.eq.1))call wavsfield(13)
          if((icycle.eq.1).and.(io_power_vz.eq.1))call wavsfield(14)
          if(((time-prnt_last).gt.prntflds).and.(io_power_vx.eq.1))then
            call wavsfield(12)
          endif
          if(((time-prnt_last).gt.prntflds).and.(io_power_vy.eq.1))then
            call wavsfield(13)
          endif
          if(((time-prnt_last).gt.prntflds).and.(io_power_vz.eq.1))then
            call wavsfield(14)
          endif
        endif

      else if (keyword(1:4).eq.'flow') then
        if (myid .eq. 0) then
c         if((icycle.eq.1).and.(io_vis.eq.1))call wavsfield(9)
         if((icycle.eq.1).and.(io_vx.eq.1))call wavsfield(2)
         if((icycle.eq.1).and.(io_vy.eq.1))call wavsfield(3)
         if((icycle.eq.1).and.(io_vz.eq.1))call wavsfield(4) 
         if((icycle.eq.1).and.(io_v3.eq.1))call wavsfield(5)
         if(((time-prnt_last).gt.prntflds).and.(io_ft.ge.1))then
          if(io_ft .eq. 1)then
            call wavsfield(1)
          elseif(io_ft .eq. 2)then
            call wavsfield(-1)
          endif
         endif
c         if(((time-prnt_last).gt.prntflds).and.(io_vis.eq.1))then
c          call wavsfield(9)
c         endif
         if(((time-prnt_last).gt.prntflds).and.(io_vx.eq.1))then
          call wavsfield(2)
         endif
         if(((time-prnt_last).gt.prntflds).and.(io_vy.eq.1))then
          call wavsfield(3)
         endif
         if(((time-prnt_last).gt.prntflds).and.(io_vz.eq.1))then
          call wavsfield(4)
         endif
         if(((time-prnt_last).gt.prntflds).and.(io_v3.eq.1))then
          call wavsfield(5)
         endif
        endif
      continue

      else if (keyword(1:3).eq.'vis') then
        if (myid .eq. 0) then
         if((icycle.eq.1).and.(io_vis.eq.1))call wavsfield(9)
          if(((time-prnt_last).gt.prntflds).and.(io_vis.eq.1))then
          call wavsfield(9)
         endif
        continue 
       endif
      else if (keyword(1:5).eq.'geoid')then
        if (myid .eq. 0) then
         if(((icycle.eq.1).and.(io_geoid.eq.1)).or.
     $    (((time-prnt_last).gt.prntflds).and.(io_geoid.eq.1)))then
          ijob_gd=0
          call geoid(ijob_gd)
          if(j3.gt.1)call wavsfield(37)	
         endif
       endif
      else if (keyword(1:4).eq.'topo')then
        if (myid .eq. 0) then
         if(((icycle.eq.1).and.(io_topo.eq.1)).or.
     $    (((time-prnt_last).gt.prntflds).and.(io_topo.eq.1)))then
          ijob_gd=3
          call geoid(ijob_gd)
          if(j3.gt.1)call wavsfield(38)
         endif
        endif
      else if (keyword(1:3).eq.'tau') then
        if (myid .eq. 0) then
         if((io_xz.eq.1).or.(io_yz.eq.1))then
          ipttype_pttotau=0
          ijob_pttotau=0
          call pttotau(ipttype_pttotau,ijob_pttotau)
         if((icycle.eq.1).and.(io_xz.eq.1))call wavsfield(25)
         if((icycle.eq.1).and.(io_yz.eq.1))call wavsfield(26)
         if(((time-prnt_last).gt.prntflds).and.((io_xz.eq.1).or.
     & (io_yz.eq.1)))then
          call wavsfield(25)
          call wavsfield(26)
         endif
          else
          return
         endif
        endif
c      if((time-prnt_last).gt.prntflds)prnt_last=prnt_last+prntflds

      else if (keyword(1:6).eq.'tstats') then
c        if(itstats.eq.1)then
c         if((((time-prnt_last).gt.prntflds).or.
c     1 (icycle.eq.1)).and.(io_ft.ge.1))then
c         ijob_tempstats=2
c         call tempstats(ijob_tempstats)
c         else
         ijob_tempstats=1
         call tempstats(ijob_tempstats)
c         endif
c        endif

      else if (keyword(1:8).eq.'printout') then

        if((time-prnt_last).gt.prntflds)prnt_last=prnt_last+prntflds


      else if (keyword(1:4).eq.'heat') then
c        *** After each step of the time-integrator
       if (myid .eq. 0) then
         call output(7)

c        *** Time-series information
         call output(9)

       endif

      else if (keyword(1:3).eq.'end') then
       if (myid .eq. 0) then
         if(io_ft.ge.1)then
            if(io_ft .eq. 1)then
              call wavsfield(1)
            elseif(io_ft .eq. 2)then
              call wavsfield(-1)
            endif
         endif
         if(io_vx.eq.1)call wavsfield(2)
         if(io_vy.eq.1)call wavsfield(3)
         if(io_vz.eq.1)call wavsfield(4)
         if(io_v3.eq.1)call wavsfield(5)
         if(io_xz.eq.1)call wavsfield(25)
         if(io_yz.eq.1)call wavsfield(26)

         call output(10)

c        *** Finishing up

         if(io_geoid.eq.1) then
          ijob_gd=0
          call geoid(ijob_gd)
          if((io_geoid.eq.1).and.(j3.eq.1))call wavsfield(37)
         endif
         if(io_topo.eq.1) then
          ijob_gd=3
          call geoid(ijob_gd)
          if((io_topo.eq.1).and.(j3.eq.1))call wavsfield(38)
         endif
       endif
c
         if(itstats.eq.1) then
c          ijob_tempstats=2
          ijob_tempstats=1
          call tempstats(ijob_tempstats)
         endif

      else if (keyword(1:8).eq.'pttouv-1') then

       if (myid .eq. 0) then
        sfact = (xl*yl)/float(nxr*nyr)
c        write(6,*) 'pttouv-1',sfact,pfac,tfac
c       *** power spectrum of P/T
        if ((icycle.eq.1).or.((time-prnt_last).gt.prntflds))then
           write (17,80)icycle,time
           do ll = 1,nz
              akep(ll)   = zero
              aket(ll)   = zero
              akephv(ll) = zero
              do j = 1,nyr
                do i = 1,nxr
                  t1 = ((fli(i,j)*u(i,j,ll))**2+
     1            (flj(i,j)*u(i,j,ll))**2)
                  akep(ll)   = t1+akep(ll)
                  akephv(ll) = t1+(w(i,j,ll))**2+akephv(ll)
                  aket(ll)   = ((fli(i,j)*v(i,j,ll))**2+
     1                (flj(i,j)*v(i,j,ll))**2)+aket(ll)
                enddo
              enddo
              akep(ll) = sqrt(akep(ll))*sfact
              aket(ll) = sqrt(aket(ll))*sfact
              akephv(ll) = sqrt(akephv(ll))*sfact
              if (akephv(ll).eq.0.) then
                ake(ll) = 1e35
              else
                ake(ll) = aket(ll)/akephv(ll)
              endif
              write (17,90) z(ll),aket(ll),akep(ll),
     v              akephv(ll),ake(ll)
            enddo
        endif
       endif

      else if (keyword(1:8).eq.'pttouv-2') then

       if (myid .eq. 0) then
        sfact = (xl*yl)/float(nxr*nyr)
c        write(6,*) 'pttouv-2',sfact,pfac,tfac
        if ((icycle.eq.1).or.((time-prnt_last).gt.prntflds))then
          write (17,80)icycle,time
c       write(6,*) 'nz=',nz,nxr,nyr
c        do ll = 1,nz
c           do j = 1,nyr
c              do i = 1,nxr
c                 w(i,j,ll) = v3(i,j,ll)
c              enddo
c           enddo
c        enddo
c        call chvar (w,inx,nxr,lbd,jnx,nyr,mbd,nz,1)
        do ll = 1,nz
          akep(ll)   = zero
          aket(ll)   = zero
          akephv(ll) = zero
c          do j = 1,nyr
c             do i = 1,nxr
c               u(i,j,ll) = pfac*fli(i,j)*vp(i,j,ll)+
c     1              tfac*flj(i,j)*vtf(i,j,ll)
c               v(i,j,ll) = pfac*flj(i,j)*vp(i,j,ll)-
c     1              tfac*fli(i,j)*vtf(i,j,ll)
c             enddo
c          enddo

c         *** use 2D trapezoidal rule
c         *** corner points weights are 0.25
          do j=1,nyr,nyr
            do i=1,nxr,nxr
               akep(ll) = 0.25*((fli(i,j)*vp(i,j,ll))**2+
     1              (flj(i,j)*vp(i,j,ll))**2)+akep(ll)
               akephv(ll) = 0.25*((fli(i,j)*vp(i,j,ll))**2+
     1                (flj(i,j)*vp(i,j,ll))**2)+
     2                          (w(i,j,ll) **2)+akephv(ll)
               aket(ll) = 0.25*((fli(i,j)*vtf(i,j,ll))**2+
     1              (flj(i,j)*vtf(i,j,ll))**2)+ aket(ll)
             enddo
          enddo
c         *** side points 0.5
          do j=1,nyr,nyr
             do i=2,nxr-1
               akep(ll) = 0.5*((fli(i,j)*vp(i,j,ll))**2+
     1              (flj(i,j)*vp(i,j,ll))**2)+akep(ll)
               akephv(ll) = 0.5*((fli(i,j)*vp(i,j,ll))**2+
     1                (flj(i,j)*vp(i,j,ll))**2)+
     2                          (w(i,j,ll) **2)+akephv(ll)
               aket(ll) = 0.5*((fli(i,j)*vtf(i,j,ll))**2+
     1              (flj(i,j)*vtf(i,j,ll))**2)+ aket(ll)
             enddo
          enddo
          do j=2,nyr-1
             do i=1,nxr,nxr
               akep(ll) = 0.5*((fli(i,j)*vp(i,j,ll))**2+
     1              (flj(i,j)*vp(i,j,ll))**2)+akep(ll)
               akephv(ll) = 0.5*((fli(i,j)*vp(i,j,ll))**2+
     1                (flj(i,j)*vp(i,j,ll))**2)+
     2                          (w(i,j,ll) **2)+akephv(ll)
               aket(ll) = 0.5*((fli(i,j)*vtf(i,j,ll))**2+
     1              (flj(i,j)*vtf(i,j,ll))**2)+ aket(ll)
             enddo
          enddo
c         *** internal points
          do j=2,nyr-1
             do i=2,nxr-1
               akep(ll) = ((fli(i,j)*vp(i,j,ll))**2+
     1              (flj(i,j)*vp(i,j,ll))**2)+akep(ll)
               akephv(ll) = ((fli(i,j)*vp(i,j,ll))**2+
     1                (flj(i,j)*vp(i,j,ll))**2)+
     2                          (w(i,j,ll) **2)+akephv(ll)
               aket(ll) = ((fli(i,j)*vtf(i,j,ll))**2+
     1              (flj(i,j)*vtf(i,j,ll))**2)+ aket(ll)
             enddo
          enddo

          akep  (ll) = pfac*sqrt(  akep(ll))*sfact
          akephv(ll) = pfac*sqrt(akephv(ll))*sfact
          aket  (ll) = tfac*sqrt(  aket(ll))*sfact
          if (akephv(ll).eq.0.) then
            ake(ll) = 1e35
          else
            ake(ll) = aket(ll)/akephv(ll)
          endif
          write (17,90) z(ll),aket(ll),akep(ll),akephv(ll),ake(ll) 
        enddo
        endif
       endif
      else
         write(6,*) 'PERROR(userout): unknown keyword'
         write(6,*) '  keyword = ',keyword
         stop 
      endif

   80 format (i8,e14.7,/,
     1"PT coupling fac. ", /,15x,
     2"   z        aket        akep       akephv    ake")
   90 format (11x,5(e10.3,2x))

      return
      end
