c $Log:   /pvcs.config/mc3dvv/src/userout.F_a  $
c
c **********************************************************************
c *   USEROUT
c *
c *   General output routine, which may be adapted at the user level
c *   Provides windows into the computational part of MC3D by
c *   reverse communication.
c *
c *   Function:
c *   Reads the input file 'input.io' to define the variables 'fldsout' 
c *   (specifies time interval between field dumps) and 'fieldname' (the
c *   fields which are to be output). In combination with the subroutine 
c *   argument ('keyword'), these define what is printed out and when.
c *
c *   'input.io' should contain (each item left justified):
c *            First   - a value for 'prntflds'
c *            Then    - a list of the fields which are to be output
c *            Finally - the keyword "end" should be used to end the list.
c *
c *            Currently, the fieldnames which may be listed for output 
c *            are:
c *               'temperature'
c *  ********     'velocity' (producing dumps of u,v and w) **********
c *  ******** very communication intensive - only use if really needed 
c *               'tstats' (produces dumps of temperature profiles)
c *               'vstats'
c *   
c *   Input argument:
c *       KEYWORD - Character string, describes at which point USEROUT
c *                 is called (from Mantle.F):
c *                 'init'   After initialisation 
c *                 'flow'   After solution for velocity/stress
c *                          (subroutine flow)
c *                 'heat'   After one step of the time-integrator
c *                 'end'    Just before the program finishes
c * DaN - removed various, now defunct options:
c *                 'spectral' During flow (following call to pttouv)
c *                          That is, after solution for Spectral
c *                          coefficients for u and v but prior to
c *                          conversion of u and v to Cartesian space 
c *                 'geoid'  Output geoid at the same time as T-field
c *                 'topo'   Output surface topography at the same time
c *                          as the T-field
c *                 'visco'  At the end of each viscosity calculation
c *                          (subroutine viscos)
c *                 'pttouv-1' Called from pttouv for calculation of
c *                          the power spectrum of P/T
c *                 'pttouv-2' Called from pttouv for calculation of
c *                          the power spectrum of P/T
c *                 'tau'    Follows "flow" call and produces output of tau_ij
c *
c **********************************************************************
c * DaN - this drastically reduced version calls (the appended) 
c *       subroutines:
c *               1. printT
c *               2. printvel
c *               3. printout
c *               4. tapes_out
c *               5. namefile1,
c *  that, between them, totally replace calls to OUTPUT and WAVSFIELD 
c **********************************************************************
c **********************************************************************

      subroutine userout(keyword)
      use module_precision
      use module_param
      use module_a3
      use module_userout
#ifdef MPI
      include 'mpif.h'
#endif
      common/io_stats/io_vel, ivstats
      character*10 keyword
      character*20 fieldname
c
#ifdef MPI
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )
#else
      myid=0
#endif

      if (keyword(1:4).eq.'init') then
      io_ft=0
      io_vel=0
      itstats=0
      ivstats=0

c **********************************************************************
c *      Read input info from 'input.io' and output initial T field
c **********************************************************************
         open (unit=33,file='input.io',status='old',
     1    form='formatted',iostat=ioerr)
         if(ioerr.ne.0)then
           if (myid .eq. 0) then
             write(*,*) '***********************'
             write(*,*) ' input.io not specified'
             write(*,*) 'no AVS output requested'
             write(*,*) '***********************'
           endif
           prntflds=1000.0
           goto 10
         endif
         read (33,*) prntflds
         if (myid .eq. 0) then
            write(*,*) '***************************'
            write(*,*) 'The following fields will'
            write(*,*) 'be output in AVS format at'
            write(*,*) 'intervals of:',prntflds
         endif
5        read (33,'(a)') fieldname

         if(fieldname(1:3).eq.'end')then
           goto 10
         elseif(fieldname(1:3).eq.'tem')then
             io_ft=1
           if (myid .eq. 0) write(*,*) 'temperature'
         elseif(fieldname(1:3).eq.'vel')then
           io_vel=1
           if (myid .eq. 0) then
             write(*,*) 'vx velocity magnitudes'
             write(*,*) 'vy velocity magnitudes'
             write(*,*) 'vz velocity magnitudes'
           endif
         elseif(fieldname(1:3).eq.'tst') then
           itstats=1
           if (myid .eq. 0) then
             write(*,*) 'T-field stats will be written by tempstats'
           endif
         elseif(fieldname(1:3).eq.'vst') then
           ivstats=1
           if (myid .eq. 0) then
             write(*,*) 'V-field stats will be written by velstats'
           endif
         else
           if (myid .eq. 0) then
             write(*,*) 'Requested AVS field output of'
             write(*,*) fieldname(1:20), 'not recognized' 
             write(*,*) 'the program will stop.'
           endif
           stop
         endif

         goto 5
10       continue
         close(33)

         if (myid .eq. 0) then
           write(*,*) '***************************'
           write(6,*) 'nx = ',nx
           write(6,*) 'ny = ',ny
           write(6,*) 'nz = ',nz
         endif

         if (myid .eq. 0) then
           if(io_ft .eq. 1) call printT

           write(6,*) 'dx= ',dx
           write(6,*) 'dy= ',dy
           write(6,*) 'dz= ',1.0d0/real(nz-1)
           write(6,*) 'dt= ',dt
         endif
         prnt_last=0.0
  
      else if (keyword(1:4).eq.'flow') then
************************************************************************
*  Output temperature and velocity fields at alloted time if required  *
************************************************************************
        if(((time-prnt_last).gt.prntflds).and.(io_ft.eq.1))then
          if(myid.eq.0) call printT
        endif

        if(io_vel.eq.1) then
          if((icycle.eq.1).or.((time-prnt_last).gt.prntflds))then
            call printvel(1)
            call printvel(2)
            call printvel(3)
          endif
        endif
       
      else if (keyword(1:6).eq.'tstats') then
************************************************************************
*       Write temperature statistics                                   *
************************************************************************
         if((((time-prnt_last).gt.prntflds).or.
     1          (icycle.eq.1)).and.(itstats.eq.1))then
            ijob_tempstats=2
            call tempstats(ijob_tempstats)
         else
            ijob_tempstats=1
            call tempstats(ijob_tempstats)
         endif

      else if (keyword(1:6).eq.'vstats') then
************************************************************************
*       Write velocity statistics                                      *
************************************************************************
        if(ivstats.eq.1) call velstats

      else if (keyword(1:8).eq.'printout') then
************************************************************************
*       update printout timing					       *
************************************************************************
        if((time-prnt_last).gt.prntflds)prnt_last=prnt_last+prntflds


      else if (keyword(1:4).eq.'heat') then
************************************************************************
*       output time-series information                                 *
************************************************************************      
        iendit=0
        if (myid .eq. 0) call tapes_out(iendit)

      else if (keyword(1:3).eq.'end') then
************************************************************************
*       final output before finish                                     *
************************************************************************

        if (myid .eq. 0) then
          iendit=1
          call tapes_out(iendit)
          if(io_ft .eq. 1) call printT
        endif
        if(io_vel.eq.1)then 
          call printvel(1)
          call printvel(2)
          call printvel(3)
        endif
c
        if(itstats.eq.1) then
          ijob_tempstats=2
          call tempstats(ijob_tempstats)
        endif

      else
************************************************************************
*        keyword unrecognised                                          *
************************************************************************
         write(6,*) 'PERROR(userout_DaN): ',keyword,' = unknown keyword'
         stop 
      endif

      return
      end

************************************************************************

      subroutine printT
c  *********************************************************************
c  *    The following routine controls temperature output              *
c  *    and has been bastardised from WAVSFIELD(1)                     *
c  *********************************************************************
      use module_precision
      use module_param
      use module_a3
      use module_a12
      use module_spwork
      character*25 root
      if(icycle.eq.0) icalltemp = 0
      
      icalltemp=icalltemp+1

      nxr_save=nxr
      nyr_save=nyr
      nxr=nx-1
      nyr=ny-1
      call ttoft_spwk  
      nxr=nxr_save
      nyr=nyr_save

      do k = 1,kmx
        do j = 1,jmx
          do i = 1,imx
            spwk(i,j,k) = spwk(i,j,k)/rat
          enddo
        enddo
      enddo
 
      root='ft'
      call printout(root)

      return 
      end

************************************************************************

      subroutine printvel(ivel)
c  **********************************************************************
c  *     This routine prepares a velocity field for printing. The value *
c  *     of 'ivel' given (1, 2, or 3) controls which field (u, v or w)  *
c  *     is sent to subroutine 'printout' (in array 'spwk') for writing *
c  **********************************************************************
      use module_precision
      use module_param
      use module_a2
      use module_a3
      use module_spwork
#ifdef MPI
      include 'mpif.h'
#endif

      character*25 root
#ifdef MPI
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )
      call MPI_COMM_SIZE( MPI_COMM_WORLD, numprocs, ierr )
#else
      numprocs=1
      myid=0
#endif
      call mpilayer(nz,numprocs,myid,izstart,izstop)
      izstart=max(izstart - 2,1)
      izstop =min(izstop + 2, nz)

      if(ivel.eq.1) then
        root='vx'
        do i=1,inx
          do j=1,jnx
            do k=izstart,izstop
              spwk(i,j,k) = u(i,j,k-izstart+1)
            enddo
          enddo
        enddo
      elseif(ivel.eq.2) then
        root='vy'
        do i=1,inx
          do j=1,jnx
            do k=izstart,izstop
              spwk(i,j,k) = v(i,j,k-izstart+1)
            enddo
          enddo
        enddo
      elseif(ivel.eq.3) then
        root='vz'
        do i=1,inx
          do j=1,jnx
            do k=izstart,izstop
              spwk(i,j,k) = w(i,j,k-izstart+1)
            enddo
          enddo
        enddo
      endif
#ifdef MPI
      call mpiprint(-1,nx,j3,nz,spwk,inx,jnx,knx)
#endif

      if(myid.eq.0) call printout(root)
      return
      end

************************************************************************

      subroutine printout(root)
c  *********************************************************************
c  *    Opens a file defined by the character variable 'root' and the  *
c  *    number of times it has been asked to print out ('icalltemp')   *
c  *    e.g. ft10001 on the first timestep, where 'ft', 'vx', 'vy' and *
c  *    'vz' are the currently availble options. It then writes        *
c  *    the current field stored in 'spwk' to that file, and closes    *
c  *    the file.                                                      *
c  ********************************************************************* 
      use module_precision
      use module_param
      use module_a3
      use module_spwork

      character*7 fname
      character*25 root
      lu_out = 55 
      itail = 0
      iformat = 1

      call namefile1(icalltemp,lu_out,iformat,root,itail,fname)
      write(lu_out) 
     & ((((real(spwk(i,j,k),kind=4)),i=1,imx), j=1,jmx), k=1,kmx) 
      close(lu_out)
      write(6,*) '********* writing field to ',fname,' *********'

      return
      end

************************************************************************

      subroutine tapes_out(iendit)
c  *********************************************************************
c  *   Outputs time-series information each iteration after heatflo.   *
c  *   Also writes restart files every 500 time-steps (the             * 
c  *   communications are expensive (dealt with in mantle.f))          *
c  *   and on the final time-step.                                     *
c  *********************************************************************

      use module_precision
      use module_param
      use module_a1
      use module_a3
      use module_nu

      data qfso, znu1o, energko, tznuo / 0., 0., 0., 0. /

c     Output of Nu, Vrms as calculated in heatflo
      if(iendit.eq.0) then
        write (6,78) icycle,time,znu(1),vrms
78      format (i8,4e15.7)

        if (icycle.eq.1) then
          open (unit=34,file='tape34',form='formatted')
          open (unit=35,file='tape35',form='formatted')
        endif

c     Write out time-series information
        qfs = znu(nz)
        write(35,80) icycle,time,qfs,znu(1),vrms,tznu
        dqfs = qfso-qfs
        dznu1 = znu1o-znu(1)
        denergk = energko-energk
        dtznu = tznuo-tznu
        write(34,80) icycle,dt,dqfs,dznu1,denergk,dtznu
80      format (i8,5e15.7)
        qfso=qfs
        znu1o=znu(1)
        energko=energk
        tznuo=tznu
        call flush(35)
        call flush(34)
      endif

c   Write restart file every 500 time-steps (write 2 incase of
c   crash whilst writing)
      if (((icycle/500)*500.eq.icycle)) then
        if (((icycle/1000)*1000.eq.icycle)) then
          open (unit=7,file='tape7_restart',form='unformatted')
          rewind (7)
          write (7) nx1,j4,nz1
          write (7) (((t(i,j,k),i=1,nx1),j=1,j4),k=1,nz1)
          close (7)
        else
          open (unit=9,file='tape9_restart',form='unformatted')
          rewind (9)
          write (9) nx1,j4,nz1
          write (9) (((t(i,j,k),i=1,nx1),j=1,j4),k=1,nz1)
          close (9)
        endif
      endif
     
c     On final time-step write restart file
      if (iendit.eq.1) then
        open (unit=7,file='tape7s',form='unformatted')
        rewind (7)
        write (7) nx1,j4,nz1
        write (7) (((t(i,j,k),i=1,nx1),j=1,j4),k=1,nz1)
        close (7)
      endif
      
      return
      end

************************************************************************

      subroutine namefile1(icall,lund,iformat,root,itail,fname)
c
c     icall= the number of times this routine has been called.
c            This value is used in putting a suffix on the output
c            file.  The file names will have sequential suffixes.  Suffix
c            is between 10001 and 99999 if ical .gt. 0 and .lt. 89999.
c            You are on your own if you want more than 90000 files!
c
c     lund = logical unit number of file to be opened.
c
c     root = file's name root to which suffix will be added.
c
c     itail = file's name tail will be added to the end of the file name.
c
c     iformat = 1 unformatted file is opened
c               2 formatted file is opened
c
      character*60 fname
      character*25 root
      character*25 tail
      character*5 ch5
c      fname='                                                   '
c            12345678901234567890123456789012345678901234567890
c
c       NAME OUTPUT FILE
c
      if(itail.eq.1)then
       tail = '.fld'
      else
       tail = ' '
      endif

      do 91 i=1,25
        if(tail(i:i).eq.' ') goto 2
   91 continue

    2 nchar_tail=i-1
c       if(iformat .eq. 2) then
c         print *,root,tail,i
c         endif

      do 95 i=1,25
        if(root(i:i).eq.' ') goto 3
   95 continue
    3 nchar_root=i-1

      ncall=icall+10000
      write(ch5,5) ncall
    5 format (i5)
      do 96 i=1,nchar_root
        fname(i:i)=root(i:i)
   96 continue
      do 97 i=1,5
        fname(nchar_root+i:nchar_root+i)=ch5(i:i)
   97 continue
      do 98 i=1,nchar_tail
        fname(nchar_root+5+i:nchar_root+5+i)=tail(i:i)
   98 continue
c
c       OPEN TO OUTPUT FILE
c

      if (iformat .eq. 1) then
      open(unit=lund, file=fname, form='unformatted')
      elseif (iformat .eq. 2) then
      open(unit=lund, file=fname, form='formatted')
      else
      continue
      endif
c
      return
      end

