      subroutine raster3(icall,
     1                   inx,
     2                   jnx,
     3                   knx,
     4                   nx,
     5                   ny,
     6                   nz,
     7                   t,
     8                   ifscal,
     9                   iscal,
     1                   scal,
     1                   root)
c
c     Each call to RASTER will produce an output file.
c
c     icall= the number of times this routine has been called.
c            This value is used in putting a suffix on the output
c            file. The file names will have sequential suffixes so they can
c            be easily read by imagetool in animate mode. Suffix will
c            be between 101 and 999 if icall .gt. 0 and .lt. 899.
c            You are on your own if you want more than 900 files!
c
c     t=    two dimensional array of real numbers to be
c           sent to a raster file for input into imagetool.
c           This routine assumes you have scaled the array
c           to have values between 0 and 1. 
c
c     inx=  first dimension of array t
c
c     jnx=  second dimension of array t   (note: knx not nec = inx)
c
c     knx=  third dimension of array t   (note: knx not nec = inx)
c
c     nx= number of data in x direction
c
c     ny= number of data in x direction
c
c     nz= number of data in z direction
c
c     ifscal = 0 auto scale of input values
c            = 1 max min scale is set in 2D aray scal, values
c              outside of scale will clip
c
c     scal   = 2d array with REAL min and max values
c
c     iscal   = 2d array with INTEGER min and max values
c                 10-250
c
c     root=raster file's name root to which suffix will be added.
c          must be declared as character*72 on MAIN.
c
c     The maximum size is 256x256=65536 data(note: knx not nec = inx). 
c     32x32x32 apx = 32000  
c               now set to 516x129=66564
c               now set to 33x33x129=140481
c     This may be changed in
c     the parameter statement below however the dimension of BUF in
c     C program cior.c must also be changed.
c
      parameter(max=140481)
      dimension t(inx,jnx,knx)
      dimension iscal(2)
      dimension scal(2)
      integer*4 lu
      integer*2 itemp
      character*1 a1(2) 
      pointer(pa2,a2)
      character*1 a2(1)

      character*80 fname
      character*72 root
      character*5 ch5
      common/name0/fname
      equivalence (a1,itemp)

      pa2 = mallocf(inx*jnx*knx)

      if(ifscal .eq. 0)then
c
c     cheak range of input
c 
      rmax=-1.e20
      rmin=1.e20     
      do 80 k=1,nz
       do 80 j=1,ny
        do 80 i=1,nx
           rmax=amax1(rmax,t(i,j,k))
           rmin=amin1(rmin,t(i,j,k))
   80 continue
      scal(1)=rmin
      scal(2)=rmax
      else
         rmin=scal(1)
         rmax=scal(2)
      endif
c
c     scale as integer between 20 and 250
c     assumine that t is between rmin and rmax
c     outside the range will be clipped
c     compress to values between imin and imax
c     scale using y=ax+b
c     where
c       a=(imax-imin)/(rmax-rmin)
c       b=(rmax*imin-rmin*imax)/(rmax-rmin)
c     write into a2 as 8 bit integer
c
      imin=20
      imax=250
      iscal(1)=20
      iscal(2)=250
      a=float(imax-imin)/(rmax-rmin)
      b=(rmax*float(imin)-rmin*float(imax))/(rmax-rmin)
      icount=1

      write(6,*) 'rmax, rmin: ',rmax,rmin,a,b

      if(ifscal .eq. 0)then
        do 90 k=1,nz
         do 90 j=1,ny
           do 90 i=1,nx
             write(6,*) 'raster3: ',icount,itemp,t(i,j,k)
             itemp=a*t(i,j,k)+b
             a2(icount)=a1(2)
             icount=icount+1
   90   continue
      else
        do 190 k=1,nz
         do 190 j=1,ny
           do 190 i=1,nx
             if(t(i,j,k) .ge. rmax)then
                itemp=a*rmax+b
             else if(t(i,j,k) .le. rmin)then
                itemp=a*rmin+b
             else
                itemp=a*t(i,j,k)+b
c                print *,itemp
             endif
             a2(icount)=a1(2)
             icount=icount+1
  190   continue
      endif
      
      nbyte=nx*ny*nz
      nbyte=icount-1
c
c     name output file
c
      do 95 i=1,72
        if(root(i:i).eq.' ') goto 3
 95     continue
 3      namlen=i-1
      ncall=icall+10000
      write(ch5,5)ncall
    5 format(i5)
      do 96 i=1,namlen
       fname(i:i)=root(i:i)
  96  continue
      do 97 i=1,5
       fname(namlen+i:namlen+i)=ch5(i:i)
   97 continue
c
c     write to output file
c
      call ropen(lu,1)
      call rwrite(lu,a2,max,nbyte)
      call rclose(lu)
c

      call free(pa2)

  999 format('ERROR RASTER3:Output dimension bigger than work array',/,
     1 'inx = ',i6,' jnx = ',i6,' knx = ',i6,/,
     2  ' inx*jnx*knx = ',i10,' max = ',i10)
      return
      end
