      subroutine xerror(messg,nmessg,nerr,level)                        xerror 4
c***begin prologue  xerror                                              xerror 5
c***date written   790801   (yymmdd)                                    xerror 6
c***revision date  861211   (yymmdd)                                    bb816249
c***category no.  r3c                                                   bb816250
c***keywords  library=slatec(xerror),type=all(xerror-a),error           bb816251
c***author  jones, r. e., (snla)                                        bb816252
c***purpose  process an error (diagnostic) message.                     bb816253
c***description                                                         xerror12
c                                                                       xerror13
c     abstract                                                          xerror14
c        xerror processes a diagnostic message, in a manner             xerror15
c        determined by the value of level and the current value         xerror16
c        of the library error control flag, kontrl.                     xerror17
c        (see subroutine xsetf for details.)                            xerror18
c                                                                       xerror19
c     description of parameters                                         xerror20
c      --input--                                                        xerror21
c        messg - the hollerith message to be processed, containing      xerror22
c                no more than 72 characters.                            xerror23
c        nmessg- the actual number of characters in messg.              xerror24
c        nerr  - the error number associated with this message.         xerror25
c                nerr must not be zero.                                 xerror26
c        level - error category.                                        xerror27
c                =2 means this is an unconditionally fatal error.       xerror28
c                =1 means this is a recoverable error.  (i.e., it is    xerror29
c                   non-fatal if xsetf has been appropriately called.)  xerror30
c                =0 means this is a warning message only.               xerror31
c                =-1 means this is a warning message which is to be     xerror32
c                   printed at most once, regardless of how many        xerror33
c                   times this call is executed.                        xerror34
c                                                                       xerror35
c     examples                                                          xerror36
c        call xerror('smooth -- num was zero.',23,1,2)                  bb816254
c        call xerror('integ  -- less than full accuracy achieved.',     bb816255
c    1                43,2,1)                                           bb816256
c        call xerror('rooter -- actual zero of f found before interval fbb816257
c    1ully collapsed.',65,3,0)                                          bb816258
c        call xerror('exp    -- underflows being set to zero.',39,1,-1) bb816259
c                                                                       bb816260
c     written by ron jones, with slatec common math library subcommitteebb816261
c***references  jones r.e., kahaner d.k., 'xerror, the slatec error-    bb816262
c                 handling package', sand82-0800, sandia laboratories,  bb816263
c                 1982.                                                 bb816264
c***routines called  xerrwv                                             bb816265
c***end prologue  xerror                                                bb816266
      character*(*) messg                                               bb816267
c***first executable statement  xerror                                  xerror53
      call xerrwv(messg,nmessg,nerr,level,0,0,0,0,0.,0.)                xerror54
      return                                                            xerror55
      end                                                               xerror56
      subroutine xerrwv(messg,nmessg,nerr,level,ni,i1,i2,nr,r1,r2)      xerrwv 4
c***begin prologue  xerrwv                                              xerrwv 5
c***date written   800319   (yymmdd)                                    xerrwv 6
c***revision date  861211   (yymmdd)                                    bb816268
c***category no.  r3c                                                   bb816269
c***keywords  library=slatec(xerror),type=all(xerrwv-a),error           bb816270
c***author  jones, r. e., (snla)                                        bb816271
c***purpose  process an error message allowing 2 integer and 2 real     bb816272
c            values to be included in the message.                      xerrwv12
c***description                                                         xerrwv13
c                                                                       xerrwv14
c     abstract                                                          xerrwv15
c        xerrwv processes a diagnostic message, in a manner             xerrwv16
c        determined by the value of level and the current value         xerrwv17
c        of the library error control flag, kontrl.                     xerrwv18
c        (see subroutine xsetf for details.)                            xerrwv19
c        in addition, up to two integer values and two real             xerrwv20
c        values may be printed along with the message.                  xerrwv21
c                                                                       xerrwv22
c     description of parameters                                         xerrwv23
c      --input--                                                        xerrwv24
c        messg - the hollerith message to be processed.                 xerrwv25
c        nmessg- the actual number of characters in messg.              xerrwv26
c        nerr  - the error number associated with this message.         xerrwv27
c                nerr must not be zero.                                 xerrwv28
c        level - error category.                                        xerrwv29
c                =2 means this is an unconditionally fatal error.       xerrwv30
c                =1 means this is a recoverable error.  (i.e., it is    xerrwv31
c                   non-fatal if xsetf has been appropriately called.)  xerrwv32
c                =0 means this is a warning message only.               xerrwv33
c                =-1 means this is a warning message which is to be     xerrwv34
c                   printed at most once, regardless of how many        xerrwv35
c                   times this call is executed.                        xerrwv36
c        ni    - number of integer values to be printed. (0 to 2)       xerrwv37
c        i1    - first integer value.                                   xerrwv38
c        i2    - second integer value.                                  xerrwv39
c        nr    - number of real values to be printed. (0 to 2)          xerrwv40
c        r1    - first real value.                                      xerrwv41
c        r2    - second real value.                                     xerrwv42
c                                                                       xerrwv43
c     examples                                                          xerrwv44
c        call xerrwv('smooth -- num (=i1) was zero.',29,1,2,            bb816273
c    1   1,num,0,0,0.,0.)                                               bb816274
c        call xerrwv('quadxy -- requested error (r1) less than minimum (bb816275
c    1r2).,54,77,1,0,0,0,2,errreq,errmin)                               bb816276
c                                                                       bb816277
c     latest revision ---  1 august 1985                                bb816278
c     written by ron jones, with slatec common math library subcommitteebb816279
c***references  jones r.e., kahaner d.k., 'xerror, the slatec error-    bb816280
c                 handling package', sand82-0800, sandia laboratories,  bb816281
c                 1982.                                                 bb816282
c***routines called  fdump,i1mach,j4save,xerabt,xerctl,xerprt,xersav,   bb816283
c                    xgetua                                             bb816284
c***end prologue  xerrwv                                                bb816285
      character*(*) messg                                               bb816286
      character*20 lfirst                                               bb816287
      character*37 form                                                 bb816288
      dimension lun(5)                                                  bb816289
c     get flags                                                         bb816290
c***first executable statement  xerrwv                                  bb816291
      lkntrl = j4save(2,0,.false.)                                      bb816292
      maxmes = j4save(4,0,.false.)                                      bb816293
c     check for valid input                                             bb816294
      if ((nmessg.gt.0).and.(nerr.ne.0).and.                            bb816295
     1    (level.ge.(-1)).and.(level.le.2)) go to 10                    bb816296
         if (lkntrl.gt.0) call xerprt('fatal error in...',17)           bb816297
         call xerprt('xerror -- invalid input',23)                      bb816298
         if (lkntrl.gt.0) call fdump                                    bb816299
         if (lkntrl.gt.0) call xerprt('job abort due to fatal error.',  bb816300
     1  29)                                                             bb816301
         if (lkntrl.gt.0) call xersav(' ',0,0,0,kdummy)                 bb816302
         call xerabt('xerror -- invalid input',23)                      bb816303
         return                                                         bb816304
   10 continue                                                          bb816305
c     record message                                                    bb816306
      junk = j4save(1,nerr,.true.)                                      bb816307
      call xersav(messg,nmessg,nerr,level,kount)                        bb816308
c     let user override                                                 bb816309
      lfirst = messg                                                    bb816310
      lmessg = nmessg                                                   xerrwv80
      lerr = nerr                                                       xerrwv81
      llevel = level                                                    xerrwv82
      call xerctl(lfirst,lmessg,lerr,llevel,lkntrl)                     xerrwv83
c     reset to original values                                          xerrwv84
      lmessg = nmessg                                                   xerrwv85
      lerr = nerr                                                       xerrwv86
      llevel = level                                                    xerrwv87
      lkntrl = max0(-2,min0(2,lkntrl))                                  xerrwv88
      mkntrl = iabs(lkntrl)                                             xerrwv89
c     decide whether to print message                                   xerrwv90
      if ((llevel.lt.2).and.(lkntrl.eq.0)) go to 100                    xerrwv91
      if (((llevel.eq.(-1)).and.(kount.gt.min0(1,maxmes)))              xerrwv92
     1.or.((llevel.eq.0)   .and.(kount.gt.maxmes))                      xerrwv93
     2.or.((llevel.eq.1)   .and.(kount.gt.maxmes).and.(mkntrl.eq.1))    xerrwv94
     3.or.((llevel.eq.2)   .and.(kount.gt.max0(1,maxmes)))) go to 100   xerrwv95
         if (lkntrl.le.0) go to 20                                      xerrwv96
            call xerprt(' ',1)                                          bb816311
c           introduction                                                xerrwv98
            if (llevel.eq.(-1)) call xerprt                             xerrwv99
     1('warning message...this message will only be printed once.',57)  bb816312
            if (llevel.eq.0) call xerprt('warning in...',13)            bb816313
            if (llevel.eq.1) call xerprt                                bb816314
     1      ('recoverable error in...',23)                              bb816315
            if (llevel.eq.2) call xerprt('fatal error in...',17)        bb816316
   20    continue                                                       bb816317
c        message                                                        bb816318
         call xerprt(messg,lmessg)                                      bb816319
         call xgetua(lun,nunit)                                         bb816320
         isizei = log10(float(i1mach(9))) + 1.0                         bb816321
         isizef = log10(float(i1mach(10))**i1mach(11)) + 1.0            bb816322
         do 50 kunit=1,nunit                                            bb816323
            iunit = lun(kunit)                                          bb816324
            if (iunit.eq.0) iunit = i1mach(4)                           bb816325
            do 22 i=1,min(ni,2)                                         bb816326
               write (form,21) i,isizei                                 bb816327
   21          format ('(11x,21hin above message, i',i1,'=,i',i2,')   ')bb816328
               if (i.eq.1) write (iunit,form) i1                        bb816329
               if (i.eq.2) write (iunit,form) i2                        bb816330
   22       continue                                                    bb816331
            do 24 i=1,min(nr,2)                                         bb816332
               write (form,23) i,isizef+10,isizef                       bb816333
   23          format ('(11x,21hin above message, r',i1,'=,e',          bb816334
     1         i2,'.',i2,')')                                           bb816335
               if (i.eq.1) write (iunit,form) r1                        bb816336
               if (i.eq.2) write (iunit,form) r2                        bb816337
   24       continue                                                    bb816338
            if (lkntrl.le.0) go to 40                                   bb816339
c              error number                                             bb816340
               write (iunit,30) lerr                                    bb816341
   30          format (15h error number =,i10)                          bb816342
   40       continue                                                    bb816343
   50    continue                                                       bb816344
c        trace-back                                                     bb816345
         if (lkntrl.gt.0) call fdump                                    bb816346
  100 continue                                                          bb816347
      ifatal = 0                                                        bb816348
      if ((llevel.eq.2).or.((llevel.eq.1).and.(mkntrl.eq.2)))           bb816349
     1ifatal = 1                                                        bb816350
c     quit here if message is not fatal                                 bb816351
      if (ifatal.le.0) return                                           bb816352
      if ((lkntrl.le.0).or.(kount.gt.max0(1,maxmes))) go to 120         bb816353
c        print reason for abort                                         bb816354
         if (llevel.eq.1) call xerprt                                   bb816355
     1   ('job abort due to unrecovered error.',35)                     bb816356
         if (llevel.eq.2) call xerprt                                   bb816357
     1   ('job abort due to fatal error.',29)                           bb816358
c        print error summary                                            bb816359
         call xersav(' ',-1,0,0,kdummy)                                 bb816360
  120 continue                                                          xerrw142
c     abort                                                             xerrw143
      if ((llevel.eq.2).and.(kount.gt.max0(1,maxmes))) lmessg = 0       xerrw144
      call xerabt(messg,lmessg)                                         xerrw145
      return                                                            xerrw146
      end                                                               xerrw147
      function j4save(iwhich,ivalue,iset)                               j4save 4
c***begin prologue  j4save                                              j4save 5
c***refer to  xerror                                                    bb815658
c***routines called  (none)                                             bb815659
c***description                                                         j4save13
c                                                                       j4save14
c     abstract                                                          j4save15
c        j4save saves and recalls several global variables needed       j4save16
c        by the library error handling routines.                        j4save17
c                                                                       j4save18
c     description of parameters                                         j4save19
c      --input--                                                        j4save20
c        iwhich - index of item desired.                                bb815660
c                = 1 refers to current error number.                    bb815661
c                = 2 refers to current error control flag.              bb815662
c                 = 3 refers to current unit number to which error      bb815663
c                    messages are to be sent.  (0 means use standard.)  bb815664
c                 = 4 refers to the maximum number of times any         bb815665
c                     message is to be printed (as set by xermax).      bb815666
c                 = 5 refers to the total number of units to which      bb815667
c                     each error message is to be written.              bb815668
c                 = 6 refers to the 2nd unit for error messages         bb815669
c                 = 7 refers to the 3rd unit for error messages         bb815670
c                 = 8 refers to the 4th unit for error messages         bb815671
c                 = 9 refers to the 5th unit for error messages         bb815672
c        ivalue - the value to be set for the iwhich-th parameter,      bb815673
c                 if iset is .true. .                                   bb815674
c        iset   - if iset=.true., the iwhich-th parameter will be       bb815675
c                 given the value, ivalue.  if iset=.false., the        bb815676
c                 iwhich-th parameter will be unchanged, and ivalue     bb815677
c                 is a dummy parameter.                                 bb815678
c      --output--                                                       bb815679
c        the (old) value of the iwhich-th parameter will be returned    bb815680
c        in the function value, j4save.                                 bb815681
c                                                                       bb815682
c     written by ron jones, with slatec common math library subcommitteebb815683
c    adapted from bell laboratories port library error handler          bb815684
c     latest revision ---  1 august 1985                                bb815685
c***references  jones r.e., kahaner d.k., 'xerror, the slatec error-    bb815686
c                 handling package', sand82-0800, sandia laboratories,  bb815687
c                 1982.                                                 bb815688
c***end prologue  j4save                                                bb815689
      logical iset                                                      bb815690
      integer iparam(9)                                                 bb815691
      save iparam                                                       bb815692
      data iparam(1),iparam(2),iparam(3),iparam(4)/0,2,0,10/            bb815693
      data iparam(5)/1/                                                 j4save56
      data iparam(6),iparam(7),iparam(8),iparam(9)/0,0,0,0/             j4save57
c***first executable statement  j4save                                  j4save58
      j4save = iparam(iwhich)                                           j4save59
      if (iset) iparam(iwhich) = ivalue                                 j4save60
      return                                                            j4save61
      end                                                               j4save62
      subroutine xerabt(messg,nmessg)                                   xerabt 4
c***begin prologue  xerabt                                              xerabt 5
c***date written   790801   (yymmdd)                                    xerabt 6
c***revision date  861211   (yymmdd)                                    bb816160
c***category no.  r3c                                                   bb816161
c***keywords  library=slatec(xerror),type=all(xerabt-a),error           bb816162
c***author  jones, r. e., (snla)                                        bb816163
c***purpose  abort program execution and print error message.           bb816164
c***description                                                         xerabt12
c                                                                       xerabt13
c     abstract                                                          xerabt14
c        ***note*** machine dependent routine                           xerabt15
c        xerabt aborts the execution of the program.                    xerabt16
c        the error message causing the abort is given in the calling    xerabt17
c        sequence, in case one needs it for printing on a dayfile,      xerabt18
c        for example.                                                   xerabt19
c                                                                       xerabt20
c     description of parameters                                         xerabt21
c        messg and nmessg are as in xerror, except that nmessg may      xerabt22
c        be zero, in which case no message is being supplied.           xerabt23
c                                                                       xerabt24
c     written by ron jones, with slatec common math library subcommitteexerabt25
c     latest revision ---  1 august 1982                                bb816165
c***references  jones r.e., kahaner d.k., 'xerror, the slatec error-    bb816166
c                 handling package', sand82-0800, sandia laboratories,  bb816167
c                 1982.                                                 bb816168
c***routines called  (none)                                             bb816169
c***end prologue  xerabt                                                bb816170
      character*(*) messg                                               bb816171
c***first executable statement  xerabt                                  bb816172
      stop                                                              bb816173
      end                                                               xerabt37
      subroutine xerctl(messg1,nmessg,nerr,level,kontrl)                xerctl 4
c***begin prologue  xerctl                                              xerctl 5
c***date written   790801   (yymmdd)                                    xerctl 6
c***revision date  861211   (yymmdd)                                    bb816184
c***category no.  r3c                                                   bb816185
c***keywords  library=slatec(xerror),type=all(xerctl-a),error           bb816186
c***author  jones, r. e., (snla)                                        bb816187
c***purpose  allow user control over handling of errors.                bb816188
c***description                                                         xerctl12
c                                                                       xerctl13
c     abstract                                                          xerctl14
c        allows user control over handling of individual errors.        xerctl15
c        just after each message is recorded, but before it is          xerctl16
c        processed any further (i.e., before it is printed or           xerctl17
c        a decision to abort is made), a call is made to xerctl.        xerctl18
c        if the user has provided his own version of xerctl, he         xerctl19
c        can then override the value of kontrol used in processing      xerctl20
c        this message by redefining its value.                          xerctl21
c        kontrl may be set to any value from -2 to 2.                   xerctl22
c        the meanings for kontrl are the same as in xsetf, except       xerctl23
c        that the value of kontrl changes only for this message.        xerctl24
c        if kontrl is set to a value outside the range from -2 to 2,    xerctl25
c        it will be moved back into that range.                         xerctl26
c                                                                       xerctl27
c     description of parameters                                         xerctl28
c                                                                       xerctl29
c      --input--                                                        xerctl30
c        messg1 - the first word (only) of the error message.           xerctl31
c        nmessg - same as in the call to xerror or xerrwv.              xerctl32
c        nerr   - same as in the call to xerror or xerrwv.              xerctl33
c        level  - same as in the call to xerror or xerrwv.              xerctl34
c        kontrl - the current value of the control flag as set          xerctl35
c                 by a call to xsetf.                                   xerctl36
c                                                                       xerctl37
c      --output--                                                       xerctl38
c        kontrl - the new value of kontrl.  if kontrl is not            xerctl39
c                 defined, it will remain at its original value.        xerctl40
c                 this changed value of control affects only            xerctl41
c                 the current occurrence of the current message.        xerctl42
c***references  jones r.e., kahaner d.k., 'xerror, the slatec error-    bb816189
c                 handling package', sand82-0800, sandia laboratories,  bb816190
c                 1982.                                                 bb816191
c***routines called  (none)                                             bb816192
c***end prologue  xerctl                                                bb816193
      character*20 messg1                                               bb816194
c***first executable statement  xerctl                                  xerctl49
      return                                                            xerctl50
      end                                                               xerctl51
      subroutine xerprt(messg,nmessg)                                   xerprt 4
c***begin prologue  xerprt                                              xerprt 5
c***date written   790801   (yymmdd)                                    xerprt 6
c***revision date  861211   (yymmdd)                                    bb816218
c***category no.  r3                                                    bb816219
c***keywords  library=slatec(xerror),type=all(xerprt-a),error           bb816220
c***author  jones, r. e., (snla)                                        bb816221
c***purpose  print error messages.                                      bb816222
c***description                                                         bb816223
c                                                                       bb816224
c     abstract                                                          bb816225
c        print the hollerith message in messg, of length nmessg,        bb816226
c        on each file indicated by xgetua.                              bb816227
c     latest revision ---  1 august 1985                                bb816228
c***references  jones r.e., kahaner d.k., 'xerror, the slatec error-    bb816229
c                 handling package', sand82-0800, sandia laboratories,  bb816230
c                 1982.                                                 bb816231
c***routines called  i1mach,xgetua                                      bb816232
c***end prologue  xerprt                                                bb816233
      integer lun(5)                                                    bb816234
      character*(*) messg                                               bb816235
      save f, g, la, lcom, lblank                                       bb816236
c     obtain unit numbers and write line to each unit                   bb816237
c***first executable statement  xerprt                                  bb816238
      call xgetua(lun,nunit)                                            bb816239
      lenmes = len(messg)                                               bb816240
      do 20 kunit=1,nunit                                               bb816241
         iunit = lun(kunit)                                             bb816242
         if (iunit.eq.0) iunit = i1mach(4)                              bb816243
         do 10 ichar=1,lenmes,72                                        bb816244
            last = min0(ichar+71 , lenmes)                              bb816245
            write (iunit,'(1x,a)') messg(ichar:last)                    bb816246
   10    continue                                                       bb816247
   20 continue                                                          bb816248
      return                                                            xerprt72
      end                                                               xerprt73
      subroutine xersav(messg,nmessg,nerr,level,icount)                 xersav 4
c***begin prologue  xersav                                              xersav 5
c***date written   800319   (yymmdd)                                    xersav 6
c***revision date  861211   (yymmdd)                                    bb816361
c***category no.  r3                                                    bb816362
c***keywords  library=slatec(xerror),type=all(xersav-a),error           bb816363
c***author  jones, r. e., (snla)                                        bb816364
c***purpose  record that an error has occurred.                         bb816365
c***description                                                         xersav12
c                                                                       xersav13
c     abstract                                                          xersav14
c        record that this error occurred.                               xersav15
c                                                                       xersav16
c     description of parameters                                         xersav17
c     --input--                                                         xersav18
c       messg, nmessg, nerr, level are as in xerror,                    xersav19
c       except that when nmessg=0 the tables will be                    xersav20
c       dumped and cleared, and when nmessg is less than zero the       xersav21
c       tables will be dumped and not cleared.                          xersav22
c     --output--                                                        xersav23
c       icount will be the number of times this message has             xersav24
c       been seen, or zero if the table has overflowed and              xersav25
c       does not contain this message specifically.                     xersav26
c       when nmessg=0, icount will not be altered.                      xersav27
c                                                                       xersav28
c     written by ron jones, with slatec common math library subcommitteexersav29
c     latest revision ---  1 august 1985                                bb816366
c***references  jones r.e., kahaner d.k., 'xerror, the slatec error-    bb816367
c                 handling package', sand82-0800, sandia laboratories,  bb816368
c                 1982.                                                 bb816369
c***routines called  i1mach,xgetua                                      bb816370
c***end prologue  xersav                                                bb816371
      integer lun(5)                                                    bb816372
      character*(*) messg                                               bb816373
      character*20 mestab(10),mes                                       bb816374
      dimension nertab(10),levtab(10),kount(10)                         bb816375
      save mestab,nertab,levtab,kount,kountx                            bb816376
c     next two data statements are necessary to provide a blank         bb816377
c     error table initially                                             bb816378
      data kount(1),kount(2),kount(3),kount(4),kount(5),                bb816379
     1     kount(6),kount(7),kount(8),kount(9),kount(10)                bb816380
     2     /0,0,0,0,0,0,0,0,0,0/                                        bb816381
      data kountx/0/                                                    bb816382
c***first executable statement  xersav                                  bb816383
      if (nmessg.gt.0) go to 80                                         bb816384
c     dump the table                                                    bb816385
         if (kount(1).eq.0) return                                      bb816386
c        print to each unit                                             bb816387
         call xgetua(lun,nunit)                                         bb816388
         do 60 kunit=1,nunit                                            bb816389
            iunit = lun(kunit)                                          bb816390
            if (iunit.eq.0) iunit = i1mach(4)                           bb816391
c           print table header                                          bb816392
            write (iunit,10)                                            bb816393
   10       format (32h0          error message summary/                bb816394
     1      51h message start             nerr     level     count)     bb816395
c           print body of table                                         bb816396
            do 20 i=1,10                                                bb816397
               if (kount(i).eq.0) go to 30                              bb816398
               write (iunit,15) mestab(i),nertab(i),levtab(i),kount(i)  bb816399
   15          format (1x,a20,3i10)                                     bb816400
   20       continue                                                    xersav79
   30       continue                                                    xersav80
c           print number of other errors                                xersav81
            if (kountx.ne.0) write (iunit,40) kountx                    xersav82
   40       format (41h0other errors not individually tabulated=,i10)   xersav83
            write (iunit,50)                                            xersav84
   50       format (1x)                                                 xersav85
   60    continue                                                       xersav86
         if (nmessg.lt.0) return                                        xersav87
c        clear the error tables                                         xersav88
         do 70 i=1,10                                                   xersav89
   70       kount(i) = 0                                                xersav90
         kountx = 0                                                     xersav91
         return                                                         xersav92
   80 continue                                                          xersav93
c     process a message...                                              xersav94
c     search for this messg, or else an empty slot for this messg,      xersav95
c     or else determine that the error table is full.                   xersav96
      mes = messg                                                       bb816401
      do 90 i=1,10                                                      xersav97
         ii = i                                                         xersav98
         if (kount(i).eq.0) go to 110                                   xersav99
         if (mes.ne.mestab(i)) go to 90                                 bb816402
         if (nerr.ne.nertab(i)) go to 90                                xersa101
         if (level.ne.levtab(i)) go to 90                               xersa102
         go to 100                                                      xersa103
   90 continue                                                          xersa104
c     three possible cases...                                           xersa105
c     table is full                                                     xersa106
         kountx = kountx+1                                              xersa107
         icount = 1                                                     xersa108
         return                                                         xersa109
c     message found in table                                            xersa110
  100    kount(ii) = kount(ii) + 1                                      xersa111
         icount = kount(ii)                                             xersa112
         return                                                         xersa113
c     empty slot found for new message                                  xersa114
  110    mestab(ii) = mes                                               bb816403
         nertab(ii) = nerr                                              xersa116
         levtab(ii) = level                                             xersa117
         kount(ii)  = 1                                                 xersa118
         icount = 1                                                     xersa119
         return                                                         xersa120
      end                                                               xersa121
      subroutine xgetua(iunita,n)                                       bb816414
c***begin prologue  xgetua                                              xgetua 5
c***date written   790801   (yymmdd)                                    xgetua 6
c***revision date  861211   (yymmdd)                                    bb816415
c***category no.  r3c                                                   bb816416
c***keywords  library=slatec(xerror),type=all(xgetua-a),error           bb816417
c***author  jones, r. e., (snla)                                        bb816418
c***purpose  return unit number(s) to which error messages are being    bb816419
c            sent.                                                      bb816420
c***description                                                         xgetua13
c                                                                       xgetua14
c     abstract                                                          xgetua15
c        xgetua may be called to determine the unit number or numbers   xgetua16
c        to which error messages are being sent.                        xgetua17
c        these unit numbers may have been set by a call to xsetun,      xgetua18
c        or a call to xsetua, or may be a default value.                xgetua19
c                                                                       xgetua20
c     description of parameters                                         xgetua21
c      --output--                                                       xgetua22
c        iunit - an array of one to five unit numbers, depending        xgetua23
c                on the value of n.  a value of zero refers to the      xgetua24
c                default unit, as defined by the i1mach machine         xgetua25
c                constant routine.  only iunit(1),...,iunit(n) are      xgetua26
c                defined by xgetua.  the values of iunit(n+1),...,      xgetua27
c                iunit(5) are not defined (for n .lt. 5) or altered     xgetua28
c                in any way by xgetua.                                  xgetua29
c        n     - the number of units to which copies of the             xgetua30
c                error messages are being sent.  n will be in the       xgetua31
c                range from 1 to 5.                                     xgetua32
c                                                                       xgetua33
c     latest revision ---  19 mar 1980                                  bb816421
c     written by ron jones, with slatec common math library subcommitteexgetua34
c***references  jones r.e., kahaner d.k., 'xerror, the slatec error-    bb816422
c                 handling package', sand82-0800, sandia laboratories,  bb816423
c                 1982.                                                 bb816424
c***routines called  j4save                                             bb816425
c***end prologue  xgetua                                                bb816426
      dimension iunita(5)                                               bb816427
c***first executable statement  xgetua                                  bb816428
      n = j4save(5,0,.false.)                                           bb816429
      do 30 i=1,n                                                       bb816430
         index = i+4                                                    bb816431
         if (i.eq.1) index = 3                                          bb816432
         iunita(i) = j4save(index,0,.false.)                            bb816433
   30 continue                                                          xgetua48
      return                                                            xgetua49
      end                                                               xgetua50
      subroutine fdump                                                  fdump  4
c***begin prologue  fdump                                               fdump  5
c***date written   790801   (yymmdd)                                    fdump  6
c***revision date  861211   (yymmdd)                                    bb815146
c***category no.  r3                                                    bb815147
c***keywords  library=slatec(xerror),type=all(fdump-a),error            bb815148
c***author  jones, r. e., (snla)                                        bb815149
c***purpose  symbolic dump (should be locally written).                 bb815150
c***description                                                         fdump 12
c                                                                       fdump 13
c        ***note*** machine dependent routine                           fdump 14
c        fdump is intended to be replaced by a locally written          fdump 15
c        version which produces a symbolic dump.  failing this,         fdump 16
c        it should be replaced by a version which prints the            fdump 17
c        subprogram nesting list.  note that this dump must be          fdump 18
c        printed on each of up to five files, as indicated by the       fdump 19
c        xgetua routine.  see xsetua and xgetua for details.            fdump 20
c                                                                       fdump 21
c     written by ron jones, with slatec common math library subcommitteefdump 22
c***references  (none)                                                  bb815151
c***routines called  (none)                                             bb815152
c***end prologue  fdump                                                 bb815153
c***first executable statement  fdump                                   fdump 30
      return                                                            fdump 31
      end                                                               fdump 32

